/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CActuators class
 * 				The CActuators class is responsible for producing the agent's actions.
 * Created: 17/12/98
 * Last modified: 23/7/99
 */

#include <stdlib.h>
#include <sys/time.h> 
#include <unistd.h>
#include <iostream.h>
#include <stdio.h>		// Used for the sprintf function call
#include "CBall.H"
#include "CActuators.H"

// Constructor that tales Mediator ////////////////////////////////
CActuators::CActuators(CMediator* Mediator):CModule(Mediator)
{
cout << "[Creating Legs                    ]" << endl;
m_SendBuffer[0] = '\0';				// Initialise m_SendBuffer
pthread_mutex_init(&m_Mutex, NULL);	// Initialise mutex
pthread_cond_init(&m_Cond, NULL);	// Initialise condition variable
m_IsNewAction = 0;					// Initialise new action predicate
m_Said = 0;							// Initialise said messages to 0

// Initialise here for safety reasons...  Values read from the configuration file will overwritte these
m_SimStep = 10000000;
m_RecvStep = 10000;
}

// DISPLAY ////////////////////////////////////////////////////////
void CActuators::Display() const	// Displays all params.  Just for debugging
{
cout << "ACTUATORS Buffer: " << m_SendBuffer << endl;
}

// Lock/Unlock and signal functions ////////////////////////////////
// LOCK ////////////////////////////////////////////////////
void CActuators::Lock()		// Lock Actuators
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING Actuators..." << endl;
#endif

status = pthread_mutex_lock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CActuators: Lock: Locking error..." << endl;
}

// UNLOCK //////////////////////////////////////////////////
void CActuators::Unlock()	// Unlock Actuators
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING Actuators..." << endl;
#endif

status = pthread_mutex_unlock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CActuators: Unlock: Unlocking error..." << endl;
}

// SIGNAL //////////////////////////////////////////////////
void CActuators::Signal()		// Signal the condition variable
{
int status;						// Signal status

status = pthread_cond_signal(&m_Cond);
if (status != 0)
	cerr << "WARNING: CActuators: Signal: Signaling error..." << endl;
}

// ABORT ///////////////////////////////////////////////
void CActuators::Abort(char* Msg, int Code)	// Display error message and exit
{
cerr << GetTime() << " Agent " << GetUNum() << " Actuators: " << Msg << endl;
DoExit(Code);
}

// ACTUATORTHREAD //////////////////////////////////////
void CActuators::ActuatorThread()	// The thread that keeps sending actions to the server
{
cout << "ACTUATORS thread started!" << endl;

struct timeval tv;
struct timespec ts;
int sent, status;
const CBall* pBall = 0;	// Pointer to ball data

while (GetStatus() == 1)
	{
	Lock();								// Lock mutex
	//cout << GetTime() << " : Starting Actuator Cycle --------------" << endl;
	sent = -999;						// Initialise sent for each loop
	if (gettimeofday(&tv, NULL) < 0)	// If gettimeofday fails, display error
			cerr << "WARNING: ActuatorThread: gettimeofday returned error..." << endl;	// and display warning
	
	ts.tv_sec = tv.tv_sec;				// Convert and store the timeval to
	ts.tv_nsec = tv.tv_usec * 1000;		// the timespec structure
		
	ts.tv_nsec += m_RecvStep * 1000;	// and increase by server's polling time
	
	if (m_IsNewAction == 1)				// If skipping predicate
		{
		//cout << "Skipping CACTUATORS PREDICATE!" << endl;
		#ifdef DEBUG_PREDS
		cerr << "WARNING: CActuators: ActuatorThread: Skipping actuator predicate at: " << GetTime() << endl;
		#endif
		//usleep(20000);					// wait !!!
		//m_IsNewAction = 0;				// and force entry into the loop
		}
	

	while (m_IsNewAction == 0)										// Test Predicate
		{
		//cout << "In actuators, BEFORE cond wait m_SendBuffer is:" << m_SendBuffer << endl;
		status = pthread_cond_timedwait(&m_Cond, &m_Mutex, &ts);	// Wait 100msec on condition variable
		// If timed out, it's time to send an action
		// If signalled it means that current action can go on this cycle...(i.e. change_view, turn_neck, etc.)
		if (status == ETIMEDOUT)
			{
			//cout << "Status ETIMEDOUT" << endl;
			break;
			}
		else if (status != 0)					// If error occured
			{
			cerr << GetTime() << " WARNING: CActuators: ActuatorThread: Conditional wait error..." << endl;
			break;
			}
		else									// It has been signalled
			{
			//cout << "Status: " << status << endl;
			break;
			}
		}
		//cout << "In actuators, AFTER cond wait m_SendBuffer is:" << m_SendBuffer << endl;
		
	if (m_SendBuffer[0] != '\0')				// If action in the buffer
		{
		#ifdef DEBUG_ACTUATORS
		if ((GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))
			cout << GetTime() << " SENDING: " << m_SendBuffer;
		#endif
		sent = SendMessage(m_SendBuffer);		// then send it
		}
	else										// else (this is an idle cycle...do what you like)
		{
		SendMessage(m_SendBuffer);				// send NULL to test server is still up	
		sent = 1;								// We don't need to increase action count or copy buffer for this action
		}			
		
	if (sent == -1)								// If message NOT sent, inform the user
		{
		cerr << GetTime() << " ERROR: CActuators: SensorThread: Problem while sending data...Has server died?" << endl;
		}
	else if (sent == 0)							// else update Memory for the action just sent
		{
		LockAgent();							// Lock Agent's mutex
		SetLastAction(GetActionCount(), m_SendBuffer);		// Update Agent for the current action
		SetActionCount(GetActionCount() + 1);	// Increase action count
		if (GetActionCount() >= MAXACTIONCOUNT)	// Just a safeguard, but if this happens, we are in serious trouble
			{
//			if ((GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))	// This however will happen at time 0, 3000, 6000 so skip the warning!
//				cerr << GetTime() << " WARNING: Critical Malfunction !!! Action count reached maximal count..." << endl;
			SetActionCount(MAXACTIONCOUNT - 1);
			}
		UnlockAgent();							// Unlock Agent's mutex
		m_SendBuffer[0] = '\0';					// Reset m_SendBuffer
		}
		
	Changed();									// Report the change in the mediator (i.e. the Agent)
	Unlock();									// Unlock mutex
	}
	
CloseConnection();								// and close connection
pthread_exit(NULL);								// Exit this thread
}
