/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CActuators class
 * 				The CActuators class is responsible for producing the agent's actions.
 * Created: 17/12/98
 * Last modified: 23/7/99
 */

#ifndef _CACTUATORS_H
#define _CACTUATORS_H

#include <string.h>
#include <pthread.h>
#include <errno.h>
#include "CModule.H"

class CActuators : public CModule
{
// Private member data ///////////////////////////////////////////
private:

	pthread_mutex_t	m_Mutex;				// Actuators's mutex
	pthread_cond_t m_Cond;					// Actuator's condition variable
	int		m_IsNewAction;					// New action predicate
	
	Socket	m_Socket;						// The UDP socket
	char 	m_SendBuffer[MAXSENDDATALEN];	//Send buffer
	long	m_SimStep;						// Simulator step
	int		m_RecvStep;						// Time step for acception of commands
	
	int		m_Said;							// Number of messages said this cycle

// Public member functions ////////////////////////////////////////
public:

// Constructor, Display and GetThis /////////////////////////////
	CActuators(CMediator*);			// Constructor that takes Mediator
	void Display() const;	// Displays all params.  Just for debugging
	CActuators *GetThis()	// Function that returns the "this" pointer.
	{ return this; }		// Used in the externs to start the threads.
	
// Lock/Unlock and signal functions ////////////////////////////////////
	void Lock();					// Lock Actuators
	void Unlock();					// Unlock Actuators
	void Signal();					// Signal the condition variable
	
// Error reporting functions ////////////////////////////////////////
	void	Abort	(char*, int);	// Display message and exit with a given code
	
// Basic actuator attribute get/set functions //////////////////////////

	void SetIsNewAction (int IsNewAction)		// Set m_IsNewAction
	{ m_IsNewAction = IsNewAction; }
	int GetIsNewAction	() const				// Get m_IsNewAction
	{ return m_IsNewAction; }
	
	void    SetSocket		(Socket Sock)	// Store socket info to m_Sock
	{ m_Socket = Sock; }
	Socket  GetSocket		() const		// Get socket info from m_Sock
	{ return m_Socket; }
	
	void	SetSendBuffer	(const char* Buffer)	//	Set SendBuffer
	{ strncpy(m_SendBuffer, Buffer, MAXSENDDATALEN); }
	const char*	GetSendBuffer	()					//	Get SendBuffer
	{ return m_SendBuffer; }
	
	void SetSimStep(long SimStep)			// Set simulator step
	{ m_SimStep = SimStep; }
	long GetSimStep() const					// Get simulator step
	{ return m_SimStep; }
	
	void SetRecvStep (int RecvStep)			// Set receive step (server's polling time)
	{ m_RecvStep = RecvStep; }
	int GetRecvStep	() const				// Get receive step (server's polling time)
	{ return m_RecvStep; }
	
	void SetSaid (int Said)					// Set number of messages said this cycle
	{ m_Said = Said; }
	int GetSaid	() const					// Get number of messages said this cycle
	{ return m_Said; }
	
// Communication functions //////////////////////////////////////////
	Socket	InitConnection	(const char*, int) const;	// Open connection with the server
	void	CloseConnection	() const;					// Close connection with the server

	int		SendMessage		(char*) const;				// Send message to the server

// Send actions to server functions /////////////////////////////////
	void	ActuatorThread	();		// The thread that keeps sending actions to the server

};
 
#endif
