/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CBall class
 * 				The CBall class holds information regarding a ball on the field.
 * Created: 14/12/98
 * Last modified: 12/7/99
 */

#include <iostream.h> 
#include "CBall.H"

// Default constructor /////////////////////////////////////
CBall::CBall()
{
m_Time = -999;
m_RPos = CVector(0, 0);
m_DistChng = 0.0;
m_DirChng = 0.0;
m_Close = 0;
m_Conf = 0.0;

m_GPos = CVector(0, 0);
m_Velocity = CVector(0, 0);

m_VelocityValid = 0;
m_PositionValid = 0;
m_LastCatchCycle = 0;
}

// Default destructor //////////////////////////////////////
CBall::~CBall()
{
}

// INIT ///////////////////////////////////////////////////
void CBall::Init(float Size, float Decay, float Rand, float Weight, float SpeedMax, float Kickable, float MobConfThr
, int CatchBanCycle, float CatchAreaLength, float CatchAreaWidth)
{
m_Size = Size;
m_Decay = Decay;
m_Rand = Rand;
m_Weight = Weight;
m_SpeedMax = SpeedMax;
m_Kickable = Kickable;
m_MobConfThr = MobConfThr;
m_CatchBanCycle = CatchBanCycle;
m_CatchAreaLength = CatchAreaLength;
m_CatchAreaWidth = CatchAreaWidth;
}

// DISPLAY ////////////////////////////////////////////////
void CBall::Display() const		// Display all params.  Just for debugging
{
cout << "\tBALL Time: " << m_Time << endl;
cout << "\tBALL RDist: " << m_RPos.GetMag() << endl;
cout << "\tBALL RDir: " << m_RPos.GetDir() << endl;
cout << "\tBALL DistChng: " << m_DistChng << endl;
cout << "\tBALL DirChng: " << m_DirChng << endl;
cout << "\tBALL Conf: " << m_Conf << endl;
cout << "\tBALL GX: " << m_GPos.GetX() << endl;
cout << "\tBALL GY: " << m_GPos.GetY() << endl;
cout << "\tBALL VX: " << m_Velocity.GetX() << endl;
cout << "\tBALL VY: " << m_Velocity.GetY() << endl;
cout << "\tBALL Mag: " << m_Velocity.GetMag() << endl;
cout << "\tBALL Dir: " << m_Velocity.GetDir() << endl;
cout << "\tBALL VelocityValid: " << m_VelocityValid << endl;
cout << "\tBALL PositionValid: " << m_PositionValid << endl;
}

// UPDATERPOS ///////////////////////////////////////////////////////////////
void CBall::UpdateRPos(CVector GPos, float GFacing, int Time)	// Update ball's relative position
{
/*
	Both these tests are necessary here (rather than just the time test)
	because the looking at behaviours might oscilate doing 80, -80, 80...and so
	on turns trying to locate an object
*/

if ((m_Time < Time) && (m_Conf > m_MobConfThr))		// If object remembered but not seen NOW
	{
	m_RPos = m_GPos - GPos;							// Update it's relative position
	m_RPos = m_RPos.Rotate(-GFacing);
	m_Time = Time;
	}
}

// GETRELATIVEVELOCITY ////////////////////////////////////
CVector CBall::GetRelativeVelocity(CVector MyVelocity, float GFacing) const		// Get ball's relative velocity vector
{
CVector tmp = m_Velocity;		// Copy balls absolute velocity to a temp vector
//tmp -= MyVelocity;		// Adding that line gives us the true relative velocity
tmp = tmp.Rotate(-GFacing);		// Otherwise we get absolute speed, relative direction
//cout << "BALL GRV- X, Y, Mag, Dir: " << tmp.GetX() << ", " << tmp.GetY() << ", " << tmp.GetMag() << ", " 
//	<< tmp.GetDir() << endl;
return tmp;
}

// ISCATCHABLE ////////////////////////////////////////////
int	CBall::IsCatchable(CVector GPos, float Direction, int Time)	// Return true if ball can be caught
{																// Direction is BodyDir+CatchDir
/*
	Maybe here it would be enough to do 
		if (dist < m_CatchAreaLength) return true; return false;
	Leave it like this for now since we are out of time !
*/
CVector temp;											// temp vector to store ball global after the rotation
float Left, Right, Top, Bottom;							// The edges of the rectangle we want the ball to be within

if ((Time - m_LastCatchCycle) <= m_CatchBanCycle)		// If last catch was recent, another one is not allowed yet
	return 0;											// return 0

/*
	Initialise edges so that our catch area looks like
	
	**********	|
	G*********	W   ---> DIR
	**********	|

	|<---L--->|
	
	REMEMBER: The direction of the catchable rectangle (DIR)
	is the goalie's BODY direction PLUS the direction of the
	CATCH command.  The NECK direction is IRRELEVANT.
*/				

Left = 0;
Right = m_CatchAreaLength;
Top = -(m_CatchAreaWidth/2);
Bottom = m_CatchAreaWidth/2;

// Now we need to rotate the ball based on our global direction (body_dir + catch_dir)
temp = m_GPos - GPos;
temp = temp.Rotate(-Direction);

	
// Now we need to check if ball is within the catchable area and return true if it is, false otherwise
return ((temp.GetX() >= Left) && (temp.GetX() <= Right) && (temp.GetY() >= Top) && (temp.GetY() <= Bottom));
}
