/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CBall class
 * 				The CBall class holds information regarding a ball on the field.
 * Created: 14/12/98
 * Last modified: 10/7/99
 */

#ifndef _CBALL_H
#define _CBALL_H

#include "CVector.H"
 
class CBall
{
// Private member data ///////////////////////////////////
private:
// These are the ball default values in server.conf and agent.conf
	float	m_Size;
	float	m_Decay;
	float	m_Rand;
	float	m_Weight;
	float	m_SpeedMax;
	float	m_MobConfThr;
	int		m_CatchBanCycle;
	float	m_CatchAreaLength;
	float	m_CatchAreaWidth;
	
// These values are obtained from sensors ////////////////
	int 	m_Time;			// Time of observation
	CVector	m_RPos;			// Relative position
	float	m_DistChng;		// Distance change
	float	m_DirChng;		// Direction change
	int		m_Close;		// Is ball too close (i.e. Ball or ball)
		
// These values are calculated ///////////////////////////
	float 	m_Conf;			// Confidence
	CVector m_GPos;			// Global position

	CVector m_Velocity;		// Ball's global velocity
	
	int		m_VelocityValid;		// Is our velocity info valid
	int		m_PositionValid;		// Is our position info valid

	float	m_Kickable;				// Max kickable range (kickable_margin + ball_size + player_size)	
	int		m_LastCatchCycle;		// The last cycle that we send a catch command
	
// Public member functions //////////////////////////////////
public:

// Constructor/Destructor, Init, and Display functions ///////////////
	CBall();				// Default constructor
	~CBall();				// Default destructor
	void Init(float, float, float, float, float, float, float, int, float, float);	// Used to initialise ball
	void Display() const;	// Displays all params.  Just for debugging
	const CBall *GetThis() const	// Function that returns the "this" pointer
	{ return this; }				// Used by Get*** in Memory
	CBall *GetNonConstThis()		// Function that returns a non-const "this" pointer
	{ return this; }				// Used by Intercept in Behaviours when ball needs to be modelled
		
// Basic ball data set/get functions /////////////////////////////////	
	void	SetTime	(int Time)			// Set ball's time
	{ m_Time = Time; }
	int		GetTime	() const			// Get ball's time
	{ return m_Time; }
	
	void	SetRPos	(CVector RPos)		// Set ball's relative position
	{ m_RPos = RPos; }
	CVector	GetRPos	() const			// Get ball's relative position
	{ return m_RPos; }
	void	UpdateRPos(CVector, float, int);	// Update ball's relative position if not seen

// Accessor function for RPOS ///////////////////////////////////////
	
	void	SetRDir	(float	Dir)		// Set relative direction
	{ m_RPos.SetDir(Dir); }
	float	GetRDir	() const			// Get relative direction
	{ return m_RPos.GetDir(); }
	
	void	SetRMag	(float	Mag)		// Set relative magnitude
	{ m_RPos.SetMag(Mag); }
	float	GetRMag	() const			// Get relative magnitude
	{ return m_RPos.GetMag(); }

	void	SetRX	(float	X)			// Set relative X co-ordinate
	{ m_RPos.SetX(X); }
	float	GetRX	() const			// Get relative X co-ordinate
	{ return m_RPos.GetX(); }	
	
	void	SetRY	(float	Y)			// Set relative Y co-ordinate
	{ m_RPos.SetY(Y); }
	float	GetRY	() const			// Get relative Y co-ordinate
	{ return m_RPos.GetY(); }	
	
/////////////////////////////////////////////////////////////////////

	void	SetDistChng	(float DistChng)// Set ball's distance change
	{ m_DistChng = DistChng; }
	float	GetDistChng	() const		// Get ball's distance change
	{ return m_DistChng; }
	
	void	SetDirChng	(float DirChng)	// Set ball's direction change
	{ m_DirChng = DirChng; }
	float	GetDirChng	() const		// Get ball's direction change
	{ return m_DirChng; }
	
	void	SetClose	(int Close)		// Set ball's close attribute
	{ m_Close = Close; }
	int	GetClose	() const			// Get ball's close attribute
	{ return m_Close; }
	
	void	SetConf	(float Conf)		// Set ball's confidence
	{ m_Conf = Conf; }
	float	GetConf	() const			// Get ball's confidence
	{ return m_Conf; }

	void	SetGPos	(CVector GPos)		// Set ball's global position
	{ m_GPos = GPos; }
	CVector	GetGPos	() const			// Get ball's global position
	{ return m_GPos; }
	
// Accessor function for GPOS ///////////////////////////////////////
	void	SetGDir	(float	Dir)		// Set relative direction
	{ m_GPos.SetDir(Dir); }
	float	GetGDir	() const			// Get relative direction
	{ return m_GPos.GetDir(); }
	
	void	SetGMag	(float	Mag)		// Set relative magnitude
	{ m_GPos.SetMag(Mag); }
	float	GetGMag	() const			// Get relative magnitude
	{ return m_GPos.GetMag(); }

	void	SetGX	(float	X)			// Set global X co-ordinate
	{ m_GPos.SetX(X); }
	float	GetGX	() const			// Get global X co-ordinate
	{ return m_GPos.GetX(); }	
	
	void	SetGY	(float	Y)			// Set global Y co-ordinate
	{ m_GPos.SetY(Y); }
	float	GetGY	() const			// Get global Y co-ordinate
	{ return m_GPos.GetY(); }	
		
/////////////////////////////////////////////////////////////////////	
	
	void	SetVelocity	(CVector CV)	// Set ball's global velocity
	{ m_Velocity = CV; }
	CVector GetVelocity	() const		// Get ball's global velocity
	{ return m_Velocity; }

// Accessor function for Velocity ///////////////////////////////////////
	void	SetDir	(float	Dir)		// Set velocity direction
	{ m_Velocity.SetDir(Dir); }
	float	GetDir	() const			// Get velocity direction
	{ return m_Velocity.GetDir(); }
	
	void	SetMag	(float	Mag)		// Set velocity magnitude
	{ m_Velocity.SetMag(Mag); }
	float	GetMag	() const		// Get velocity magnitude
	{ return m_Velocity.GetMag(); }
/////////////////////////////////////////////////////////////////////

	void	SetVelocityValid	(int Value)				// Set velocity valid
	{ m_VelocityValid = Value; }
	int		GetVelocityValid	() const				// Get velocity valid
	{ return m_VelocityValid; }

	void	SetPositionValid	(int Value)				// Set position valid
	{ m_PositionValid = Value; }
	int		GetPositionValid	() const				// Get position valid
	{ return m_PositionValid; }

	void	SetLastCatchCycle	(int LastCatchCycle)	// Set last catch cycle
	{ m_LastCatchCycle = LastCatchCycle; }
	int		GetLastCatchCycle	() const				// Get last catch cycle
	{ return m_LastCatchCycle; }

// Additional ball functions /////////////////////////////////////////////
	CVector	GetRelativeVelocity	(CVector, float) const;		// Get ball's relative velocity vector given my global velocity and facing
	int		IsKickable			() const					// Return true if ball is within kicking range
	{ return (m_RPos.GetMag() < m_Kickable); }
	int		IsCatchable			(CVector, float, int);// Return true if ball can be caught
	
};
 
#endif
