/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CBehaviours class
 * 				The CBehaviours class is responsible for all the agent's thinking.
 * Created: 17/12/98
 * Last modified: 19/7/99
 */

#include <iostream.h>
#include "CGetParams.H" 
#include "CBehaviours.H"

// Constructor that takes Mediator ////////////////////////////
CBehaviours::CBehaviours(CMediator* Mediator):CModule(Mediator)
{
cout << "[Creating Brain                   ]" << endl;
m_ActionBuffer[0] = '\0';					// Initialise m_ActionBuffer
pthread_mutex_init(&m_Mutex, NULL);			// Initialise mutex
pthread_cond_init(&m_Cond, NULL);			// Initialise condition variable
pthread_mutex_init(&m_ExecMutex, NULL);		// Initialise Exec mutex
pthread_cond_init(&m_ExecCond, NULL);		// Initialise Exec condition variable
pthread_mutex_init(&m_NewCycleMutex, NULL);	// Initialise NewCycle mutex
pthread_cond_init(&m_NewCycleCond, NULL);	// Initialise NewCycle condition variable
m_IsNewVisual = 0;							// Initialise m_IsNewVisual
m_IsNewCycle = 0;							// Initialise m_IsNewCycle

if (MCPLPosLibInit() < 0)				// Try and read findpos.ini, and if that failed ABORT
	Abort("Failed to read findpos.ini file.  Make sure file is in the current directory and file is readable.", 2);

// Initialise positioning pointers to NULL
m_GPS334N = 0;
m_GPS334Defend = 0;
m_GPS334BallTracker = 0;
m_GPS334OffsideTrap = 0;

m_GPS433N = 0;
m_GPS433Defend = 0;
m_GPS433BallTracker = 0;
m_GPS433OffsideTrap = 0;
		
m_GPS532N = 0;
m_GPS532Defend = 0;
m_GPS532BallTracker = 0;
m_GPS532OffsideTrap = 0;

// Initialise here for safety reasons...  Values read from the configuration file will overwritte these
m_SimStep = 100000000;

m_FormationID = 0;

m_ScoreDiff = 0;
m_ChangeFormationThr = 3;
m_LosingFormation = 334;
m_NormalFormation = 433;
m_WinningFormation = 532;

m_PitchLength = 105.0;
m_PitchWidth = 68.0;
m_PlayerSize = 0.3;
m_BallSize = 0.085;
m_PlayerDecay = 0.4;
m_BallDecay = 0.94;
m_PlayerSpeedMax = 1.0;
m_BallSpeedMax = 2.7;
m_KickableMargin = 0.7;
m_Kickable = m_KickableMargin + m_PlayerSize + m_BallSize;
m_MaxPower = 100;
m_MinPower = -100;
m_MaxMoment = 180;
m_MinMoment = -180;

m_MaxNeckMoment = 180;
m_MinNeckMoment = -180;
m_MaxNeckAng = 90;
m_MinNeckAng = -90;

m_InertiaMoment = 5.0;
m_KickPowerRate = 0.016;
m_DashPowerRate = 0.006;
m_RecvStep = 10000;

// Optimal control distance = PlayerSize + 0.8*KickableMargin
m_OptCtrlDist = m_PlayerSize + 0.8 * m_KickableMargin;
m_ClosestDist = m_PlayerSize + 2.0 * m_BallSize;
m_KickFactor = 0.22;

m_StopPass = 0.0;
m_SoftPass = 0.3;
m_MediumPass = 0.7;
m_HardPass = 1.2;

m_Epsilon = 0.0001;
m_StatConfDrop = 0.95;
m_StatConfThr = 0.50;
m_MobConfDrop = 0.90;
m_MobConfThr = 0.40;
m_Slow = 20.0;
m_Normal = 50.0;
m_Fast = 100.0;
m_MaxGoAngle = 5.0;
m_MinDistToPoint = 1.0;
m_MaxMoveDashes = 3;
m_MaxInterceptDashes = 1;
m_InterceptStep = 2;
m_InterceptTol = 0.15;
m_SpinDirTol = 5;
m_MaxSpinPower = 20.0;
m_MaxStopSpeed = 0.2;
m_MinOppWorryDist = 5;
m_DodgeDistBuf = 3.5;
m_DribbleDodgeCloseDist = 2;
m_DribbleDodgeAngleErr = 15;
m_BallDribbleDist = 0.7;
m_MaxDribbleExpAngle = 10;
m_DribbleNormalAngle = 60.0;
m_CollisionBuffer = 0.1;
}

// INIT /////////////////////////////////////////////////////////////
void CBehaviours::Init(CGetParams BehaviourParams)	// Initialise all fields using BehaviourParams
{
m_SimStep = BehaviourParams.GetAP_sim_step();					// Read simulation step from params

m_ScoreDiff = BehaviourParams.GetAP_score_diff();				// Read score difference from params
m_ChangeFormationThr = BehaviourParams.GetAP_change_formation_thr();// Threashold value for m_ScoreDiff used to change between formations from params

SetScoreDiff(m_ScoreDiff);		// Call that so that we have an up to date fomration ID before any goals

m_LosingFormation = BehaviourParams.GetAP_losing_formation();	// Read formation when losing from params
m_NormalFormation = BehaviourParams.GetAP_normal_formation();	// Read normal formation from params
m_WinningFormation = BehaviourParams.GetAP_winning_formation();	// Read formation when winning from params

m_PitchLength = BehaviourParams.GetSP_pitch_length();			// Pitch length from params
m_PitchWidth = BehaviourParams.GetSP_pitch_width();				// Pitch width from params		
m_PlayerSize = BehaviourParams.GetSP_player_size();				// Read player size from params
m_BallSize = BehaviourParams.GetSP_ball_size();					// Read ball size from params
m_PlayerDecay = BehaviourParams.GetSP_player_decay();			// Player decay from params
m_BallDecay = BehaviourParams.GetSP_ball_decay();				// Ball decay from params
m_PlayerSpeedMax = BehaviourParams.GetSP_player_speed_max();	// Max speed for players from params
m_BallSpeedMax = BehaviourParams.GetSP_ball_speed_max();		// Max speed for ball from params
m_KickableMargin = BehaviourParams.GetSP_kickable_margin();		// Kickable margin from params
m_Kickable =  m_KickableMargin + m_BallSize + m_PlayerSize;		// Calculate max kickable range
m_MaxPower = BehaviourParams.GetSP_maxpower();					// Read max power from params
m_MinPower = BehaviourParams.GetSP_minpower();					// Read min power from params
m_MaxMoment = BehaviourParams.GetSP_maxmoment();				// Read max moment from params
m_MinMoment = BehaviourParams.GetSP_minmoment();				// Read min moment from params

m_MaxNeckMoment = BehaviourParams.GetSP_maxneckmoment();		// Read max turn neck moment from params;
m_MinNeckMoment = BehaviourParams.GetSP_minneckmoment();		// Read min turn neck moment from params;
m_MaxNeckAng = BehaviourParams.GetSP_maxneckang();				// Read max neck angle from params;
m_MinNeckAng = BehaviourParams.GetSP_minneckang();				// Read min neck angle from params;

m_InertiaMoment = BehaviourParams.GetSP_inertia_moment();		// Read inertia moment from params
m_KickPowerRate = BehaviourParams.GetSP_kick_power_rate();		// Read kick power rate from params
m_DashPowerRate = BehaviourParams.GetSP_dash_power_rate();		// Dash power rate from params
m_RecvStep = BehaviourParams.GetSP_recv_step() * 1000;			// Receive step from params (polling time by the server)
																// converted to msec
// Optimal control distance = PlayerSize + 0.8*KickableMargin
m_OptCtrlDist = m_PlayerSize + 0.8 * m_KickableMargin;
m_ClosestDist = m_PlayerSize + 2.0 * m_BallSize;				// The closest distance the ball can get (= PlayerSize + 2*BallSize)
m_KickFactor = 0.22;			// Kick factor

m_StopPass = BehaviourParams.GetAP_stop_pass();					// Target velocity for passing from params
m_SoftPass = BehaviourParams.GetAP_soft_pass();					//
m_MediumPass = BehaviourParams.GetAP_medium_pass();				//
m_HardPass = BehaviourParams.GetAP_hard_pass();					//

m_Epsilon = BehaviourParams.GetAP_epsilon();					// Get epsilon from params
m_StatConfDrop = BehaviourParams.GetAP_stat_conf_drop();		// Rate at which confidence of stationary objects drops after each cycle from params
m_StatConfThr = BehaviourParams.GetAP_stat_conf_thr();			// Minimum confidence threshold for stationary objects from params
m_MobConfDrop = BehaviourParams.GetAP_mob_conf_drop();			// Rate at which confidence of mobile objects drops after each cycle from params
m_MobConfThr = BehaviourParams.GetAP_mob_conf_thr();			// Minimum confidence threshold for mobile objects from params
m_Slow = BehaviourParams.GetAP_slow();							// Slow dash power from params
m_Normal = BehaviourParams.GetAP_normal();						// Normal dash power from params
m_Fast = BehaviourParams.GetAP_fast();							// Fast dash power from params
m_MaxGoAngle = BehaviourParams.GetAP_max_go_angle();			// If target point less than X degrees, don't turn from params
m_MinDistToPoint = BehaviourParams.GetAP_min_dist_to_point();	// In MoveToPos or Chase, if target point is closer than X meters, consider it ok from params
m_MaxMoveDashes = BehaviourParams.GetAP_max_move_dashes();		// Maximum dashes in a MoveToPos behaviour from params
m_MaxInterceptDashes = BehaviourParams.GetAP_max_intercept_dashes();// Maximum dashes in a MoveToIntercept behaviour (without seeing the ball) from params
m_InterceptStep = BehaviourParams.GetAP_intercept_step();		// Intercept cycle step from params
m_InterceptTol = BehaviourParams.GetAP_intercept_tol();			// Intercept tolerance from params
m_SpinDirTol = BehaviourParams.GetAP_spin_dir_tol();			// Direction +/- spin_dir_tol is used to determine if ball is at the right place from params
m_MaxSpinPower = BehaviourParams.GetAP_max_spin_power();		// Power used in kick when spinning ball round body from params
m_MaxStopSpeed = BehaviourParams.GetAP_max_stop_speed();		// If ball is going slower than that, consider it stopped from params
m_MinOppWorryDist = BehaviourParams.GetAP_min_opp_worry_dist();	// If an opponent is closer than that, we worry about it from params
m_DodgeDistBuf = BehaviourParams.GetAP_min_opp_worry_dist();	// Dodge distance buffer from params
m_DribbleDodgeCloseDist = BehaviourParams.GetAP_dribble_dodge_close_dist();// Dribble dodge close distance from params
m_DribbleDodgeAngleErr = BehaviourParams.GetAP_dribble_dodge_angle_err();// Dribble dodge angle error from params
m_BallDribbleDist = BehaviourParams.GetAP_ball_dribble_dist();	// Dribble ball dist (if you change that, Dribble might NOT work) from params
m_MaxDribbleExpAngle = BehaviourParams.GetAP_max_dribble_exp_angle();// Used when dribbling to check if ball will end up behind us from params
m_DribbleNormalAngle = BehaviourParams.GetAP_dribble_normal_angle();// Normal dribble angle from params
m_CollisionBuffer = BehaviourParams.GetAP_collision_buffer();// Collision buffer used by WillXXXBeCollision functions from params
}

// DISPLAY ///////////////////////////////////////////////////
void CBehaviours::Display() const		// Displays all params.  Just for debugging
{
}

// Lock/Unlock and signal functions ////////////////////////////////////
// LOCK ////////////////////////////////////////////////////
void CBehaviours::Lock()	// Lock Behaviours
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING Behaviours..." << endl;
#endif

status = pthread_mutex_lock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: Lock: Locking error..." << endl;
}

// UNLOCK //////////////////////////////////////////////////
void CBehaviours::Unlock()	// Unlock Behaviours
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING Behaviours..." << endl;
#endif

status = pthread_mutex_unlock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: Unlock: Unlocking error..." << endl;
}

// SIGNAL //////////////////////////////////////////////////
void CBehaviours::Signal()		// Signal the condition variable
{
int status;						// Signal status

status = pthread_cond_signal(&m_Cond);
if (status != 0)
	cerr << "WARNING: CBehaviours: Signal: Signaling error..." << endl;
}

// EXECLOCK ////////////////////////////////////////////////////
void CBehaviours::ExecLock()	// ExecLock Behaviours
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING Exec in Behaviours..." << endl;
#endif

status = pthread_mutex_lock(&m_ExecMutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: ExecLock: Locking error..." << endl;
}

// EXECUNLOCK //////////////////////////////////////////////////
void CBehaviours::ExecUnlock()	// ExecUnlock Behaviours
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING Exec in Behaviours..." << endl;
#endif

status = pthread_mutex_unlock(&m_ExecMutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: ExecUnlock: Unlocking error..." << endl;
}

// EXECSIGNAL //////////////////////////////////////////////////
void CBehaviours::ExecSignal()		// Signal the Exec condition variable
{
int status;						// Signal status

status = pthread_cond_signal(&m_ExecCond);
if (status != 0)
	cerr << "WARNING: CBehaviours: ExecSignal: Signaling error..." << endl;
}

// NEWCYCLELOCK ////////////////////////////////////////////////////
void CBehaviours::NewCycleLock()// NewCycle lock Behaviours
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING NewCycle in Behaviours..." << endl;
#endif

status = pthread_mutex_lock(&m_NewCycleMutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: NewCycleLock: Locking error..." << endl;
}

// NEWCYCLELOCK //////////////////////////////////////////////////
void CBehaviours::NewCycleUnlock()	// NewCycle unlock Behaviours
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING NewCycle in Behaviours..." << endl;
#endif

status = pthread_mutex_unlock(&m_NewCycleMutex);
if (status != 0)
	cerr << "WARNING: CBehaviours: NewCycleUnlock: Unlocking error..." << endl;
}

// NEWCYCLESIGNAL //////////////////////////////////////////////////
void CBehaviours::NewCycleSignal()		// Signal the NewCycle condition variable
{
int status;						// Signal status

status = pthread_cond_signal(&m_NewCycleCond);
if (status != 0)
	cerr << "WARNING: CBehaviours: NewCycleSignal: Signaling error..." << endl;
}

// ABORT ///////////////////////////////////////////////
void CBehaviours::Abort(char* Msg, int Code)	// Display error message and exit
{
cerr << endl;
cerr << "Behaviours: " << Msg;
cerr << endl;
DoExit(Code);
}

// SETSCOREDIFF /////////////////////////////////////////
void CBehaviours::SetScoreDiff(int Diff)		// Set score difference and change formation if necessary
{
m_ScoreDiff = Diff;								// Set score difference;
if (m_ScoreDiff <= -m_ChangeFormationThr)		// If less than minus threashold
	m_FormationID = -1;							// Set formation ID to losing
else if (m_ScoreDiff >= m_ChangeFormationThr)	// If greater than threashold
	m_FormationID = 1;							// Set formation ID to winning
else											// else if in between
	m_FormationID = 0;							// Set formation ID to normal
}
