/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CBehaviours class
 * 				The CBehaviours class is responsible for all the agent's thinking.
 * Created: 17/12/98
 * Last modified: 24/7/99
 */

#ifndef _CBEHAVIOURS_H
#define _CBEHAVIOURS_H

class CGetParams;		// Need a forward declaration

#include <string.h>
#include <pthread.h>
#include "CModule.H"
#include "MCPLPosLib.h"
 
class CBehaviours: public CModule
{
// Private member data ////////////////////////////////////////////
private:

		pthread_mutex_t	m_Mutex;				// Behaviour's mutex
		pthread_cond_t m_Cond;					// Behaviour's condition variable
		pthread_mutex_t	m_ExecMutex;			// Behaviour's Exec mutex
		pthread_cond_t m_ExecCond;				// Behaviour's Exec condition variable		
		pthread_mutex_t	m_NewCycleMutex;		// Behaviour's NewCycle mutex
		pthread_cond_t m_NewCycleCond;			// Behaviour's NewCycle condition variable		
		
		int		m_IsNewVisual;					// Predicate
		int		m_IsNewCycle;					// Predicate
		long	m_SimStep;						// Simulator step
		
		char 	m_ActionBuffer[MAXSENDDATALEN];	// Action buffer
		int		m_ScoreDiff;					// Current score difference (-ve means I am losing)
		int		m_ChangeFormationThr;			// Threashold value for m_ScoreDiff used to change between formations (from agent.conf)
		int		m_FormationID;					// Current formation ID: (-1)-Losing, 0-Normal, 1-Winning
		int		m_LosingFormation;				// Formation when losing
		int		m_NormalFormation;				// Normal formation
		int		m_WinningFormation;				// Formation when winning

		float	m_PitchLength;					// Pitch length from params
		float	m_PitchWidth;					// Pitch width from params		
		float	m_PlayerSize;					// Player size from params
		float	m_BallSize;						// Ball size from params
		float	m_PlayerDecay;					// Player decay from params
		float	m_BallDecay;					// Ball decay from params
		float	m_PlayerSpeedMax;				// Max speed for players from params
		float	m_BallSpeedMax;					// Max speed for ball from params
		float	m_KickableMargin;				// Kickable margin from params
		float	m_Kickable;						// Max kickable range (kickable_margin + ball_size + player_size)
		int		m_MaxPower;						// Max power of dash and kick from params
		int		m_MinPower;						// Min power of dash and kick from params
		int		m_MaxMoment;					// Max moment of dash and kick from params
		int		m_MinMoment;					// Min moment of dash and kick from params
		
		int		m_MaxNeckMoment;				// Max moment for turn neck from params
		int		m_MinNeckMoment;				// Min moment for turn neck from params
		int		m_MaxNeckAng;					// Max neck angle from params
		int		m_MinNeckAng;					// Min neck angle from params
		
		float	m_InertiaMoment;				// Inertia moment from params
		float	m_KickPowerRate;				// Kick power rate from params
		float	m_DashPowerRate;				// Dash power rate from params

		int		m_RecvStep;						// Receive step from params (polling time by the server)

		float	m_OptCtrlDist;					// Optimal control distance = m_PlayerSize + 0.8 * SP_kick_margin
		float	m_ClosestDist;					// The closest distance the ball can get (= PlayerSize + 2*BallSize)
		float	m_KickFactor;					// Kick factor
		
		float	m_StopPass;						// Target velocity is 0.0
		float	m_SoftPass;						// Target velocity is 0.3
		float	m_MediumPass;					// Target velocity is 0.7
		float	m_HardPass;						// Target velocity is 1.2
				
		MCPLStatus m_MCPLStatus;				// Structure passed to the positioning module
		
		// 334 Formations //////////////////////////////////////////////////////////////////////
		MCPLPositioner* m_GPS334N;				// Pointer to the positioner module
		MCPLPositioner* m_GPS334Defend;			// Pointer to the positioner module for the Defend specialisation
		MCPLPositioner* m_GPS334BallTracker;	// Pointer to the positioner module for the BallTracker specialisation
		MCPLPositioner* m_GPS334OffsideTrap;	// Pointer to the positioner module for the OffsideTrap specialisation
		////////////////////////////////////////////////////////////////////////////////////////
		
		// 433 Formations //////////////////////////////////////////////////////////////////////
		MCPLPositioner* m_GPS433N;				// Pointer to the positioner module
		MCPLPositioner* m_GPS433Defend;			// Pointer to the positioner module for the Defend specialisation
		MCPLPositioner* m_GPS433BallTracker;	// Pointer to the positioner module for the BallTracker specialisation
		MCPLPositioner* m_GPS433OffsideTrap;	// Pointer to the positioner module for the OffsideTrap specialisation
		////////////////////////////////////////////////////////////////////////////////////////
		
		// 532 Formations //////////////////////////////////////////////////////////////////////
		MCPLPositioner* m_GPS532N;				// Pointer to the positioner module
		MCPLPositioner* m_GPS532Defend;			// Pointer to the positioner module for the Defend specialisation
		MCPLPositioner* m_GPS532BallTracker;	// Pointer to the positioner module for the BallTracker specialisation
		MCPLPositioner* m_GPS532OffsideTrap;	// Pointer to the positioner module for the OffsideTrap specialisation
		////////////////////////////////////////////////////////////////////////////////////////
		
		float	m_Epsilon;						// Epsilon value used for float equality test (from agent.conf)
		float	m_StatConfDrop;					// Rate at which confidence of stationary objects drops after each cycle (from agent.conf)
		float	m_StatConfThr;					// Minimum confidence threshold for stationary objects (from agent.conf)
		float	m_MobConfDrop;					// Rate at which confidence of mobile objects drops after each cycle (from agent.conf)
		float	m_MobConfThr;					// Minimum confidence threshold for mobile objects (from agent.conf)
		float	m_Slow;							// Slow dash power (from agent.conf)
		float	m_Normal;						// Normal dash power (from agent.conf)
		float	m_Fast;							// Fast dash power (from agent.conf)
		float	m_MaxGoAngle;					// If target point less than X degrees, don't turn (from agent.conf)
		float	m_MinDistToPoint;				// In MoveToPos or Chase, if target point is closer than X meters, consider it ok (from agent.conf)
		int		m_MaxMoveDashes;				// Maximum dashes in a MoveToPos behaviour (from agent.conf)
		int		m_MaxInterceptDashes;			// Maximum dashes in a MoveToIntercept behaviour (without seeing the ball) (from agent.conf)
		int		m_InterceptStep;				// Intercept cycle step (from agent.conf)
		float	m_InterceptTol;					// Intercept tolerance (from agent.conf)
		int		m_SpinDirTol;					// Direction +/- spin_dir_tol is used to determine if ball is at the right place (from agent.conf)
		float	m_MaxSpinPower;					// Power used in kick when spinning ball round body (from agent.conf)
		float	m_MaxStopSpeed;					// If ball is going slower than that, consider it stopped (from agent.conf)
		int		m_MinOppWorryDist;				// If an opponent is closer than that, we worry about it ! (from agent.conf)
		float	m_DodgeDistBuf;					// Dodge distance buffer (from agent.conf)
		int		m_DribbleDodgeCloseDist;		// Dribble dodge close distance (from agent.conf)
		int		m_DribbleDodgeAngleErr;			// Dribble dodge angle error (from agent.conf)
		float	m_BallDribbleDist;				// Dribble ball dist (if you change that, Dribble might NOT work) (from agent.conf)
		int		m_MaxDribbleExpAngle;			// Used when dribbling to check if ball will end up behind us (from agent.conf)
		float	m_DribbleNormalAngle;			// Normal dribble angle (from agent.conf)
		float	m_CollisionBuffer;				// Collision buffer used by WillXXXBeCollision functions (from agent.conf)
		
// Public member functions //////////////////////////////////////////
public:

// Constructor and Display functions //////////////////////////////
	CBehaviours(CMediator*);	// Constructor that takes Mediator
	void Init(CGetParams);		// Initialise all fields using BehavioursParams
	void Display() const;		// Displays all params.  Just for debugging

// Lock/Unlock and Signal functions ////////////////////////////////////
	void Lock();					// Lock Behaviours
	void Unlock();					// Unlock Behaviours
	void Signal();					// Signal the condition variable
	
	void ExecLock();				// ExecLock Behaviours
	void ExecUnlock();				// ExecUnlock Behaviours
	void ExecSignal();				// Signal the Exec condition variable
	
	void NewCycleLock();			// NewCycleLock Behaviours
	void NewCycleUnlock();			// NewCycleUnlock Behaviours
	void NewCycleSignal();			// Signal the NewCycle condition variable
	
// Error reporting functions ////////////////////////////////////////
	void	Abort	(char* Msg, int Code);	// Display message and exit with a given code

// Basic behaviour attribute get/set functions //////////////////////////
	void	SetIsNewVisual	(int IsNewVisual)	// Set m_IsNewVisual
	{ m_IsNewVisual = IsNewVisual; }
	int		GetIsNewVisual	()					// Get m_IsNewVisual
	{ return m_IsNewVisual; }

	void	SetIsNewCycle	(int IsNewCycle)	// Set m_IsNewCycle
	{ m_IsNewCycle = IsNewCycle; }
	int		GetIsNewCycle	()					// Get m_IsNewCycle
	{ return m_IsNewCycle; }

	void SetSimStep(long SimStep)			// Set simulator step
	{ m_SimStep = SimStep; }
	long GetSimStep() const					// Get simulator step
	{ return m_SimStep; }
	
	void SetRecvStep (int RecvStep)			// Set receive step (server's polling time)
	{ m_RecvStep = RecvStep; }
	int GetRecvStep	() const				// Get receive step (server's polling time)
	{ return m_RecvStep; }
	
	void	SetActionBuffer	(char* Buffer)		//	Set m_ActionBuffer
	{ strncpy(m_ActionBuffer, Buffer, MAXSENDDATALEN); }
	const char*	GetActionBuffer	()				//	Get m_ActionBuffer
	{ return m_ActionBuffer; }

	void SetScoreDiff(int Diff);				// Set score difference and change formation if necessary
	int GetScoreDiff() const					// Get score difference
	{ return m_ScoreDiff; }
	
	void SetFormationID(int FormationID)		// Set formation ID
	{ m_FormationID = FormationID; }
	int GetFormationID() const					// Get formation ID
	{ return m_FormationID; }
	
	void SetLosingFormation(int LosingFormation)	// Set formation when losing
	{ m_LosingFormation = LosingFormation; }
	int GetLosingFormation() const					// Get formation when losing
	{ return m_LosingFormation; }
	
	void SetNormalFormation(int NormalFormation)	// Set normal formation
	{ m_NormalFormation = NormalFormation; }
	int GetNormalFormation() const					// Get normal formation
	{ return m_NormalFormation; }
	
	void SetWinningFormation(int WinningFormation)	// Set formation when winning
	{ m_WinningFormation = WinningFormation; }
	int GetWinningFormation() const					// Get formation when losing
	{ return m_WinningFormation; }

	void SetPlayerSize(float PlayerSize)			// Set player size
	{ m_PlayerSize = PlayerSize; }
	float GetPlayerSize() const						// Get player size
	{ return m_PlayerSize; }
	
	void SetBallSize(float BallSize)				// Set ball size
	{ m_BallSize = BallSize; }
	float GetBallSize() const						// Get ball size
	{ return m_BallSize; }
	
	void SetKickable(float Kickable)				// Set kickable range
	{ m_Kickable = Kickable; }
	float GetKickable() const						// Get kickable range
	{ return m_Kickable; }
	
	void SetMaxPower(int MaxPower)					// Set max power
	{ m_MaxPower = MaxPower; }
	int GetMaxPower() const							// Get max power
	{ return m_MaxPower; }
	
	void SetMinPower(int MinPower)					// Set min power
	{ m_MinPower = MinPower; }
	int GetMinPower() const							// Get min power
	{ return m_MinPower; }
	
	void SetMaxMoment(int MaxMoment)				// Set max moment
	{ MaxMoment = MaxMoment; }
	int GetMaxMoment() const						// Get max moment
	{ return m_MaxMoment; }
	
	void SetMinMoment(int MinMoment)				// Set min moment
	{ m_MinMoment = MinMoment; }
	int GetMinMoment() const						// Get min moment
	{ return m_MinMoment; }
	
	void SetInertiaMoment(float InertiaMoment)		// Set inertia moment
	{ m_InertiaMoment = InertiaMoment; }
	float GetInertiaMoment() const					// Get inertia moment
	{ return m_InertiaMoment; }
	
	void SetKickPowerRate(float KickPowerRate)		// Set kick power rate
	{ m_KickPowerRate = KickPowerRate; }
	float GetKickPowerRate() const					// Get kick power rate
	{ return m_KickPowerRate; }

	void SetMCPLStatus	(MCPLStatus S)				// Set the MCPLStatus
	{ m_MCPLStatus = S;}
	MCPLStatus GetMCPLStatus ()						// Get the MCPLStatus
	{ return m_MCPLStatus; }
	MCPLStatus* GetPMCPLStatus ()					// Get pointer to MCPLStatus
	{ return &m_MCPLStatus; }
	
// Basic Utilities //////////////////////////////////////////////////
	float 	NormaliseAngle	(float ang)				// Normalise angle
	{ while(ang >  180.0) ang -= 360.0;
	  while(ang < -180.0) ang += 360.0;
	  return ang; }

	float	GoAngle			(float dist)			// Decide if it's worth turning
	{ if (dist > 1)
		return (5 + 10/dist);		
	  else
	  	return 15; }

	float	SignF			(float X)				// Return 1 or -1 depending on sign
	{ if (X > 0.0)
		return (1.0);
	  else
	  	return (-1.0); }
		
// Behavioural Utilities /////////////////////////////////////////////
	float SumInfGeomSeries(float X, float R)
	{ return (X / (1 - R)); } 	
	float SolveForSumInfGeomSeries(float R, float S)
	{ return (S * (1 - R)); } 
	float SumGeomSeries(float, float, int);				// 
	float SolveForSumGeomSeries(float, float, float);	// Returns -1 on error
	// Solves the line circle intersection problem and returns the number of solutions found		
	int	LineCircleIntersect	(CVector, CVector, float, CVector, CVector*, CVector*);
	CVector GetClosestPointToLine(CVector, CVector, CVector);		// Calculate closest point of intersection	
// Localisation functions ///////////////////////////////////////////
	const 	CLine*		InsideLine	();				// Calculate the line to use for localisation
	const 	CFlag*		ClosestFlag	();				// Calculate the flag to use for localisation
	float	GFaceDir	(const CLine*);				// Calculate the global facing direction given a line
	void	Localise	();							// Calculate global position given a line and a flag

// Basic Actions ////////////////////////////////////////////////////
	void	SayBye		();							// Send bye command
	int		Move		(int, int);					// Send move command
	int		BasicTurn	(float);					// Send turn command
	int		Dash		(float);					// Send dash command
	int		BasicKick	(float, float);				// Send kick command
	int		BasicTurnNeck(float);					// Send turn_neck command
	int		Say			(char*);					// Send say command
	int		ChangeView	(char*, char*);				// Send change_view command
	int		Catch		(float);					// Send catch command
	int		SenseBody	();							// Send sense_body command	

// Advanced Actions //////////////////////////////////////////////////
	int		Turn			(float);				// Turning that takes inertia into account
	int		Kick			(float, float, float);	// Kick so that balls travels distance at direction, given a distance factor
	//int		TurnNeck		(float);				// Turn neck that check neckmoment
	//int		ResetNeck		();						// Align neck with body
	//int		ResetBody		();						// Align body with neck
	
// Basic Behaviours //////////////////////////////////////////////////	  
	void	InitialMove		();						// Move player to formation (used before kick-off and after goals)
	int		Wait4NewVisual	();						// Wait for new visual data
	void	WaitToExec		();						// Wait for the given action to get executed
	void	Wait4NewCycle	();						// Wait till we enter a new cycle
	void	LookAround		();						// Do a complete circle and gather visual data
	float	Collision		();						// Return collision direction

	// Parameter is Mode
	int		MoveToPos		(float);				// Move to your predefined global position (depending on formation)
	int		MoveToStopClearance (float);			// Move to block opponents clearance
	
	// Parameter is position and power
	int		MoveToPos		(CVector, float);		// Move to (X,Y) with the given Power
	int		MoveToPos		(Point P, float Power)	// Another way to call MoveToPos (using Points)
	{ CVector CV(P.X, P.Y); return MoveToPos(CV, Power); }
	int		MoveToPos		(float X, float Y, float Power)	// Another way to call MoveToPos (using co-ordinates)
	{ CVector CV(X, Y); return MoveToPos(CV, Power); }
	// Backward moving behaviours
	int		MoveToPosBW		(CVector, float);		// Move to (X,Y) with the given Power
	int		MoveToPosBW		(Point P, float Power)	// Another way to call MoveToPos (using Points)
	{ CVector CV(P.X, P.Y); return MoveToPosBW(CV, Power); }
	int		MoveToPosBW		(float X, float Y, float Power)	// Another way to call MoveToPos (using co-ordinates)
	{ CVector CV(X, Y); return MoveToPosBW(CV, Power); }
	
	float	LookingAtBall	(int);					// Find ball and return its distance
	float	LookingAtBall	()						// By default these behaviours to not TURN to face target
	{ return LookingAtBall(0); }
	float	LookingAtGoal	(int);					// Find enemy goal and return its distance	
	float	LookingAtGoal	()						// By default these behaviours to not TURN to face target
	{ return LookingAtGoal(0); }
	float	LookingAtMyGoal	(int);					// Find my goal and return its distance
	float	LookingAtMyGoal	()						// By default these behaviours to not TURN to face target
	{ return LookingAtMyGoal(0); }
	
	float	AdjustPower		(float);				// Adjust power of movement depending on stamina
	float	CalcPow4Goalie	();						// Calculate movement power for goalie
	float	CalcPow4Def		();						// Calculate movement power for defenders
	float	CalcPow4Mid		();						// Calculate movement power for midfielders
	float	CalcPow4Fw		();						// Calculate movement power for forwards
	
	void 	Recharge		();						// Check if we can afford to inrease our stamina
	
// Advanced Behaviours ////////////////////////////////////////////////
	int		ControlBall		();						// Stop a moving ball (if moving too fast)
	int		ShootToGoal		(int);					// Shoot the ball to the enemy goal
	int		HavePossession	();						// Determines which team has possession of the ball
													// Returns 1: Ours, 0: None, -1: Enemy
	int 	ClosestToBall	(int);					// Determines who is closer to the ball (i.e. me, teammate or enemy)
													// (The closest player is the one who can intercept quickest)
													// Returns 1 if me, 0 if a teammate, -1 if an enemy,
													// and -2 if enemy is closer than closer friend
	CVector	IsShot			();						// Determines if there is an incoming shot and returns
													// last point inbounds if it is else mag = -1
													
	float	ClearUsefulness	(float);				// Calculation of the usefulness function for clearing
	float	SendUsefulness	(float);				// Calculation of the usefulness function for sending
	float	CentreUsefulness(float, float, float);	//
	float	ESInfluence		(float, float, float, float, float);	//
	float	ExpectedSuccess	(float, float);			// Calculation of the expected success of an angle
	float	ShootAngle		(float, float, float,float) ;	//
	float	PassAngle		(float, float, float,float);			//
	float	ClearingAngle	(float, float);			// Calculates the best clearing angle
	float	SendingAngle	(float, float);			// Calculates the best sending angle
	
	int		ClearBall		(float);				// Defensive clearing of the ball (the action)
	int		SendBall		(float);				// Offensive sending of the ball (the action)
	int		CrossBall		();						// Make a cross towards the penalty spot

// Interception behaviours /////////////////////////////////////////////
	// Active intercept ///////////////////////////////////////////////////////
	CVector	ChaseCloseBall	(float);				// Chase a ball that is close (2 step look-ahead) with a given power
	int		CyclesToIntercept(float, int, int, const CPlayer*, CVector &);	// Calculate the number of cycles it takes for a 
																			// given player to intercept the ball
	int		CyclesToIntercept(float Power, int MaxLookAhead, int IsMe, const CPlayer* P)// Overloaded for players
	{ CVector temp; return CyclesToIntercept(Power, MaxLookAhead, IsMe, P, temp); }
	int		CyclesToIntercept(float Power, int MaxLookAhead, CVector& InterceptPoint)	// Overloaded for me
	{ return CyclesToIntercept(Power, MaxLookAhead, 1, 0, InterceptPoint); }
	///////////////////////////////////////////////////////////////////////////
	
	// Passive intercept //////////////////////////////////////////////////////
	int		CyclesToClosestPointOnPath	(int, CVector &);					// Calculate the point that is
																			// closest to the ball's path
	int		CyclesToClosestPointOnPath	(CVector& InterceptPoint)			// Same but with MAXMODELSTEPS
	{ return CyclesToClosestPointOnPath(MAXMODELSTEPS, InterceptPoint); }
	///////////////////////////////////////////////////////////////////////////
	
	int		ChaseBall		(float, int);			// Chase ball with a given dash power and mode (active/passive)
	int		ChaseBall		(float Power)			// Default chase mode is active
	{ return ChaseBall(Power, 0); }					// (i.e. the IsPassive flag is false)
	int		MoveToIntercept	(CVector, float);		// Same as MoveToPos but with use of TurnNeck and ChangeView
													// so that player can maintain visual with the ball during chase
	int		GoalieChase		(int);					// Chase behaviour specialised for the goalie
													// Return 0: false, 1: kickable, 2: catchable
	int		GoalieMoveToIntercept(CVector, int);			// Same as move to intercept but check for catching as well
	
// Passing behaviours /////////////////////////////////////////////////////////
	const CPlayer* DTPass			(char*, int);	// Find best candidate according to DT depending on Mode
												/*
												Modes are (string combining any of these):	
												0: Most likely			1: Only positive DT
												2: Most offensive		3: Most offensive pos only
												4: Most wide			5: Most wide pos only
												6: Most centred			7: Most centred pos only
												(e.g. "753" ORDER IS IMPORTANT)
												
												The boolean is used to determine if I want the receiver
												to have a higher X co-ord relative to me
											*/
	const CPlayer* OffensivePass	();				// Find most offensive candidate for a pass
	int		Pass			(const CPlayer*, float);// Pass to player with ball travelling at a given speed when it gets there
	int		Pass			(const CPlayer* P)		// Default target speed is Soft
	{ return Pass(P, m_SoftPass); }

// DT Passing /////////////////////////////////////////////////////////
	const	CPlayer* DTBest	(char*, int, const CPlayer*const*,const CPlayer*const*,const CPlayer*const*,const CPlayer*const*);	// Picks best candidate for a pass
	int		create_example	(const CPlayer*const*,const CPlayer*const*,float*,const CPlayer*);		// Creates the attribs for the DT
	float	assess			(float*); 				// Returns the DT value

// Kicking behaviours /////////////////////////////////////////////////
	float	Point2KickVelocity	(CVector, float);	// Velocity at point 2 velocity for kick converter
	int		CanKickStraight		(float, float);		// Check if we can kick the ball straight without collision
	float	CalcKickPower		(float);			// Calculate the actual power of the shot
	int		CalcSpinDirection	(float, float);		// Calculate spinning direction.  (Return 1:CCW, -1:CW)
	int		CalcOppAvoidSpinDirection(float, float);// Calculate spinning direction based on nearest opponent.  (Return 1:CCW, -1:CW)
	void	PlayerMovementCorrection(float*, float*);	// Correct for movement of player (used before calling Kick)
	int 	SpinKick			(float, float, int, float);	// Kick ball, but spin round body if necessary
	int		SpinRound			(float Direction, int StopBall)			// Just spin ball round body
	{ return SpinKick(m_OptCtrlDist, Direction, StopBall, m_KickFactor); }
	int		StraightKick(float, float, float);			// If stright kick possible, use this one
	
// Dribbling behaviours ///////////////////////////////////////////////
	int		DribbleDash			(CVector, float, float);	// See if a dash would keep us in control
	int		DribbleKick			(CVector, float, float);	// Figure kick to keep dribbling
	// Dribble towards a direction with the given power, dodging things if necessary
	int		Dribble				(CVector, float, float, float, int, CVector);	// Dribble to a given point dodging things if necessary
	int		Dribble				(CVector, float, float);	// Simpler version of the above.  No dodging
	int		CleverDribble		(CVector, float, float);	// High level dribbling with opponent avoidance

// Positioning Behaviours /////////////////////////////////////////////
	void	InitPositioner		();	// Initialise positioner object
	void	ProduceMCPLStatus	();	// Using memory, create or update the MCPLStatus
	int		DefensivePosition	();	// Move towards best defensive position given your unum & the current formation
	int		OffensivePosition	();	// Move towards best offensive position given your unum & the current formation

// Prediction Behaviours /////////////////////////////////////////////
// For ball //////////////////////////////////////////////////////////
	CVector BallPredPosAfterKick(int, float, float);		// Predict ball's position after a kick
	CVector	BallPredPos			(int Steps)					// Predict ball's position after a number of steps
	{ return BallPredPosAfterKick(Steps, 0, 0); }
	CVector BallPredPos			()							// Predict ball's position for the next cycle
	{ return BallPredPosAfterKick(1, 0, 0); }
	int		BallWillBeKickable	(int, float);				// Will ball be kickable after a number of steps?
	
	
// For me ////////////////////////////////////////////////////////
	CVector PredPosAfterDashing	(int, float);				// Predicted my position after continuous dashing
	CVector PredPosAfterDash	(float Power)				// Predict my position after a single dash
	{ return PredPosAfterDashing(1, Power); }
	CVector PredPosAfterTurn	(int, float, float, int);	// Predict my position after turn (and dashes)
	CVector PredPos				()							// Predict my position for the next cycle
	{ return PredPosAfterTurn(1, 0, 0, 0); }

// For player ////////////////////////////////////////////////////////
	CVector PPredPosAfterDashing(int, float, const CPlayer*);			// Predicted player's position after continuous dashing
	CVector PPredPosAfterDash	(float Power, const CPlayer* P)			// Predict player's position after a single dash
	{ return PPredPosAfterDashing(1, Power, P); }
	CVector PPredPosAfterTurn	(int, float, float, int, const CPlayer*);// Predict player's position after turn
	CVector PPredPos			(int, CVector, const CPlayer*);			// Predict player's position using a velocity vector
	CVector PPredPos			(const CPlayer* P)						// Predict player's position for the next cycle
	{ return PPredPosAfterTurn(1, 0, 0, 0, P); }

// Others ////////////////////////////////////////////////////////////
	int		WillDashBeCollision	(float);					// Will dash lead to a collision with the ball?
	int		WillKickBeCollision	(float, float);				// Will kick lead to a collision with the ball?
	
// Play Styles ///////////////////////////////////////////////////////
	// Parameter is the previous and current play mode
	void	PlayAsGoalKeeper	(int, int);					// Goal keeper
	void	PlayAsDefender		(int, int);					// Defender
	void	PlayAsMidfielder	(int, int);					// Midfielder
	void	PlayAsForward		(int, int);					// Forward
	
};
 
#endif
