/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CFlag class
 * 				The CFlag class holds information regarding a flag on the field.
 * Created: 14/12/98
 * Last modified: 10/7/99
 */

#ifndef _CFLAG_H
#define _CFLAG_H
 
#include "CVector.H"
 
class CFlag
{
// Private member data /////////////////////////////////////////////
private:

// These are default values in server.conf and agent.conf
	float	m_StatConfThr;

// These values are fixed /////////////////////////////////////
	int 	m_ID;			// Flag ID:
	CVector m_GPos;			// Global position
	
// These values are obtained from sensors /////////////////////
	int 	m_Time;			// Time of observation
	CVector	m_RPos;			// Relative position
	float	m_DistChng;		// Distance change
	float	m_DirChng;		// Direction change
	
// These values are calculated ///////////////////////////////
	float 	m_Conf;			// Confidence

// Public member functions ///////////////////////////////////////////
public:

// Constructor/Destructor, Init, and Display functions ///////////////
	CFlag();						// Default constructor
	~CFlag();						// Default destructor
	void Init(int, float, float, float);	// Used to initialise flags
	void RevertSigns();				// Revert signs for right side players
	void Display() const;			// Displays all params.  Just for debugging
	const CFlag *GetThis() const	// Function that returns the "this" pointer
	{ return this; }				// Used by Get*** in Memory
	
// Basic flag data set/get functions /////////////////////////////////
	void	SetID	(int ID)		// Set flag's ID
	{ m_ID = ID; }
	int		GetID	() const		// Get flag's ID
	{ return m_ID; }

	void	SetGPos	(CVector GPos)	// Set flag's position
	{ m_GPos = GPos; }
	CVector	GetGPos	() const		// Get flag's position
	{ return m_GPos; }

// Accessor function for GPOS ///////////////////////////////////////
	
	void	SetGDir	(float	Dir)		// Set relative direction
	{ m_GPos.SetDir(Dir); }
	float	GetGDir	() const			// Get relative direction
	{ return m_GPos.GetDir(); }
	
	void	SetGMag	(float	Mag)		// Set relative magnitude
	{ m_GPos.SetMag(Mag); }
	float	GetGMag	() const			// Get relative magnitude
	{ return m_GPos.GetMag(); }

	void	SetGX	(float	X)			// Set global X co-ordinate
	{ m_GPos.SetX(X); }
	float	GetGX	() const			// Get global X co-ordinate
	{ return m_GPos.GetX(); }	
	
	void	SetGY	(float	Y)			// Set global Y co-ordinate
	{ m_GPos.SetY(Y); }
	float	GetGY	() const			// Get global Y co-ordinate
	{ return m_GPos.GetY(); }	
		
/////////////////////////////////////////////////////////////////////	
	
	void	SetTime	(int Time)		// Set flag's time
	{ m_Time = Time; }
	int		GetTime	() const		// Get flag's time
	{ return m_Time; }
	
	void	SetRPos	(CVector RPos)	// Set flag's relative position
	{ m_RPos = RPos; }
	CVector	GetRPos	() const		// Get flag's relative position
	{ return m_RPos; }
	void	UpdateRPos(CVector, float, int);	// Update flag's relative position if not seen
	
// Accessor function for RPOS ///////////////////////////////////////
	
	void	SetRDir	(float	Dir)		// Set relative direction
	{ m_RPos.SetDir(Dir); }
	float	GetRDir	() const			// Get relative direction
	{ return m_RPos.GetDir(); }
	
	void	SetRMag	(float	Mag)		// Set relative magnitude
	{ m_RPos.SetMag(Mag); }
	float	GetRMag	() const			// Get relative magnitude
	{ return m_RPos.GetMag(); }
	
	void	SetRX	(float	X)			// Set relative X co-ordinate
	{ m_RPos.SetX(X); }
	float	GetRX	() const			// Get relative X co-ordinate
	{ return m_RPos.GetX(); }	
	
	void	SetRY	(float	Y)			// Set relative Y co-ordinate
	{ m_RPos.SetY(Y); }
	float	GetRY	() const			// Get relative Y co-ordinate
	{ return m_RPos.GetY(); }	
	
/////////////////////////////////////////////////////////////////////
	
	void	SetDistChng	(float DistChng)// Set flag's distance change
	{ m_DistChng = DistChng; }
	float	GetDistChng	() const		// Get flag's distance change
	{ return m_DistChng; }
	
	void	SetDirChng	(float DirChng)	// Set flag's direction change
	{ m_DirChng = DirChng; }
	float	GetDirChng	() const		// Get flag's direction change
	{ return m_DirChng; }
	
	void	SetConf	(float Conf)	// Set flag's confidence
	{ m_Conf = Conf; }
	float	GetConf	() const		// Get flag's confidence
	{ return m_Conf; }

};
 
#endif
