/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CGetParams class
 * 				The CGetParam class reads the agent.conf, and server.conf files and stores the data
 *				for later use by the agent.
 * Created: 11/12/98
 * Last modified: 10/7/99
 */

#include <stdio.h>
#include <fstream.h>
#include "CGetParams.H"

// Default constructor ////////////////////////////////////////////////////
CGetParams::CGetParams()
{
}

// LOADPARAMS /////////////////////////////////////////////////////////////
void CGetParams::LoadParams(char* ConfigurationFile)	// Load parameters from file
{
char str[256], str2[256];				// Store temporary strings
int	l;									// Store skip char number for sscanf

SP_pitch_length = 105.0;				// Set parameters for the size of the pitch and the
SP_pitch_width = 68.0;					// penalty areas
SP_penalty_area_length = 16.5;
SP_penalty_area_width = 40.32;
SP_pitch_margin = 5.0;

ifstream fin("server.conf");	// Open server.conf for reading

if (fin.fail())					// If open failed, notify and exit
	{
	cerr << "Read failed for \"server.conf\"" << endl;
	cerr << "Check file is in the current directory, and file is readable." << endl;
	exit(100);
	}
else if (fin.good())			// else if open was ok
	{
	while(fin.getline(str,255))	// Read line by line and
		{
		// If not a comment or a blank line, store value of appropriate variable
		if ( (str[0] != '#') && (strlen(str) > 1) )
			{
			if  (!strncmp(str, "goal_width", 10))
				sscanf(str, "goal_width : %f", &SP_goal_width);
				
			else if  (!strncmp(str, "player_size", 11))
				sscanf(str, "player_size : %f", &SP_player_size);
				
			else if  (!strncmp(str, "player_decay", 12))
				sscanf(str, "player_decay : %f", &SP_player_decay);
				
			else if  (!strncmp(str, "player_rand", 11))
				sscanf(str, "player_rand : %f", &SP_player_rand);
				
			else if  (!strncmp(str, "player_weight", 13))
				sscanf(str, "player_weight : %f", &SP_player_weight);
				
			else if  (!strncmp(str, "player_speed_max", 16))
				sscanf(str, "player_speed_max : %f", &SP_player_speed_max);
				
			else if  (!strncmp(str, "stamina_max", 11))
				sscanf(str, "stamina_max : %f", &SP_stamina_max);
				
			else if  (!strncmp(str, "stamina_inc_max", 15))
				sscanf(str, "stamina_inc_max : %f", &SP_stamina_inc_max);
				
			else if  (!strncmp(str, "recover_dec_thr", 15))
				sscanf(str, "recover_dec_thr : %f", &SP_recover_dec_thr);

			else if  (!strncmp(str, "recover_dec", 11))
				sscanf(str, "recover_dec : %f", &SP_recover_dec);
				
			else if  (!strncmp(str, "recover_min", 11))
				sscanf(str, "recover_min : %f", &SP_recover_min);
				
			else if  (!strncmp(str, "effort_dec_thr", 14))
				sscanf(str, "effort_dec_thr : %f", &SP_effort_dec_thr);
				
			else if  (!strncmp(str, "effort_dec", 10))
				sscanf(str, "effort_dec : %f", &SP_effort_dec);
				
			else if  (!strncmp(str, "effort_inc_thr", 14))
				sscanf(str, "effort_inc_thr : %f", &SP_effort_inc_thr);
				
			else if  (!strncmp(str, "effort_inc", 10))
				sscanf(str, "effort_inc : %f", &SP_effort_inc);
				
			else if  (!strncmp(str, "effort_min", 10))
				sscanf(str, "effort_min : %f", &SP_effort_min);
				
			else if  (!strncmp(str, "hear_max", 8))
				sscanf(str, "hear_max : %d", &SP_hear_max);
				
			else if  (!strncmp(str, "hear_inc", 8))
				sscanf(str, "hear_inc : %d", &SP_hear_inc);
				
			else if  (!strncmp(str, "hear_decay", 10))
				sscanf(str, "hear_decay : %d", &SP_hear_decay);
				
			else if  (!strncmp(str, "inertia_moment", 14))
				sscanf(str, "inertia_moment : %f", &SP_inertia_moment);

			else if  (!strncmp(str, "sense_body_step", 15))
				sscanf(str, "sense_body_step : %d", &SP_sense_body_step);
				
			else if  (!strncmp(str, "catchable_area_l", 16))
				sscanf(str, "catchable_area_l : %f", &SP_catchable_area_l);
				
			else if  (!strncmp(str, "catchable_area_w", 16))
				sscanf(str, "catchable_area_w : %f", &SP_catchable_area_w);
				
			else if  (!strncmp(str, "catch_probability", 17))
				sscanf(str, "catch_probability : %f", &SP_catch_probability);
				
			else if  (!strncmp(str, "catch_ban_cycle", 15))
				sscanf(str, "catch_ban_cycle : %d", &SP_catch_ban_cycle);
				
			else if  (!strncmp(str, "ball_size", 9))
				sscanf(str, "ball_size : %f", &SP_ball_size);

			else if  (!strncmp(str, "ball_decay", 10))
				sscanf(str, "ball_decay : %f", &SP_ball_decay);
				
			else if  (!strncmp(str, "ball_rand", 9))
				sscanf(str, "ball_rand : %f", &SP_ball_rand);
				
			else if  (!strncmp(str, "ball_weight", 11))
				sscanf(str, "ball_weight : %f", &SP_ball_weight);
				
			else if  (!strncmp(str, "ball_speed_max", 14))
				sscanf(str, "ball_speed_max : %f", &SP_ball_speed_max);
				
			else if  (!strncmp(str, "wind_force", 10))
				sscanf(str, "wind_force : %f", &SP_wind_force);
				
			else if  (!strncmp(str, "wind_dir", 8))
				sscanf(str, "wind_dir : %f", &SP_wind_dir);
				
			else if  (!strncmp(str, "wind_rand", 9))
				sscanf(str, "wind_rand : %f", &SP_wind_rand);
				
			else if  (!strncmp(str, "kickable_margin", 15))
				sscanf(str, "kickable_margin : %f", &SP_kickable_margin);
				
			else if  (!strncmp(str, "ckick_margin", 12))
				sscanf(str, "ckick_margin : %f", &SP_ckick_margin);
				
			else if  (!strncmp(str, "dash_power_rate", 15))
				sscanf(str, "dash_power_rate : %f", &SP_dash_power_rate);
				
			else if  (!strncmp(str, "kick_power_rate", 15))
				sscanf(str, "kick_power_rate : %f", &SP_kick_power_rate);
				
			else if  (!strncmp(str, "visible_angle", 13))
				sscanf(str, "visible_angle : %f", &SP_visible_angle);
				
			else if  (!strncmp(str, "audio_cut_dist", 14))
				sscanf(str, "audio_cut_dist : %f", &SP_audio_cut_dist);
				
			else if  ( (!strncmp(str, "quantize_step", 13)) && (strncmp(str, "quantize_step_l", 15)) )
				sscanf(str, "quantize_step : %f", &SP_quantize_step);
				
			else if  (!strncmp(str, "quantize_step_l", 15))
				sscanf(str, "quantize_step_l : %f", &SP_quantize_step_l);				

			else if  (!strncmp(str, "maxpower", 8))
				sscanf(str, "maxpower : %d", &SP_maxpower);
				
			else if  (!strncmp(str, "minpower", 8))
				sscanf(str, "minpower : %d", &SP_minpower);
				
			else if  (!strncmp(str, "maxmoment", 9))
				sscanf(str, "maxmoment : %d", &SP_maxmoment);
				
			else if  (!strncmp(str, "minmoment", 9))
				sscanf(str, "minmoment : %d", &SP_minmoment);

			else if  (!strncmp(str, "maxneckmoment", 13))
				sscanf(str, "maxneckmoment : %d", &SP_maxneckmoment);
				
			else if  (!strncmp(str, "minneckmoment", 13))
				sscanf(str, "minneckmoment : %d", &SP_minneckmoment);
				
			else if  (!strncmp(str, "maxneckang", 10))
				sscanf(str, "maxneckang : %d", &SP_maxneckang);
				
			else if  (!strncmp(str, "minneckang", 10))
				sscanf(str, "minneckang : %d", &SP_minneckang);
				
			else if  (!strncmp(str, "port", 4))
				sscanf(str, "port : %d", &SP_port);
				
			else if  (!strncmp(str, "coach_port", 10))
				sscanf(str, "coach_port : %d", &SP_coach_port);

			else if  (!strncmp(str, "say_coach_cnt_max", 17))
				sscanf(str, "say_coach_cnt_max : %d", &SP_say_coach_cnt_max);

			else if  (!strncmp(str, "say_coach_msg_size", 18))
				sscanf(str, "say_coach_msg_size : %d", &SP_say_coach_msg_size);
				
			else if  (!strncmp(str, "send_vi_step", 12))
				sscanf(str, "send_vi_step : %d", &SP_send_vi_step);	
								
			else if  (!strncmp(str, "simulator_step", 14))
				sscanf(str, "simulator_step : %d", &SP_simulator_step);

			else if  (!strncmp(str, "send_step", 9))
				sscanf(str, "send_step : %d", &SP_send_step);
				
			else if  (!strncmp(str, "recv_step", 9))
				sscanf(str, "recv_step : %d", &SP_recv_step);
				
			else if  (!strncmp(str, "half_time", 9))
				sscanf(str, "half_time : %d", &SP_half_time);
	
			else if  (!strncmp(str, "say_msg_size", 12))
				sscanf(str, "say_msg_size : %d", &SP_say_msg_size);
	
			else if  (!strncmp(str, "use_offside", 11))
				{
				sscanf(str, "use_offside : %[A-Za-z]", &str2);
				if ( (!strncmp(str2, "off", 3)) || (!strncmp(str2, "OFF", 3)) )
					SP_use_offside = 0;
				else
					SP_use_offside = 1;
				}
				
			else if  (!strncmp(str, "offside_active_area_size", 24))
				sscanf(str, "offside_active_area_size : %f", &SP_offside_active_area_size);
	
			else if  (!strncmp(str, "forbid_kick_off_offside", 23))
				{
				sscanf(str, "forbid_kick_off_offside : %[A-Za-z]", &str2);
				if ( (!strncmp(str2, "off", 3)) || (!strncmp(str2, "OFF", 3)) )
					SP_forbid_kick_off_offside = 0;
				else
					SP_forbid_kick_off_offside = 1;
				}
				
			else if  (!strncmp(str, "verbose", 7))
				{
				sscanf(str, "verbose : %[A-Za-z]", &str2);
				if ( (!strncmp(str2, "on", 2)) || (!strncmp(str2, "ON", 2)) )
					SP_verbose = 1;
				else
					SP_verbose = 0;
				}
			
			else if  (!strncmp(str, "log_file", 8))
				sscanf(str, "log_file : %s", &SP_log_file);
			
			else if  (!strncmp(str, "record_version", 14))
				sscanf(str, "record_version : %d", &SP_record_version);
				
			else if  (!strncmp(str, "record_log", 10))
				{
				sscanf(str, "record_log : %[A-Za-z]", &str2);
				if ( (!strncmp(str2, "on", 2)) || (!strncmp(str2, "ON", 2)) )
					SP_record_log = 1;
				else
					SP_record_log = 0;
				}
				
			else if  (!strncmp(str, "send_log", 8))
				{
				sscanf(str, "send_log : %[A-Za-z]", &str2);
				if ( (!strncmp(str2, "off", 3)) || (!strncmp(str2, "OFF", 3)) )
					SP_send_log = 0;
				else
					SP_send_log = 1;
				}

			else if  (!strncmp(str, "offside_kick_margin", 19))
				sscanf(str, "offside_kick_margin : %f", &SP_offside_kick_margin);
			
			else				// else, don't recognise parameter, better to exit!
				{
				cerr << "Unrecognised parameter in \"server.conf\"..." << endl;
				cerr << str << endl;
				exit(101);
				}
			}
		}
	}
else							// else, don't know what went wrong, so exit!
	{
	cerr << "An error has occurred while reading file \"server.conf\"" << endl;
	exit(110);
	}
	
fin.close();					// Close server.conf

ifstream fin2(ConfigurationFile);	// Open configuration file for reading

if (fin2.fail())				// If open failed, notify and exit
	{
	cerr << "Read failed for \"" << ConfigurationFile << "\"" << endl;
	cerr << "Check file is in the current directory, and file is readable." << endl;
	exit(100);
	}
else if (fin2.good())			// else if open was ok
	{
	while(fin2.getline(str,255))// Read line by line and
		{
		// If not a comment or a blank line, store value of appropriate variable
		if ( (str[0] != '#') && (strlen(str) > 1) )
			{
			if (!strncmp(str, "team_name", 9))
				sscanf(str, "team_name : %s", &AP_team_name);

			else if (!strncmp(str, "hostname", 8))
				sscanf(str, "hostname : %s", &AP_hostname);
				
			else if (!strncmp(str, "port", 4))
				sscanf(str, "port : %d", &AP_port);
			
			else if (!strncmp(str, "score_diff", 10))
				sscanf(str, "score_diff : %d", &AP_score_diff);

			else if (!strncmp(str, "change_formation_thr", 20))
				sscanf(str, "change_formation_thr : %d", &AP_change_formation_thr);
				
			else if  (!strncmp(str, "losing_formation", 16))
				sscanf(str, "losing_formation : %d", &AP_losing_formation);
				
			else if  (!strncmp(str, "normal_formation", 16))
				sscanf(str, "normal_formation : %d", &AP_normal_formation);
			
			else if  (!strncmp(str, "winning_formation", 17))
				sscanf(str, "winning_formation : %d", &AP_winning_formation);
			
			else if  (!strncmp(str, "sim_step", 8))
				sscanf(str, "sim_step : %ld", &AP_sim_step);
			
			else if  (!strncmp(str, "mob_conf_drop", 13))
				sscanf(str, "mob_conf_drop : %f", &AP_mob_conf_drop);
			
			else if  (!strncmp(str, "stat_conf_drop", 14))
				sscanf(str, "stat_conf_drop : %f", &AP_stat_conf_drop);
			
			else if  (!strncmp(str, "mob_conf_thr", 12))
				sscanf(str, "mob_conf_thr : %f", &AP_mob_conf_thr);
			
			else if  (!strncmp(str, "stat_conf_thr", 13))
				sscanf(str, "stat_conf_thr : %f", &AP_stat_conf_thr);
			
			else if  (!strncmp(str, "epsilon", 7))
				sscanf(str, "epsilon : %f", &AP_epsilon);
			
			else if  (!strncmp(str, "slow", 4))
				sscanf(str, "slow : %f", &AP_slow);
			
			else if  (!strncmp(str, "normal", 6))
				sscanf(str, "normal : %f", &AP_normal);
			
			else if  (!strncmp(str, "fast", 4))
				sscanf(str, "fast : %f", &AP_fast);
			
			else if  (!strncmp(str, "max_go_angle", 12))
				sscanf(str, "max_go_angle : %f", &AP_max_go_angle);
			
			else if  (!strncmp(str, "min_dist_to_point", 17))
				sscanf(str, "min_dist_to_point : %f", &AP_min_dist_to_point);
			
			else if  (!strncmp(str, "max_move_dashes", 15))
				sscanf(str, "max_move_dashes : %d", &AP_max_move_dashes);
				
			else if  (!strncmp(str, "max_intercept_dashes", 20))
				sscanf(str, "max_intercept_dashes : %d", &AP_max_intercept_dashes);
				
			else if  (!strncmp(str, "def_zone", 8))
				sscanf(str, "def_zone : %d", &AP_def_zone);
				
			else if  (!strncmp(str, "att_zone", 8))
				sscanf(str, "att_zone : %d", &AP_att_zone);
				
			else if  (!strncmp(str, "intercept_step", 14))
				sscanf(str, "intercept_step : %d", &AP_intercept_step);
				
			else if  (!strncmp(str, "intercept_tol", 13))
				sscanf(str, "intercept_tol : %f", &AP_intercept_tol);
			
			else if  (!strncmp(str, "inner_ring", 10))
				sscanf(str, "inner_ring : %f", &AP_inner_ring);
			
			else if  (!strncmp(str, "outer_ring", 10))
				sscanf(str, "outer_ring : %f", &AP_outer_ring);
			
			else if  (!strncmp(str, "pow_to_stop", 11))
				sscanf(str, "pow_to_stop : %f", &AP_pow_to_stop);
			
			else if  (!strncmp(str, "stop_pass", 9))
				sscanf(str, "stop_pass : %f", &AP_stop_pass);
			
			else if  (!strncmp(str, "soft_pass", 9))
				sscanf(str, "soft_pass : %f", &AP_soft_pass);
			
			else if  (!strncmp(str, "medium_pass", 11))
				sscanf(str, "medium_pass : %f", &AP_medium_pass);
			
			else if  (!strncmp(str, "hard_pass", 9))
				sscanf(str, "hard_pass : %f", &AP_hard_pass);
			
			else if  (!strncmp(str, "spin_dir_tol", 12))
				sscanf(str, "spin_dir_tol : %d", &AP_spin_dir_tol);
				
			else if  (!strncmp(str, "max_spin_power", 14))
				sscanf(str, "max_spin_power : %f", &AP_max_spin_power);
				
			else if  (!strncmp(str, "max_stop_speed", 14))
				sscanf(str, "max_stop_speed : %f", &AP_max_stop_speed);
				
			else if  (!strncmp(str, "min_opp_worry_dist", 18))
				sscanf(str, "min_opp_worry_dist : %d", &AP_min_opp_worry_dist);
				
			else if  (!strncmp(str, "dodge_dist_buf", 14))
				sscanf(str, "dodge_dist_buf : %f", &AP_dodge_dist_buf);
			
			else if  (!strncmp(str, "dribble_dodge_close_dist", 24))
				sscanf(str, "dribble_dodge_close_dist : %d", &AP_dribble_dodge_close_dist);
				
			else if  (!strncmp(str, "dribble_dodge_angle_err", 23))
				sscanf(str, "dribble_dodge_angle_err : %d", &AP_dribble_dodge_angle_err);
				
			else if  (!strncmp(str, "ball_dribble_dist", 17))
				sscanf(str, "ball_dribble_dist : %f", &AP_ball_dribble_dist);
			
			else if  (!strncmp(str, "max_dribble_exp_angle", 21))
				sscanf(str, "max_dribble_exp_angle : %d", &AP_max_dribble_exp_angle);
				
			else if  (!strncmp(str, "dribble_normal_angle", 20))
				sscanf(str, "dribble_normal_angle : %f", &AP_dribble_normal_angle);
			
			else if  (!strncmp(str, "collision_buffer", 16))
				sscanf(str, "collision_buffer : %f", &AP_collision_buffer);
				
			else				// else, don't recognise parameter, better to exit!
				{
				cerr << "ERROR: CGetParams: CGetParams: Unrecognised parameter in \"" << ConfigurationFile << "\"..." << endl;
				exit(102);
				}
			}
		}
	}
else							// else, don't know what went wrong, so exit!
	{
	cerr << "ERROR: CGetParams: CGetParams: An error has occurred while reading file \"" << ConfigurationFile << "\"" << endl;
	exit(110);
	}
	
fin2.close();					// Close agent.conf
}

// Default destructor ////////////////////////////////////////////////////////
CGetParams::~CGetParams()
{
}

// DISPLAY ////////////////////////////////////////////////////////////////////
void CGetParams::Display() const	// Displays all params.  Just for debugging
{
cout << "server.conf is as follows:" << endl;
cout << "width of goals" << endl;
cout << SP_goal_width << endl;

cout << "size, decay, random parameter, weight, maximum speed of players" << endl;
cout << SP_player_size << endl;
cout << SP_player_decay << endl;
cout << SP_player_rand << endl;
cout << SP_player_weight << endl;
cout << SP_player_speed_max << endl;

cout << "maximum and recovery step of player's stamina (default 2500.0, 50.0)" << endl;
cout << SP_stamina_max << endl;
cout << SP_stamina_inc_max << endl;

cout << "decriment threshold ,decriment step and minimum of player's recovery" << endl;
cout << SP_recover_dec_thr << endl;
cout << SP_recover_dec << endl;
cout << SP_recover_min << endl;

cout << "decriment threshold, decriment step, incriment threshold, incriment step" << endl;
cout << "and minimum of player's effort (default 0.4, 0.05, 0.9, 0.05, 0.1)" << endl; 
cout << SP_effort_dec_thr << endl;
cout << SP_effort_dec << endl;
cout << SP_effort_inc_thr << endl;
cout << SP_effort_inc << endl;
cout << SP_effort_min << endl;

cout << "maximum, recovery step and decay of player's hear capacity (default 2, 1, 1)" << endl;
cout << SP_hear_max << endl;
cout << SP_hear_inc << endl;
cout << SP_hear_decay << endl;

cout << "inertia moment of player (default 5.0)" << endl;
cout << SP_inertia_moment << endl;

cout << "interval of sense_body (default 100)" << endl;
cout << SP_sense_body_step << endl;
	
cout << "goalie catchable area length, width (default 2.0, 1.0)" << endl;
cout << SP_catchable_area_l << endl;
cout << SP_catchable_area_w << endl;

cout << "goalie catchbale probability (default 1.0)" << endl;
cout << SP_catch_probability << endl;

cout << "goalie catch ban cycle (default 5)" << endl;
cout << SP_catch_ban_cycle << endl;

cout << "size, decay, random parameter, weight and maximum speed of a ball" << endl;
cout << "(default 0.15, 0.8, 0.2, 0.2, 32.0)" << endl;
cout << SP_ball_size << endl;
cout << SP_ball_decay << endl;
cout << SP_ball_rand << endl;
cout << SP_ball_weight << endl;
cout << SP_ball_speed_max << endl;

cout << "force, direction and random parameter of wind (default 10.0, 0.0, 0.3)" << endl;
cout << SP_wind_force << endl;
cout << SP_wind_dir << endl;
cout << SP_wind_rand << endl;

cout << "kickable margin (default 1.0) kickable_area = kickable_margin + bsize + psize" << endl;
cout << SP_kickable_margin << endl;

cout << "corner kick margin (default 1.0)" << endl;
cout << SP_ckick_margin << endl;

cout << "magnification of power in dash, kick  (default 0.1, 0.1)" << endl;
cout << SP_dash_power_rate << endl;
cout << SP_kick_power_rate << endl;

cout << "angle of view corn [unit: degree] (default 90.0)" << endl;
cout << SP_visible_angle << endl;

cout << "audio cut off distance (default 50.0)" << endl;
cout << SP_audio_cut_dist << endl;

cout << "quantize step of distance for move_object, landmark (default 0.1, 0.01)" << endl;
cout << SP_quantize_step << endl;
cout << SP_quantize_step_l << endl;

cout << "max and min of power in dash and kick (default 100, -30)" << endl;
cout << SP_maxpower << endl;
cout << SP_minpower << endl;

cout << "max and min of power in turn (default 180, -180)" << endl;
cout << SP_maxmoment << endl;
cout << SP_minmoment << endl;

cout << "max and min of power in turn_neck (default 180,-180)" << endl;
cout << SP_maxneckmoment << endl;
cout << SP_minneckmoment << endl;

cout << "max and min of neck angle (default 90, -90)" << endl;
cout << SP_maxneckang << endl;
cout << SP_minneckang << endl;

cout << "Default port number (default 6000)" << endl;
cout << SP_port << endl;

cout << "Default coach port number (default 6001)" << endl;
cout << SP_coach_port << endl;

cout << "Default upper limit of the number of online coach's message (default 128)" << endl;
cout << SP_say_coach_cnt_max << endl;

cout << "Default upper limit of length of online coach's message (default 128)" << endl;
cout << SP_say_coach_msg_size << endl;

cout << "Default interval of online coach's look (default 100)" << endl;
cout << SP_send_vi_step << endl;

cout << "time step of simulation [unit:msec] (default 100)" << endl;
cout << SP_simulator_step << endl;

cout << "time step of visual information [unit:msec] (default 150)" << endl;
cout << SP_send_step << endl;

cout << "time step of acception of command [unit: msec] (defalut 20)" << endl;
cout << SP_recv_step << endl;

cout << "length of half of game [unit:sec] (default 6000)" << endl;
cout << "(if negative, a game does not stop automatically)" << endl;
cout << SP_half_time << endl;

cout << "string size of say message [unit:byte] (default 256)" << endl;
cout << SP_say_msg_size << endl;

cout << "flag for using off side rule. [on/off] (default on)" << endl;
cout << SP_use_offside << endl;

cout << "offside active area size (default 9.15)" << endl;
cout << SP_offside_active_area_size << endl;

cout << "forbid kick off offside. [on/off] (default on)" << endl; 
cout << SP_forbid_kick_off_offside << endl;

cout << "flag for verbose mode. [on/off] (default off)" << endl;
cout << SP_verbose << endl;

cout << "Record log file" << endl;
cout << SP_log_file << endl;

cout << "flag for record log. (default 2)" << endl; 
cout << "(old version 1)" << endl;
cout << SP_record_version << endl;

cout << "flag for record client command log. [on/off] (default off)" << endl;
cout << SP_record_log << endl;

cout << "flag for send client command log. [on/off] (default on)" << endl;
cout << SP_send_log << endl;

cout << "offside kick margin. (default 9.15)" << endl;
cout << SP_offside_kick_margin << endl;

cout << endl;

cout << "agent.conf is as follows:" << endl;
cout << "Team name" << endl;
cout << AP_team_name << endl;

cout << "hostname" << endl;
cout << AP_hostname << endl;

cout << "port" << endl;
cout << AP_port << endl;

cout << "Current score difference (-ve means I am losing)" << endl;
cout << AP_score_diff << endl;

cout << "Threashold value for m_ScoreDiff used to change between formations" << endl;
cout << AP_change_formation_thr << endl;

cout << "losing formation" << endl;
cout << AP_losing_formation << endl;

cout << "normal formation" << endl;
cout << AP_normal_formation << endl;

cout << "winning formation" << endl;
cout << AP_winning_formation << endl;

cout << "Simulator step. Simulator step in nano-sec.  A bit less than 100ms is used" << endl;
cout << "to account for some calculations in the actuators class (this is a guestimate" << endl;
cout << "and is machine and OS dependent)" << endl;
cout << AP_sim_step << endl;

cout << "Rate at which confidence of mobile objects drops after each cycle" << endl;
cout << AP_mob_conf_drop << endl;

cout << "Rate at which confidence of stationary objects drops after each cycle" << endl;
cout << AP_stat_conf_drop << endl;

cout << "Minimum confidence threshold for mobile objects (appr. after 4 cycles)" << endl;
cout << AP_mob_conf_thr << endl;

cout << "Minimum confidence threshold for stationary objects (appr. after 8 cycles)" << endl;
cout << AP_stat_conf_thr << endl;

cout << "In float numbers (0.9 == 0.9) is NOT true." << endl;
cout << "The alternative is to say (0.9 > (0.9 - EPSILON))" << endl;
cout << AP_epsilon << endl;

cout << "Slow dash power" << endl;
cout << AP_slow << endl;

cout << "Normal dash power" << endl;
cout << AP_normal << endl;

cout << "Fast dash power" << endl;
cout << AP_fast << endl;

cout << "In MoveToPos, Chase, Intercept, if target point less than 5 degrees, don't turn" << endl;
cout << AP_max_go_angle << endl;

cout << "In MoveToPos or Chase, if target point is closer than 1 meter, consider it ok" << endl;
cout << AP_min_dist_to_point << endl;

cout << "Maximum dashes in a MoveToPos behaviour" << endl;
cout << AP_max_move_dashes << endl;

cout << "Maximum dashes in a MoveToIntercept behaviour (without seeing the ball)" << endl;
cout << AP_max_intercept_dashes << endl;

cout << "Defensive zone ends here (X co-ord)" << endl;
cout << AP_def_zone << endl;

cout << "Attack zone ends here (X co-ord)" << endl;
cout << AP_att_zone << endl;

cout << "Intercept cycle step" << endl;
cout << AP_intercept_step << endl;

cout << "Intercept tolerance" << endl;
cout << AP_intercept_tol << endl;

cout << "Used in SpinRound() and CanKickDirect() so that ball does not get to close" << endl;
cout << AP_inner_ring << endl;

cout << "Used in SpinRound() and CanKickDirect() so that ball does not go to far" << endl;
cout << AP_outer_ring << endl;

cout << "Power to stop the ball is ball's velocity has not been calculated" << endl;
cout << AP_pow_to_stop << endl;

cout << "Velocity of ball when ball reaches the desired point (used in passing)" << endl;
cout << AP_stop_pass << endl;
cout << AP_soft_pass << endl;
cout << AP_medium_pass << endl;
cout << AP_hard_pass << endl;

cout << "Direction +/- spin_dir_tol is used to determine if ball is at the right place" << endl;
cout << AP_spin_dir_tol << endl;

cout << "Power used in kick when spinning ball round body" << endl;
cout << AP_max_spin_power << endl;

cout << "If ball is going slower than that, consider it stopped" << endl;
cout << AP_max_stop_speed << endl;

cout << "If an opponent is closer than that, we worry about it !" << endl;
cout << AP_min_opp_worry_dist << endl;

cout << "Dodge distance buffer" << endl;
cout << AP_dodge_dist_buf << endl;

cout << "Dribble dodge close distance" << endl;
cout << AP_dribble_dodge_close_dist << endl;

cout << "Dribble dodge angle error" << endl;
cout << AP_dribble_dodge_angle_err << endl;

cout << "Dribble ball dist (if you change that, Dribble might NOT work)" << endl;
cout << AP_ball_dribble_dist << endl;

cout << "Used when dribbling to check if ball will end up behind us" << endl;
cout << AP_max_dribble_exp_angle << endl;

cout << "Normal dribble angle" << endl;
cout << AP_dribble_normal_angle << endl;

cout << "Collision buffer (used by WillXXXBeCollision functions)" << endl;
cout << AP_collision_buffer << endl;
	
cout << endl;
}
