/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CGetParams class
 * 				The CGetParam class reads the agent.conf, and server.conf files and stores the data
 *				for later use by the agent.
 * Created: 11/12/98
 * Last modified: 12/7/99
 */

#ifndef _CGETPARAMS_H
#define _CGETPARAMS_H

#include <string.h>
#include "../Agent_Others/GlobalParams.H"

class CGetParams
{
// Private member data ////////////////////////////////////////////////////////////////
private:

// Parameters for the size of the pitch and the penalty areas
	float	SP_pitch_length;
	float	SP_pitch_width;
	float	SP_penalty_area_length;
	float	SP_penalty_area_width;
	float	SP_pitch_margin;

// Configuration parameters for soccerserver 

// width of goals
	float	SP_goal_width;

// size, decay, random parameter, weight, maximum speed of players
	float	SP_player_size;
	float	SP_player_decay;
	float	SP_player_rand;
	float	SP_player_weight;
	float	SP_player_speed_max;

// maximum and recovery step of player's stamina (default 2500.0, 50.0)
	float	SP_stamina_max;
	float	SP_stamina_inc_max;

// decriment threshold ,decriment step and minimum of player's recovery
	float	SP_recover_dec_thr;
	float	SP_recover_dec;
	float	SP_recover_min;

// decriment threshold, decriment step, incriment threshold, incriment step
// and minimum of player's effort (default 0.4, 0.05, 0.9, 0.05, 0.1) 
	float	SP_effort_dec_thr;
	float	SP_effort_dec;
	float	SP_effort_inc_thr;
	float	SP_effort_inc;
	float	SP_effort_min;

// maximum, recovery step and decay of player's hear capacity (default 2, 1, 1)
	int		SP_hear_max;
	int		SP_hear_inc;
	int		SP_hear_decay;

// inertia moment of player (default 5.0)
	float	SP_inertia_moment;

// interval of sense_body (default 100)
	int		SP_sense_body_step;

// goalie catchable area length, width (default 2.0, 1.0)
	float	SP_catchable_area_l;
	float	SP_catchable_area_w;

// goalie catchbale probability (default 1.0)
	float	SP_catch_probability;

// goalie catch ban cycle (default 5)
	int	SP_catch_ban_cycle;

// size, decay, random parameter, weight and maximum speed of a ball
// (default 0.15, 0.8, 0.2, 0.2, 32.0)
	float	SP_ball_size;
	float	SP_ball_decay;
	float	SP_ball_rand;
	float	SP_ball_weight;
	float	SP_ball_speed_max;

// force, direction and random parameter of wind (default 10.0, 0.0, 0.3)
	float	SP_wind_force;
	float	SP_wind_dir;
	float	SP_wind_rand;

// kickable margin (default 1.0) kickable_area = kickable_margin + bsize + psize
	float	SP_kickable_margin;

// corner kick margin (default 1.0)
	float	SP_ckick_margin;

// magnification of power in dash, kick  (default 0.1, 0.1)
	float	SP_dash_power_rate;
	float	SP_kick_power_rate;

// angle of view corn [unit: degree] (default 90.0)
	float	SP_visible_angle;

// audio cut off distance (default 50.0)
	float	SP_audio_cut_dist;

// quantize step of distance for move_object, landmark (default 0.1, 0.01)
	float	SP_quantize_step;
	float	SP_quantize_step_l;

// max and min of power in dash and kick (default 100, -30)
	int		SP_maxpower;
	int		SP_minpower;

// max and min of power in turn (default 180, -180)
	int		SP_maxmoment;
	int		SP_minmoment;

// max and min of power in turn_neck (default 180,-180)
	int		SP_maxneckmoment;
	int		SP_minneckmoment;

// max and min of neck angle (default 90, -90)
	int		SP_maxneckang;
	int		SP_minneckang;

// Default port number (default 6000)
	int		SP_port;

// Default coach port number (default 6001)
	int		SP_coach_port;

// Default upper limit of the number of online coach's message (default 128)
	int		SP_say_coach_cnt_max;

// Default upper limit of length of online coach's message (default 128)
	int		SP_say_coach_msg_size;

// Default interval of online coach's look (default 100)
	int		SP_send_vi_step; 

// time step of simulation [unit:msec] (default 100)
	int		SP_simulator_step;

// time step of visual information [unit:msec] (default 150)
	int		SP_send_step;

// time step of acception of command [unit: msec] (defalut 20)
	int		SP_recv_step;

// length of half of game [unit:sec] (default 3000)
// (if negative, a game does not stop automatically)
	int		SP_half_time;

// string size of say message [unit:byte] (default 256)
	int		SP_say_msg_size;

// flag for using off side rule. [on/off] (default on)
	int		SP_use_offside;

// offside active area size (default 9.15)
	float	SP_offside_active_area_size;

// forbid kick off offside. [on/off] (default on) 
	int		SP_forbid_kick_off_offside;

// flag for verbose mode. [on/off] (default off)
	int		SP_verbose;

// Record log file
	char	SP_log_file[256];

// flag for record log. (default 2) 
// (old version 1)
	int		SP_record_version;

// flag for record client command log. [on/off] (default off)
	int		SP_record_log;

// flag for send client command log. [on/off] (default on)
	int		SP_send_log;

// offside kick margin. (default 9.15)
	float	SP_offside_kick_margin;

// Configuration parameters for the agent

// Team name
	char	AP_team_name[MAXTEAMNAMELEN];
	
// Hostname
	char	AP_hostname[MAXHOSTNAMELEN];
	
//	Port
	int		AP_port;

// Score difference
	int		AP_score_diff;

// Threashold value for m_ScoreDiff used to change between formations
	int		AP_change_formation_thr;
	
// Formation when losing
	int		AP_losing_formation;
	
// Normal formation
	int		AP_normal_formation;

// Formation when winning
	int		AP_winning_formation;

// Simulator step. Simulator step in nano-sec.  A bit less than 100ms is used
// to account for some calculations in the actuators class (this is a guestimate
// and is machine and OS dependent)
	long	AP_sim_step;

// Rate at which confidence of mobile objects drops after each cycle
	float	AP_mob_conf_drop;

// Rate at which confidence of stationary objects drops after each cycle
	float	AP_stat_conf_drop;

// Minimum confidence threshold for mobile objects (appr. after 4 cycles)
	float	AP_mob_conf_thr;

// Minimum confidence threshold for stationary objects (appr. after 8 cycles)
	float	AP_stat_conf_thr;

// In float numbers (0.9 == 0.9) is NOT true.
// The alternative is to say (0.9 > (0.9 - EPSILON))
	float	AP_epsilon;

// Slow dash power
	float	AP_slow;

// Normal dash power
	float	AP_normal;

// Fast dash power
	float	AP_fast;

// In MoveToPos, Chase, Intercept, if target point less than 5 degrees, don't turn
	float	AP_max_go_angle;

// In MoveToPos or Chase, if target point is closer than 1 meter, consider it ok
	float	AP_min_dist_to_point;

// Maximum dashes in a MoveToPos behaviour
	int		AP_max_move_dashes;

// Maximum dashes in a MoveToIntercept behaviour (without seeing the ball)
	int		AP_max_intercept_dashes;

// Defensive zone ends here (X co-ord)
	int		AP_def_zone;

// Attack zone ends here (X co-ord)
	int		AP_att_zone;

// Intercept cycle step
	int		AP_intercept_step;

// Intercept tolerance
	float	AP_intercept_tol;

// Used in SpinRound() and CanKickDirect() so that ball does not get to close.
	float	AP_inner_ring;

// Used in SpinRound() and CanKickDirect() so that ball does not go to far
	float	AP_outer_ring;

// Power to stop the ball is ball's velocity has not been calculated
	float	AP_pow_to_stop;

// Velocity of ball when ball reaches the desired point (used in passing)
	float	AP_stop_pass;
	float	AP_soft_pass;
	float	AP_medium_pass;
	float	AP_hard_pass;

// Direction +/- spin_dir_tol is used to determine if ball is at the right place
	int		AP_spin_dir_tol;

// Power used in kick when spinning ball round body
	float	AP_max_spin_power;

// If ball is going slower than that, consider it stopped
	float	AP_max_stop_speed;

// If an opponent is closer than that, we worry about it !
	int		AP_min_opp_worry_dist;

// Dodge distance buffer
	float	AP_dodge_dist_buf;

// Dribble dodge close distance
	int		AP_dribble_dodge_close_dist;

// Dribble dodge angle error
	int		AP_dribble_dodge_angle_err;

// Dribble ball dist (if you change that, Dribble might NOT work)
	float	AP_ball_dribble_dist;

// Used when dribbling to check if ball will end up behind us
	int		AP_max_dribble_exp_angle;

// Normal dribble angle
	float	AP_dribble_normal_angle;
	
// Collision buffer (used by WillXXXBeCollision functions)
	float	AP_collision_buffer;

// Public member functions ////////////////////////////////////////////////////
public:

// Constructor/Destructor, and Display functions //////////////////////////////
	CGetParams();			// Constructor
	void LoadParams(char*);	// Load parameters from file
	~CGetParams();			// Destructor
	void Display() const;	// Displays all params.  Just for debugging

// Basic Set/Get functions for the member variables ////////////////////////////
// No comments have been added here since things are pretty clear... ///////////
	void SetSP_pitch_length(float PitchLength)
	{ SP_pitch_length = PitchLength; }
	float GetSP_pitch_length() const
	{ return SP_pitch_length; }

	void SetSP_pitch_width(float PitchWidth)
	{ SP_pitch_width = PitchWidth; }
	float GetSP_pitch_width() const
	{ return SP_pitch_width; }

	void SetSP_penalty_area_length(float PenaltyAreaLength)
	{ SP_penalty_area_length = PenaltyAreaLength; }
	float GetSP_penalty_area_length() const
	{ return SP_penalty_area_length; }

	void SetSP_penalty_area_width(float PenaltyAreaWidth)
	{ SP_penalty_area_width = PenaltyAreaWidth; }
	float GetSP_penalty_area_width() const
	{ return SP_penalty_area_width; }

	void SetSP_pitch_margin(float PitchMargin)
	{ SP_pitch_margin = PitchMargin; }
	float GetSP_pitch_margin() const
	{ return SP_pitch_margin; }

	void SetSP_goal_width(float GoalWidth)
	{ SP_goal_width = GoalWidth; }
	float GetSP_goal_width() const
	{ return SP_goal_width; }
	
	void SetSP_player_size(float PlayerSize)
	{ SP_player_size = PlayerSize; }
	float GetSP_player_size() const
	{ return SP_player_size; }
	
	void SetSP_player_decay(float PlayerDecay)
	{ SP_player_decay = PlayerDecay; }
	float GetSP_player_decay() const
	{ return SP_player_decay; }
	
	void SetSP_player_rand(float PlayerRand)
	{ SP_player_rand = PlayerRand; }
	float GetSP_player_rand() const
	{ return SP_player_rand; }
	
	void SetSP_player_weight(float PlayerWeight)
	{ SP_player_weight = PlayerWeight; }
	float GetSP_player_weight() const
	{ return SP_player_weight; }
	
	void SetSP_player_speed_max(float PlayerSpeedMax)
	{ SP_player_speed_max = PlayerSpeedMax; }
	float GetSP_player_speed_max() const
	{ return SP_player_speed_max; }
	
	void SetSP_stamina_max(float StaminaMax)
	{ SP_stamina_max = StaminaMax; }
	float GetSP_stamina_max() const
	{ return SP_stamina_max; }
	
	void SetSP_stamina_inc_max(float StaminaIncMax)
	{ SP_stamina_inc_max = StaminaIncMax; }
	float GetSP_stamina_inc_max() const
	{ return SP_stamina_inc_max; }
	
	void SetSP_recover_dec_thr(float RecoverDecThr)
	{ SP_recover_dec_thr = RecoverDecThr; }
	float GetSP_recover_dec_thr() const
	{ return SP_recover_dec_thr; }
	
	void SetSP_recover_dec(float RecoverDec)
	{ SP_recover_dec = RecoverDec; }
	float GetSP_recover_dec() const
	{ return SP_recover_dec; }
	
	void SetSP_recover_min(float RecoverMin)
	{ SP_recover_min = RecoverMin; }
	float GetSP_recover_min() const
	{ return SP_recover_min; }
	
	void SetSP_effort_dec_thr(float EffortDecThr)
	{ SP_effort_dec_thr = EffortDecThr; }
	float GetSP_effort_dec_thr() const
	{ return SP_effort_dec_thr; }
	
	void SetSP_effort_dec(float EffortDec)
	{ SP_effort_dec = EffortDec; }
	float GetSP_effort_dec() const
	{ return SP_effort_dec; }
	
	void SetSP_effort_inc_thr(float EffortIncThr)
	{ SP_effort_inc_thr = EffortIncThr; }
	float GetSP_effort_inc_thr() const
	{ return SP_effort_inc_thr; }
	
	void SetSP_effort_inc(float EffortInc)
	{ SP_effort_inc = EffortInc; }
	float GetSP_effort_inc() const
	{ return SP_effort_inc; }
	
	void SetSP_effort_min(float EffortMin)
	{ SP_effort_min = EffortMin; }
	float GetSP_effort_min() const
	{ return SP_effort_min; }
	
	void SetSP_hear_max(int HearMax)
	{ SP_hear_max = HearMax; }
	int GetSP_hear_max() const
	{ return SP_hear_max; }
	
	void SetSP_hear_inc(int HearInc)
	{ SP_hear_inc = HearInc; }
	int GetSP_hear_inc() const
	{ return SP_hear_inc; }
	
	void SetSP_hear_decay(int HearDecay)
	{ SP_hear_decay = HearDecay; }
	int GetSP_hear_decay() const
	{ return SP_hear_decay; }
	
	void SetSP_inertia_moment(float InertiaMoment)
	{ SP_inertia_moment = InertiaMoment; }
	float GetSP_inertia_moment() const
	{ return SP_inertia_moment; }
	
	void SetSP_sense_body_step(int SenseBodyStep)
	{ SP_sense_body_step = SenseBodyStep; }
	int GetSP_sense_body_step() const
	{ return SP_sense_body_step; }
	
	void SetSP_catchable_area_l(float CatchableAreaL)
	{ SP_catchable_area_l = CatchableAreaL; }
	float GetSP_catchable_area_l() const
	{ return SP_catchable_area_l; }
	
	void SetSP_catchable_area_w(float CatchableAreaW)
	{ SP_catchable_area_w = CatchableAreaW; }
	float GetSP_catchable_area_w() const
	{ return SP_catchable_area_w; }
	
	void SetSP_catch_probability(float CatchProbability)
	{ SP_catch_probability = CatchProbability; }
	float GetSP_catch_probability() const
	{ return SP_catch_probability; }
	
	void SetSP_catch_ban_cycle(int CatchBanCycle)
	{ SP_catch_ban_cycle = CatchBanCycle; }
	int GetSP_catch_ban_cycle() const
	{ return SP_catch_ban_cycle; }
	
	void SetSP_ball_size(float BallSize)
	{ SP_ball_size = BallSize; }
	float GetSP_ball_size() const
	{ return SP_ball_size; }
	
	void SetSP_ball_decay(float BallDecay)
	{ SP_ball_decay = BallDecay; }
	float GetSP_ball_decay() const
	{ return SP_ball_decay; }
	
	void SetSP_ball_rand(float BallRand)
	{ SP_ball_rand = BallRand; }
	float GetSP_ball_rand() const
	{ return SP_ball_rand; }
	
	void SetSP_ball_weight(float BallWeight)
	{ SP_ball_weight = BallWeight; }
	float GetSP_ball_weight() const
	{ return SP_ball_weight; }
	
	void SetSP_ball_speed_max(float BallSpeedMax)
	{ SP_ball_speed_max = BallSpeedMax; }
	float GetSP_ball_speed_max() const
	{ return SP_ball_speed_max; }
	
	void SetSP_wind_force(float WindForce)
	{ SP_wind_force = WindForce; }
	float GetSP_wind_force() const
	{ return SP_wind_force; }
	
	void SetSP_wind_dir(float WindDir)
	{ SP_wind_dir = WindDir; }
	float GetSP_wind_dir() const
	{ return SP_wind_dir; }
	
	void SetSP_wind_rand(float WindRand)
	{ SP_wind_rand = WindRand; }
	float GetSP_wind_rand() const
	{ return SP_wind_rand; }
	
	void SetSP_kickable_margin(float KickableMargin)
	{ SP_kickable_margin = KickableMargin; }
	float GetSP_kickable_margin() const
	{ return SP_kickable_margin; }
	
	void SetSP_ckick_margin(float CKickMargin)
	{ SP_ckick_margin = CKickMargin; }
	float GetSP_ckick_margin() const
	{ return SP_ckick_margin; }
	
	void SetSP_dash_power_rate(float DashPowerRate)
	{ SP_dash_power_rate = DashPowerRate; }
	float GetSP_dash_power_rate() const
	{ return SP_dash_power_rate; }
	
	void SetSP_kick_power_rate(float KickPowerRate)
	{ SP_kick_power_rate = KickPowerRate; }
	float GetSP_kick_power_rate() const
	{ return SP_kick_power_rate; }
	
	void SetSP_visible_angle(float VisibleAngle)
	{ SP_visible_angle = VisibleAngle; }
	float GetSP_visible_angle() const
	{ return SP_visible_angle; }
	
	void SetSP_audio_cut_dist(float AudioCutDist)
	{ SP_audio_cut_dist = AudioCutDist; }
	float GetSP_audio_cut_dist() const
	{ return SP_audio_cut_dist; }
	
	void SetSP_quantize_step(float QuantizeStep)
	{ SP_quantize_step = QuantizeStep; }
	float GetSP_quantize_step() const
	{ return SP_quantize_step; }
	
	void SetSP_quantize_step_l(float QuantizeStepL)
	{ SP_quantize_step_l = QuantizeStepL; }
	float GetSP_quantize_step_l() const
	{ return SP_quantize_step_l; }
	
	void SetSP_maxpower(int MaxPower)
	{ SP_maxpower = MaxPower; }
	int GetSP_maxpower() const
	{ return SP_maxpower; }
	
	void SetSP_minpower(int MinPower)
	{ SP_minpower = MinPower; }
	int GetSP_minpower() const
	{ return SP_minpower; }
	
	void SetSP_maxmoment(int MaxMoment)
	{ SP_maxmoment = MaxMoment; }
	int GetSP_maxmoment() const
	{ return SP_maxmoment; }
	
	void SetSP_minmoment(int MinMoment)
	{ SP_minmoment = MinMoment; }
	int GetSP_minmoment() const
	{ return SP_minmoment; }

	void SetSP_maxneckmoment(int MaxNeckMoment)
	{ SP_maxneckmoment = MaxNeckMoment; }
	int	GetSP_maxneckmoment() const
	{ return SP_maxneckmoment; }

	void SetSP_minneckmoment(int MinNeckMoment)
	{ SP_minneckmoment = MinNeckMoment; }
	int	GetSP_minneckmoment() const
	{ return SP_minneckmoment; }
	
	void SetSP_maxneckang(int MaxNeckAng)
	{ SP_maxneckang = MaxNeckAng; }
	int	GetSP_maxneckang() const
	{ return SP_maxneckang; }
	
	void SetSP_minneckang(int MinNeckAng)
	{ SP_minneckang = MinNeckAng; }
	int	GetSP_minneckang() const
	{ return SP_minneckang; }
	
	void SetSP_port(int Port)
	{ SP_port = Port; }
	int GetSP_port() const
	{ return SP_port; }
	
	void SetSP_coach_port(int CoachPort)
	{ SP_coach_port = CoachPort; }
	int GetSP_coach_port() const
	{ return SP_coach_port; }
	
	void SetSP_say_coach_cnt_max(int SayCoachCntMax)
	{ SP_say_coach_cnt_max = SayCoachCntMax; }
	int GetSP_say_coach_cnt_max() const
	{ return SP_say_coach_cnt_max; }
	
	void SetSP_say_coach_msg_size(int SayCoachMsgSize)
	{ SP_say_coach_msg_size = SayCoachMsgSize; }
	int GetSP_say_coach_msg_size() const
	{ return SP_say_coach_msg_size; }
	
	void SetSP_send_vi_step(int SendViStep)
	{ SP_send_vi_step = SendViStep; }
	int GetSP_send_vi_step() const
	{ return SP_send_vi_step; }
	
	void SetSP_simulator_step(int SimulatorStep)
	{ SP_simulator_step = SimulatorStep; }
	int GetSP_simulator_step() const
	{ return SP_simulator_step; }
	
	void SetSP_send_step(int SendStep)
	{ SP_send_step = SendStep; }
	int GetSP_send_step() const
	{ return SP_send_step; }
	
	void SetSP_recv_step(int RecvStep)
	{ SP_recv_step = RecvStep; }
	int GetSP_recv_step() const
	{ return SP_recv_step; }
	
	void SetSP_half_time(int HalfTime)
	{ SP_half_time = HalfTime; }
	int GetSP_half_time() const
	{ return SP_half_time; }
	
	void SetSP_say_msg_size(int SayMsgSize)
	{ SP_say_msg_size = SayMsgSize; }
	int GetSP_say_msg_size() const
	{ return SP_say_msg_size; }
	
	void SetSP_use_offside(int UseOffside)
	{ SP_use_offside = UseOffside; }
	int GetSP_use_offside() const
	{ return SP_use_offside; }
	
	void SetSP_offside_active_area_size(float OffsideActiveAreaSize)
	{ SP_offside_active_area_size = OffsideActiveAreaSize; }
	float GetSP_offside_active_area_size() const
	{ return SP_offside_active_area_size; }
	
	void SetSP_forbid_kick_off_offside(int ForbidKickOffOffside)
	{ SP_forbid_kick_off_offside = ForbidKickOffOffside; }
	int GetSP_forbid_kick_off_offside() const
	{ return SP_forbid_kick_off_offside; }
	
	void SetSP_verbose(int Verbose)
	{ SP_verbose = Verbose; }
	int GetSP_verbose() const
	{ return SP_verbose; }
	
	void SetSP_log_file(char* LogFile)
	{ strncpy(SP_log_file, LogFile, 256); }
	const char* GetSP_log_file() const
	{ return SP_log_file; }
	
	void SetSP_record_version(int RecordVersion)
	{ SP_record_version = RecordVersion; }
	int GetSP_record_version() const
	{ return SP_record_version; }
	
	void SetSP_record_log(int RecordLog)
	{ SP_record_log = RecordLog; }
	int GetSP_record_log() const
	{ return SP_record_log; }
	
	void SetSP_send_log(int SendLog)
	{ SP_send_log = SendLog; }
	int GetSP_send_log() const
	{ return SP_send_log; }
	
	void SetAP_team_name(char* TName)
	{ strncpy(AP_team_name, TName, MAXTEAMNAMELEN); }
	const char* GetAP_team_name() const
	{ return AP_team_name; }
	
	void GetAP_hostname(char* Hostname)
	{ strncpy(AP_hostname, Hostname, MAXHOSTNAMELEN); }
	const char* GetAP_hostname() const
	{ return AP_hostname; }
	
	void SetAP_port(int Port)
	{ AP_port = Port; }
	int GetAP_port() const
	{ return AP_port; }
	
	void SetAP_score_diff(int ScoreDiff)
	{ AP_score_diff = ScoreDiff; }
	int GetAP_score_diff() const
	{ return AP_score_diff; }
	
	void SetAP_change_formation_thr(int ChangeFormationThr)
	{ AP_change_formation_thr = ChangeFormationThr; }
	int GetAP_change_formation_thr() const
	{ return AP_change_formation_thr; }
	
	void SetAP_losing_formation(int LosingFormation)
	{ AP_losing_formation = LosingFormation; }
	int GetAP_losing_formation() const
	{ return AP_losing_formation; }
	
	void SetAP_normal_formation(int NormalFormation)
	{ AP_normal_formation = NormalFormation; }
	int GetAP_normal_formation() const
	{ return AP_normal_formation; }
	
	void SetAP_winning_formation(int WinningFormation)
	{ AP_winning_formation = WinningFormation; }
	int GetAP_winning_formation() const
	{ return AP_winning_formation; }
	
	void SetAP_sim_step(long SimStep)
	{ AP_sim_step = SimStep; }
	long GetAP_sim_step() const
	{ return AP_sim_step; }
	
	void SetAP_mob_conf_drop(float MobConfDrop)
	{ AP_mob_conf_drop = MobConfDrop; }
	float GetAP_mob_conf_drop() const
	{ return AP_mob_conf_drop; }
	
	void SetAP_stat_conf_drop(float StatConfDrop)
	{ AP_stat_conf_drop = StatConfDrop; }
	float GetAP_stat_conf_drop() const
	{ return AP_stat_conf_drop; }
	
	void SetAP_mob_conf_thr(float MobConfThr)
	{ AP_mob_conf_thr = MobConfThr; }
	float GetAP_mob_conf_thr() const
	{ return AP_mob_conf_thr; }
	
	void SetAP_stat_conf_thr(float StatConfThr)
	{ AP_stat_conf_thr = StatConfThr; }
	float GetAP_stat_conf_thr() const
	{ return AP_stat_conf_thr; }
	
	void SetAP_epsilon(float Epsilon)
	{ AP_epsilon = Epsilon; }
	float GetAP_epsilon() const
	{ return AP_epsilon; }
	
	void SetAP_slow(float Slow)
	{ AP_slow = Slow; }
	float GetAP_slow() const
	{ return AP_slow; }
	
	void SetAP_normal(float Normal)
	{ AP_normal = Normal; }
	float GetAP_normal() const
	{ return AP_normal; }
	
	void SetAP_fast(float Fast)
	{ AP_fast = Fast; }
	float GetAP_fast() const
	{ return AP_fast; }
	
	void SetAP_max_go_angle(float MaxGoAngle)
	{ AP_max_go_angle = MaxGoAngle; }
	float GetAP_max_go_angle() const
	{ return AP_max_go_angle; }
	
	void SetAP_min_dist_to_point(float MinDistToPoint)
	{ AP_min_dist_to_point = MinDistToPoint; }
	float GetAP_min_dist_to_point() const
	{ return AP_min_dist_to_point; }
	
	void SetAP_max_move_dashes(int MaxMoveDashes)
	{ AP_max_move_dashes = MaxMoveDashes; }
	int GetAP_max_move_dashes() const
	{ return AP_max_move_dashes; }
	
	void SetAP_max_intercept_dashes(int MaxInterceptDashes)
	{ AP_max_intercept_dashes = MaxInterceptDashes; }
	int GetAP_max_intercept_dashes() const
	{ return AP_max_intercept_dashes; }
	
	void SetAP_def_zone(int DefZone)
	{ AP_def_zone = DefZone; }
	int GetAP_def_zone() const
	{ return AP_def_zone; }
	
	void SetAP_att_zone(int AttZone)
	{ AP_att_zone = AttZone; }
	int GetAP_att_zone() const
	{ return AP_att_zone; }
	
	void SetAP_intercept_step(int InterceptStep)
	{ AP_intercept_step = InterceptStep; }
	int GetAP_intercept_step() const
	{ return AP_intercept_step; }
	
	void SetAP_intercept_tol(float InterceptTol)
	{ AP_intercept_tol = InterceptTol; }
	float GetAP_intercept_tol() const
	{ return AP_intercept_tol; }
	
	void SetAP_inner_ring(float InnerRing)
	{ AP_inner_ring = InnerRing; }
	float GetAP_inner_ring() const
	{ return AP_inner_ring; }
	
	void SetAP_outer_ring(float OuterRing)
	{ AP_outer_ring = OuterRing; }
	float GetAP_outer_ring() const
	{ return AP_outer_ring; }
	
	void SetAP_pow_to_stop(float PowToStop)
	{ AP_pow_to_stop = PowToStop; }
	float GetAP_pow_to_stop() const
	{ return AP_pow_to_stop; }
	
	void SetAP_stop_pass(float StopPass)
	{ AP_stop_pass = StopPass; }
	float GetAP_stop_pass() const
	{ return AP_stop_pass; }
	
	void SetAP_soft_pass(float SoftPass)
	{ AP_soft_pass = SoftPass; }
	float GetAP_soft_pass() const
	{ return AP_soft_pass; }
	
	void SetAP_medium_pass(float MediumPass)
	{ AP_medium_pass = MediumPass; }
	float GetAP_medium_pass() const
	{ return AP_medium_pass; }
	
	void SetAP_hard_pass(float HardPass)
	{ AP_hard_pass = HardPass; }
	float GetAP_hard_pass() const
	{ return AP_hard_pass; }
	
	void SetAP_spin_dir_tol(int SpinDirTol)
	{ AP_spin_dir_tol = SpinDirTol; }
	int GetAP_spin_dir_tol() const
	{ return AP_spin_dir_tol; }
	
	void SetAP_max_spin_power(float MaxSpinPower)
	{ AP_max_spin_power = MaxSpinPower; }
	float GetAP_max_spin_power() const
	{ return AP_max_spin_power; }
	
	void SetAP_max_stop_speed(float MaxStopSpeed)
	{ AP_max_stop_speed = MaxStopSpeed; }
	float GetAP_max_stop_speed() const
	{ return AP_max_stop_speed; }
	
	void SetAP_min_opp_worry_dist(int MinOppWorryDist)
	{ AP_min_opp_worry_dist = MinOppWorryDist; }
	int GetAP_min_opp_worry_dist() const
	{ return AP_min_opp_worry_dist; }
	
	void SetAP_dodge_dist_buf(float DodgeDistBuf)
	{ AP_dodge_dist_buf = DodgeDistBuf; }
	float GetAP_dodge_dist_buf() const
	{ return AP_dodge_dist_buf; }
	
	void SetAP_dribble_dodge_close_dist(int DribbleDodgeCloseDist)
	{ AP_dribble_dodge_close_dist = DribbleDodgeCloseDist; }
	int GetAP_dribble_dodge_close_dist() const
	{ return AP_dribble_dodge_close_dist; }
	
	void SetAP_dribble_dodge_angle_err(int DribbleDodgeAngleErr)
	{ AP_dribble_dodge_angle_err = DribbleDodgeAngleErr; }
	int GetAP_dribble_dodge_angle_err() const
	{ return AP_dribble_dodge_angle_err; }
	
	void SetAP_ball_dribble_dist(float BallDribbleDist)
	{ AP_ball_dribble_dist = BallDribbleDist; }
	float GetAP_ball_dribble_dist() const
	{ return AP_ball_dribble_dist; }
	
	void SetAP_max_dribble_exp_angle(int MaxDribbleExpAngle)
	{ AP_max_dribble_exp_angle = MaxDribbleExpAngle; }
	int GetAP_max_dribble_exp_angle() const
	{ return AP_max_dribble_exp_angle; }
	
	void SetAP_dribble_normal_angle(float DribbleNormalAngle)
	{ AP_dribble_normal_angle = DribbleNormalAngle; }
	float GetAP_dribble_normal_angle() const
	{ return AP_dribble_normal_angle; }

	void SetAP_collision_buffer(float CollisionBuffer)
	{ AP_collision_buffer = CollisionBuffer; }
	float GetAP_collision_buffer() const
	{ return AP_collision_buffer; }
	
};

#endif
