/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CLine class
 * 				The CLine class holds information regarding a line on the field.
 * Created: 14/12/98
 * Last modified: 10/7/99
 */

#ifndef _CLINE_H
#define _CLINE_H
 
#include "CVector.H"
 
class CLine
{
// Private member data /////////////////////////////////////////////////////////
private:

// These are default values in server.conf and agent.conf
	float	m_StatConfThr;

// These values are fixed ///////////////////////////////////////////////
	char 	m_ID;			// Line ID: l-left, t-top, r-right, b-bottom, u-unknown
	CVector m_GPos;			// Global position
	
// These values are obtained from sensors ///////////////////////////////
	int 	m_Time;			// Time of observation
	CVector m_RPos;			// Relative position
	
// These values are calculated //////////////////////////////////////////
	float 	m_Conf;			// Confidence

// Public member functions ///////////////////////////////////////////////////////
public:

// Constructor/Destructor, Init, and Display functions ///////////////
	CLine();				// Default constructor
	~CLine();				// Default destructor
	void Init(char, float, float, float);		// Used to initialise lines
	void RevertSigns();				// Revert signs for right side players
	void Display() const;			// Displays all params.  Just for debugging
	const CLine *GetThis() const	// Function that returns the "this" pointer
	{ return this; }				// Used by Get*** in Memory
		
// Basic line data set/get functions /////////////////////////////////
	void	SetID	(char ID)			// Set line's ID
	{ m_ID = ID; }
	char	GetID	() const			// Get line's ID
	{ return m_ID; }

	void	SetGPos	(CVector GPos)	// Set goal's position
	{ m_GPos = GPos; }
	CVector	GetGPos	() const		// Get goal's position
	{ return m_GPos; }

// Accessor function for GPOS ///////////////////////////////////////

	void	SetGDir	(float	Dir)		// Set relative direction
	{ m_GPos.SetDir(Dir); }
	float	GetGDir	() const			// Get relative direction
	{ return m_GPos.GetDir(); }
	
	void	SetGMag	(float	Mag)		// Set relative magnitude
	{ m_GPos.SetMag(Mag); }
	float	GetGMag	() const			// Get relative magnitude
	{ return m_GPos.GetMag(); }

	void	SetGX	(float	X)			// Set global X co-ordinate
	{ m_GPos.SetX(X); }
	float	GetGX	() const			// Get global X co-ordinate
	{ return m_GPos.GetX(); }	
	
	void	SetGY	(float	Y)			// Set global Y co-ordinate
	{ m_GPos.SetY(Y); }
	float	GetGY	() const			// Get global Y co-ordinate
	{ return m_GPos.GetY(); }	
		
/////////////////////////////////////////////////////////////////////	

	void	SetTime	(int Time)			// Set line's time
	{ m_Time = Time; }
	int		GetTime	() const			// Get line's time
	{ return m_Time; }
	
	void	SetRPos	(CVector RPos)		// Set line's relative position
	{ m_RPos = RPos; }
	CVector	GetRPos	() const			// Get line's relative position
	{ return m_RPos; }
	void	UpdateRPos(CVector, float, int);	// Update line's relative position if not seen
	
// Accessor function for RPOS ///////////////////////////////////////
	
	void	SetRDir	(float	Dir)		// Set relative direction
	{ m_RPos.SetDir(Dir); }
	float	GetRDir	() const			// Get relative direction
	{ return m_RPos.GetDir(); }
	
	void	SetRMag	(float	Mag)		// Set relative magnitude
	{ m_RPos.SetMag(Mag); }
	float	GetRMag	() const			// Get relative magnitude
	{ return m_RPos.GetMag(); }
	
	void	SetRX	(float	X)			// Set relative X co-ordinate
	{ m_RPos.SetX(X); }
	float	GetRX	() const			// Get relative X co-ordinate
	{ return m_RPos.GetX(); }	
	
	void	SetRY	(float	Y)			// Set relative Y co-ordinate
	{ m_RPos.SetY(Y); }
	float	GetRY	() const			// Get relative Y co-ordinate
	{ return m_RPos.GetY(); }	
	
/////////////////////////////////////////////////////////////////////
	
	void	SetConf	(float Conf)		// Set line's confidence
	{ m_Conf = Conf; }
	float	GetConf	() const			// Get line's confidence
	{ return m_Conf; }
};
 
#endif
