/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CMemory class
 * 				The CMemory class holds information regarding all the objects on the field.
 * Created: 17/12/98
 * Last modified: 10/7/99
 */

#include <iostream.h>
#include "CGetParams.H"
#include "CLine.H"
#include "CFlag.H"
#include "CGoal.H"
#include "CPlayer.H"
#include "CBall.H"
#include "CVector.H" 
#include "CMemory.H"

// Constructor that takes Mediator //////////////////////////////////
CMemory::CMemory(CMediator* Mediator):CModule(Mediator)
{
cout << "[Creating & Initialising Memory   ]" << endl;

// Initialise here for safety reasons...  Values read from the configuration file will overwritte these
m_PlayerSize = 0.3;
m_BallSize = 0.085;
m_PlayerDecay = 0.4;
m_BallDecay = 0.94;
m_PlayerSpeedMax = 1.0;
m_BallSpeedMax = 2.7;
m_KickableMargin = 0.7;
m_Kickable = m_KickableMargin + m_PlayerSize + m_BallSize;
m_InertiaMoment = 5.0;
m_KickPowerRate = 0.016;
m_DashPowerRate = 0.006;
m_StaminaIncMax = 35.0;

m_ClosestDist = m_PlayerSize + 2.0 * m_BallSize;
m_Epsilon = 0.0001;
m_StatConfDrop = 0.95;
m_StatConfThr = 0.50;
m_MobConfDrop = 0.90;
m_MobConfThr = 0.40;
}

// INIT /////////////////////////////////////////////////////////////
void CMemory::Init(CGetParams MemoryParams)	// Initialise all fields using MemoryParams
{
float PL, PW;	// Pitch length and width
float PAL, PAW;	// Penalty areas length and width
float GW, PM;	// Goal width, and pitch margin

pthread_mutex_init(&m_Mutex, NULL);				// Initialise mutex

PL = MemoryParams.GetSP_pitch_length();			// Set parameters needed
PW = MemoryParams.GetSP_pitch_width();
PAL = MemoryParams.GetSP_penalty_area_length();
PAW = MemoryParams.GetSP_penalty_area_width();
GW = MemoryParams.GetSP_goal_width();
PM = MemoryParams.GetSP_pitch_margin();

m_GoalClose.Init('u', 0.0, 0.0, MemoryParams.GetAP_stat_conf_thr());	// Close objects (i.e. Flag, Line, Goal...)
m_FlagClose.Init(0, 0.0, 0.0, MemoryParams.GetAP_stat_conf_thr());
m_LineClose.Init('u', 0.0, 0.0, MemoryParams.GetAP_stat_conf_thr());
m_PlayerClose.Init(0, 'u', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());

m_LineL.Init('l', -PL/2.0, 0.0, MemoryParams.GetAP_stat_conf_thr());		// A field has 4 lines
m_LineT.Init('t', 0.0, -PW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_LineR.Init('r', PL/2.0, 0.0, MemoryParams.GetAP_stat_conf_thr());
m_LineB.Init('b', 0.0, PW/2.0, MemoryParams.GetAP_stat_conf_thr());

m_FlagC.Init(1, 0.0, 0.0, MemoryParams.GetAP_stat_conf_thr());	// A field has 53 flags.  A center flag

m_FlagPLB.Init(2, -PL/2.0+PAL, PAW/2.0, MemoryParams.GetAP_stat_conf_thr());	// 6 penalty flags
m_FlagPLC.Init(3, -PL/2.0+PAL, 0, MemoryParams.GetAP_stat_conf_thr());
m_FlagPLT.Init(4, -PL/2.0+PAL, -PAW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagPRT.Init(5, PL/2.0-PAL, -PAW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagPRC.Init(6, PL/2.0-PAL, 0, MemoryParams.GetAP_stat_conf_thr());
m_FlagPRB.Init(7, PL/2.0-PAL, PAW/2.0, MemoryParams.GetAP_stat_conf_thr());

m_FlagLB.Init(8, -PL/2.0, PW/2.0, MemoryParams.GetAP_stat_conf_thr());			// 10 flags on the lines
m_FlagGLB.Init(9, -PL/2.0, GW/2.0, MemoryParams.GetAP_stat_conf_thr());			// Out of which 4 are goal flags
m_FlagGLT.Init(10, -PL/2.0, -GW/2.0, MemoryParams.GetAP_stat_conf_thr());		//
m_FlagLT.Init(11, -PL/2.0, -PW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagCT.Init(12, 0, -PW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagRT.Init(13, PL/2.0, -PW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagGRT.Init(14, PL/2.0, -GW/2.0, MemoryParams.GetAP_stat_conf_thr());		//
m_FlagGRB.Init(15, PL/2.0, GW/2.0, MemoryParams.GetAP_stat_conf_thr());			//
m_FlagRB.Init(16, PL/2.0, PW/2.0, MemoryParams.GetAP_stat_conf_thr());
m_FlagCB.Init(17, 0, PW/2.0, MemoryParams.GetAP_stat_conf_thr());

m_FlagLB30.Init(18, -PL/2.0-PM, 30, MemoryParams.GetAP_stat_conf_thr());		// And 36 flags outside the pitch
m_FlagLB20.Init(19, -PL/2.0-PM, 20, MemoryParams.GetAP_stat_conf_thr());
m_FlagLB10.Init(20, -PL/2.0-PM, 10, MemoryParams.GetAP_stat_conf_thr());
m_FlagL0.Init(21, -PL/2.0-PM, 0, MemoryParams.GetAP_stat_conf_thr());
m_FlagLT10.Init(22, -PL/2.0-PM, -10, MemoryParams.GetAP_stat_conf_thr());
m_FlagLT20.Init(23, -PL/2.0-PM, -20, MemoryParams.GetAP_stat_conf_thr());
m_FlagLT30.Init(24, -PL/2.0-PM, -30, MemoryParams.GetAP_stat_conf_thr());
m_FlagTL50.Init(25, -50, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTL40.Init(26, -40, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTL30.Init(27, -30, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTL20.Init(28, -20, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTL10.Init(29, -10, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagT0.Init(30, 0, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTR10.Init(31, 10, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTR20.Init(32, 20, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTR30.Init(33, 30, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTR40.Init(34, 40, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagTR50.Init(35, 50, -PW/2.0-PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagRT30.Init(36, PL/2.0+PM, -30, MemoryParams.GetAP_stat_conf_thr());
m_FlagRT20.Init(37, PL/2.0+PM, -20, MemoryParams.GetAP_stat_conf_thr());
m_FlagRT10.Init(38, PL/2.0+PM, -10, MemoryParams.GetAP_stat_conf_thr());
m_FlagR0.Init(39, PL/2.0+PM, 0, MemoryParams.GetAP_stat_conf_thr());
m_FlagRB10.Init(40, PL/2.0+PM, 10, MemoryParams.GetAP_stat_conf_thr());
m_FlagRB20.Init(41, PL/2.0+PM, 20, MemoryParams.GetAP_stat_conf_thr());
m_FlagRB30.Init(42, PL/2.0+PM, 30, MemoryParams.GetAP_stat_conf_thr());
m_FlagBR50.Init(43, 50, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBR40.Init(44, 40, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBR30.Init(45, 30, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBR20.Init(46, 20, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBR10.Init(47, 10, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagB0.Init(48, 0, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBL10.Init(49, -10, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBL20.Init(50, -20, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBL30.Init(51, -30, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBL40.Init(52, -40, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());
m_FlagBL50.Init(53, -50, PW/2.0+PM, MemoryParams.GetAP_stat_conf_thr());

m_GoalL.Init('l', -PL/2.0, 0.0, MemoryParams.GetAP_stat_conf_thr());	// A field has 2 goals
m_GoalR.Init('r', PL/2.0, 0.0, MemoryParams.GetAP_stat_conf_thr());

// A field has 22 players
m_PlayerL1.Init(1, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL2.Init(2, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL3.Init(3, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL4.Init(4, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL5.Init(5, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL6.Init(6, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL7.Init(7, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL8.Init(8, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL9.Init(9, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL10.Init(10, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerL11.Init(11, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR1.Init(1, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR2.Init(2, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR3.Init(3, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR4.Init(4, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR5.Init(5, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR6.Init(6, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR7.Init(7, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR8.Init(8, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR9.Init(9, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR10.Init(10, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());
m_PlayerR11.Init(11, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());

m_InertiaMoment = MemoryParams.GetSP_inertia_moment();		// Read inertia moment from params
m_DashPowerRate = MemoryParams.GetSP_dash_power_rate();		// Dash power rate from params
m_KickPowerRate = MemoryParams.GetSP_kick_power_rate();		// Kick power rate from params
m_PlayerDecay = MemoryParams.GetSP_player_decay();			// Player decay from params
m_PlayerSize = MemoryParams.GetSP_player_size();			// Player size from params
m_PlayerSpeedMax = MemoryParams.GetSP_player_speed_max();	// Max speed for players from params
m_BallDecay = MemoryParams.GetSP_ball_decay();				// Ball decay from params
m_BallSize = MemoryParams.GetSP_ball_size();				// Ball size from params
m_BallSpeedMax = MemoryParams.GetSP_ball_speed_max();		// Max speed for ball from params
m_KickableMargin = MemoryParams.GetSP_kickable_margin();	// Kickable margin from params
m_Kickable = m_KickableMargin + m_BallSize + m_PlayerSize;	// Calculate max kickable range
m_StaminaIncMax = MemoryParams.GetSP_stamina_inc_max();		// Stamina increase max from params
m_ClosestDist = m_PlayerSize + 2.0 * m_BallSize;			// The closest distance the ball can get (= PlayerSize + 2*BallSize)

m_Ball.Init(MemoryParams.GetSP_ball_size(), MemoryParams.GetSP_ball_decay(), MemoryParams.GetSP_ball_rand(),
MemoryParams.GetSP_ball_weight(), MemoryParams.GetSP_ball_speed_max(), m_Kickable, MemoryParams.GetAP_mob_conf_thr(),
MemoryParams.GetSP_catch_ban_cycle(), MemoryParams.GetSP_catchable_area_l(), MemoryParams.GetSP_catchable_area_w());	// Init ball

m_Epsilon = MemoryParams.GetAP_epsilon();					// Get epsilon from params
m_StatConfDrop = MemoryParams.GetAP_stat_conf_drop();		// Rate at which confidence of stationary objects drops after each cycle from params
m_StatConfThr = MemoryParams.GetAP_stat_conf_thr();			// Minimum confidence threshold for stationary objects from params
m_MobConfDrop = MemoryParams.GetAP_mob_conf_drop();			// Rate at which confidence of mobile objects drops after each cycle from params
m_MobConfThr = MemoryParams.GetAP_mob_conf_thr();			// Minimum confidence threshold for mobile objects from params
}

// REVERTSIGNS //////////////////////////////////////////////////
void CMemory::RevertSigns()									// Revert signs for right side players
{
m_LineL.RevertSigns();		// A field has 4 lines
m_LineT.RevertSigns();
m_LineR.RevertSigns();
m_LineB.RevertSigns();

m_FlagC.RevertSigns();

m_FlagPLB.RevertSigns();
m_FlagPLC.RevertSigns();
m_FlagPLT.RevertSigns();
m_FlagPRT.RevertSigns();
m_FlagPRC.RevertSigns();
m_FlagPRB.RevertSigns();

m_FlagLB.RevertSigns();
m_FlagGLB.RevertSigns();
m_FlagGLT.RevertSigns();
m_FlagLT.RevertSigns();
m_FlagCT.RevertSigns();
m_FlagRT.RevertSigns();
m_FlagGRT.RevertSigns();
m_FlagGRB.RevertSigns();
m_FlagRB.RevertSigns();
m_FlagCB.RevertSigns();

m_FlagLB30.RevertSigns();
m_FlagLB20.RevertSigns();
m_FlagLB10.RevertSigns();
m_FlagL0.RevertSigns();
m_FlagLT10.RevertSigns();
m_FlagLT20.RevertSigns();
m_FlagLT30.RevertSigns();
m_FlagTL50.RevertSigns();
m_FlagTL40.RevertSigns();
m_FlagTL30.RevertSigns();
m_FlagTL20.RevertSigns();
m_FlagTL10.RevertSigns();
m_FlagT0.RevertSigns();
m_FlagTR10.RevertSigns();
m_FlagTR20.RevertSigns();
m_FlagTR30.RevertSigns();
m_FlagTR40.RevertSigns();
m_FlagTR50.RevertSigns();
m_FlagRT30.RevertSigns();
m_FlagRT20.RevertSigns();
m_FlagRT10.RevertSigns();
m_FlagR0.RevertSigns();
m_FlagRB10.RevertSigns();
m_FlagRB20.RevertSigns();
m_FlagRB30.RevertSigns();
m_FlagBR50.RevertSigns();
m_FlagBR40.RevertSigns();
m_FlagBR30.RevertSigns();
m_FlagBR20.RevertSigns();
m_FlagBR10.RevertSigns();
m_FlagB0.RevertSigns();
m_FlagBL10.RevertSigns();
m_FlagBL20.RevertSigns();
m_FlagBL30.RevertSigns();
m_FlagBL40.RevertSigns();
m_FlagBL50.RevertSigns();

m_GoalL.RevertSigns();
m_GoalR.RevertSigns();
}

// INITPLAYERARRAYS /////////////////////////////////////////////
void CMemory::InitPlayerArrays(char Side, CGetParams MemoryParams)		// Initialise Friend/Foe arrays
{
int i;											// for loop counter

if (Side == 'l')								// If left side
	{
	m_Friend[0] = m_PlayerL1.GetNonConstThis();	// Assign pointers
	m_Friend[1] = m_PlayerL2.GetNonConstThis();
	m_Friend[2] = m_PlayerL3.GetNonConstThis();
	m_Friend[3] = m_PlayerL4.GetNonConstThis();
	m_Friend[4] = m_PlayerL5.GetNonConstThis();
	m_Friend[5] = m_PlayerL6.GetNonConstThis();
	m_Friend[6] = m_PlayerL7.GetNonConstThis();
	m_Friend[7] = m_PlayerL8.GetNonConstThis();
	m_Friend[8] = m_PlayerL9.GetNonConstThis();
	m_Friend[9] = m_PlayerL10.GetNonConstThis();
	m_Friend[10] = m_PlayerL11.GetNonConstThis();
	
	m_Foe[0] = m_PlayerR1.GetNonConstThis();
	m_Foe[1] = m_PlayerR2.GetNonConstThis();
	m_Foe[2] = m_PlayerR3.GetNonConstThis();
	m_Foe[3] = m_PlayerR4.GetNonConstThis();
	m_Foe[4] = m_PlayerR5.GetNonConstThis();
	m_Foe[5] = m_PlayerR6.GetNonConstThis();
	m_Foe[6] = m_PlayerR7.GetNonConstThis();
	m_Foe[7] = m_PlayerR8.GetNonConstThis();
	m_Foe[8] = m_PlayerR9.GetNonConstThis();
	m_Foe[9] = m_PlayerR10.GetNonConstThis();
	m_Foe[10] = m_PlayerR11.GetNonConstThis();
	}

else if (Side == 'r')							// If right side
	{
	m_Foe[0] = m_PlayerL1.GetNonConstThis();			// Assign pointers
	m_Foe[1] = m_PlayerL2.GetNonConstThis();
	m_Foe[2] = m_PlayerL3.GetNonConstThis();
	m_Foe[3] = m_PlayerL4.GetNonConstThis();
	m_Foe[4] = m_PlayerL5.GetNonConstThis();
	m_Foe[5] = m_PlayerL6.GetNonConstThis();
	m_Foe[6] = m_PlayerL7.GetNonConstThis();
	m_Foe[7] = m_PlayerL8.GetNonConstThis();
	m_Foe[8] = m_PlayerL9.GetNonConstThis();
	m_Foe[9] = m_PlayerL10.GetNonConstThis();
	m_Foe[10] = m_PlayerL11.GetNonConstThis();
	
	m_Friend[0] = m_PlayerR1.GetNonConstThis();
	m_Friend[1] = m_PlayerR2.GetNonConstThis();
	m_Friend[2] = m_PlayerR3.GetNonConstThis();
	m_Friend[3] = m_PlayerR4.GetNonConstThis();
	m_Friend[4] = m_PlayerR5.GetNonConstThis();
	m_Friend[5] = m_PlayerR6.GetNonConstThis();
	m_Friend[6] = m_PlayerR7.GetNonConstThis();
	m_Friend[7] = m_PlayerR8.GetNonConstThis();
	m_Friend[8] = m_PlayerR9.GetNonConstThis();
	m_Friend[9] = m_PlayerR10.GetNonConstThis();
	m_Friend[10] = m_PlayerR11.GetNonConstThis();
	}

else
	cerr << "WARNING: CMemory: InitPlayerArrays: What side is that -" << Side << "-?" << endl;

for (i=0; i<11; i++)					// Also create the Far??? player arrays
	{
	m_FarFriend[i] = new CPlayer;		// Create the players
	m_FarFoe[i] = new CPlayer;			// 
	if (Side == 'l')					// Then depending on side
		{
		m_FarFriend[i]->Init(0, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());	// initialise them
		m_FarFoe[i]->Init(0, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());		//
		}
	else if (Side == 'r')				//
		{
		m_FarFriend[i]->Init(0, 'r', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());	//
		m_FarFoe[i]->Init(0, 'l', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());		//
		}
	else
		cerr << "WARNING: CMemory: InitPlayerArrays: What side is that -" << Side << "-?" << endl;
	}
	
for (i=0; i<22; i++)					// Create the 22 unknown players
	{
	m_UnknownPlayer[i] = new CPlayer;	// and initialise them
	m_UnknownPlayer[i]->Init(0, 'u', MemoryParams.GetSP_player_size(), MemoryParams.GetSP_player_decay(), MemoryParams.GetSP_player_rand(),
MemoryParams.GetSP_player_weight(), MemoryParams.GetSP_player_speed_max(), MemoryParams.GetAP_mob_conf_thr());	// as 'unknown'
	}
}

// DELETEPLAYERARRAYS /////////////////////////////////////////////
void CMemory::DeletePlayerArrays()		// Delete player arrays (the ones that have been new-ed i.e.
{										// m_FarFriend, m_FarFoe, and m_UnknownPlayer)
int i;							// for loop counter

for (i=0; i<11; i++)			// For all 11 Far??? players
	{
	delete m_FarFriend[i];		// Delete them and set pointer to NULL
	m_FarFriend[i] = 0;			// just to be safe
	delete m_FarFoe[i];			//
	m_FarFoe[i] = 0;			//
	}

for (i=0; i<22; i++)			// For all 22 Unknown players
	{
	delete m_UnknownPlayer[i];	// Delete them and set pointer to NULL
	m_UnknownPlayer[i] = 0;		// just to be safe
	}
}

// DISPLAY /////////////////////////////////////////////////////
void CMemory::Display() const	// Displays all params.  Just for debugging
{
int i;						// Used in for loop

cout << "MEMORY GoalClose: " << endl;
m_GoalClose.Display();
cout << "MEMORY FlagClose: " << endl;
m_FlagClose.Display();
cout << "MEMORY LineClose: " << endl;
m_LineClose.Display();
cout << "MEMORY PlayerClose: " << endl;
m_PlayerClose.Display();

cout << "MEMORY LineL: " << endl;
m_LineL.Display();
cout << "MEMORY LineT: " << endl;
m_LineT.Display();
cout << "MEMORY LineR: " << endl;
m_LineR.Display();
cout << "MEMORY LineB: " << endl;
m_LineB.Display();

cout << "MEMORY FlagC: " << endl;
m_FlagC.Display();
cout << "MEMORY FlagPLB: " << endl;
m_FlagPLB.Display();
cout << "MEMORY FlagPLC: " << endl;
m_FlagPLC.Display();
cout << "MEMORY FlagPLT: " << endl;
m_FlagPLT.Display();
cout << "MEMORY FlagPRT: " << endl;
m_FlagPRT.Display();
cout << "MEMORY FlagPRC: " << endl;
m_FlagPRC.Display();
cout << "MEMORY FlagPRB: " << endl;
m_FlagPRB.Display();
cout << "MEMORY FlagLB: " << endl;
m_FlagLB.Display();
cout << "MEMORY FlagGLB: " << endl;
m_FlagGLB.Display();
cout << "MEMORY FlagGLT: " << endl;
m_FlagGLT.Display();
cout << "MEMORY FlagLT: " << endl;
m_FlagLT.Display();
cout << "MEMORY FlagCT: " << endl;
m_FlagCT.Display();
cout << "MEMORY FlagRT: " << endl;
m_FlagRT.Display();
cout << "MEMORY FlagGRT: " << endl;
m_FlagGRT.Display();
cout << "MEMORY FlagGRB: " << endl;
m_FlagGRB.Display();
cout << "MEMORY FlagRB: " << endl;
m_FlagRB.Display();
cout << "MEMORY FlagCB: " << endl;
m_FlagCB.Display();
cout << "MEMORY FlagLB30: " << endl;
m_FlagLB30.Display();
cout << "MEMORY FlagLB20: " << endl;
m_FlagLB20.Display();
cout << "MEMORY FlagLB10: " << endl;
m_FlagLB10.Display();
cout << "MEMORY FlagL0: " << endl;
m_FlagL0.Display();
cout << "MEMORY FlagLT10: " << endl;
m_FlagLT10.Display();
cout << "MEMORY FlagLT20: " << endl;
m_FlagLT20.Display();
cout << "MEMORY FlagLT30: " << endl;
m_FlagLT30.Display();
cout << "MEMORY FlagTL50: " << endl;
m_FlagTL50.Display();
cout << "MEMORY FlagTL40: " << endl;
m_FlagTL40.Display();
cout << "MEMORY FlagTL30: " << endl;
m_FlagTL30.Display();
cout << "MEMORY FlagTL20: " << endl;
m_FlagTL20.Display();
cout << "MEMORY FlagTL10: " << endl;
m_FlagTL10.Display();
cout << "MEMORY FlagT0: " << endl;
m_FlagT0.Display();
cout << "MEMORY FlagTR10: " << endl;
m_FlagTR10.Display();
cout << "MEMORY FlagTR20: " << endl;
m_FlagTR20.Display();
cout << "MEMORY FlagTR30: " << endl;
m_FlagTR30.Display();
cout << "MEMORY FlagTR40: " << endl;
m_FlagTR40.Display();
cout << "MEMORY FlagTR50: " << endl;
m_FlagTR50.Display();
cout << "MEMORY FlagRT30: " << endl;
m_FlagRT30.Display();
cout << "MEMORY FlagRT20: " << endl;
m_FlagRT20.Display();
cout << "MEMORY FlagRT10: " << endl;
m_FlagRT10.Display();
cout << "MEMORY FlagR0: " << endl;
m_FlagR0.Display();
cout << "MEMORY FlagRB10: " << endl;
m_FlagRB10.Display();
cout << "MEMORY FlagRB20: " << endl;
m_FlagRB20.Display();
cout << "MEMORY FlagRB30: " << endl;
m_FlagRB30.Display();
cout << "MEMORY FlagBR50: " << endl;
m_FlagBR50.Display();
cout << "MEMORY FlagBR40: " << endl;
m_FlagBR40.Display();
cout << "MEMORY FlagBR30: " << endl;
m_FlagBR30.Display();
cout << "MEMORY FlagBR20: " << endl;
m_FlagBR20.Display();
cout << "MEMORY FlagBR10: " << endl;
m_FlagBR10.Display();
cout << "MEMORY FlagB0: " << endl;
m_FlagB0.Display();
cout << "MEMORY FlagBL10: " << endl;
m_FlagBL10.Display();
cout << "MEMORY FlagBL20: " << endl;
m_FlagBL20.Display();
cout << "MEMORY FlagBL30: " << endl;
m_FlagBL30.Display();
cout << "MEMORY FlagBL40: " << endl;
m_FlagBL40.Display();
cout << "MEMORY FlagBL50: " << endl;
m_FlagBL50.Display();

cout << "MEMORY GoalL: " << endl;
m_GoalL.Display();
cout << "MEMORY GoalR: " << endl;
m_GoalR.Display();

cout << "MEMORY PlayerL1: " << endl;
m_PlayerL1.Display();
cout << "MEMORY PlayerL2: " << endl;
m_PlayerL2.Display();
cout << "MEMORY PlayerL3: " << endl;
m_PlayerL3.Display();
cout << "MEMORY PlayerL4: " << endl;
m_PlayerL4.Display();
cout << "MEMORY PlayerL5: " << endl;
m_PlayerL5.Display();
cout << "MEMORY PlayerL6: " << endl;
m_PlayerL6.Display();
cout << "MEMORY PlayerL7: " << endl;
m_PlayerL7.Display();
cout << "MEMORY PlayerL8: " << endl;
m_PlayerL8.Display();
cout << "MEMORY PlayerL9: " << endl;
m_PlayerL9.Display();
cout << "MEMORY PlayerL10: " << endl;
m_PlayerL10.Display();
cout << "MEMORY PlayerL11: " << endl;
m_PlayerL11.Display();
cout << "MEMORY PlayerR1: " << endl;
m_PlayerR1.Display();
cout << "MEMORY PlayerR2: " << endl;
m_PlayerR2.Display();
cout << "MEMORY PlayerR3: " << endl;
m_PlayerR3.Display();
cout << "MEMORY PlayerR4: " << endl;
m_PlayerR4.Display();
cout << "MEMORY PlayerR5: " << endl;
m_PlayerR5.Display();
cout << "MEMORY PlayerR6: " << endl;
m_PlayerR6.Display();
cout << "MEMORY PlayerR7: " << endl;
m_PlayerR7.Display();
cout << "MEMORY PlayerR8: " << endl;
m_PlayerR8.Display();
cout << "MEMORY PlayerR9: " << endl;
m_PlayerR9.Display();
cout << "MEMORY PlayerR10: " << endl;
m_PlayerR10.Display();
cout << "MEMORY PlayerR11: " << endl;
m_PlayerR11.Display();

cout << "MEMORY Ball: " << endl;
m_Ball.Display();

for (i=0; i<11; i++)
	{
	cout << "MEMORY FarFriend[" << i << "]:" << endl;
	if (m_FarFriend[i] != 0)
		m_FarFriend[i]->Display();
	else
		cout << "\tNULL" << endl;
	}
	
for (i=0; i<11; i++)
	{
	cout << "MEMORY FarFoe[" << i << "]:" << endl;
	if (m_FarFoe[i] != 0)
		m_FarFoe[i]->Display();
	else
		cout << "\tNULL" << endl;
	}
	
for (i=0; i<22; i++)
	{
	cout << "MEMORY UnknownPlayer[" << i << "]:" << endl;
	if (m_UnknownPlayer[i] != 0)
		m_UnknownPlayer[i]->Display();
	else
		cout << "\tNULL" << endl;
	}
}

// Lock/Unlock functions ////////////////////////////////////
// LOCK ////////////////////////////////////////////////////
void CMemory::Lock()					// Lock Memory
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING Memory..." << endl;
#endif

status = pthread_mutex_lock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CMemory: Lock: Locking error..." << endl;
}

// UNLOCK //////////////////////////////////////////////////
void CMemory::Unlock()					// Unlock Memory
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING Memory..." << endl;
#endif

status = pthread_mutex_unlock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CMemory: Unlock: Unlocking error..." << endl;
}
	
// Accessor functions ////////////////////////////////////////////////////////////////////////
// SETPLAYER /////////////////////////////////////////////////////////////////////////////////
void CMemory::SetPlayer(int Time, int UNum, char Side, float Dist, float Dir, float DistChng, float DirChng, float BFacing, float Facing) 				// Write player to Memory.
{
const float GFacing = GetFacing();			// Store my global facing direction here
CVector* pVelocity = GetNonConstVelocity();	// Get my own velocity
CVector tmp;								// Temprarily store velocity here

//cout << GetTime() << " PLAYER INFO: Time, UNum, Side, Dist, Dir, DistChng, DirChng, BFacing, Facing: "
//	<< Time << ", " << UNum << ", " << Side << ", " << Dist << ", " << Dir << ", " << DistChng << ", " << DirChng
//	<< ", " << BFacing << ", " << Facing << endl;
	
// If possible, calculate player's velocity.  In other words,
if ((DistChng != -999) && (DirChng != -999))	// If DistChng and DirChng info is available
	{
	CVector tmp(DistChng, Dist * tan(DEG2RAD * DirChng));	// Calculate balls relative velocity
	tmp = tmp.Rotate(Dir);									//
		
	tmp = *pVelocity + tmp.Rotate(GFacing);					// Convert tmp to player's absolute velocity	
	}
else												// else set it to 0
	{
	tmp = CVector(0, 0);
	}

if (Side == 'l')			// Depending on side and unum, store in the appropriate object
	{
	switch (UNum)
		{
		case 1:
			m_PlayerL1.SetTime(Time);
			m_PlayerL1.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL1.SetDistChng(DistChng);
			m_PlayerL1.SetDirChng(DirChng);
			m_PlayerL1.SetRBFacing(BFacing);
			m_PlayerL1.SetRFacing(Facing);
			m_PlayerL1.SetGBFacing(NormaliseAngle(m_PlayerL1.GetRBFacing() + GFacing));
			m_PlayerL1.SetGFacing(NormaliseAngle(m_PlayerL1.GetRFacing() + GFacing));
			m_PlayerL1.SetConf(1.0);
			m_PlayerL1.SetVelocity(tmp);
			m_PlayerL1.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL1.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 2:
			m_PlayerL2.SetTime(Time);
			m_PlayerL2.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL2.SetDistChng(DistChng);
			m_PlayerL2.SetDirChng(DirChng);
			m_PlayerL2.SetRBFacing(BFacing);
			m_PlayerL2.SetRFacing(Facing);
			m_PlayerL2.SetGBFacing(NormaliseAngle(m_PlayerL2.GetRBFacing() + GFacing));
			m_PlayerL2.SetGFacing(NormaliseAngle(m_PlayerL2.GetRFacing() + GFacing));
			m_PlayerL2.SetConf(1.0);
			m_PlayerL2.SetVelocity(tmp);
			m_PlayerL2.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL2.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 3:
			m_PlayerL3.SetTime(Time);
			m_PlayerL3.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL3.SetDistChng(DistChng);
			m_PlayerL3.SetDirChng(DirChng);
			m_PlayerL3.SetRBFacing(BFacing);
			m_PlayerL3.SetRFacing(Facing);
			m_PlayerL3.SetGBFacing(NormaliseAngle(m_PlayerL3.GetRBFacing() + GFacing));
			m_PlayerL3.SetGFacing(NormaliseAngle(m_PlayerL3.GetRFacing() + GFacing));
			m_PlayerL3.SetConf(1.0);
			m_PlayerL3.SetVelocity(tmp);
			m_PlayerL3.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL3.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 4:
			m_PlayerL4.SetTime(Time);
			m_PlayerL4.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL4.SetDistChng(DistChng);
			m_PlayerL4.SetDirChng(DirChng);
			m_PlayerL4.SetRBFacing(BFacing);
			m_PlayerL4.SetRFacing(Facing);
			m_PlayerL4.SetGBFacing(NormaliseAngle(m_PlayerL4.GetRBFacing() + GFacing));
			m_PlayerL4.SetGFacing(NormaliseAngle(m_PlayerL4.GetRFacing() + GFacing));
			m_PlayerL4.SetConf(1.0);
			m_PlayerL4.SetVelocity(tmp);
			m_PlayerL4.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL4.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 5:
			m_PlayerL5.SetTime(Time);
			m_PlayerL5.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL5.SetDistChng(DistChng);
			m_PlayerL5.SetDirChng(DirChng);
			m_PlayerL5.SetRBFacing(BFacing);
			m_PlayerL5.SetRFacing(Facing);
			m_PlayerL5.SetGBFacing(NormaliseAngle(m_PlayerL5.GetRBFacing() + GFacing));
			m_PlayerL5.SetGFacing(NormaliseAngle(m_PlayerL5.GetRFacing() + GFacing));
			m_PlayerL5.SetConf(1.0);
			m_PlayerL5.SetVelocity(tmp);
			m_PlayerL5.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL5.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 6:
			m_PlayerL6.SetTime(Time);
			m_PlayerL6.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL6.SetDistChng(DistChng);
			m_PlayerL6.SetDirChng(DirChng);
			m_PlayerL6.SetRBFacing(BFacing);
			m_PlayerL6.SetRFacing(Facing);
			m_PlayerL6.SetGBFacing(NormaliseAngle(m_PlayerL6.GetRBFacing() + GFacing));
			m_PlayerL6.SetGFacing(NormaliseAngle(m_PlayerL6.GetRFacing() + GFacing));
			m_PlayerL6.SetConf(1.0);
			m_PlayerL6.SetVelocity(tmp);
			m_PlayerL6.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL6.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 7:
			m_PlayerL7.SetTime(Time);
			m_PlayerL7.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL7.SetDistChng(DistChng);
			m_PlayerL7.SetDirChng(DirChng);
			m_PlayerL7.SetRBFacing(BFacing);
			m_PlayerL7.SetRFacing(Facing);
			m_PlayerL7.SetGBFacing(NormaliseAngle(m_PlayerL7.GetRBFacing() + GFacing));
			m_PlayerL7.SetGFacing(NormaliseAngle(m_PlayerL7.GetRFacing() + GFacing));
			m_PlayerL7.SetConf(1.0);
			m_PlayerL7.SetVelocity(tmp);
			m_PlayerL7.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL7.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 8:
			m_PlayerL8.SetTime(Time);
			m_PlayerL8.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL8.SetDistChng(DistChng);
			m_PlayerL8.SetDirChng(DirChng);
			m_PlayerL8.SetRBFacing(BFacing);
			m_PlayerL8.SetRFacing(Facing);
			m_PlayerL8.SetGBFacing(NormaliseAngle(m_PlayerL8.GetRBFacing() + GFacing));
			m_PlayerL8.SetGFacing(NormaliseAngle(m_PlayerL8.GetRFacing() + GFacing));
			m_PlayerL8.SetConf(1.0);
			m_PlayerL8.SetVelocity(tmp);
			m_PlayerL8.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL8.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 9:
			m_PlayerL9.SetTime(Time);
			m_PlayerL9.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL9.SetDistChng(DistChng);
			m_PlayerL9.SetDirChng(DirChng);
			m_PlayerL9.SetRBFacing(BFacing);
			m_PlayerL9.SetRFacing(Facing);
			m_PlayerL9.SetGBFacing(NormaliseAngle(m_PlayerL9.GetRBFacing() + GFacing));
			m_PlayerL9.SetGFacing(NormaliseAngle(m_PlayerL9.GetRFacing() + GFacing));
			m_PlayerL9.SetConf(1.0);
			m_PlayerL9.SetVelocity(tmp);
			m_PlayerL9.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL9.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 10:
			m_PlayerL10.SetTime(Time);
			m_PlayerL10.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL10.SetDistChng(DistChng);
			m_PlayerL10.SetDirChng(DirChng);
			m_PlayerL10.SetRBFacing(BFacing);
			m_PlayerL10.SetRFacing(Facing);
			m_PlayerL10.SetGBFacing(NormaliseAngle(m_PlayerL10.GetRBFacing() + GFacing));
			m_PlayerL10.SetGFacing(NormaliseAngle(m_PlayerL10.GetRFacing() + GFacing));
			m_PlayerL10.SetConf(1.0);
			m_PlayerL10.SetVelocity(tmp);
			m_PlayerL10.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL10.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 11:
			m_PlayerL11.SetTime(Time);
			m_PlayerL11.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerL11.SetDistChng(DistChng);
			m_PlayerL11.SetDirChng(DirChng);
			m_PlayerL11.SetRBFacing(BFacing);
			m_PlayerL11.SetRFacing(Facing);
			m_PlayerL11.SetGBFacing(NormaliseAngle(m_PlayerL11.GetRBFacing() + GFacing));
			m_PlayerL11.SetGFacing(NormaliseAngle(m_PlayerL11.GetRFacing() + GFacing));
			m_PlayerL11.SetConf(1.0);
			m_PlayerL11.SetVelocity(tmp);
			m_PlayerL11.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerL11.SetPositionValid(1);			// including probable unknown velocity
			break;
		case -999:		// Unum unknown
			if (GetSide() == 'l')							// Then he is a friend
				{					
				int	STime = GetTime();						// Sensor time
				int PTime = m_FarFriend[0]->GetTime();		// Array time
				int counter = m_FarFriend[0]->GetUNum();	// Number of players stored so far (STARTING from 1)
			
				if (STime == PTime)							// If time is the same
					{
					counter++;								// Increase local counter
					// AND global counter (since "counter" will go out of scope when this function returns)
					m_FarFriend[0]->SetUNum(counter);		
							
					if (counter > 10)						// Post fence error when time is still (probably when time is 0)
						{
						m_FarFriend[0]->SetUNum(1);			// Just reset
						counter = 1;						// It doesn't matter if we lose data here anyway
						}
						
					m_FarFriend[counter]->SetTime(Time);			
					m_FarFriend[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFriend[counter]->SetDistChng(DistChng);//
					m_FarFriend[counter]->SetDirChng(DirChng);	//
					m_FarFriend[counter]->SetRBFacing(BFacing);	//
					m_FarFriend[counter]->SetRFacing(Facing);	//
					m_FarFriend[counter]->SetGBFacing(NormaliseAngle(m_FarFriend[counter]->GetRBFacing() + GFacing));
					m_FarFriend[counter]->SetGFacing(NormaliseAngle(m_FarFriend[counter]->GetRFacing() + GFacing));
					m_FarFriend[counter]->SetConf(1);			//
					m_FarFriend[counter]->SetVelocity(tmp);
					m_FarFriend[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFriend[counter]->SetPositionValid(1);			// including probable unknown velocity
					}
				else if (STime != PTime)
					{
					m_FarFriend[0]->SetTime(STime);			// Update time
					m_FarFriend[0]->SetUNum(1);				// Reset counter
					counter = 1;							// and update counter so that we can store current player
				
					m_FarFriend[counter]->SetTime(Time);			
					m_FarFriend[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFriend[counter]->SetDistChng(DistChng);//
					m_FarFriend[counter]->SetDirChng(DirChng);	//
					m_FarFriend[counter]->SetRBFacing(BFacing);	//
					m_FarFriend[counter]->SetRFacing(Facing);	//
					m_FarFriend[counter]->SetGBFacing(NormaliseAngle(m_FarFriend[counter]->GetRBFacing() + GFacing));
					m_FarFriend[counter]->SetGFacing(NormaliseAngle(m_FarFriend[counter]->GetRFacing() + GFacing));
					m_FarFriend[counter]->SetConf(1);			//
					m_FarFriend[counter]->SetVelocity(tmp);
					m_FarFriend[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFriend[counter]->SetPositionValid(1);			// including probable unknown velocity
					// After this the other player will follow the "if" above since time has been updated here
					}
				else
					cerr << "WARNING: CMemory: SetPlayer: Time comparison problem for uniform number -999..." << endl;
				}
			else if (GetSide() == 'r')						// Then he is a foe
				{
				int	STime = GetTime();						// Sensor time
				int PTime = m_FarFoe[0]->GetTime();			// Array time
				int counter = m_FarFoe[0]->GetUNum();		// Number of players stored so far (STARTING from 1)
				
				if (STime == PTime)							// If time is the same
					{
					counter++;								// Increase local counter
					// AND global counter (since "counter" will go out of scope when this function returns)
					m_FarFoe[0]->SetUNum(counter);		
							
					if (counter > 10)						// Post fence error when time is still (probably when time is 0)
						{
						m_FarFoe[0]->SetUNum(1);			// Just reset
						counter = 1;						// It doesn't matter if we lose data here anyway
						}
				
					m_FarFoe[counter]->SetTime(Time);			
					m_FarFoe[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFoe[counter]->SetDistChng(DistChng);//
					m_FarFoe[counter]->SetDirChng(DirChng);	//
					m_FarFoe[counter]->SetRBFacing(BFacing);//
					m_FarFoe[counter]->SetRFacing(Facing);	//
					m_FarFoe[counter]->SetGBFacing(NormaliseAngle(m_FarFoe[counter]->GetRBFacing() + GFacing));
					m_FarFoe[counter]->SetGFacing(NormaliseAngle(m_FarFoe[counter]->GetRFacing() + GFacing));
					m_FarFoe[counter]->SetConf(1);			//
					m_FarFoe[counter]->SetVelocity(tmp);
					m_FarFoe[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFoe[counter]->SetPositionValid(1);			// including probable unknown velocity
					}
				else if (STime != PTime)
					{
					m_FarFoe[0]->SetTime(STime);			// Update time
					m_FarFoe[0]->SetUNum(1);				// Reset counter
					counter = 1;							// and update counter so that we can store current player
				
					m_FarFoe[counter]->SetTime(Time);			
					m_FarFoe[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFoe[counter]->SetDistChng(DistChng);//
					m_FarFoe[counter]->SetDirChng(DirChng);	//
					m_FarFoe[counter]->SetRBFacing(BFacing);//
					m_FarFoe[counter]->SetRFacing(Facing);	//
					m_FarFoe[counter]->SetGBFacing(NormaliseAngle(m_FarFoe[counter]->GetRBFacing() + GFacing));
					m_FarFoe[counter]->SetGFacing(NormaliseAngle(m_FarFoe[counter]->GetRFacing() + GFacing));
					m_FarFoe[counter]->SetConf(1);			//
					m_FarFoe[counter]->SetVelocity(tmp);
					m_FarFoe[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFoe[counter]->SetPositionValid(1);			// including probable unknown velocity
					// After this the other player will follow the "if" above since time has been updated here
					}
				else
					cerr << "WARNING: CMemory: SetPlayer: Time comparison problem for uniform number -999..." << endl;
				}
			else
				cerr << "WARNING: CMemory: SetPlayer: What side is that?" << endl;
			break;			
		default:
			cerr << "WARNING: CMemory: SetPlayer: What unum is that?" << endl;
			break;
		}
	}
else if (Side == 'r')
	{
	switch (UNum)
		{
		case 1:
			m_PlayerR1.SetTime(Time);
			m_PlayerR1.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR1.SetDistChng(DistChng);
			m_PlayerR1.SetDirChng(DirChng);
			m_PlayerR1.SetRBFacing(BFacing);
			m_PlayerR1.SetRFacing(Facing);
			m_PlayerR1.SetGBFacing(NormaliseAngle(m_PlayerR1.GetRBFacing() + GFacing));
			m_PlayerR1.SetGFacing(NormaliseAngle(m_PlayerR1.GetRFacing() + GFacing));
			m_PlayerR1.SetConf(1.0);
			m_PlayerR1.SetVelocity(tmp);
			m_PlayerR1.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR1.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 2:
			m_PlayerR2.SetTime(Time);
			m_PlayerR2.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR2.SetDistChng(DistChng);
			m_PlayerR2.SetDirChng(DirChng);
			m_PlayerR2.SetRBFacing(BFacing);
			m_PlayerR2.SetRFacing(Facing);
			m_PlayerR2.SetGBFacing(NormaliseAngle(m_PlayerR2.GetRBFacing() + GFacing));
			m_PlayerR2.SetGFacing(NormaliseAngle(m_PlayerR2.GetRFacing() + GFacing));
			m_PlayerR2.SetConf(1.0);
			m_PlayerR2.SetVelocity(tmp);
			m_PlayerR2.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR2.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 3:
			m_PlayerR3.SetTime(Time);
			m_PlayerR3.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR3.SetDistChng(DistChng);
			m_PlayerR3.SetDirChng(DirChng);
			m_PlayerR3.SetRBFacing(BFacing);
			m_PlayerR3.SetRFacing(Facing);
			m_PlayerR3.SetGBFacing(NormaliseAngle(m_PlayerR3.GetRBFacing() + GFacing));
			m_PlayerR3.SetGFacing(NormaliseAngle(m_PlayerR3.GetRFacing() + GFacing));
			m_PlayerR3.SetConf(1.0);
			m_PlayerR3.SetVelocity(tmp);
			m_PlayerR3.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR3.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 4:
			m_PlayerR4.SetTime(Time);
			m_PlayerR4.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR4.SetDistChng(DistChng);
			m_PlayerR4.SetDirChng(DirChng);
			m_PlayerR4.SetRBFacing(BFacing);
			m_PlayerR4.SetRFacing(Facing);
			m_PlayerR4.SetGBFacing(NormaliseAngle(m_PlayerR4.GetRBFacing() + GFacing));
			m_PlayerR4.SetGFacing(NormaliseAngle(m_PlayerR4.GetRFacing() + GFacing));
			m_PlayerR4.SetConf(1.0);
			m_PlayerR4.SetVelocity(tmp);
			m_PlayerR4.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR4.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 5:
			m_PlayerR5.SetTime(Time);
			m_PlayerR5.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR5.SetDistChng(DistChng);
			m_PlayerR5.SetDirChng(DirChng);
			m_PlayerR5.SetRBFacing(BFacing);
			m_PlayerR5.SetRFacing(Facing);
			m_PlayerR5.SetGBFacing(NormaliseAngle(m_PlayerR5.GetRBFacing() + GFacing));
			m_PlayerR5.SetGFacing(NormaliseAngle(m_PlayerR5.GetRFacing() + GFacing));
			m_PlayerR5.SetConf(1.0);
			m_PlayerR5.SetVelocity(tmp);
			m_PlayerR5.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR5.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 6:
			m_PlayerR6.SetTime(Time);
			m_PlayerR6.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR6.SetDistChng(DistChng);
			m_PlayerR6.SetDirChng(DirChng);
			m_PlayerR6.SetRBFacing(BFacing);
			m_PlayerR6.SetRFacing(Facing);
			m_PlayerR6.SetGBFacing(NormaliseAngle(m_PlayerR6.GetRBFacing() + GFacing));
			m_PlayerR6.SetGFacing(NormaliseAngle(m_PlayerR6.GetRFacing() + GFacing));
			m_PlayerR6.SetConf(1.0);
			m_PlayerR6.SetVelocity(tmp);
			m_PlayerR6.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR6.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 7:
			m_PlayerR7.SetTime(Time);
			m_PlayerR7.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR7.SetDistChng(DistChng);
			m_PlayerR7.SetDirChng(DirChng);
			m_PlayerR7.SetRBFacing(BFacing);
			m_PlayerR7.SetRFacing(Facing);
			m_PlayerR7.SetGBFacing(NormaliseAngle(m_PlayerR7.GetRBFacing() + GFacing));
			m_PlayerR7.SetGFacing(NormaliseAngle(m_PlayerR7.GetRFacing() + GFacing));
			m_PlayerR7.SetConf(1.0);
			m_PlayerR7.SetVelocity(tmp);
			m_PlayerR7.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR7.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 8:
			m_PlayerR8.SetTime(Time);
			m_PlayerR8.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR8.SetDistChng(DistChng);
			m_PlayerR8.SetDirChng(DirChng);
			m_PlayerR8.SetRBFacing(BFacing);
			m_PlayerR8.SetRFacing(Facing);
			m_PlayerR8.SetGBFacing(NormaliseAngle(m_PlayerR8.GetRBFacing() + GFacing));
			m_PlayerR8.SetGFacing(NormaliseAngle(m_PlayerR8.GetRFacing() + GFacing));
			m_PlayerR8.SetConf(1.0);
			m_PlayerR8.SetVelocity(tmp);
			m_PlayerR8.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR8.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 9:
			m_PlayerR9.SetTime(Time);
			m_PlayerR9.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR9.SetDistChng(DistChng);
			m_PlayerR9.SetDirChng(DirChng);
			m_PlayerR9.SetRBFacing(BFacing);
			m_PlayerR9.SetRFacing(Facing);
			m_PlayerR9.SetGBFacing(NormaliseAngle(m_PlayerR9.GetRBFacing() + GFacing));
			m_PlayerR9.SetGFacing(NormaliseAngle(m_PlayerR9.GetRFacing() + GFacing));
			m_PlayerR9.SetConf(1.0);
			m_PlayerR9.SetVelocity(tmp);
			m_PlayerR9.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR9.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 10:
			m_PlayerR10.SetTime(Time);
			m_PlayerR10.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR10.SetDistChng(DistChng);
			m_PlayerR10.SetDirChng(DirChng);
			m_PlayerR10.SetRBFacing(BFacing);
			m_PlayerR10.SetRFacing(Facing);
			m_PlayerR10.SetGBFacing(NormaliseAngle(m_PlayerR10.GetRBFacing() + GFacing));
			m_PlayerR10.SetGFacing(NormaliseAngle(m_PlayerR10.GetRFacing() + GFacing));
			m_PlayerR10.SetConf(1.0);
			m_PlayerR10.SetVelocity(tmp);
			m_PlayerR10.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR10.SetPositionValid(1);			// including probable unknown velocity
			break;
		case 11:
			m_PlayerR11.SetTime(Time);
			m_PlayerR11.SetRPos(Polar2CVector(Dist, Dir));
			m_PlayerR11.SetDistChng(DistChng);
			m_PlayerR11.SetDirChng(DirChng);
			m_PlayerR11.SetRBFacing(BFacing);
			m_PlayerR11.SetRFacing(Facing);
			m_PlayerR11.SetGBFacing(NormaliseAngle(m_PlayerR11.GetRBFacing() + GFacing));
			m_PlayerR11.SetGFacing(NormaliseAngle(m_PlayerR11.GetRFacing() + GFacing));
			m_PlayerR11.SetConf(1.0);
			m_PlayerR11.SetVelocity(tmp);
			m_PlayerR11.SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_PlayerR11.SetPositionValid(1);			// including probable unknown velocity
			break;
		case -999:		// Unum unknown
			if (GetSide() == 'r')							// Then he is a friend
				{					
				int	STime = GetTime();						// Sensor time
				int PTime = m_FarFriend[0]->GetTime();		// Array time
				int counter = m_FarFriend[0]->GetUNum();	// Number of players stored so far (STARTING from 1)
			
				if (STime == PTime)							// If time is the same
					{
					counter++;								// Increase local counter
					// AND global counter (since "counter" will go out of scope when this function returns)
					m_FarFriend[0]->SetUNum(counter);		
							
					if (counter > 10)						// Post fence error when time is still (probably when time is 0)
						{
						m_FarFriend[0]->SetUNum(1);			// Just reset
						counter = 1;						// It doesn't matter if we lose data here anyway
						}
							
					m_FarFriend[counter]->SetTime(Time);
					m_FarFriend[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFriend[counter]->SetDistChng(DistChng);//
					m_FarFriend[counter]->SetDirChng(DirChng);	//
					m_FarFriend[counter]->SetRBFacing(BFacing);	//
					m_FarFriend[counter]->SetRFacing(Facing);	//
					m_FarFriend[counter]->SetGBFacing(NormaliseAngle(m_FarFriend[counter]->GetRBFacing() + GFacing));
					m_FarFriend[counter]->SetGFacing(NormaliseAngle(m_FarFriend[counter]->GetRFacing() + GFacing));
					m_FarFriend[counter]->SetConf(1);			//
					m_FarFriend[counter]->SetVelocity(tmp);
					m_FarFriend[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFriend[counter]->SetPositionValid(1);			// including probable unknown velocity
					}
				else if (STime != PTime)
					{
					m_FarFriend[0]->SetTime(STime);			// Update time
					m_FarFriend[0]->SetUNum(1);				// Reset counter
					counter = 1;							// and update counter so that we can store current player
							
					m_FarFriend[counter]->SetTime(Time);
					m_FarFriend[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFriend[counter]->SetDistChng(DistChng);//
					m_FarFriend[counter]->SetDirChng(DirChng);	//
					m_FarFriend[counter]->SetRBFacing(BFacing);	//
					m_FarFriend[counter]->SetRFacing(Facing);	//
					m_FarFriend[counter]->SetGBFacing(NormaliseAngle(m_FarFriend[counter]->GetRBFacing() + GFacing));
					m_FarFriend[counter]->SetGFacing(NormaliseAngle(m_FarFriend[counter]->GetRFacing() + GFacing));
					m_FarFriend[counter]->SetConf(1);			//
					m_FarFriend[counter]->SetVelocity(tmp);		
					m_FarFriend[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFriend[counter]->SetPositionValid(1);			// including probable unknown velocity
					// After this the other player will follow the "if" above since time has been updated here
					}
				else
					cerr << "WARNING: CMemory: SetPlayer: Time comparison problem for uniform number -999..." << endl;
				}
			else if (GetSide() == 'l')						// Then he is a foe
				{
				int	STime = GetTime();						// Sensor time
				int PTime = m_FarFoe[0]->GetTime();			// Array time
				int counter = m_FarFoe[0]->GetUNum();		// Number of players stored so far (STARTING from 1)
				
				if (STime == PTime)							// If time is the same
					{
					counter++;								// Increase local counter
					// AND global counter (since "counter" will go out of scope when this function returns)
					m_FarFoe[0]->SetUNum(counter);		
							
					if (counter > 10)						// Post fence error when time is still (probably when time is 0)
						{
						m_FarFoe[0]->SetUNum(1);			// Just reset
						counter = 1;						// It doesn't matter if we lose data here anyway
						}
							
					m_FarFoe[counter]->SetTime(Time);
					m_FarFoe[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFoe[counter]->SetDistChng(DistChng);//
					m_FarFoe[counter]->SetDirChng(DirChng);	//
					m_FarFoe[counter]->SetRBFacing(BFacing);//
					m_FarFoe[counter]->SetRFacing(Facing);	//
					m_FarFoe[counter]->SetGBFacing(NormaliseAngle(m_FarFoe[counter]->GetRBFacing() + GFacing));
					m_FarFoe[counter]->SetGFacing(NormaliseAngle(m_FarFoe[counter]->GetRFacing() + GFacing));
					m_FarFoe[counter]->SetConf(1);			//
					m_FarFoe[counter]->SetVelocity(tmp);
					m_FarFoe[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFoe[counter]->SetPositionValid(1);			// including probable unknown velocity
					}
				else if (STime != PTime)
					{
					m_FarFoe[0]->SetTime(STime);			// Update time
					m_FarFoe[0]->SetUNum(1);				// Reset counter
					counter = 1;							// and update counter so that we can store current player
							
					m_FarFoe[counter]->SetTime(Time);
					m_FarFoe[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
					m_FarFoe[counter]->SetDistChng(DistChng);//
					m_FarFoe[counter]->SetDirChng(DirChng);	//
					m_FarFoe[counter]->SetRBFacing(BFacing);//
					m_FarFoe[counter]->SetRFacing(Facing);	//
					m_FarFoe[counter]->SetGBFacing(NormaliseAngle(m_FarFoe[counter]->GetRBFacing() + GFacing));
					m_FarFoe[counter]->SetGFacing(NormaliseAngle(m_FarFoe[counter]->GetRFacing() + GFacing));
					m_FarFoe[counter]->SetConf(1);			//
					m_FarFoe[counter]->SetVelocity(tmp);
					m_FarFoe[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
					m_FarFoe[counter]->SetPositionValid(1);			// including probable unknown velocity
					// After this the other player will follow the "if" above since time has been updated here
					}
				else
					cerr << "WARNING: CMemory: SetPlayer: Time comparison problem for uniform number -999..." << endl;
				}
			else
				cerr << "WARNING: CMemory: SetPlayer: What side is that?" << endl;
			break;
		default:
			cerr << "WARNING: CMemory: SetPlayer: What unum is that?" << endl;
			break;
		}
	}
else if (Side == 'u')
	{
	if (UNum == 0)								// If close player
		{
		m_PlayerClose.SetTime(Time);
		m_PlayerClose.SetRPos(Polar2CVector(Dist, Dir));
		m_PlayerClose.SetDistChng(DistChng);
		m_PlayerClose.SetDirChng(DirChng);
		m_PlayerClose.SetRBFacing(BFacing);
		m_PlayerClose.SetRFacing(Facing);
		m_PlayerClose.SetGBFacing(NormaliseAngle(m_PlayerClose.GetRBFacing() + GFacing));
		m_PlayerClose.SetGFacing(NormaliseAngle(m_PlayerClose.GetRFacing() + GFacing));
		m_PlayerClose.SetConf(1.0);
		m_PlayerClose.SetVelocity(tmp);
		m_PlayerClose.SetVelocityValid(1);			// Just got info from sensors so everything is valid
		m_PlayerClose.SetPositionValid(1);			// including probable unknown velocity
		}
	else										// else if unknown player
		{	
		int	STime = GetTime();						// Sensor time
		int PTime = m_UnknownPlayer[0]->GetTime();	// Array time
		int counter = m_UnknownPlayer[0]->GetUNum();// Number of players stored so far (STARTING from 1)
				
		if (STime == PTime)							// If time is the same
			{
			counter++;								// Increase local counter
			// AND global counter (since "counter" will go out of scope when this function returns)
			m_UnknownPlayer[0]->SetUNum(counter);		
					
			if (counter > 10)						// Post fence error when time is still (probably when time is 0)
				{
				m_UnknownPlayer[0]->SetUNum(1);		// Just reset
				counter = 1;						// It doesn't matter if we lose data here anyway
				}
			
			m_UnknownPlayer[counter]->SetTime(Time);		
			m_UnknownPlayer[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
			m_UnknownPlayer[counter]->SetDistChng(DistChng);//
			m_UnknownPlayer[counter]->SetDirChng(DirChng);	//
			m_UnknownPlayer[counter]->SetRBFacing(BFacing);	//
			m_UnknownPlayer[counter]->SetRFacing(Facing);	//
			m_UnknownPlayer[counter]->SetGBFacing(NormaliseAngle(m_UnknownPlayer[counter]->GetRBFacing() + GFacing));
			m_UnknownPlayer[counter]->SetGFacing(NormaliseAngle(m_UnknownPlayer[counter]->GetRFacing() + GFacing));
			m_UnknownPlayer[counter]->SetConf(1);			//
			m_UnknownPlayer[counter]->SetVelocity(tmp);
			m_UnknownPlayer[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_UnknownPlayer[counter]->SetPositionValid(1);			// including probable unknown velocity
			}
		else if (STime != PTime)
			{
			m_UnknownPlayer[0]->SetTime(STime);			// Update time
			m_UnknownPlayer[0]->SetUNum(1);				// Reset counter
			counter = 1;							// and update counter so that we can store current player
					
			m_UnknownPlayer[counter]->SetTime(Time);
			m_UnknownPlayer[counter]->SetRPos(Polar2CVector(Dist, Dir));		// And store player
			m_UnknownPlayer[counter]->SetDistChng(DistChng);//
			m_UnknownPlayer[counter]->SetDirChng(DirChng);	//
			m_UnknownPlayer[counter]->SetRBFacing(BFacing);	//
			m_UnknownPlayer[counter]->SetRFacing(Facing);	//
			m_UnknownPlayer[counter]->SetGBFacing(NormaliseAngle(m_UnknownPlayer[counter]->GetRBFacing() + GFacing));
			m_UnknownPlayer[counter]->SetGFacing(NormaliseAngle(m_UnknownPlayer[counter]->GetRFacing() + GFacing));
			m_UnknownPlayer[counter]->SetConf(1);			//
			m_UnknownPlayer[counter]->SetVelocity(tmp);	
			m_UnknownPlayer[counter]->SetVelocityValid(1);			// Just got info from sensors so everything is valid
			m_UnknownPlayer[counter]->SetPositionValid(1);			// including probable unknown velocity
			// After this the other player will follow the "if" above since time has been updated here
			}
		else
			cerr << "WARNING: CMemory: SetPlayer: Time comparison problem for uniform number -999..." << endl;
		}
	}
else
	cerr << "WARNING: CMemory: SetPlayer: What side is that?" << endl;
}

// GETPLAYER ////////////////////////////////////////////////////////////////////////////////
const CPlayer* CMemory::GetPlayer(int UNum, char Side) const // Return pointer to player
{
if (Side == 'l')			// Depending on side and unum, read from the appropriate object
	{
	switch (UNum)
		{
		case 1:
			return m_PlayerL1.GetThis();
			break;
		case 2:
			return m_PlayerL2.GetThis();
			break;
		case 3:
			return m_PlayerL3.GetThis();
			break;
		case 4:
			return m_PlayerL4.GetThis();
			break;
		case 5:
			return m_PlayerL5.GetThis();
			break;
		case 6:
			return m_PlayerL6.GetThis();
			break;
		case 7:
			return m_PlayerL7.GetThis();
			break;
		case 8:
			return m_PlayerL8.GetThis();
			break;
		case 9:
			return m_PlayerL9.GetThis();
			break;
		case 10:
			return m_PlayerL10.GetThis();
			break;
		case 11:
			return m_PlayerL11.GetThis();
			break;
		default:
			cerr << "WARNING: CMemory: GetPlayer: What unum is that?" << endl;
			break;
		}
	}
else if (Side == 'r')
	{
	switch (UNum)
		{
		case 1:
			return m_PlayerR1.GetThis();
			break;
		case 2:
			return m_PlayerR2.GetThis();
			break;
		case 3:
			return m_PlayerR3.GetThis();
			break;
		case 4:
			return m_PlayerR4.GetThis();
			break;
		case 5:
			return m_PlayerR5.GetThis();
			break;
		case 6:
			return m_PlayerR6.GetThis();
			break;
		case 7:
			return m_PlayerR7.GetThis();
			break;
		case 8:
			return m_PlayerR8.GetThis();
			break;
		case 9:
			return m_PlayerR9.GetThis();
			break;
		case 10:
			return m_PlayerR10.GetThis();
			break;
		case 11:
			return m_PlayerR11.GetThis();
			break;
		default:
			cerr << "WARNING: CMemory: GetPlayer: What unum is that?" << endl;
			break;
		}
	}
else if ((Side == 'u') && (UNum == 0))
	{
	return m_PlayerClose.GetThis();
	}
else
	cerr << "WARNING: CMemory: GetPlayer: What side is that?" << endl;
}

// GETPLAYERARRAY /////////////////////////////////////////////////////////////////////////////
const CPlayer*const* CMemory::GetPlayerArray(int Far, int Friend) const		// Return pointer to array of players
{
if (Far == 0)					// If request for close player arrays
	{
	if (Friend == 0)
		return m_Foe;
	
	if (Friend == 1)
		return m_Friend;
	}

if (Far == 1)					// If request for far player arrays
	{
	if (Friend == 0)
		return m_FarFoe;
	
	if (Friend == 1)
		return m_FarFriend;
		
	if (Friend == 2)
		return m_UnknownPlayer;
	}
}

// SETGOAL ////////////////////////////////////////////////////////////////////////////////////
void CMemory::SetGoal(int Time, char ID, float Dist, float Dir)					// Write goal to Memory
{
switch (ID)
	{
	case 'l':		// If left goal
		m_GoalL.SetTime(Time);
		m_GoalL.SetRPos(Polar2CVector(Dist, Dir));
		m_GoalL.SetConf(1.0);
		break;
		
	case 'r':		// If right goal
		m_GoalR.SetTime(Time);
		m_GoalR.SetRPos(Polar2CVector(Dist, Dir));
		m_GoalR.SetConf(1.0);
		break;

	case 'u':		// If close goal
		m_GoalClose.SetTime(Time);
		m_GoalClose.SetRPos(Polar2CVector(Dist, Dir));
		m_GoalClose.SetConf(1.0);
		break;
		
	default:
		cerr << "WARNING: CMemory: SetGoal: What goal is that?" << endl;
		break;
	}
}
	
// GETGOAL ////////////////////////////////////////////////////////////////////////////////////
const CGoal* CMemory::GetGoal(char ID) const	// Return pointer to goal
{
switch (ID)
	{
	case 'l':		// If left goal
		return m_GoalL.GetThis();
		break;
		
	case 'r':		// If right goal
		return m_GoalR.GetThis();
		break;
		
	case 'u':		// If close goal
		return m_GoalClose.GetThis();
		break;
		
	default:
		cerr << "WARNING: CMemory: GetGoal: What goal is that?" << endl;
		break;
	}
}	

// GETNONCONSTGOAL /////////////////////////////////////////////////////////////////
CGoal* CMemory::GetNonConstGoal(char ID)		// Return non const pointer to goal (For use in UpdateGPos)
{
switch (ID)
	{
	case 'l':		// If left goal
		return m_GoalL.GetNonConstThis();
		break;
		
	case 'r':		// If right goal
		return m_GoalR.GetNonConstThis();
		break;
		
	case 'u':		// If close goal
		return m_GoalClose.GetNonConstThis();
		break;
		
	default:
		cerr << "WARNING: CMemory: GetGoal: What goal is that?" << endl;
		break;
	}
}

// SETBALL ////////////////////////////////////////////////////////////////////////////////////
void CMemory::SetBall(int Time, float Dist, float Dir, float DistChng, float DirChng, int Close)					// Write ball to Memory
{
const float GFacing = GetFacing();			// Store global facing direction here
CVector* pVelocity = GetNonConstVelocity();	// Get my own velocity

m_Ball.SetTime(Time);
m_Ball.SetRPos(Polar2CVector(Dist, Dir));
m_Ball.SetDistChng(DistChng);
m_Ball.SetDirChng(DirChng);
m_Ball.SetClose(Close);
m_Ball.SetConf(1.0);

if ((DistChng != -999) && (DirChng != -999))	// If DistChng and DirChng info is available
	{
	CVector tmp(DistChng, Dist * tan(DEG2RAD * DirChng));	// Calculate balls relative velocity
	tmp = tmp.Rotate(Dir);									//
		
	tmp = *pVelocity + tmp.Rotate(GFacing);			// Convert tmp to ball's absolute velocity
	m_Ball.SetVelocity(tmp);						// And then set the ball's absolute velocity
	}
else												// else
	{
	if (!Close)										// If ball not close
		m_Ball.SetVelocity(CVector(0, 0));			// assume zero velocity
	}
m_Ball.SetVelocityValid(1);			// Just got info from sensors so everything is valid
m_Ball.SetPositionValid(1);			// including probable unknown velocity

//cout << GetTime() << " Ball from MEMORY is:" << endl;	
//m_Ball.Display();
}

// GETBALL ////////////////////////////////////////////////////////////////////////////////////
const CBall* CMemory::GetBall() const	// Return pointer to ball
{
return m_Ball.GetThis();
}

// GETNONCONSTBALL ////////////////////////////////////////////////////////////////////////////
CBall* CMemory::GetNonConstBall()		// Return pointer to non-const ball (For use in CalcIntPoint)
{
return m_Ball.GetNonConstThis();
}

// SETFLAG ////////////////////////////////////////////////////////////////////////////////////
void CMemory::SetFlag(int Time, int ID, float Dist, float Dir, float DistChng, float DirChng)					// Write flag to Memory
{
switch (ID)
	{
	case 1:								// If center flag
		m_FlagC.SetTime(Time);
		m_FlagC.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagC.SetDistChng(DistChng);
		m_FlagC.SetDirChng(DirChng);
		m_FlagC.SetConf(1.0);
		break;
		
	case 2:								// If PLB flag
		m_FlagPLB.SetTime(Time);
		m_FlagPLB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPLB.SetDistChng(DistChng);
		m_FlagPLB.SetDirChng(DirChng);
		m_FlagPLB.SetConf(1.0);
		break;
		
	case 3:								// If PLC flag
		m_FlagPLC.SetTime(Time);
		m_FlagPLC.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPLC.SetDistChng(DistChng);
		m_FlagPLC.SetDirChng(DirChng);
		m_FlagPLC.SetConf(1.0);
		break;
		
	case 4:								// If PLT flag
		m_FlagPLT.SetTime(Time);
		m_FlagPLT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPLT.SetDistChng(DistChng);
		m_FlagPLT.SetDirChng(DirChng);
		m_FlagPLT.SetConf(1.0);
		break;
		
	case 5:								// If PRT flag
		m_FlagPRT.SetTime(Time);
		m_FlagPRT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPRT.SetDistChng(DistChng);
		m_FlagPRT.SetDirChng(DirChng);
		m_FlagPRT.SetConf(1.0);
		break;
		
	case 6:								// If PRC flag
		m_FlagPRC.SetTime(Time);
		m_FlagPRC.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPRC.SetDistChng(DistChng);
		m_FlagPRC.SetDirChng(DirChng);
		m_FlagPRC.SetConf(1.0);
		break;
		
	case 7:								// If PRB flag
		m_FlagPRB.SetTime(Time);
		m_FlagPRB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagPRB.SetDistChng(DistChng);
		m_FlagPRB.SetDirChng(DirChng);
		m_FlagPRB.SetConf(1.0);
		break;
		
	case 8:								// If LB flag
		m_FlagLB.SetTime(Time);
		m_FlagLB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLB.SetDistChng(DistChng);
		m_FlagLB.SetDirChng(DirChng);
		m_FlagLB.SetConf(1.0);
		break;
		
	case 9:								// If GLB flag
		m_FlagGLB.SetTime(Time);
		m_FlagGLB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagGLB.SetDistChng(DistChng);
		m_FlagGLB.SetDirChng(DirChng);
		m_FlagGLB.SetConf(1.0);
		break;
		
	case 10:							// If GLT flag
		m_FlagGLT.SetTime(Time);
		m_FlagGLT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagGLT.SetDistChng(DistChng);
		m_FlagGLT.SetDirChng(DirChng);
		m_FlagGLT.SetConf(1.0);
		break;
		
	case 11:							// If LT flag
		m_FlagLT.SetTime(Time);
		m_FlagLT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLT.SetDistChng(DistChng);
		m_FlagLT.SetDirChng(DirChng);
		m_FlagLT.SetConf(1.0);
		break;
		
	case 12:							// If CT flag
		m_FlagCT.SetTime(Time);
		m_FlagCT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagCT.SetDistChng(DistChng);
		m_FlagCT.SetDirChng(DirChng);
		m_FlagCT.SetConf(1.0);
		break;
		
	case 13:							// If RT flag
		m_FlagRT.SetTime(Time);
		m_FlagRT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRT.SetDistChng(DistChng);
		m_FlagRT.SetDirChng(DirChng);
		m_FlagRT.SetConf(1.0);
		break;
		
	case 14:							// If GRT flag
		m_FlagGRT.SetTime(Time);
		m_FlagGRT.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagGRT.SetDistChng(DistChng);
		m_FlagGRT.SetDirChng(DirChng);
		m_FlagGRT.SetConf(1.0);
		break;
		
	case 15:							// If GRB flag
		m_FlagGRB.SetTime(Time);
		m_FlagGRB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagGRB.SetDistChng(DistChng);
		m_FlagGRB.SetDirChng(DirChng);
		m_FlagGRB.SetConf(1.0);
		break;
		
	case 16:							// If RB flag
		m_FlagRB.SetTime(Time);
		m_FlagRB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRB.SetDistChng(DistChng);
		m_FlagRB.SetDirChng(DirChng);
		m_FlagRB.SetConf(1.0);
		break;
		
	case 17:							// If CB flag
		m_FlagCB.SetTime(Time);
		m_FlagCB.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagCB.SetDistChng(DistChng);
		m_FlagCB.SetDirChng(DirChng);
		m_FlagCB.SetConf(1.0);
		break;
		
	case 18:							// If LB30 flag
		m_FlagLB30.SetTime(Time);
		m_FlagLB30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLB30.SetDistChng(DistChng);
		m_FlagLB30.SetDirChng(DirChng);
		m_FlagLB30.SetConf(1.0);
		break;
		
	case 19:							// If LB20 flag
		m_FlagLB20.SetTime(Time);
		m_FlagLB20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLB20.SetDistChng(DistChng);
		m_FlagLB20.SetDirChng(DirChng);
		m_FlagLB20.SetConf(1.0);
		break;
		
	case 20:							// If LB10 flag
		m_FlagLB10.SetTime(Time);
		m_FlagLB10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLB10.SetDistChng(DistChng);
		m_FlagLB10.SetDirChng(DirChng);
		m_FlagLB10.SetConf(1.0);
		break;
		
	case 21:							// If L0 flag
		m_FlagL0.SetTime(Time);
		m_FlagL0.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagL0.SetDistChng(DistChng);
		m_FlagL0.SetDirChng(DirChng);
		m_FlagL0.SetConf(1.0);
		break;
		
	case 22:							// If LT10 flag
		m_FlagLT10.SetTime(Time);
		m_FlagLT10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLT10.SetDistChng(DistChng);
		m_FlagLT10.SetDirChng(DirChng);
		m_FlagLT10.SetConf(1.0);
		break;
		
	case 23:							// If LT20 flag
		m_FlagLT20.SetTime(Time);
		m_FlagLT20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLT20.SetDistChng(DistChng);
		m_FlagLT20.SetDirChng(DirChng);
		m_FlagLT20.SetConf(1.0);
		break;
		
	case 24:							// If LT30 flag
		m_FlagLT30.SetTime(Time);
		m_FlagLT30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagLT30.SetDistChng(DistChng);
		m_FlagLT30.SetDirChng(DirChng);
		m_FlagLT30.SetConf(1.0);
		break;
		
	case 25:							// If TL50 flag
		m_FlagTL50.SetTime(Time);
		m_FlagTL50.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTL50.SetDistChng(DistChng);
		m_FlagTL50.SetDirChng(DirChng);
		m_FlagTL50.SetConf(1.0);
		break;
		
	case 26:							// If TL40 flag
		m_FlagTL40.SetTime(Time);
		m_FlagTL40.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTL40.SetDistChng(DistChng);
		m_FlagTL40.SetDirChng(DirChng);
		m_FlagTL40.SetConf(1.0);
		break;
			
	case 27:							// If TL30 flag
		m_FlagTL30.SetTime(Time);
		m_FlagTL30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTL30.SetDistChng(DistChng);
		m_FlagTL30.SetDirChng(DirChng);
		m_FlagTL30.SetConf(1.0);
		break;
		
	case 28:							// If TL20 flag
		m_FlagTL20.SetTime(Time);
		m_FlagTL20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTL20.SetDistChng(DistChng);
		m_FlagTL20.SetDirChng(DirChng);
		m_FlagTL20.SetConf(1.0);
		break;
		
	case 29:							// If TL10 flag
		m_FlagTL10.SetTime(Time);
		m_FlagTL10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTL10.SetDistChng(DistChng);
		m_FlagTL10.SetDirChng(DirChng);
		m_FlagTL10.SetConf(1.0);
		break;
		
	case 30:							// If T0 flag
		m_FlagT0.SetTime(Time);
		m_FlagT0.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagT0.SetDistChng(DistChng);
		m_FlagT0.SetDirChng(DirChng);
		m_FlagT0.SetConf(1.0);
		break;
		
	case 31:							// If TR10 flag
		m_FlagTR10.SetTime(Time);
		m_FlagTR10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTR10.SetDistChng(DistChng);
		m_FlagTR10.SetDirChng(DirChng);
		m_FlagTR10.SetConf(1.0);
		break;
		
	case 32:							// If TR20 flag
		m_FlagTR20.SetTime(Time);
		m_FlagTR20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTR20.SetDistChng(DistChng);
		m_FlagTR20.SetDirChng(DirChng);
		m_FlagTR20.SetConf(1.0);
		break;
		
	case 33:							// If TR30 flag
		m_FlagTR30.SetTime(Time);
		m_FlagTR30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTR30.SetDistChng(DistChng);
		m_FlagTR30.SetDirChng(DirChng);
		m_FlagTR30.SetConf(1.0);
		break;
		
	case 34:							// If TR40 flag
		m_FlagTR40.SetTime(Time);
		m_FlagTR40.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTR40.SetDistChng(DistChng);
		m_FlagTR40.SetDirChng(DirChng);
		m_FlagTR40.SetConf(1.0);
		break;
		
	case 35:							// If TR50 flag
		m_FlagTR50.SetTime(Time);
		m_FlagTR50.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagTR50.SetDistChng(DistChng);
		m_FlagTR50.SetDirChng(DirChng);
		m_FlagTR50.SetConf(1.0);
		break;
		
	case 36:							// If RT30 flag
		m_FlagRT30.SetTime(Time);
		m_FlagRT30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRT30.SetDistChng(DistChng);
		m_FlagRT30.SetDirChng(DirChng);
		m_FlagRT30.SetConf(1.0);
		break;
		
	case 37:							// If RT20 flag
		m_FlagRT20.SetTime(Time);
		m_FlagRT20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRT20.SetDistChng(DistChng);
		m_FlagRT20.SetDirChng(DirChng);
		m_FlagRT20.SetConf(1.0);
		break;
		
	case 38:							// If RT10 flag
		m_FlagRT10.SetTime(Time);
		m_FlagRT10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRT10.SetDistChng(DistChng);
		m_FlagRT10.SetDirChng(DirChng);
		m_FlagRT10.SetConf(1.0);
		break;
		
	case 39:							// If R0 flag
		m_FlagR0.SetTime(Time);
		m_FlagR0.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagR0.SetDistChng(DistChng);
		m_FlagR0.SetDirChng(DirChng);
		m_FlagR0.SetConf(1.0);
		break;
		
	case 40:							// If RB10 flag
		m_FlagRB10.SetTime(Time);
		m_FlagRB10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRB10.SetDistChng(DistChng);
		m_FlagRB10.SetDirChng(DirChng);
		m_FlagRB10.SetConf(1.0);
		break;
		
	case 41:							// If RB20 flag
		m_FlagRB20.SetTime(Time);
		m_FlagRB20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRB20.SetDistChng(DistChng);
		m_FlagRB20.SetDirChng(DirChng);
		m_FlagRB20.SetConf(1.0);
		break;
		
	case 42:							// If RB30 flag
		m_FlagRB30.SetTime(Time);
		m_FlagRB30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagRB30.SetDistChng(DistChng);
		m_FlagRB30.SetDirChng(DirChng);
		m_FlagRB30.SetConf(1.0);
		break;
		
	case 43:							// If BR50 flag
		m_FlagBR50.SetTime(Time);
		m_FlagBR50.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBR50.SetDistChng(DistChng);
		m_FlagBR50.SetDirChng(DirChng);
		m_FlagBR50.SetConf(1.0);
		break;
		
	case 44:							// If BR40 flag
		m_FlagBR40.SetTime(Time);
		m_FlagBR40.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBR40.SetDistChng(DistChng);
		m_FlagBR40.SetDirChng(DirChng);
		m_FlagBR40.SetConf(1.0);
		break;
		
	case 45:							// If BR30 flag
		m_FlagBR30.SetTime(Time);
		m_FlagBR30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBR30.SetDistChng(DistChng);
		m_FlagBR30.SetDirChng(DirChng);
		m_FlagBR30.SetConf(1.0);
		break;
		
	case 46:							// If BR20 flag
		m_FlagBR20.SetTime(Time);
		m_FlagBR20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBR20.SetDistChng(DistChng);
		m_FlagBR20.SetDirChng(DirChng);
		m_FlagBR20.SetConf(1.0);
		break;
		
	case 47:							// If BR10 flag
		m_FlagBR10.SetTime(Time);
		m_FlagBR10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBR10.SetDistChng(DistChng);
		m_FlagBR10.SetDirChng(DirChng);
		m_FlagBR10.SetConf(1.0);
		break;
		
	case 48:							// If B0 flag
		m_FlagB0.SetTime(Time);
		m_FlagB0.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagB0.SetDistChng(DistChng);
		m_FlagB0.SetDirChng(DirChng);
		m_FlagB0.SetConf(1.0);
		break;
		
	case 49:							// If BL10 flag
		m_FlagBL10.SetTime(Time);
		m_FlagBL10.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBL10.SetDistChng(DistChng);
		m_FlagBL10.SetDirChng(DirChng);
		m_FlagBL10.SetConf(1.0);
		break;
		
	case 50:							// If BL20 flag
		m_FlagBL20.SetTime(Time);
		m_FlagBL20.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBL20.SetDistChng(DistChng);
		m_FlagBL20.SetDirChng(DirChng);
		m_FlagBL20.SetConf(1.0);
		break;
		
	case 51:							// If BL30 flag
		m_FlagBL30.SetTime(Time);
		m_FlagBL30.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBL30.SetDistChng(DistChng);
		m_FlagBL30.SetDirChng(DirChng);
		m_FlagBL30.SetConf(1.0);
		break;
		
	case 52:							// If BL40 flag
		m_FlagBL40.SetTime(Time);
		m_FlagBL40.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBL40.SetDistChng(DistChng);
		m_FlagBL40.SetDirChng(DirChng);
		m_FlagBL40.SetConf(1.0);
		break;
		
	case 53:							// If BL50 flag
		m_FlagBL50.SetTime(Time);
		m_FlagBL50.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagBL50.SetDistChng(DistChng);
		m_FlagBL50.SetDirChng(DirChng);
		m_FlagBL50.SetConf(1.0);
		break;

	case 0:								// If close flag
		m_FlagClose.SetTime(Time);
		m_FlagClose.SetRPos(Polar2CVector(Dist, Dir));
		m_FlagClose.SetDistChng(DistChng);
		m_FlagClose.SetDirChng(DirChng);
		m_FlagClose.SetConf(1.0);
		break;
				
	default:
		cerr << "WARNING: CMemory: SetFlag: What flag is that?" << endl;
		break;
	}
}

// GETFLAG ////////////////////////////////////////////////////////////////////////////////////
const CFlag* CMemory::GetFlag(int ID) const	// Return pointer to flag
{
switch (ID)
	{
	case 1:								// If center flag
		return m_FlagC.GetThis();
		break;
		
	case 2:								// If PLB flag
		return m_FlagPLB.GetThis();
		break;
		
	case 3:								// If PLC flag
		return m_FlagPLC.GetThis();
		break;
		
	case 4:								// If PLT flag
		return m_FlagPLT.GetThis();
		break;
		
	case 5:								// If PRT flag
		return m_FlagPRT.GetThis();
		break;
		
	case 6:								// If PRC flag
		return m_FlagPRC.GetThis();
		break;
		
	case 7:								// If PRB flag
		return m_FlagPRB.GetThis();
		break;
		
	case 8:								// If LB flag
		return m_FlagLB.GetThis();
		break;
		
	case 9:								// If GLB flag
		return m_FlagGLB.GetThis();
		break;
		
	case 10:							// If GLT flag
		return m_FlagGLT.GetThis();
		break;
		
	case 11:							// If LT flag
		return m_FlagLT.GetThis();
		break;
		
	case 12:							// If CT flag
		return m_FlagCT.GetThis();
		break;
		
	case 13:							// If RT flag
		return m_FlagRT.GetThis();
		break;
		
	case 14:							// If GRT flag
		return m_FlagGRT.GetThis();
		break;
		
	case 15:							// If GRB flag
		return m_FlagGRB.GetThis();
		break;
		
	case 16:							// If RB flag
		return m_FlagRB.GetThis();
		break;
		
	case 17:							// If CB flag
		return m_FlagCB.GetThis();
		break;
		
	case 18:							// If LB30 flag
		return m_FlagLB30.GetThis();
		break;
		
	case 19:							// If LB20 flag
		return m_FlagLB20.GetThis();
		break;
		
	case 20:							// If LB10 flag
		return m_FlagLB10.GetThis();
		break;
		
	case 21:							// If L0 flag
		return m_FlagL0.GetThis();
		break;
		
	case 22:							// If LT10 flag
		return m_FlagLT10.GetThis();
		break;
		
	case 23:							// If LT20 flag
		return m_FlagLT20.GetThis();
		break;
		
	case 24:							// If LT30 flag
		return m_FlagLT30.GetThis();
		break;
		
	case 25:							// If TL50 flag
		return m_FlagTL50.GetThis();
		break;
		
	case 26:							// If TL40 flag
		return m_FlagTL40.GetThis();
		break;
			
	case 27:							// If TL30 flag
		return m_FlagTL30.GetThis();
		break;
		
	case 28:							// If TL20 flag
		return m_FlagTL20.GetThis();
		break;
		
	case 29:							// If TL10 flag
		return m_FlagTL10.GetThis();
		break;
		
	case 30:							// If T0 flag
		return m_FlagT0.GetThis();
		break;
		
	case 31:							// If TR10 flag
		return m_FlagTR10.GetThis();
		break;
		
	case 32:							// If TR20 flag
		return m_FlagTR20.GetThis();
		break;
		
	case 33:							// If TR30 flag
		return m_FlagTR30.GetThis();
		break;
		
	case 34:							// If TR40 flag
		return m_FlagTR40.GetThis();
		break;
		
	case 35:							// If TR50 flag
		return m_FlagTR50.GetThis();
		break;
		
	case 36:							// If RT30 flag
		return m_FlagRT30.GetThis();
		break;
		
	case 37:							// If RT20 flag
		return m_FlagRT20.GetThis();
		break;
		
	case 38:							// If RT10 flag
		return m_FlagRT10.GetThis();
		break;
		
	case 39:							// If R0 flag
		return m_FlagR0.GetThis();
		break;
		
	case 40:							// If RB10 flag
		return m_FlagRB10.GetThis();
		break;
		
	case 41:							// If RB20 flag
		return m_FlagRB20.GetThis();
		break;
		
	case 42:							// If RB30 flag
		return m_FlagRB30.GetThis();
		break;
		
	case 43:							// If BR50 flag
		return m_FlagBR50.GetThis();
		break;
		
	case 44:							// If BR40 flag
		return m_FlagBR40.GetThis();
		break;
		
	case 45:							// If BR30 flag
		return m_FlagBR30.GetThis();
		break;
		
	case 46:							// If BR20 flag
		return m_FlagBR20.GetThis();
		break;
		
	case 47:							// If BR10 flag
		return m_FlagBR10.GetThis();
		break;
		
	case 48:							// If B0 flag
		return m_FlagB0.GetThis();
		break;
		
	case 49:							// If BL10 flag
		return m_FlagBL10.GetThis();
		break;
		
	case 50:							// If BL20 flag
		return m_FlagBL20.GetThis();
		break;
		
	case 51:							// If BL30 flag
		return m_FlagBL30.GetThis();
		break;
		
	case 52:							// If BL40 flag
		return m_FlagBL40.GetThis();
		break;
		
	case 53:							// If BL50 flag
		return m_FlagBL50.GetThis();
		break;
		
	default:
		cerr << "WARNING: CMemory: GetFlag: What flag is that?" << endl;
		break;
	}
}

// SETLINE ////////////////////////////////////////////////////////////////////////////////////
void CMemory::SetLine(int Time, char ID, float Dist, float Dir)					// Write line to Memory
{
switch (ID)
	{
	case 'l':		// If left line
		m_LineL.SetTime(Time);
		m_LineL.SetRPos(Polar2CVector(Dist, Dir));
		m_LineL.SetConf(1.0);
		break;
			
	case 't':		// If top line
		m_LineT.SetTime(Time);
		m_LineT.SetRPos(Polar2CVector(Dist, Dir));
		m_LineT.SetConf(1.0);
		break;
			
	case 'r':		// If right line
		m_LineR.SetTime(Time);
		m_LineR.SetRPos(Polar2CVector(Dist, Dir));
		m_LineR.SetConf(1.0);
		break;
			
	case 'b':		// If bottom line
		m_LineB.SetTime(Time);
		m_LineB.SetRPos(Polar2CVector(Dist, Dir));
		m_LineB.SetConf(1.0);
		break;
	
	case 'u':		// If close line
		m_LineClose.SetTime(Time);
		m_LineClose.SetRPos(Polar2CVector(Dist, Dir));
		m_LineClose.SetConf(1.0);
		break;
					
	default:
		cerr << "WARNING: CMemory: SetLine: What line is that?" << endl;
		break;
	}
}

// GETLINE ///////////////////////////////////////////////////
const CLine* CMemory::GetLine(char ID) const				// Return pointer to line	
{
switch (ID)
	{
	case 'l':		// If left line
		return m_LineL.GetThis();
		break;
		
	case 't':		// If top line
		return m_LineT.GetThis();
		break;
			
	case 'r':		// If right line
		return m_LineR.GetThis();
		break;
			
	case 'b':		// If bottom line
		return m_LineB.GetThis();
		break;
			
	default:
		cerr << "WARNING: CMemory: GetLine: What line is that?" << endl;
		break;
	}
}
