/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CMemory class
 * 				The CMemory class holds information regarding all the objects on the field.
 * Created: 17/12/98
 * Last modified: 16/7/99
 */

#ifndef _CMEMORY_H
#define _CMEMORY_H

class CGetParams;		// Need a forward declaration
class CVector;			// Need a forward declaration

#include <pthread.h>
#include "CModule.H"
#include "CLine.H"
#include "CFlag.H"
#include "CGoal.H"
#include "CPlayer.H"
#include "CBall.H"

class CMemory: public CModule
{
// Private member data ///////////////////////////////////////////////
private:

	pthread_mutex_t	m_Mutex;		// Memory's mutex
	
	CGoal	m_GoalClose;// Close objects (i.e. Flag, Line, Goal...)
	CFlag	m_FlagClose;
	CLine	m_LineClose;
	CPlayer	m_PlayerClose;
	
	CLine	m_LineL;	// A field has 4 lines
	CLine	m_LineT;
	CLine	m_LineR;
	CLine	m_LineB;

	CFlag	m_FlagC;	// A field has 53 flags.  A center flag

	CFlag	m_FlagPLB;	// 6 penalty flags
	CFlag	m_FlagPLC;
	CFlag	m_FlagPLT;
	CFlag	m_FlagPRT;
	CFlag	m_FlagPRC;
	CFlag	m_FlagPRB;

	CFlag	m_FlagLB;	// 10 flags on the lines
	CFlag	m_FlagGLB;	// Out of which 4 are goal flags
	CFlag	m_FlagGLT;	//
	CFlag	m_FlagLT;
	CFlag	m_FlagCT;
	CFlag	m_FlagRT;
	CFlag	m_FlagGRT;	//
	CFlag	m_FlagGRB;	//
	CFlag	m_FlagRB;
	CFlag	m_FlagCB;

	CFlag	m_FlagLB30;	// And 36 flags outside the pitch
	CFlag	m_FlagLB20;
	CFlag	m_FlagLB10;
	CFlag	m_FlagL0;
	CFlag	m_FlagLT10;
	CFlag	m_FlagLT20;
	CFlag	m_FlagLT30;
	CFlag	m_FlagTL50;
	CFlag	m_FlagTL40;
	CFlag	m_FlagTL30;
	CFlag	m_FlagTL20;
	CFlag	m_FlagTL10;
	CFlag	m_FlagT0;
	CFlag	m_FlagTR10;
	CFlag	m_FlagTR20;
	CFlag	m_FlagTR30;
	CFlag	m_FlagTR40;
	CFlag	m_FlagTR50;
	CFlag	m_FlagRT30;
	CFlag	m_FlagRT20;
	CFlag	m_FlagRT10;
	CFlag	m_FlagR0;
	CFlag	m_FlagRB10;
	CFlag	m_FlagRB20;
	CFlag	m_FlagRB30;
	CFlag	m_FlagBR50;
	CFlag	m_FlagBR40;
	CFlag	m_FlagBR30;
	CFlag	m_FlagBR20;
	CFlag	m_FlagBR10;
	CFlag	m_FlagB0;
	CFlag	m_FlagBL10;
	CFlag	m_FlagBL20;
	CFlag	m_FlagBL30;
	CFlag	m_FlagBL40;
	CFlag	m_FlagBL50;

	CGoal	m_GoalL;	// A field has 2 goals
	CGoal	m_GoalR;

	CPlayer	m_PlayerL1;	// A field has 22 players
	CPlayer	m_PlayerL2;
	CPlayer	m_PlayerL3;
	CPlayer	m_PlayerL4;
	CPlayer	m_PlayerL5;
	CPlayer	m_PlayerL6;
	CPlayer	m_PlayerL7;
	CPlayer	m_PlayerL8;
	CPlayer	m_PlayerL9;
	CPlayer	m_PlayerL10;
	CPlayer	m_PlayerL11;
	CPlayer	m_PlayerR1;
	CPlayer	m_PlayerR2;
	CPlayer	m_PlayerR3;
	CPlayer	m_PlayerR4;
	CPlayer	m_PlayerR5;
	CPlayer	m_PlayerR6;
	CPlayer	m_PlayerR7;
	CPlayer	m_PlayerR8;
	CPlayer	m_PlayerR9;
	CPlayer	m_PlayerR10;
	CPlayer	m_PlayerR11;

	CBall	m_Ball;		// A field has a ball

	CPlayer *m_Friend[11];			// 11 pointers to friendly players (both team name and unum known)
	CPlayer *m_FarFriend[11];		// 11 pointers to friendly players (only team name known)
	CPlayer *m_Foe[11];				// 11 pointer to enemy players (both team name and unum known)
	CPlayer *m_FarFoe[11];			// 11 pointers to enemy players (only team name known)
	CPlayer *m_UnknownPlayer[22];	// 22 pointers to players (neither team name nor unum known)

/*
		In the Far??? and UnknownPlayer arrays the first player (i.e. ???[0]) is not used
		as a player, but is used as a special structure.  The .Time field indicates
		the last time the array was updated, and the .UNum field is the counter so that
		we know in which position to store the player.  Also when reading these arrays,
		the for loop is executed ???[0].Unum times STARTING from 1.
*/
	
	float	m_KickableMargin;		// Kickable margin from params
	float	m_Kickable;				// Max kickable range (kickable_margin + ball_size + player_size)
	float	m_InertiaMoment;		// Inertia moment from params
	float	m_DashPowerRate;		// Dash power rate from params
	float	m_KickPowerRate;		// Kick power rate from params
	float	m_PlayerDecay;			// Player decay from params
	float	m_PlayerSize;			// Player size from params
	float	m_PlayerSpeedMax;		// Max speed for players from params
	float	m_BallDecay;			// Ball decay from params
	float	m_BallSize;				// Ball size from params
	float	m_BallSpeedMax;			// Max speed for ball from params
	float	m_StaminaIncMax;		// Stamina increase max from params
	float	m_ClosestDist;			// The closest distance the ball can get (= PlayerSize + 2*BallSize)

	float	m_Epsilon;				// Epsilon value used for float equality test (from agent.conf)
	float	m_StatConfDrop;			// Rate at which confidence of stationary objects drops after each cycle (from agent.conf)
	float	m_StatConfThr;			// Minimum confidence threshold for stationary objects (from agent.conf)
	float	m_MobConfDrop;			// Rate at which confidence of mobile objects drops after each cycle (from agent.conf)
	float	m_MobConfThr;			// Minimum confidence threshold for mobile objects (from agent.conf)
	
// Public member functions /////////////////////////////////////////////
public:

// Constructor, Init, and Display functions ////////////////
	CMemory(CMediator*);				// Constructor that takes Mediator
	void Init(CGetParams);				// Initialise all fields using MemoryParams
	void RevertSigns();					// Revert signs for right side players
	void InitPlayerArrays(char, CGetParams);		// Initialise Friend/Foe arrays
	void DeletePlayerArrays();			// Delete player arrays (the ones that have been new-ed i.e.
										// m_FarFriend, m_FarFoe, and m_UnknownPlayer)
	void Display() const;				// Displays all params.  Just for debugging

// Basic memory attribute get/set functions ////////////////////////// 
	void SetInertiaMoment(float InertiaMoment)		// Set inertia moment
	{ m_InertiaMoment = InertiaMoment; }
	float GetInertiaMoment() const					// Get inertia moment
	{ return m_InertiaMoment; }
	
// Basic Utilities //////////////////////////////////////////////////
	float 	NormaliseAngle	(float ang)				// Normalise angle
	{ while(ang >  180.0) ang -= 360.0;
	  while(ang < -180.0) ang += 360.0;
	  return ang; }
	  
// Lock/Unlock functions ////////////////////////////////////
	void Lock();					// Lock Memory
	void Unlock();					// Unlock Memory
		
// Accessor functions //////////////////////////////////////
	// Parameters are time, unum, side, dist, dir, distchng, dirchng, bfacing, facing
	// In Get*** there is an extra parameter, the confidence
	void SetPlayer(int, int, char, float, float, float, float, float, float);						// Write player to Memory.
	// Parameters are UNum and Side
	const CPlayer* GetPlayer(int, char) const;			// Return pointer to player
	// Parameters are 0 for Foe, 1 for Friend
	const CPlayer*const* GetPlayerArray(int, int) const;// Return pointer to array of players (READ .C FILE FOR VALUES)
	// Parameters are time, ID, dist, dir
	void SetGoal(int, char, float, float);				// Write goal to Memory
	// Parameters are ID
	const CGoal* GetGoal(char) const;					// Return pointer to goal
	CGoal* GetNonConstGoal(char);						// Return non const pointer to goal (For use in UpdateGPos, LookingAt??)
	// Parameters are time, dist, dir, distchng, dirchng
	void SetBall(int, float, float, float, float, int);	// Write ball to Memory
	// Parameters are none			
	const CBall* GetBall() const;						// Return pointer to ball
	CBall* GetNonConstBall();							// Return pointer to non-const ball (For use in CalcIntPoint)
	// Parameters are time, ID, dist, dir, distchng, dirchng			
	void SetFlag(int, int, float, float, float, float);	// Write flag to Memory
	// Parameters are ID
	const CFlag* GetFlag(int) const;					// Return pointer to flag
	// Parameters are time, ID, dist, dir
	void SetLine(int, char, float, float);				// Write line to Memory
	// Parameters are ID
	const CLine* GetLine(char) const;					// Return pointer to line

// Updating functions /////////////////////////////////////////
	void Update4Action(const char*);// Update Memory model depending on the action just sent
	void Update4Turn(float);		// Update for turn
	void Update4Dash(float);		// Update for dash
	void Update4Kick(float, float);	// UPdate for kick
	void Update4TurnNeck(float);	// Update for turn_neck
	
	void Update4NewCycle();			// Update Memory model for each new cycle	
	void UpdateGlobals();			// Now that we've localised, calculate global positions of selected objects	
	void UpdateRelatives();			// Updates relative co-ords of mobile objects

	void RemoveGhosts();			// Forget any objects that should have been visible, but aren't
		
////////////////////////////////////////////////////////////////////////////////////
/*
	The following functions are IDENTICAL to some of the ones in CBehaviours, but are methods
	of the CMemory class.  The only difference is that CMemory has direct access
	to m_Ball data and hence no pointer via the Mediator is needed.
	
	The reason for doing this is because action updates should also update my
	global position after each action.  Hence, these behaviours are needed
	both by the CMemory AND by the CBehaviours classes.  One solution would be
	to attach them to the Mediator like the other globaly needed behaviours.
	However these are kept separate from the Mediator for performance reasons,
	and because the Mediator is kept independend of any behavioural methods.
	
	The increase in size is minimal...
*/
////////////////////////////////////////////////////////////////////////////////////
// Prediction Behaviours /////////////////////////////////////////////
	CVector BallPredPosAfterKick(int, float, float);		// Predict ball's position after a kick
	CVector	BallPredPos			(int Steps)					// Predict ball's position
	{ return BallPredPosAfterKick(Steps, 0, 0); }
	CVector BallPredPos			()							// Predict ball's position for the next cycle
	{ return BallPredPosAfterKick(1, 0, 0); }
	CVector PredPosAfterDashing	(int, float);				// Predicted my position after continuous dashing
	CVector PredPosAfterDash	(float Power)				// Predict my position after a single dash
	{ return PredPosAfterDashing(1, Power); }
	CVector PredPosAfterTurn	(int, float, float, int);	// Predict my position after turn
	CVector PredPos				()							// Predict my position for the next cycle
	{ return PredPosAfterTurn(1, 0, 0, 0); }
	
};
 
#endif
