/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CModule class
 * Created: 1/3/98
 * Last modified: 16/7/99
 */
 
#include "CModule.H"
#include "CMediator.H"

// Constructor that assignes the Module's mediator ///////////////
CModule::CModule(CMediator* Mediator)
{
pMediator = Mediator;	// Assign your Mediator
}

// CHANGED //////////////////////////////////////////////////////
void CModule::Changed()
{
pMediator->ModuleChanged(this);	// Let your Mediator know that you've changed
}

// REMOVEGHOSTS //////////////////////////////////////////////////
void CModule::RemoveGhosts()
{
pMediator->RemoveGhosts();		// When new visual info is received from the sensors
								// right after the visual has been stored in memory,
								// this function tests for objects that should have been
								// visible but aren't.  If that is the case, it forgets
								// these objects.
}

// UPDATEGPOS ////////////////////////////////////////////////////
void CModule::UpdateGPos()	
{
pMediator->UpdateGPos();		// When new visual info is received from the sensors
								// localise and update the global position in Agent, Behaviours,
								// and Memory
								// Also this function updates the global facing direction in 
								// these modules
}

// DOEXIT ///////////////////////////////////////////////////////
void CModule::DoExit(int Code)	// Do a proper exit in case of a critical error
{
pMediator->DoExit(Code);
}

// LOCKAGENT ///////////////////////////////////////////////////
void CModule::LockAgent() 				// Lock the Agent mutex
{
pMediator->LockAgent();
}

// UNLOCKAGENT /////////////////////////////////////////////////
void CModule::UnlockAgent() 			// Unlock the Agent mutex
{
pMediator->UnlockAgent();
}

// LOCKMEMORY ///////////////////////////////////////////////////
void CModule::LockMemory() const		// Lock the Memory mutex
{
pMediator->LockMemory();
}

// UNLOCKMEMORY /////////////////////////////////////////////////
void CModule::UnlockMemory() const		// Unlock the Memory mutex
{
pMediator->UnlockMemory();
}

// LOCKBEHAVIOURS ///////////////////////////////////////////////////
void CModule::LockBehaviours() const	// Lock the Behaviours mutex
{
pMediator->LockBehaviours();
}

// UNLOCKBEHAVIOURS /////////////////////////////////////////////////
void CModule::UnlockBehaviours() const	// Unlock the Behaviours mutex
{
pMediator->UnlockBehaviours();
}

// LOCKPLAYMODE //////////////////////////////////////////////////////
void CModule::LockPlayMode() const			// Lock the PlayMode mutex
{
pMediator->LockPlayMode();
}

// UNLOCKPLAYMODE //////////////////////////////////////////////////////
void CModule::UnlockPlayMode() const		// Unlock the PlayMode mutex
{
pMediator->UnlockPlayMode();
}

// LOCKACTUATORS ////////////////////////////////////////////////////////
void CModule::LockActuators() const			// Lock the Actuators mutex
{
pMediator->LockActuators();
}

// UNLOCKACTUATORS ///////////////////////////////////////////////////////
void CModule::UnlockActuators() const		// Unlock the Actuators mutex
{
pMediator->UnlockActuators();
}
	
// SETSTATUS /////////////////////////////////////////////////////////
void CModule::SetStatus(int Status)			// Set agent's status: 1-Alive, rest are error codes
{
pMediator->SetStatus(Status);
}

// GETSTATUS /////////////////////////////////////////////////////////
int CModule::GetStatus() const				// Get agent's status
{
return pMediator->GetStatus();
}

// SETISNEWACTION //////////////////////////////////////////////////////
void CModule::SetIsNewAction(int IsNewAction)	// Set m_IsNewAction predicate
{
pMediator->SetIsNewAction(IsNewAction);
}

// GETISNEWACTION //////////////////////////////////////////////////////
int	CModule::GetIsNewAction() const				// Get m_IsNewAction predicate
{
return pMediator->GetIsNewAction();
}
		
// SETTIME ///////////////////////////////////////////////////////////
void CModule::SetTime(int Time)					// Set time of last sensory input
{
pMediator->SetTime(Time);
}

// GETTIME ///////////////////////////////////////////////////////////
int CModule::GetTime() const					// Get time of last sensory input
{
return pMediator->GetTime();
}

// SETNEWTIME /////////////////////////////////////////////////////////
void CModule::SetNewTime(int NewTime)			// Set time of new sensory input (that hasn't been processed yet)
{
pMediator->SetNewTime(NewTime);
}

// GETNEWTIME /////////////////////////////////////////////////////////
int CModule::GetNewTime() const					// Get time of new sensory input (that hasn't been processed yet)
{
return pMediator->GetNewTime();
}

// SETLASTACTION /////////////////////////////////////////////////////
void CModule::SetLastAction(int i, const char* LastAction)	// Set last action sent by actuators
{
pMediator->SetLastAction(i, LastAction);
}

// GETLASTACTION /////////////////////////////////////////////////////
const char* CModule::GetLastAction(int i) const	// Get last action sent by actuators
{
return pMediator->GetLastAction(i);
}

// SETACTIONCOUNT /////////////////////////////////////////////////////////
void CModule::SetActionCount(int ActionCount)	// Set action count
{
pMediator->SetActionCount(ActionCount);
}

// GETACTIONCOUNT /////////////////////////////////////////////////////////
int CModule::GetActionCount() const				// Get action count
{
return pMediator->GetActionCount();
}

// SETTNAME //////////////////////////////////////////////////////////
void CModule::SetTName(const char* TName)	// Set team name
{
pMediator->SetTName(TName);
}

// GETTNAME //////////////////////////////////////////////////////////
const char* CModule::GetTName() const		// Get team name
{
return pMediator->GetTName();
}
	
// SETSIDE ///////////////////////////////////////////////////////////
void CModule::SetSide(char Side)			// Set side
{
pMediator->SetSide(Side);
}

// GETSIDE ///////////////////////////////////////////////////////////
char CModule::GetSide() const				// Get side
{
return pMediator->GetSide();
}
	
// SETUNUM ///////////////////////////////////////////////////////////
void CModule::SetUNum(int UNum)				// Set uniform number
{
pMediator->SetUNum(UNum);
}
	
// GETUNUM ///////////////////////////////////////////////////////////
int CModule::GetUNum() const				// Get uniform number
{
return pMediator->GetUNum();
}
	
// SETGPOS ///////////////////////////////////////////////////////////
void CModule::SetGPos(CVector CV)			// Set global position
{
pMediator->SetGPos(CV);
}

// GETGPOS ///////////////////////////////////////////////////////////
CVector CModule::GetGPos() const			// Get global position
{
return pMediator->GetGPos();
}
	
// SETFACING ///////////////////////////////////////////////////////////
void CModule::SetFacing(float Facing)		// Set global facing direction
{
pMediator->SetFacing(Facing);
}

// GETFACING ///////////////////////////////////////////////////////////
float CModule::GetFacing() const			// Get global facing direction
{
return pMediator->GetFacing();
}
	
// SETBFACING ///////////////////////////////////////////////////////////
void CModule::SetBFacing(float BFacing)		// Set global body facing direction
{
pMediator->SetBFacing(BFacing);
}

// GETBFACING ///////////////////////////////////////////////////////////
float CModule::GetBFacing() const			// Get global body facing direction
{
return pMediator->GetBFacing();
}	
	
// SETSB ///////////////////////////////////////////////////////////
void CModule::SetSB(SenseBody SB)			// Set sense body info
{
pMediator->SetSB(SB);
}

// GETSB ///////////////////////////////////////////////////////////
SenseBody CModule::GetSB() const			// Get sense body info				
{
return pMediator->GetSB();
}

// SETVELOCITY /////////////////////////////////////////////////////
void CModule::SetVelocity (CVector& V)		// Set velocity vector
{
pMediator->SetVelocity(V);
}

// GETVELOCITY /////////////////////////////////////////////////////
const CVector* CModule::GetVelocity () const// Get velocity vector
{
return pMediator->GetVelocity();
}

// GETNONCONSTVELOCITY /////////////////////////////////////////////
CVector* CModule::GetNonConstVelocity()		// Get non-const velocity vector
{
return pMediator->GetNonConstVelocity();
}
	
// Sensors to Memory Communication Functions /////////////////////////////////////////////////////
// S2MWRITEPLAYER ////////////////////////////////////////////////////////////////////////
void CModule::S2MWritePlayer(int Time, int UNum, char Side, float Dist, float Dir, float DistChng, float DirChng, float BFacing, float Facing) const // Write player to Memory.
{
pMediator->S2MWritePlayer(Time, UNum, Side, Dist, Dir, DistChng, DirChng, BFacing, Facing);
}

// S2MWRITEGOAL //////////////////////////////////////////////////////////////////////////
void CModule::S2MWriteGoal(int Time, char ID, float Dist, float Dir) const		// Write goal to Memory
{
pMediator->S2MWriteGoal(Time, ID, Dist, Dir);
}

// S2MWRITEBALL //////////////////////////////////////////////////////////////////////////
void CModule::S2MWriteBall(int Time, float Dist, float Dir, float DistChng, float DirChng, int Close) const	// Write ball to Memory
{
pMediator->S2MWriteBall(Time, Dist, Dir, DistChng, DirChng, Close);
}

// S2MWRITEFLAG //////////////////////////////////////////////////////////////////////////
void CModule::S2MWriteFlag(int Time, int ID, float Dist, float Dir, float DistChng, float DirChng) const	// Write flag to Memory 
{
pMediator->S2MWriteFlag(Time, ID, Dist, Dir, DistChng, DirChng);
}

// S2MWRITELINE //////////////////////////////////////////////////////////////////////////
void CModule::S2MWriteLine(int Time, char ID, float Dist, float Dir) const	// Write line to Memory
{
pMediator->S2MWriteLine(Time, ID, Dist, Dir);
}

// S2MUPDATE4NEWVISUAL ////////////////////////////////////////////////////////////////////
void CModule::S2MUpdate4NewCycle()		// Update Memory for each new cycle
{
pMediator->S2MUpdate4NewCycle();
}

// Sensors to Behaviours Communication Functions ////////////////////////////////////////////////
// S2BWRITEISNEWVISUAL ////////////////////////////////////////////////////////////////
void CModule::S2BWriteIsNewVisual(int IsTrue) const			// Write IsNewVisual to Behaviours
{
pMediator->S2BWriteIsNewVisual(IsTrue);
}

// S2BSIGNAL ///////////////////////////////////////////////////////////////////////////
void CModule::S2BSignal() const								// Signal Behaviours condition variable
{
pMediator->S2BSignal();
}

// Sensors to PlayMode Communication Functions ////////////////////////////////////////////////////
// S2PWRITEPLAYMODE ///////////////////////////////////////////////////////////////////////////////
void CModule::S2PWritePlayMode(char* PlayMode)				// Update the current playmode
{
pMediator->S2PWritePlayMode(PlayMode);
}
		
// PlayMode to Behaviours Communication Functions /////////////////////////////////////////////////
// P2BREADPLAYMODE ////////////////////////////////////////////////////////////////////////////////
int CModule::P2BReadPlayMode(int Current) const				// Read play mode from PlayMode
{
return pMediator->P2BReadPlayMode(Current);
}

// P2BREADSCORE //////////////////////////////////////////////////////////////////////////////////
int CModule::P2BReadScoreDiff() const						// Read score difference from Behaviours
{
return pMediator->P2BReadScoreDiff();
}

// P2BWRITESCORE /////////////////////////////////////////////////////////////////////////////////
void CModule::P2BWriteScoreDiff(int Diff) const				// Write score difference to Behaviours
{
pMediator->P2BWriteScoreDiff(Diff);
}

// Memory to Behaviours Communication Functions ///////////////////////////////////////////////////
// M2BREADPLAYER //////////////////////////////////////////////////////////////////////////////////
const CPlayer* CModule::M2BReadPlayer(int UNum, char Side) const // Return pointer to player
{
return pMediator->M2BReadPlayer(UNum, Side);
}

// M2BREADPLAYERARRAY /////////////////////////////////////////////////////////////////////////////
const CPlayer*const* CModule::M2BReadPlayerArray(int Far, int Friend) const	// Return pointer to array of players
{
return pMediator->M2BReadPlayerArray(Far, Friend);
}
	
// M2BREADGOAL ////////////////////////////////////////////////////////////////////////////////////
const CGoal* CModule::M2BReadGoal(char ID) const	// Return pointer to goal
{
return pMediator->M2BReadGoal(ID);
}

// M2BREADNONCONSTGOAL ///////////////////////////////////////////////////////////////////////////
CGoal* CModule::M2BReadNonConstGoal(char ID)		// Return non-const pointer to goal (for LookingAt??)
{
return pMediator->M2BReadNonConstGoal(ID);
}

// M2BREADBALL ///////////////////////////////////////////////////////////////////////////////////
const CBall* CModule::M2BReadBall() const	// Return pointer to ball
{
return pMediator->M2BReadBall();
}

// M2BREADNONCONSTBALL ///////////////////////////////////////////////////////////////////////////
CBall* CModule::M2BReadNonConstBall()		// Return non-const pointer to ball (for CalcIntPoint)
{
return pMediator->M2BReadNonConstBall();
}
	
// M2BREADFLAG ////////////////////////////////////////////////////////////////////////////////////
const CFlag* CModule::M2BReadFlag(int ID) const				// Return pointer to flag
{
return pMediator->M2BReadFlag(ID);
}

// M2BREADLINE ////////////////////////////////////////////////////////////////////////////////////
const CLine* CModule::M2BReadLine(char ID) const			// Return pointer to line
{
return pMediator->M2BReadLine(ID);
}

// Actuators to Memory Communication Functions ////////////////////////////////////////////////////
// A2MUPDATE4ACTION ///////////////////////////////////////////////////////////////////////////////
void CModule::A2MUpdate4Action(const char* sendbuffer)		// Update Memory model depending on the action just sent
{
pMediator->A2MUpdate4Action(sendbuffer);
}
