/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CPlayMode class
 * 				The CPlayMode class holds information regarding the current playmode.
 * Created: 15/12/98
 * Last modified: 23/7/99
 */

#include <string.h> 
#include <iostream.h>
#include <pthread.h>
#include "CPlayMode.H"

// Constructor that take Mediator ////////////////////////////////
CPlayMode::CPlayMode(CMediator* Mediator):CModule(Mediator)
{
cout << "[Creating & Initialising Play Mode]" << endl;
pthread_mutex_init(&m_Mutex, NULL);		// Initialise mutex
m_PreviousID = 0;						// Initialise playe-mode IDs
m_ID = 0;								//
}

// DISPLAY ///////////////////////////////////////////////////
void CPlayMode::Display() const	// Displays current playermode.  Just for debugging
{
cout << "PLAYMODE Previous ID: " << m_PreviousID << endl;
cout << "PLAYMODE Previous PlayMode: " << m_PreviousPlayMode << endl;
cout << "PLAYMODE ID: " << m_ID << endl;
cout << "PLAYMODE PlayMode: " << m_PlayMode << endl;
}

// Lock/Unlock functions ////////////////////////////////////
// LOCK ////////////////////////////////////////////////////
void CPlayMode::Lock()	// Lock PlayMode
{
int status;					// Locking status

#ifdef DEBUG_SYNC
cout << "LOCKING PlayMode..." << endl;
#endif

status = pthread_mutex_lock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CPlayMode: Lock: Locking error..." << endl;
}

// UNLOCK //////////////////////////////////////////////////
void CPlayMode::Unlock()	// Unlock PlayMode
{
int status;					// Unlocking status

#ifdef DEBUG_SYNC
cout << "UNLOCKING PlayMode..." << endl;
#endif

status = pthread_mutex_unlock(&m_Mutex);
if (status != 0)
	cerr << "WARNING: CPlayMode: Unlock: Unlocking error..." << endl;
}

// SETPLAYMODE ///////////////////////////////////////////////
void CPlayMode::SetPlayMode(char* PlayMode)	// Set new play-mode and copy current to previous
/*
	The play mode IDs have the follows format:
		0 indicates before_kick_off
		Positive numbers indicate event for my side
		(i.e. corner_kick_l if I am left)
		Negative numbers indicate event for enemy side
		(i.e. corner_kick_l if I am right)
		The IDs for each mode are the same
		(i.e. if 1 indicates kick_off_? then
		-1 indicates also kick_off_?) with the sign
		determining if the event if for or against
		my team.
		
		These IDs are as follows:
			0:	before_kick_off
			1:	kick_off
			2:	kick_in
			3:	corner_kick
			4:	goal_kick
			5:	free_kick
			6:	offside
			
			7:	foul_?_UNum		// However it is foul_? without UNum !!!
			8:	goal_?_Point
			9:	goalie_catch_ball
			
		And then we have the modes that are only positive
		since they don't depend on the side.  These are:
		
			20:	play_on
			21:	half_time
			22:	time_over (or time_up)
			23: time_up_without_a_team
			24:	time_extended
			25:	drop_ball
		
*/
{
char side = GetSide();							// Store side here
int ScoreDiff;									// Store score difference here if we need to update

strncpy(m_PreviousPlayMode, m_PlayMode, MAXPLAYMODELEN);	// Copy current PlayMode to previous
m_PreviousID = m_ID;										// Copy current ID to previous

strncpy(m_PlayMode, PlayMode, MAXPLAYMODELEN);	// Copy new PlayMode

// And now set ID depending on playmode and side
if (!strncmp("before_kick_off", m_PlayMode, 15))
	m_ID = 0;

else if ( (!strncmp("kick_off_", m_PlayMode, 9)) && (m_PlayMode[9] == side) )
	m_ID = 1;
else if ( (!strncmp("kick_off_", m_PlayMode, 9)) && (m_PlayMode[9] != side) )
	m_ID = -1;

else if ( (!strncmp("kick_in_", m_PlayMode, 8)) && (m_PlayMode[8] == side) )
	m_ID = 2;
else if ( (!strncmp("kick_in_", m_PlayMode, 8)) && (m_PlayMode[8] != side) )
	m_ID = -2;

else if ( (!strncmp("corner_kick_", m_PlayMode, 12)) && (m_PlayMode[12] == side) )
	m_ID = 3;
else if ( (!strncmp("corner_kick_", m_PlayMode, 12)) && (m_PlayMode[12] != side) )
	m_ID = -3;
	
else if ( (!strncmp("goal_kick_", m_PlayMode, 10)) && (m_PlayMode[10] == side) )
	m_ID = 4;
else if ( (!strncmp("goal_kick_", m_PlayMode, 10)) && (m_PlayMode[10] != side) )
	m_ID = -4;
	
else if ( (!strncmp("free_kick_", m_PlayMode, 10)) && (m_PlayMode[10] == side) )
	m_ID = 5;
else if ( (!strncmp("free_kick_", m_PlayMode, 10)) && (m_PlayMode[10] != side) )
	m_ID = -5;
	
else if ( (!strncmp("offside_", m_PlayMode, 8)) && (m_PlayMode[8] == side) )
	m_ID = 6;
else if ( (!strncmp("offside_", m_PlayMode, 8)) && (m_PlayMode[8] != side) )
	m_ID = -6;
	
else if ( (!strncmp("foul_", m_PlayMode, 5)) && (m_PlayMode[5] == side) )
	m_ID = 7;
else if ( (!strncmp("foul_", m_PlayMode, 5)) && (m_PlayMode[5] != side) )
	m_ID = -7;
	
else if ( (!strncmp("goal_", m_PlayMode, 5)) && (m_PlayMode[5] == side) )
	{
	ScoreDiff = P2BReadScoreDiff();		// Read current value
	ScoreDiff++;						// Add one
	P2BWriteScoreDiff(ScoreDiff);		// Write new value
	m_ID = 8;
	}
else if ( (!strncmp("goal_", m_PlayMode, 5)) && (m_PlayMode[5] != side) )
	{
	ScoreDiff = P2BReadScoreDiff();		// Read current value
	ScoreDiff--;						// Subtract one
	P2BWriteScoreDiff(ScoreDiff);		// Write new value
	m_ID = -8;
	}
	
else if ( (!strncmp("goalie_catch_ball_", m_PlayMode, 18)) && (m_PlayMode[18] == side) )
	m_ID = 9;
else if ( (!strncmp("goalie_catch_ball_", m_PlayMode, 18)) && (m_PlayMode[18] != side) )
	m_ID = -9;	

else if (!strncmp("play_on", m_PlayMode, 7))
	m_ID = 20;

else if (!strncmp("half_time", m_PlayMode, 9))
	m_ID = 21;
	
else if ((!strncmp("time_over", m_PlayMode, 9)) || (!strncmp("time_up", m_PlayMode, 7)))
	m_ID = 22;
	
else if (!strncmp("time_up_without_a_team", m_PlayMode, 22))
	m_ID = 23;
	
else if (!strncmp("time_extended", m_PlayMode, 6))
	m_ID = 24;
	
else if (!strncmp("drop_ball", m_PlayMode, 9))
	m_ID = 25;

else
	cerr << "WARNING: CPlayMode: SetPlayMode: What playmode is that - " << m_PlayMode << " -?" << endl;
}

// GETPLAYMODE ////////////////////////////////////////////////////
int CPlayMode::GetPlayMode(int Current) const		// Get PlayMode (current or previous)
{
if (Current)				// If current PlayMode requested
	return m_ID;			// return ID
else						// else
	return m_PreviousID;	// return previous ID
}
