/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CPlayMode class
 * 				The CPlayMode class holds information regarding the current play mode.
 * Created: 15/12/98
 * Last modified: 23/7/99
 */

#ifndef _CPLAYMODE_H
#define _CPLAYMODE_H

#include "CModule.H"

class CPlayMode: public CModule
{
// Protected member data /////////////////////////////////////////////////
private:

	pthread_mutex_t	m_Mutex;			// PlayMode's mutex
	
/*
	The play mode IDs have the follows format:
		0 indicates before_kick_off
		Positive numbers indicate event for my side
		(i.e. corner_kick_l if I am left)
		Negative numbers indicate event for enemy side
		(i.e. corner_kick_l if I am right)
		The IDs for each mode are the same
		(i.e. if 1 indicates kick_off_? then
		-1 indicates also kick_off_?) with the sign
		determining if the event if for or against
		my team.
		
		These IDs are as follows:
			0:	before_kick_off
			1:	kick_off
			2:	kick_in
			3:	corner_kick
			4:	goal_kick
			5:	free_kick
			6:	offside
			
			7:	foul_?_UNum		// However it is foul_? without UNum !!!
			8:	goal_?_Point
			9:	goalie_catch_ball
			
		And then we have the modes that are only positive
		since they don't depend on the side.  These are:
		
			20:	play_on
			21:	half_time
			22:	time_over (or time_up)
			23: time_up_without_a_team
			24:	time_extended
			25:	drop_ball
		
*/
	int		m_PreviousID;						// Previous PlayMode ID
	char 	m_PreviousPlayMode[MAXPLAYMODELEN];	// Previous Playmode (in string as said by referee)
	
	int		m_ID;						// PlayMode ID
	char 	m_PlayMode[MAXPLAYMODELEN];	// Playmode (in string as said by referee)

// Public member functions ////////////////////////////////////////////////
public:

// Constructor and Display functions ///////////////////////////
	CPlayMode(CMediator*);			// Constructor that takes Mediator
	void Display() const;	// Displays current playmode.  Just for debugging

// Lock/Unlock functions ////////////////////////////////////
	void Lock();					// Lock PlayMode
	void Unlock();					// Unlock PlayMode
	
// Basic member data set/get functions /////////////////////////////////////
	void SetPlayMode(char*);		// Set new PlayMode and copy current to previous
	int GetPlayMode(int) const;		// Get PlayMode (current or previous)
	
};

#endif
