/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CPlayer class
 * 				The CPlayer class holds information regarding a player on the field.
 * Created: 14/12/98
 * Last modified: 10/7/99
 */

#ifndef _CPLAYER_H
#define _CPLAYER_H

#include <string.h>
#include "CVector.H"
#include "../Agent_Others/GlobalParams.H"

class CPlayer
{
// Protected member data /////////////////////////////////////////////
private:
// These are the player default values in server.conf and agent.conf
	float	m_Size;
	float	m_Decay;
	float	m_Rand;
	float	m_Weight;
	float	m_SpeedMax;
	float	m_MobConfThr;
	
// These values are obtained from sensors ////////////////////////////
	int		m_UNum;				// Uniform number (0 if unknown)
	char	m_TName[MAXTEAMNAMELEN];	// Team name
	char	m_Side;				// Player Side

	int		m_Time;				// Time of observation
	CVector	m_RPos;				// Relative position
	float	m_DistChng;			// Distance change
	float	m_DirChng;			// Direction change
	float	m_RBFacing;			// Relative body facing direction
	float	m_RFacing;			// Relative facing direction
	
// These values are calculated //////////////////////////////////////
	float	m_Conf;				// Confidence

	CVector	m_GPos;				// Global position
	float	m_GBFacing;			// Global body facing direction
	float	m_GFacing;			// Global facing direction

	CVector m_Velocity;			// Player's velocity

	int		m_VelocityValid;		// Is our velocity info valid
	int		m_PositionValid;		// Is our position info valid
	
// Public member functions ////////////////////////////////////////////
public:

// Constructor/Destructor, Init, and Display functions ////////////////
	CPlayer();						// Default constructor
	~CPlayer();						// Default destructor
	void Init(int, int, float, float, float, float, float, float);			// Used to initialise players
	void Display() const;			// Displays all params.  Just for debugging
	const CPlayer *GetThis() const	// Function that returns the "this" pointer
	{ return this; }				// Used by Get*** in Memory
	CPlayer *GetNonConstThis()		// Function that returns a non-const "this" pointer
	{ return this; }				// Used by InitPlayerArrays in Memory
		
// Basic player data set/get functions /////////////////////////////////	
	void	SetUNum	(int UNum)			// Set player's UNum
	{ m_UNum = UNum; }
	int		GetUNum	() const			// Get player's UNum
	{ return m_UNum; }

	void	SetTName	(char* TName)	// Set team name
	{ strncpy(m_TName, TName, MAXTEAMNAMELEN); }
	const char* GetTName	()			// Get team name
	{ return m_TName; }

	void	SetSide	(char Side)			// Set player's side
	{ m_Side = Side; }
	char		GetSide	() const		// Get player's side
	{ return m_Side; }
	
	void	SetTime	(int Time)			// Set player's time
	{ m_Time = Time; }
	int		GetTime	() const			// Get player's time
	{ return m_Time; }
	
	void	SetRPos	(CVector RPos)		// Set player's relative position
	{ m_RPos = RPos; }
	CVector	GetRPos	() const			// Get player's relative position
	{ return m_RPos; }
	void	UpdateRPos(CVector, float, int);	// Update player's relative position if not seen
	
// Accessor function for RPOS ///////////////////////////////////////
	
	void	SetRDir	(float	Dir)		// Set relative direction
	{ m_RPos.SetDir(Dir); }
	float	GetRDir	() const			// Get relative direction
	{ return m_RPos.GetDir(); }
	
	void	SetRMag	(float	Mag)		// Set relative magnitude
	{ m_RPos.SetMag(Mag); }
	float	GetRMag	() const			// Get relative magnitude
	{ return m_RPos.GetMag(); }
	
	void	SetRX	(float	X)			// Set relative X co-ordinate
	{ m_RPos.SetX(X); }
	float	GetRX	() const			// Get relative X co-ordinate
	{ return m_RPos.GetX(); }	
	
	void	SetRY	(float	Y)			// Set relative Y co-ordinate
	{ m_RPos.SetY(Y); }
	float	GetRY	() const			// Get relative Y co-ordinate
	{ return m_RPos.GetY(); }	
	
/////////////////////////////////////////////////////////////////////

	void	SetDistChng	(float DistChng)// Set player's distance change
	{ m_DistChng = DistChng; }
	float	GetDistChng	() const		// Get player's distance change
	{ return m_DistChng; }
	
	void	SetDirChng	(float DirChng)	// Set player's direction change
	{ m_DirChng = DirChng; }
	float	GetDirChng	() const		// Get player's direction change
	{ return m_DirChng; }
	
	void	SetRBFacing	(float RBFacing)// Set player's relative body facing direction
	{ m_RBFacing = RBFacing; }
	float	GetRBFacing	() const		// Get player's relative body facing direction
	{ return m_RBFacing; }
	
	void	SetRFacing	(float RFacing)	// Set player's relative facing direction
	{ m_RFacing = RFacing; }
	float	GetRFacing	() const		// Get player's relative facing direction
	{ return m_RFacing; }

	void	SetConf	(float Conf)		// Set player's confidence
	{ m_Conf = Conf; }
	float	GetConf	() const			// Get player's confidence
	{ return m_Conf; }

	void	SetGPos	(CVector GPos)		// Set player's global position
	{ m_GPos = GPos; }
	CVector	GetGPos	() const			// Get player's global position
	{ return m_GPos; }

// Accessor function for GPOS ///////////////////////////////////////
	
	void	SetGDir	(float	Dir)		// Set relative direction
	{ m_GPos.SetDir(Dir); }
	float	GetGDir	() const			// Get relative direction
	{ return m_GPos.GetDir(); }
	
	void	SetGMag	(float	Mag)		// Set relative magnitude
	{ m_GPos.SetMag(Mag); }
	float	GetGMag	() const			// Get relative magnitude
	{ return m_GPos.GetMag(); }

	void	SetGX	(float	X)			// Set global X co-ordinate
	{ m_GPos.SetX(X); }
	float	GetGX	() const			// Get global X co-ordinate
	{ return m_GPos.GetX(); }	
	
	void	SetGY	(float	Y)			// Set global Y co-ordinate
	{ m_GPos.SetY(Y); }
	float	GetGY	() const			// Get global Y co-ordinate
	{ return m_GPos.GetY(); }	
		
/////////////////////////////////////////////////////////////////////	

	void	SetGBFacing	(float GBFacing)// Set player's global body facing direction
	{ m_GBFacing = GBFacing; }
	float	GetGBFacing	() const		// Get player's global body facing direction
	{ return m_GBFacing; }
	
	void	SetGFacing	(float GFacing)	// Set player's global facing direction
	{ m_GFacing = GFacing; }
	float	GetGFacing	() const		// Get player's global facing direction
	{ return m_GFacing; }

	void	SetVelocity	(CVector CV)	// Set player's velocity
	{ m_Velocity = CV; }
	CVector GetVelocity	() const		// Get player's velocity
	{ return m_Velocity; }

// Accessor function for Velocity ///////////////////////////////////////
	void	SetDir	(float	Dir)		// Set velocity direction
	{ m_Velocity.SetDir(Dir); }
	float	GetDir	() const			// Get velocity direction
	{ return m_Velocity.GetDir(); }
	
	void	SetMag	(float	Mag)		// Set velocity magnitude
	{ m_Velocity.SetMag(Mag); }
	float	GetMag	() const		// Get velocity magnitude
	{ return m_Velocity.GetMag(); }
/////////////////////////////////////////////////////////////////////

	void	SetVelocityValid	(int Value)				// Set velocity valid
	{ m_VelocityValid = Value; }
	int		GetVelocityValid	() const				// Get velocity valid
	{ return m_VelocityValid; }

	void	SetPositionValid	(int Value)				// Set position valid
	{ m_PositionValid = Value; }
	int		GetPositionValid	() const				// Get position valid
	{ return m_PositionValid; }
	
};
 
#endif
