/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CSensors class
 * 				The CSensors class holds information regarding the inputs received by the server.
 *				The CSensors class is also responsible for processing this input.
 * Created: 17/12/98
 * Last modified: 24/7/99
 */

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <pthread.h>
#include "CBall.H" 
#include "CSensors.H"

// Constructor that takes Mediator //////////////////////////////////////
CSensors::CSensors(CMediator* Mediator):CModule(Mediator)
{
cout << "[Creating Eyes & Ears             ]" << endl;
}

// DISPLAY /////////////////////////////////////////////////////////////
void CSensors::Display() const		// Displays all params.  Just for debugging
{
cout << "SENSORS Visual: " << m_Visual << endl;
cout << "SENSORS Audio: " << m_Audio << endl;
cout << "SENSORS SenseBody: " << m_SenseBody << endl;
}

// ABORT ///////////////////////////////////////////////
void CSensors::Abort(char* Msg, int Code)	// Display error message and exit
{
cerr << GetTime() << " Agent " << GetUNum() << " Sensors: " << Msg << endl;
DoExit(Code);
}

// SENSORTHREAD //////////////////////////////////////////////////////////
void CSensors::SensorThread()		// The thread that keeps reading from the socket
{
cout << "SENSORS thread started!" << endl;
int n;				// Used for receive message byte count

while (GetStatus() == 1)			// While alive
	{
	n = ReceiveMessage(m_Temp, MAXVISUALLEN);		// Receive message
	
	switch(n)		// Check byte return code and
		{
		case 0:		// If 0, just break
			break;
		case 1:		// If 1
			LockMemory();		// Lock Memory mutex
			AnalyseData();		// Analyse new sensory input
			strncpy(m_Temp, "", 2);	// Clear receive buffer (just a safeguard)
			UnlockMemory();		// Unlock mutex
			break;
		default:	// If none of the above...display error and exit
			Abort("ERROR: CSensors: SensorThread: Problem while receiving data...Has server died?", 2000);
			exit(2000);		// Brute force exit for now...
			break;
		}
	}
	
pthread_exit(NULL);				// Exit this thread
}

// ANALYSEDATA /////////////////////////////////////////////////////////
void CSensors::AnalyseData()		// Check what type of data has just received
{
char *tmp=m_Temp;	// Look at m_Temp
tmp++;				// Skip the first character "("

#ifdef DEBUG_SENSORS
if (GetTime() != 0)
	cout << "RECEIVED: " << m_Temp << endl;
#endif

switch (*tmp)
	{
	case 'i':			// Another init command?
		cerr << "WARNING: AnalyseData: Another init command?" << endl;
		break;
	case 'e':			// Error command
		cerr << "WARNING: AnalyseData: Received error command..." << endl;
		break;
	case 'r':			// Another reconnect command?
		cerr << "WARNING: AnalyseData: Another reconnect command?" << endl;
		break;
	case 's':					// See command or sense body command
		tmp+=2;					// Skip "se" and test next character
		if (*tmp == 'e')		// if it is "e" it was a "see" info
			{
			// Memory mutex already locked here
			ParseVisual();					// so process the visual data

			RemoveGhosts();					// Forget any objects that should have been visible, but aren't			
			
			UpdateGPos();					// Update global position in Agent, then UpdateGlobals, and UpdateRelatives

			LockBehaviours();				// Lock behaviours mutex			
			S2BWriteIsNewVisual(1);			// Set new visual data predicate as true
			S2BSignal();					// Signal the Behaviours condition variable
			UnlockBehaviours();				// Unlock behaviours mutex
			
			// Memory mutex will be unlocked after this
			}
		else if (*tmp == 'n')	// if it is "n" it was a "sense" info
			{
			ParseSenseBody();	// so process the sense body data
			}
		else					// else, what on earth is it?
			cerr << "WARNING: Sensory data starting with -s- was neither visual nor sense_body..." << endl;
		break;
	case 'h':			// Hear command
		ParseAudio();	// Process the data
		break;
	default:			// None of the above?  Display warning
		cerr << "WARNING: AnalyseData: Default?" << endl;
		break;
	}
}
	
// PARSEVISUAL /////////////////////////////////////////////////////////
void CSensors::ParseVisual()		// Analyse visual data
{
/*
	Typical see string looks like:

	(see 30 ((goal r) 69.4 32) ((flag c t) 6.7 26 -0 0) ((flag r t) 58.6 2) ((flag p r t) 45.2 21) 
	((flag p r c) 55.7 41) ((flag g r t) 66 27) ((flag g r b) 73 36) ((flag t 0) 6.3 -18 0 0) 
	((flag t r 10) 16.1 -7 0 0) ((flag t r 20) 26 -4) ((flag t r 30) 36.2 -3 0 0) ((flag t r 40) 46.1 -2) 
	((flag t r 50) 56.3 -2) ((flag r t 30) 64.1 6) ((flag r t 20) 66 14) ((flag r t 10) 68.7 23) 
	((flag r 0) 73.7 30) ((flag r b 10) 79 36) ((flag r b 20) 85.6 41) 
	((player Essex_Wizards 1) 3 0 0 0 0))

	Line and ball objects are missing, but you get the general idea...
*/
int l, time;							// Skip char number, and time
char *tmp=m_Temp;						// Look at m_Temp

sscanf(tmp, "(see %d%n", &time, &l);	// Get the Time
SetNewTime(time);						// Set time of new sensory data
Changed();				// Inform mediator (i.e. Agent) of the change now that we have the new time
tmp += l;								// Advance past the time

/*
	Before entering the while loop the pointer is left to exactly
	the next character after the last token's close bracket.
	(If it is the first time in the loop it is just after the time)
	This means that if the next char is ")" then that is the end of
	the string.  If on the other hand the next char is NOT ")" then
	it is a space followed by the next token.  So if we skip the next
	3 characters i.e. " ((" we get to the first letter of the next token
	and we can start our switch again to see what it is.  It is important
	after processing each token to live the pointer to exactly the next
	character after the last ")" of the token just processed.
*/
while (*tmp != ')')						// While not end of message
	{
	tmp += 3;							// Skip " (("

	switch (*tmp)
		{
		/*				
			In the switch statement we look for things depending
			on their frequency of appearance.  Too many flags
			i.e. the come first, 22 players so they come second,
			then 4 lines, 2 goal, and 1 ball.
		*/
		case 'f':						// If flag
			l = ParseFlag(1, tmp);		// Then parse flag
			tmp += l;					// and advance the pointer
			break;

		case 'F':						// If flag very close
			l = ParseFlag(0, tmp);		// Then parse close flag
			tmp += l;					// and advance the pointer
			break;
						
		case 'p':						// If player
			l = ParsePlayer(1, tmp);	// Then parse player
			tmp += l;					// and advance the pointer
			break;

		case 'P':						// If player very close
			l = ParsePlayer(0, tmp);	// Then parse close player
			tmp += l;					// and advance the pointer
			break;

		case 'l':						// If line
			l = ParseLine(1, tmp);		// Then parse line
			tmp += l;					// and advance the pointer
			break;

		case 'L':						// If line very close
			l = ParseLine(0, tmp);		// Then parse close line
			tmp += l;					// and advance the pointer
			break;

		case 'g':						// If goal
			l = ParseGoal(1, tmp);		// Then parse goal
			tmp += l;					// and advance the pointer
			break;

		case 'G':						// If goal very close
			l = ParseGoal(0, tmp);		// Then parse close goal
			tmp += l;					// and advance the pointer
			break;

		case 'b':						// If ball
			l = ParseBall(1, tmp);		// Then parse ball
			tmp += l;					// and advance the pointer
			break;

		case 'B':						// If ball very close
			l = ParseBall(0, tmp);		// Then parse close ball
			tmp += l;					// and advance the pointer
			break;

		default:						// None of the above?  Display fragment of visual and warning
			cerr << m_Temp << endl;
			cerr << "ERROR for " << tmp << endl;
			cerr << "WARNING: ParseVisual: Default?" << endl;
			break;
		}
	}
}

// PARSEAUDIO //////////////////////////////////////////////////////////
void CSensors::ParseAudio()		// Analyse audio data
{
/*
	Typical audio strings look like:

	(hear 0 referee kick_off_l)
	(hear 4 -87 Hello World)
	
*/
int l;					// Skip char number
char *tmp=m_Temp;		// Look at m_Temp
Audio AU;				// Audio structure to store info

sscanf(tmp, "(hear %d%n", &AU.Time, &l);	// Get the Time
SetNewTime(AU.Time);						// Set time of new sensory data
Changed();				// Inform mediator (i.e. Agent) of the change now that we have the new time
tmp += l+1;									// Advance past the time and the space that follows

//cout << GetTime() << " AUDIO FROM SENSORS: " << m_Temp;
		
switch (*tmp)
	{
	case 'r':										// If referee
		/*
			You normally get something like:
			(hear 0 referee play_on)
		*/
		strncpy(AU.Sender, "referee", MAXSENDERLEN);// Set sender to referee
		sscanf(tmp, "referee  %[^)]", &AU.Message);	// Store the message
		AU.Direction = -999;						// and set direction to -999
		LockPlayMode();								// Lock PlayMode mutex
		S2PWritePlayMode(AU.Message);				// Update the current playmode
		UnlockPlayMode();							// Unlock PlayMode mutex
		break;
	case 's':										// If self
		/*
			You normally get something like:
			(hear 0 self Message goes here)
		*/
		strncpy(AU.Sender, "self", MAXSENDERLEN);	// Set sender to self
		sscanf(tmp, "self %[^)]", &AU.Message);		// Store message
		break;
	case 'o':										// It is an online-coach
		/*
			You normally get something like:
			(hear 0 online_coach_right Message goes here) OR
			(hear 0 online_coach_left Message goes here)
		*/
		if (!strncmp(tmp, "online_coach_left", 17))						// If this is the left coach
			{
			strncpy(AU.Sender, "online_coach_left", MAXSENDERLEN);		// Set sender to online_coach_left
			sscanf(tmp, "online_coach_left %[^)]", &AU.Message);		// Store message
			}
		else															// else it must be the right coach
			{
			strncpy(AU.Sender, "online_coach_right", MAXSENDERLEN);		// Set sender to online_coach_right
			sscanf(tmp, "online_coach_right %[^)]", &AU.Message);		// Store message
			}
		break;
	default:															// Otherwise it is another player
		/*
			You normally get something like:
			(hear 8 -22 Message goes here)
		*/
		strncpy(AU.Sender, "player", MAXSENDERLEN);						// Set sender to player
		sscanf(tmp, "%d  %[^)]", &AU.Direction, &AU.Message);			// Store direction of message and message
		break;
	}
		
/*
#ifdef DEBUG_SENSORS
cout << "Audio info:" << endl;
cout << "Time      = " << AU.Time << endl;
cout << "Sender    = " << AU.Sender << endl;
cout << "Direction = " << AU.Direction << endl;
cout << "Message   = " << AU.Message << endl;
#endif
*/
}

// PARSESENSEBODY /////////////////////////////////////////////////////
void CSensors::ParseSenseBody()		// Analyse sense body info
{
/*
	Typical sense body string looks like:
	
	(sense_body 0 (view_mode high normal) (stamina 2000 1) (speed 0) (head_angle 0) 
	(kick 0) (dash 2) (turn 0) (say 0) (turn_neck 0))
	
	Some of the values above, although integers here, are floating point numbers.
	If you scan for integer instead of float the program WILL crash...
*/
SenseBody	SB;						// SenseBody structure to store info
CVector* pVelocity = GetNonConstVelocity();	// Store our current velocity here

//cout << GetTime() << " SENSEBODY from SENSORS: " << m_Temp;

sscanf(m_Temp, "(sense_body %d (view_mode %s %[^)]) (stamina %f %f) (speed %f) (head_angle %f) (kick %d) (dash %d) (turn %d) (say %d) (turn_neck %d))", 
&SB.Time, &SB.VMQuality, &SB.VMMode, &SB.Stamina, &SB.Effort, &SB.Speed, &SB.HeadAngle, &SB.Kick, &SB.Dash, &SB.Turn, &SB.Say, &SB.TurnNeck);

SetNewTime(SB.Time);				// Set time of new sensory data
Changed();							// Inform mediator (i.e. Agent) of the change now that we have the new time
SetSB(SB);

/*
	Do not change direction of velocity since last command might have been turn.
	If that was the case, our global facing has changed, but our velocity's direction HAS NOT
*/
pVelocity->SetMag(SB.Speed);		// Update velocity magnitude
SetVelocity(*pVelocity);			//

/*
#ifdef DEBUG_SENSORS
cout << "Sense body info:" << endl;
cout << "Time       = " << SB.Time << endl;
cout << "VM Quality = " << SB.VMQuality << endl;
cout << "VM Mode    = " << SB.VMMode << endl;
cout << "Stamina    = " << SB.Stamina << endl;
cout << "Effort     = " << SB.Effort << endl;
cout << "Speed      = " << SB.Speed << endl;
cout << "Head angle = " << SB.HeadAngle << endl;
cout << "Kick       = " << SB.Kick << endl;
cout << "Dash       = " << SB.Dash << endl;
cout << "Turn       = " << SB.Turn << endl;
cout << "Say        = " << SB.Say << endl;
cout << "Turn neck  = " << SB.TurnNeck << endl;
#endif
*/

//if (SB.Stamina < 500)
//	cerr << GetTime() << ": Player " << GetUNum() << " is low on energy! Currently at: " << SB.Stamina << endl;
}

// PARSEPLAYER /////////////////////////////////////////////
int CSensors::ParsePlayer(int Far, char *Buffer)	// Analyse player data
{
char *tmp=Buffer;			// Look at Buffer
int l, sum=0;				// Number of chars pointer will be advanced and its sum
char tname[MAXTEAMNAMELEN];	// Player's teamname
tname[0] = '\0';			// In case you get (player) so that tname is not junk
int unum=-999;				// Player's uniform number
char side='u';				// Player's side
float dist=-999, dir=-999;	// Player's distance and direction
float distchng=-999, dirchng=-999;	// Player's distance change and direction change
float bfacing=-999, facing=-999;	// Player's body facing direction and facing direction

//cout << GetTime() << " PLAYER from SENSORS: " << tmp;

if (Far)			// If player is not close
	{
	sscanf(tmp, "player%n", &l);	// Move pointer after "player"
	tmp += l;						//
	sum += l;						// update sum

	// First layer, checking for team name //////////////////////////////////////////	
	if (*tmp == ')')				// If no more data
		{
		sscanf(tmp, ") %f %f)%n", &dist, &dir, &l);			// Get distance and direction
		tmp += l;					// Move pointer
		sum += l;					// update sum
		}
	else							// If team name available
		{
		sscanf(tmp, " %[A-Za-z0-9_.+*/?<>-]%n", &tname, &l);		// Get team name
		tmp += l;					// Move pointer
		sum += l;					// update sum
		// Since team name available, use it to determine side
		if (!strncmp(GetTName(), tname, MAXTEAMNAMELEN))	// If team name is the same
			{
			if (GetSide() == 'l')							// If my side is left
				side = 'l';									// then side is left
			else											// else
				side = 'r';									// side is right
			}
		else
			{
			if (GetSide() == 'r')							// If my side is right
				side = 'l';									// then side is left
			else											// else
				side = 'r';									// side is right
			}
		
		// Second layer, checking for uniform number ////////////////////////////////////
		if (*tmp == ')')			// If no more info
			{
			sscanf(tmp, ") %f %f)%n", &dist, &dir, &l);		// Get distance and direction
			tmp += l;				// Move pointer
			sum += l;				// update sum
			}
		else						// If uniform number available
			{
			sscanf(tmp, " %d) %f %f%n", &unum, &dist, &dir, &l);	// Get uniform num, distance and direction
			tmp += l;				// Move pointer
			sum += l;				// update sum
			// Third layer, checking for distance change and direction change ////////////
			if (*tmp == ')')		// If no more info
				{
				tmp++;				// Move past ')'
				sum++;				// Update sum
				}
			else					// If distance change, direction change and facing direction are available
				{
				// Get distance change, direction change, body facing and facing direction
				sscanf(tmp, " %f %f %f %f)%n", &distchng, &dirchng, &bfacing, &facing, &l);	// Version 5.xx
				//sscanf(tmp, " %f %f %f)%n", &distchng, &dirchng, &facing, &l);	// Version 4.xx
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			}						// End of third layer
		}							// End of second layer
	}								
else				// else if it is close
	{
	// Extract distance and direction info
	sscanf(tmp, "Player) %f %f)%n", &dist, &dir, &sum);
	unum = 0;		// and set unum to 0
	}

S2MWritePlayer(GetTime(), unum, side, dist, dir, distchng, dirchng, bfacing, facing);
return sum;			// Return number of chars that pointer will be advanced
}

// PARSEGOAL /////////////////////////////////////////////
int CSensors::ParseGoal(int Far, char *Buffer)	// Analyse goal data
{
int l;						// Number of chars pointer will be advanced
char ID;					// Left or right goal
float dist=-999, dir=-999;	// Goal's distance and direction

if (Far)			// If goal is not close
	{
	// extract all the info
	sscanf(Buffer, "goal %c) %f %f)%n", &ID, &dist, &dir, &l);
	}
else				// else if it is close
	{
	// Set ID to "u" and extract the rest of the info
	sscanf(Buffer, "Goal) %f %f)%n", &dist, &dir, &l);
	ID = 'u';		// ID unknown
	}

S2MWriteGoal(GetTime(), ID, dist, dir);
return l;			// Return number of chars that pointer will be advanced
}

// PARSEBALL /////////////////////////////////////////////
int CSensors::ParseBall(int Far, char *Buffer)	// Analyse ball data
{
char *tmp=Buffer;						// Look at buffer
int l, sum=0;							// Number of chars pointer will be advanced and its sum
int close = -999;						// Is ball too close (i.e. Ball or ball)
float dist=-999, dir=-999;				// Ball's distance and direction
float distchng=-999, dirchng=-999;		// Ball's distance change and direction change

//cout << GetTime() << " BALL from SENSORS: " << tmp;

if (Far)				// If ball is not close
	{
	close = 0;			// Set close to false
	// Extract distance and direction info
	sscanf(tmp, "ball) %f %f%n", &dist, &dir, &l);
	sum += l;			// Update sum
	tmp += l;			// Advance pointer
	if (*tmp != ')')	// Distchng and dirchng info found
		{
		// Extract distange change and direction change info
		sscanf(tmp, " %f %f)%n", &distchng, &dirchng, &l);	// Store dist/dir change info
		sum += l;		// Update sum
		}
	else				// else if no distance change and direction info found
		sum++;			// Advance past last ')'
	}
else					// else if it is close
	{
	close = 1;			// Set close to true
	sscanf(tmp, "Ball) %f %f)%n", &dist, &dir, &sum);
	}

S2MWriteBall(GetTime(), dist, dir, distchng, dirchng, close);
return sum;			// Return number of chars that pointer will be advanced
}

// PARSEFLAG /////////////////////////////////////////////
int CSensors::ParseFlag(int Far, char *Buffer)	// Analyse flag data
{
char *tmp=Buffer;			// Look at Buffer
int l, sum=0;				// Number of chars pointer will be advanced and its sum
char subid='u', hor='u', ver='u';	// Flag's optional sub-ID (in case it is goal, penalty or center flag),
									// horizontal and vertical ID (i.e. top, bottom, left, right)
									// NOTE: subid is also used to resolve conflicts.  Look further
									// down at 'l' and 'r' cases
int distid=-999;			// Flag's distance-ID (in case it is outside the field like (flag t l 50))							
float dist=-999, dir=-999;	// Flag's distance and direction
float distchng=-999, dirchng=-999;	// Flag's distance change and direction change

if (Far)			// If flag is not close
	{
	sscanf(tmp, "flag %n", &l);		// Get past the initial "flag " string
	tmp += l;						// Move pointer
	sum += l;						// Update sum

	/*
		After the switch statement we will have all the information
		regarding the flag's overall ID, distance and direction.
		The only thing left would be to check if we have distance change
		and direction change information.
	*/	
	switch (*tmp)					// Get the first character
		{
		case 't':					// If top flag outside the field
			ver = 't';				// Vertical position is top
			tmp += 2;				// Move past "t "
			sum += 2;				// Update sum
			
			if (*tmp == '0')		// If it is the (flag t 0) flag
				{
				distid = 0;			// Set dist-ID to 0
				sscanf(tmp, "0) %f %f%n", &dist, &dir, &l);	// Get distance and direction
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			else					// If it is NOT the (flag t 0) flag
				{
				// Get flag's horizontal position, distance-ID, distance and direction
				sscanf(tmp, "%c %d) %f %f%n", &hor, &distid, &dist, &dir, &l);
				tmp += l;				// Move pointer
				sum += l;				// Update sum
				}
			break;
			
		case 'b':					// If bottom flag outside the field
			ver = 'b';				// Vertical position is bottom
			tmp += 2;				// Move past "b "
			sum += 2;				// Update sum
			
			if (*tmp == '0')		// If it is the (flag b 0) flag
				{
				distid = 0;			// Set dist-ID to 0
				sscanf(tmp, "0) %f %f%n", &dist, &dir, &l);	// Get distance and direction
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			else					// If it is NOT the (flag b 0) flag
				{
				// Get flag's horizontal position, distance-ID, distance and direction
				sscanf(tmp, "%c %d) %f %f%n", &hor, &distid, &dist, &dir, &l);
				tmp += l;				// Move pointer
				sum += l;				// Update sum
				}
			break;
			
		case 'l':					// If left flag
			hor = 'l';				// Horizontal position is left
			tmp += 2;				// Move past "l "
			sum += 2;				// Update sum
			
			if (*tmp == '0')		// If it is the (flag l 0) flag
				{
				distid = 0;			// Set dist-ID to 0
				sscanf(tmp, "0) %f %f%n", &dist, &dir, &l);	// Get distance and direction
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			else					// If it is NOT the (flag l 0) flag
				{
				sscanf(tmp, "%c%n", &ver, &l);	// Get flag's vertical ID
				tmp += l;						// Move pointer
				sum += l;						// Update sum
			
				if (*tmp == ')')		// If no more data, i.e. either (flag l b) or (flag l t)
					{
					sscanf(tmp, ") %f %f%n", &dist, &dir, &l);	// Get distance and direction
					tmp += l;						// Move pointer
					sum += l;						// Update sum
					}
				else					// else if more data available, i.e. flag off the field (e.g. (flag l b 30))
					{
					// Get dist-ID, distance and direction
					sscanf(tmp, " %d) %f %f%n", &distid, &dist, &dir, &l);
					tmp += l;			// Move pointer
					sum += l;			// Update sum

					// If LB?? flag we need to set the subid to 'h' to indicate horizontal flag so
					// that we can distinguish between BL??.  That is because in both cases
					// hor='l', ver='b', and distid=30.
					subid='h';
					}
				}
			break;

		case 'r':					// If right flag
			hor = 'r';				// Horizontal position is right
			tmp += 2;				// Move past "r "
			sum += 2;				// Update sum
			
			if (*tmp == '0')		// If it is the (flag r 0) flag
				{
				distid = 0;			// Set dist-ID to 0
				sscanf(tmp, "0) %f %f%n", &dist, &dir, &l);	// Get distance and direction
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			else					// If it is NOT the (flag r 0) flag
				{
				sscanf(tmp, "%c%n", &ver, &l);	// Get flag's vertical ID
				tmp += l;							// Move pointer
				sum += l;							// Update sum
			
				if (*tmp == ')')		// If no more data, i.e. either (flag r b) or (flag r t)
					{
					sscanf(tmp, ") %f %f%n", &dist, &dir, &l);	// Get distance and direction
					tmp += l;						// Move pointer
					sum += l;						// Update sum
					}
				else					// else if more data available, i.e. flag off the field (e.g. (flag r b 30))
					{
					// Get dist-ID, distance and direction
					sscanf(tmp, " %d) %f %f%n", &distid, &dist, &dir, &l);
					tmp += l;						// Move pointer
					sum += l;						// Update sum
					
					// If RB?? flag we need to set the subid to 'h' to indicate horizontal flag so
					// that we can distinguish between BR??.  That is because in both cases
					// hor='l', ver='b', and distid=30.
					subid='h';
					}
				}
			break;
			
		case 'p':					// If penalty flag
			subid = 'p';			// Sub-ID is penalty
			// Get horizontal and vertical position, distance and direction
			sscanf(tmp, "p %c %c) %f %f%n", &hor, &ver, &dist, &dir, &l);
			tmp += l;				// Move pointer
			sum += l;				// Update sum
			break;
			
		case 'g':					// If goal flag
			subid = 'g';			// Sub-ID is goal
			// Get horizontal and vertical position, distance and direction
			sscanf(tmp, "g %c %c) %f %f%n", &hor, &ver, &dist, &dir, &l);
			tmp += l;				// Move pointer
			sum += l;				// Update sum
			break;
			
		case 'c':					// If center flag
			hor = 'c';				// Horizontal position is center
			tmp ++;					// Move pointer past 'c'
			sum ++;					// Update sum
			if (*tmp == ')')		// If it is THE center flag
				{
				sscanf(tmp, ") %f %f%n", &dist, &dir, &l);	// Get distance and direction
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			else					// else we need to get the vertical position
				{
				// Get vertical position, distance, and direction
				sscanf(tmp, " %c) %f %f%n", &ver, &dist, &dir, &l);
				tmp += l;			// Move pointer
				sum += l;			// Update sum
				}
			break;
			
		default:					// What flag is that?  Display warning
			cerr << "WARNING: ParseFlag: What flag is that?" << endl;
			break;
		}
	
	if (*tmp == ')')				// If no distange change and direction change info
		{
		tmp ++;						// Move past ')'
		sum ++;						// Update sum		
		}
	else							// If distance change and direction change info is available
		{
		sscanf(tmp, " %f %f)%n", &distchng, &dirchng, &l);	// Get distance change and direction change info
		tmp += l;					// Move pointer
		sum += l;					// Update sum
		}			
	}
else				// else if it is close
	// Extract distance and direction info
	sscanf(tmp, "Flag) %f %f)%n", &dist, &dir, &sum);

S2MWriteFlag(GetTime(), Flag2ID(subid, hor, ver, distid), dist, dir, distchng, dirchng);	// Store flag in Memory

return sum;			// Return number of chars that pointer will be advanced
}

// PARSELINE /////////////////////////////////////////////
int CSensors::ParseLine(int Far, char *Buffer)	// Analyse line data
{
int l;						// Number of chars pointer will be advanced
char ID;					// Line's ID {l, r, t, b}
float dist=-999, dir=-999;	// Line's distance and direction

if (Far)			// If line is not close
	{
	// extract all the info
	sscanf(Buffer, "line %c) %f %f)%n", &ID, &dist, &dir, &l);
	}
else				// else if it is close
	{
	// Set ID to "u" and extract the rest of the info
	sscanf(Buffer, "Line) %f %f)%n", &dist, &dir, &l);
	ID = 'u';		// ID unknown
	}

S2MWriteLine(GetTime(), ID, dist, dir);	// Store line in Memory
return l;			// Return number of chars that pointer will be advanced
}
	
// FLAG2ID ////////////////////////////////////////////////////
int CSensors::Flag2ID(char subid, char hor, char ver, int distid) const	// Convert the flag to an ID
{
if ( (distid == -999) && (hor == 'c') && (ver == 'u') && (subid == 'u') )
	return 1;
else if ( (distid == -999) && (hor == 'l') && (ver == 'b') && (subid == 'p') )
	return 2;
else if ( (distid == -999) && (hor == 'l') && (ver == 'c') && (subid == 'p') )
	return 3;
else if ( (distid == -999) && (hor == 'l') && (ver == 't') && (subid == 'p') )
	return 4;
else if ( (distid == -999) && (hor == 'r') && (ver == 't') && (subid == 'p') )
	return 5;
else if ( (distid == -999) && (hor == 'r') && (ver == 'c') && (subid == 'p') )
	return 6;
else if ( (distid == -999) && (hor == 'r') && (ver == 'b') && (subid == 'p') )
	return 7;
else if ( (distid == -999) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 8;
else if ( (distid == -999) && (hor == 'l') && (ver == 'b') && (subid == 'g') )
	return 9;
else if ( (distid == -999) && (hor == 'l') && (ver == 't') && (subid == 'g') )
	return 10;
else if ( (distid == -999) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 11;
else if ( (distid == -999) && (hor == 'c') && (ver == 't') && (subid == 'u') )
	return 12;
else if ( (distid == -999) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 13;
else if ( (distid == -999) && (hor == 'r') && (ver == 't') && (subid == 'g') )
	return 14;
else if ( (distid == -999) && (hor == 'r') && (ver == 'b') && (subid == 'g') )
	return 15;
else if ( (distid == -999) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 16;
else if ( (distid == -999) && (hor == 'c') && (ver == 'b') && (subid == 'u') )
	return 17;
else if ( (distid == 30) && (hor == 'l') && (ver == 'b') && (subid == 'h') )
	return 18;
else if ( (distid == 20) && (hor == 'l') && (ver == 'b') && (subid == 'h') )
	return 19;
else if ( (distid == 10) && (hor == 'l') && (ver == 'b') && (subid == 'h') )
	return 20;
else if ( (distid == 0) && (hor == 'l') && (ver == 'u') && (subid == 'u') )
	return 21;
else if ( (distid == 10) && (hor == 'l') && (ver == 't') && (subid == 'h') )
	return 22;
else if ( (distid == 20) && (hor == 'l') && (ver == 't') && (subid == 'h') )
	return 23;
else if ( (distid == 30) && (hor == 'l') && (ver == 't') && (subid == 'h') )
	return 24;
else if ( (distid == 50) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 25;
else if ( (distid == 40) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 26;
else if ( (distid == 30) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 27;
else if ( (distid == 20) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 28;
else if ( (distid == 10) && (hor == 'l') && (ver == 't') && (subid == 'u') )
	return 29;
else if ( (distid == 0) && (hor == 'u') && (ver == 't') && (subid == 'u') )
	return 30;
else if ( (distid == 10) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 31;
else if ( (distid == 20) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 32;
else if ( (distid == 30) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 33;
else if ( (distid == 40) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 34;
else if ( (distid == 50) && (hor == 'r') && (ver == 't') && (subid == 'u') )
	return 35;
else if ( (distid == 30) && (hor == 'r') && (ver == 't') && (subid == 'h') )
	return 36;
else if ( (distid == 20) && (hor == 'r') && (ver == 't') && (subid == 'h') )
	return 37;
else if ( (distid == 10) && (hor == 'r') && (ver == 't') && (subid == 'h') )
	return 38;
else if ( (distid == 0) && (hor == 'r') && (ver == 'u') && (subid == 'u') )
	return 39;
else if ( (distid == 10) && (hor == 'r') && (ver == 'b') && (subid == 'h') )
	return 40;
else if ( (distid == 20) && (hor == 'r') && (ver == 'b') && (subid == 'h') )
	return 41;
else if ( (distid == 30) && (hor == 'r') && (ver == 'b') && (subid == 'h') )
	return 42;
else if ( (distid == 50) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 43;
else if ( (distid == 40) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 44;
else if ( (distid == 30) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 45;
else if ( (distid == 20) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 46;
else if ( (distid == 10) && (hor == 'r') && (ver == 'b') && (subid == 'u') )
	return 47;
else if ( (distid == 0) && (hor == 'u') && (ver == 'b') && (subid == 'u') )
	return 48;
else if ( (distid == 10) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 49;
else if ( (distid == 20) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 50;
else if ( (distid == 30) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 51;
else if ( (distid == 40) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 52;
else if ( (distid == 50) && (hor == 'l') && (ver == 'b') && (subid == 'u') )
	return 53;
else if ( (distid == -999) && (hor == 'u') && (ver == 'u') && (subid == 'u') )
	return 0;	// Special case for Flag (i.e. if flag is close)
else
	cerr << "WARNING: CSensors: Flag2ID: No match found for " << subid << " " << hor << " " << ver << " " << distid << endl;
}
