/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CSensors class
 * 				The CSensors class holds information regarding the inputs received by the server.
 *				The CSensors class is also responsible for processing this input.
 * Created: 17/12/98
 * Last modified: 16/7/99
 */

#ifndef _CSENSORS_H
#define _CSENSORS_H

#include "CModule.H"
#include "../Agent_Others/GlobalParams.H"
 
class CSensors: public CModule
{
// Protected member data ////////////////////////////////////////////////
private:

	Socket	m_Socket;						// The UDP socket
	
	char	m_Temp[MAXVISUALLEN];			// Temporary storage buffer
	char	m_Visual[MAXVISUALLEN];			// Visual data buffer
	char	m_Audio[MAXAUDIOLEN];			// Audio data buffer
	char	m_SenseBody[MAXSENSEBODYLEN];	// SenseBody buffer

// Public member functions //////////////////////////////////////////////
public:

// Constructor and Display functions /////////////////////////
	CSensors(CMediator*);		// Constructor that takes Mediator
	void Display() const;		// Displays all params.  Just for debugging
	CSensors *GetThis()			// Function that returns the "this" pointer.
	{ return this; }			// Used in the externs to start the threads.
	
// Error reporting functions ////////////////////////////////////////
	void	Abort	(char*, int);		// Display message and exit with a given code
	
// Basic actuator attribute get/set functions //////////////////////////
	void    SetSocket	(Socket Sock)	// Store socket info to m_Sock
	{ m_Socket = Sock; }
	Socket  GetSocket	() const		// Get socket info from m_Sock
	{ return m_Socket; }
	
// Communication functions //////////////////////////////////////////////
	int		ReceiveMessage	(char*, int);	// Receive message from the server
		
// Read and parse data functions ////////////////////////////////////////
	void	SensorThread	();				// The thread that keeps reading from the socket
	void	AnalyseData		();				// Check what type of data was just received
	
	void	ParseVisual		();				// Analyse visual data
											// This function returns whether the new data have been stored
											// or ignored.  The data is ignored if internal update is newer.
	void	ParseAudio		();				// Analyse audio data
	void	ParseSenseBody	();				// Analyse sense body info
	
	// Parameters are Far and Buffer
	int		ParsePlayer		(int, char*);	// Analyse player data
	int		ParseGoal		(int, char*);	// Analyse goal data
	int		ParseBall		(int, char*);	// Analyse ball data
	int		ParseFlag		(int, char*);	// Analyse flag data
	int		ParseLine		(int, char*);	// Analyse line data

// Converting functions///////////////////////////////////////////////////
	// Parameters are subid, hor, ver, and distid
	int		Flag2ID			(char, char, char, int) const;		// Convert a flag to an ID		
};

#endif
