/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CVector class
 * 				The CVector class is responsible for all the vector manipulation operations.
 * Created: 6/4/98
 * Last modified: 8/7/99
 */

#include <iostream.h>
#include "CVector.H"

// DEFAULT CONSTUCTOR ///////////////////////////////////////////////
CVector::CVector()			// Default constrctor
{
m_X = 0.0;
m_Y = 0.0;
m_Mag = 0.0;
m_Dir = 0.0;
}

// CONSTUCTOR /////////////////////////////////////////////////////
CVector::CVector(float X, float Y)	// Constructor that takes X, Y
{
m_X = X;
m_Y = Y;
CalcMag();
CalcDir();
}

// DISPLAY /////////////////////////////////////////////////////////
void CVector::Display() const	// Displays all params.  Just for debugging
{
cout << "VECTOR with X, Y, Mag, Dir: " << m_X << ", " << m_Y << ", " << m_Mag << ", " << m_Dir << endl;
}

// OPERATOR = /////////////////////////////////////////////////////
void CVector::operator =(const CVector &CV)
{
m_X = CV.GetX();
m_Y = CV.GetY();
m_Mag = CV.GetMag();
m_Dir = CV.GetDir();
}

// OPERATOR += /////////////////////////////////////////////////////
void CVector::operator +=(const CVector &CV)
{
m_X += CV.GetX();
m_Y += CV.GetY();
CalcMag();
CalcDir();
}

// OPERATOR -= /////////////////////////////////////////////////////
void CVector::operator -=(const CVector &CV)
{
m_X -= CV.GetX();
m_Y -= CV.GetY();
CalcMag();
CalcDir();
}

// OPERATOR *= /////////////////////////////////////////////////////
void CVector::operator *=(const float &A)
{
m_X *= A;
m_Y *= A;
CalcMag();
CalcDir();
}

// OPERATOR /= /////////////////////////////////////////////////////
void CVector::operator /=(const float &A)
{
m_X /= A;
m_Y /= A;
CalcMag();
CalcDir();
}

// GLOBAL2RELATIVE /////////////////////////////////////////////////
CVector	CVector::Global2Relative(CVector Origin, float Angle)	// Global 2 relative vector converter		
{
return ((*this - Origin).Rotate(-Angle));
}

// RELATIVE2GLOBAL ////////////////////////////////////////////////
CVector CVector::Relative2Global(CVector Origin, float Angle)	// Relative 2 global vector converter
{
return ((Rotate(Angle) + Origin)); 
}
