/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member data and member function prototypes of the CActuators class.
 * 				The CVector class is responsible for all the vector manipulation operations.
 * Created: 6/4/98
 * Last modified: 8/7/99
 */

#ifndef _CVECTOR_H
#define _CVECTOR_H

#include <math.h>
#include <iostream.h>
#include "../Agent_Others/GlobalParams.H"

class CVector
{
// Private member data ///////////////////////////////////////////
private:

	float	m_X;	// X component
	float	m_Y;	// Y component
		
	float	m_Mag;	// Magnitude
	float	m_Dir;	// Direction

// Public member functions ////////////////////////////////////////
public:

// Constructor, Display and GetThis /////////////////////////////

	CVector();				// Default constrctor
	CVector(float, float);	// Constructor that takes X, Y
	~CVector(){};			// Destructor
	void Display() const;	// Displays all params.  Just for debugging
	const CVector* GetThis() const	// Function that returns the "this" pointer.
	{ return this; }	
	CVector* GetNonConstThis()		// Function that returns a non-const "this" pointer.
	{ return this; }

// Basic Utilities //////////////////////////////////////////////////
	float 	NormaliseAngle	(float ang)				// Normalise angle
	{ while(ang >  180.0) ang -= 360.0;
	  while(ang < -180.0) ang += 360.0;
	  return ang; }
	  	
// Basic actuator attribute get/set functions //////////////////////////

	void	SetX	(float X)			// Set X component
	{ m_X = X;}
	float	GetX	() const			// Get X component
	{ return m_X;}

	void	SetY	(float Y)			// Set Y component
	{ m_Y = Y;}
	float	GetY	() const			// Get Y component
	{ return m_Y;}

	void	SetMag	(float Mag)			// Set magnitude
	{ m_Mag = Mag;}
	float	GetMag	() const			// Get magnitude
	{ return m_Mag;}
	
	void	SetDir	(float Dir)			// Set direction
	{ m_Dir = Dir;}
	float	GetDir	() const			// Get direction
	{ return m_Dir;}

// Basic vector operations and utilities //////////////////////////////// 

	void	SetVector	(const CVector V)// Set vector
	{ *this = V; }
	CVector* GetVector	()				// Get vector
	{ return this; }	

	void	operator =	(const CVector&);	// Overloaded = operator
	CVector	operator +	(const CVector &CV)	// Overloaded + operator
	{ return CVector(m_X + CV.GetX(), m_Y + CV.GetY()); }
	CVector	operator -	(const CVector &CV)// Overloaded - operator
	{ return CVector(m_X - CV.GetX(), m_Y - CV.GetY()); }
	CVector	operator *	(const float &A)	// Overloaded * operator
	{ return CVector(m_X * A, m_Y * A); }
	CVector	operator /	(const float &A)	// Overloaded / operator
	{ return CVector(m_X / A, m_Y / A); }
	void 	operator +=	(const CVector&);	// Overloaded += operator
	void 	operator -=	(const CVector&);	// Overloaded -= operator
	void 	operator *=	(const float&);		// Overloaded *= operator
	void 	operator /=	(const float&);		// Overloaded /= operator

	void	CalcMag	()						// Calculate the magnitude of this vector
	{ m_Mag = sqrt((m_X*m_X)+(m_Y*m_Y)); }
	void	CalcDir	()						// Calculate the direction of this vector
	{ if ((m_X == 0.0) && (m_Y == 0)) m_Dir = 0.0; else m_Dir = (RAD2DEG*atan2(m_Y, m_X)); }
	
	CVector	Rotate	(float A)					// Rotate vector by angle
	{ return CVector(this->GetMag()* cos(DEG2RAD * (this->GetDir() + A)),
					this->GetMag() * sin(DEG2RAD * (this->GetDir() + A))); }

	CVector	Global2Relative	(CVector, float);	// Global 2 relative vector converter	
	CVector Relative2Global	(CVector, float);	// Relative 2 global vector converter
};

// POLAR2CVECTOR ////////////////////////////////////////////////////
inline CVector	Polar2CVector	(float Distance, float Direction)	// Return vector from polar co-ordinates
{
//cout << "P2CV- Dist, Dir: " << Distance << ", " << Direction << endl;
CVector tmp;
tmp.SetX(Distance*cos(DEG2RAD*Direction));
tmp.SetY(Distance*sin(DEG2RAD*Direction));
tmp.CalcMag();
tmp.CalcDir();
//cout << "P2CV- Vector is X, Y, Mag, Dir: " << tmp.GetX() << ", " << tmp.GetY() << ", " << tmp.GetMag() << ", " 
//	<< tmp.GetDir() << endl;
return tmp; 
}

// GLOBAL2RELATIVE2ME //////////////////////////////////////////////
inline CVector	Global2Relative2Me	(CVector CV, CVector GPos, float GFacing)		// Return vector that is global 2 relative 2 me
{
return CV.Global2Relative(GPos, GFacing);
}

#endif


