/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's advanced actions
 *				Advanced actions are the basic actions (turn, kick, etc.) that have
 * 				been modified to account for some other effects.  Eg. to turn we need
 *				to consider inertia etc.
 * Created: 18/4/99
 * Last modified: 22/7/99
 */

#include "../Agent_Classes/CBehaviours.H"

// TURN ////////////////////////////////////////////////////
int CBehaviours::Turn(float Moment)
{
float ActualMoment;			// The actual angle to turn
sensebody SB = GetSB();		// Used to get our current speed
/*
	Normally a turn can be between [-180, 180].  However there is
	concept of intertia that makes it more difficult to turn when
	you are moving.  The actual angle a player is turned is given
	by
	
	ActualAngle = Moment/(1.0 + InertiaMoment*PlayerSpeed)
	
	With this behaviour I am trying to turn by Moment irrespective
	of what the current speed is, i.e. the angle I should turn to
	turn by Moment is
	
	Moment * (1.0 + InertiaMoment * PlayerSpeed)
	
	Remember PlayerSpeed is the speed at the next cycle i.e.
	PlayerSpeed * PlayerDecay
*/

ActualMoment = Moment * (1.0 + m_InertiaMoment * SB.Speed);

//cout << GetTime() << " ***TURN request for: " << Moment << " degrees with predicted speed: " << SB.Speed << " received at: "
//	 << SB.Time << " resulted in turn by: " << ActualMoment << endl;

if (ActualMoment > m_MaxMoment)		// If result greater than maxmoment
	return BasicTurn(m_MaxMoment);	// turn by maxmoment
else if (ActualMoment < m_MinMoment)// else if less than minmoment
	return BasicTurn(m_MinMoment);	// turn by minmoment
else								// else
	return BasicTurn(ActualMoment);	// turn by ActualAngle
}
	
// KICK ////////////////////////////////////////////////////
int CBehaviours::Kick(float Distance, float Direction, float DistFactor)
{
//cout << GetTime() << " IN KICK" << endl;
if (!CanKickStraight(Distance, Direction))
	{
	return SpinKick(Distance, Direction, 0, DistFactor);
	}
else
	{
	return StraightKick(Distance, Direction, DistFactor);
	}
	
return 1;
}

/*
// TURNNECK //////////////////////////////////////////////////////////
int CBehaviours::TurnNeck(float Moment)
{
//cout << GetTime() << " In TurnNeck with Moment: " << Moment << endl;
float HeadResult;								// Corrected value if neckang exceeded
sensebody SB = GetSB();							// Used to get our head angle

// First check if we exceed the neck moment limits
if (Moment > m_MaxNeckMoment)					// If result greater than maxneckmoment
	Moment = m_MaxNeckMoment;					// adjust
else if (Moment < m_MinNeckMoment)				// else if less than minneckmoment
	Moment = m_MinNeckMoment;					// adjust

//cout << GetTime() << " After limit test Moment is: " << Moment << endl;

// Now check if we exceed the neck angle
HeadResult = SB.HeadAngle + Moment;				// Compute result

//cout << GetTime() << " HeadResult is: " << HeadResult << endl;

if (HeadResult > (m_MaxNeckAng-1))				// If result greater than maxneckang (only goes to 89)
	Moment = (m_MaxNeckAng-1) - SB.HeadAngle;	// adjust
else if (HeadResult < (m_MinNeckAng+1))			// else if less than minneckmoment (only goes to -89)
	Moment = (m_MinNeckAng+1) - SB.HeadAngle;	// adjust
	
//cout << GetTime() << " After limit test HeadResult is: " << HeadResult << " so we are turnecking by: " << Moment << endl;	
	
if (fabs(Moment) > m_Epsilon)					// if within limits and not near zero value
	return BasicTurnNeck(Moment);				// do the turn neck
}

// RESETNECK /////////////////////////////////////////////////////////
int CBehaviours::ResetNeck()					// Align neck with body
{
sensebody SB = GetSB();						// Get current SB
if (fabs(SB.HeadAngle) > m_Epsilon)			// If neck not already at 0
	{
	//cout << GetTime() << " RESETTING NECK from angle: " << SB.HeadAngle << endl;
	return TurnNeck(-SB.HeadAngle);			// Bring head back forward !
	}
}

// RESETBODY /////////////////////////////////////////////////////////
int CBehaviours::ResetBody()				// Align body with neck
{
sensebody SB = GetSB();						// Get current SB
float tmp = SB.HeadAngle;					// Need to store that as TurnNeck has to go before Turn
if (fabs(tmp) > m_Epsilon)					// If neck not already at 0
	{
	//cout << GetTime() << " RESETTING BODY from angle: " << tmp << endl;
	TurnNeck(-tmp);							// Bring head back forward !
	return Turn(tmp);						// And bring body to where head was
	}
}
*/
