/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's advanced behaviours
 * Created: 17/3/99
 * Last modified: 29/6/00
 *
 * MCH: Tidied up 29/06/00
 * Last modified: 24/7/99
 */

#include <iostream.h>
#include "../Agent_Classes/CPlayer.H"
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CGoal.H" 
#include "../Agent_Classes/CBehaviours.H"

// CONTROLBALL ///////////////////////////////////////////////////
int	CBehaviours::ControlBall()				// Stop a moving ball
{
CVector GPos = GetGPos();					// Store global position here
const CBall* pBall = M2BReadBall();			// Get pointer to ball data
float dist, dir;							// Distance and direction

dist = 0.0;									// We want the ball to travel
dir = 0.0;									// 0 meters at 0 degrees

if (!pBall->IsKickable())					// If lost control
	{
	return 0;								// return false
	}
else if (pBall->GetMag() < m_MaxStopSpeed)	// If ball moving too slow
	{
	return 1;								// do nothing and return true
	}
else										// else stop the ball
	{
	PlayerMovementCorrection(&dist, &dir);	// Correct for our movement
	Kick(dist, dir, 1.0);					// and do the kick
	return 1;
	}
	
return 1;									// We will never reach this point
}

// SHOOTTOGOAL //////////////////////////////////////////////////////////////
int CBehaviours::ShootToGoal(int Mode)
{
/*
	Mode 1 indicates a shot that avoids opponents
	Mode 2 justs shoots the ball towards the middle of the goal
*/

const CBall *pBall = M2BReadBall();		// Get pointer to ball data
float gdir=0;							// Goal direction used by Kick
const CGoal *pGoal;						// Get pointer to enemy goal here
CVector HighPost(52.5, 7.01);			// Used by ShootAngle	(READ FROM PARAMS IN THE FUTURE)
CVector LowPost(52.5, -7.01);			// Used by ShootAngle
CVector GPos = GetGPos();				// Store global position here
const float GBFacing = GetBFacing();	// Store global body facing direction here
const CPlayer* pPlayerClose = M2BReadPlayer(0, 'u');			// Get pointer to close player
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
float mindist = 999;					// Minimum distance to enemy player

if (GetSide() == 'l')					// If my side is left
	pGoal = M2BReadGoal('r');			// then the enemy goal is the right one
else									// otherwise
	pGoal = M2BReadGoal('l');			// it is the left one

gdir = pGoal->GetRDir();				// Get relative direction to the enemy goal

switch (Mode)
	{
	case 1:								// Try and avoid opponents that block the shot
		HighPost -= GPos;
		LowPost -= GPos;
		gdir = ShootAngle(20, 2, HighPost.GetDir(), LowPost.GetDir());
		gdir = NormaliseAngle(gdir - GBFacing);
		break;

	case 2:								// Quick shot towards middle of enemy goal
		break;
	
	case 3:								// Slow powerfull shot
		HighPost -= GPos;
		LowPost -= GPos;
		gdir = ShootAngle(20, 2, HighPost.GetDir(), LowPost.GetDir());
		gdir = NormaliseAngle(gdir - GBFacing);
		
		if (pPlayerClose->GetConf() > (m_MobConfDrop - m_Epsilon))	// If we have a tail
			{
			mindist = 0;			// Set min dist to 0
			}
		else
			{
			for (int i=0; i<11; i++)
				{
				if (pCloseFoe[i]->GetConf() > (m_MobConfDrop - m_Epsilon))	// if we have a tail
					{
					if (pCloseFoe[i]->GetRMag() < mindist)					// If closest so far
						mindist = pCloseFoe[i]->GetRMag();					// update mindist
					}
				}
			}
			
		if (mindist > (m_MinOppWorryDist - 1))
			{
			SpinKick(0, 180, 0, m_KickFactor);
			SpinKick(0, 180, 0, m_KickFactor);
			SpinKick(0.4, gdir, 0, 1.0);		// Make one shot (might make another one)
			}
		break;
	default:
		cerr << "WARNING: CBehaviours: ShootToGoal: What mode is that?" << endl;
		break;
	}

Kick(999, gdir, 1.0);				// Make the shot (with a HUGE end-distance (999) just to get the most powerful kick)
Kick(999, gdir, 1.0);				// Double kicks for cycle loss
return 1;							// and return true
}

// HAVEPOSSESSION /////////////////////////////////////////////////////////
int CBehaviours::HavePossession()			// Determines which team has possession of the ball
{											// Returns 1: Ours, 0: None, -1: Enemy
const CBall *pBall = M2BReadBall();								// Get pointer to ball data
const CPlayer*const* pCloseFriend = M2BReadPlayerArray(0, 1);	// Get pointer to close friendly player data
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
const CPlayer*const* pFarFriend = M2BReadPlayerArray(1, 1);		// Get pointer to close friendly player data
const CPlayer*const* pFarFoe = M2BReadPlayerArray(1, 0);		// Get pointer to close enemy player data
int i;															// Loop counter
CVector tmp;													// Temp vector

if (pBall->GetConf() > m_MobConfThr)			// If ball remembred
	{	
	for (i=0; i<11; i++)					// For all enemy players
		{
		// For close foes
		if (pCloseFoe[i]->GetConf() > m_MobConfThr)			// If remembered
			{
			tmp = pCloseFoe[i]->GetGPos() - pBall->GetGPos();// Calculate vector difference
		
			if (tmp.GetMag() < (2*m_Kickable))			// If in possesion
				return (-1);							// return enemy team
			}
		
		// For far foes
		if (pFarFoe[i]->GetConf() > m_MobConfThr)				// If remembered
			{
			tmp = pFarFoe[i]->GetGPos() - pBall->GetGPos();	// Calculate vector difference
		
			if (tmp.GetMag() < (2*m_Kickable))			// If in possesion
				return (-1);							// return enemy team
			}
		}
	
	for (i=0; i<11; i++)					// For all friends, find the closest to the ball
		{
		// For close friends
		if (pCloseFriend[i]->GetConf() > m_MobConfThr)		// If remembered
			{
			tmp = pCloseFriend[i]->GetGPos() - pBall->GetGPos();	// Calculate vector difference
		
			if (tmp.GetMag() < (1.5*m_Kickable))		// If in possesion
				return 1;								// return our team
			}
		
		// For far friends
		if (pFarFriend[i]->GetConf() > m_MobConfThr)			// If remembered
			{
			tmp = pFarFriend[i]->GetGPos() - pBall->GetGPos();	// Calculate vector difference
		
			if (tmp.GetMag() < (1.5*m_Kickable))		// If in possesion
				return 1;								// return our team
			}
		}
	}
		
return 0;									// If we reach this, cannot determine
}

// CLOSESTTOBALL ///////////////////////////////////////////////////////
/*
	Determines who is closer to the ball (i.e. me, teammate or enemy)
	(The closest player is the one who can intercept quickest)
	Returns 1 if me, 0 if a teammate, -1 if an enemy
	and -2 if enemy is closer than closer friend
*/
int CBehaviours::ClosestToBall(int Min)	
{
int Count=0;					// How many found so far
int FriendCycles=0;				// Teammate interception time
int Result = 1;					// Result is 1 if me, 0 if a teammate, -1 if an enemy,
								// and -2 if enemy is closer than closer friend
int Cycles = 999, temp;			// Min number of cycles to intercept the ball, and temp result
CPlayer BestFriend;				// Used to store the best candidates found
const CPlayer*const* pCloseFriend = M2BReadPlayerArray(0, 1);	// Get pointer to close friendly player data
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
const CPlayer*const* pFarFriend = M2BReadPlayerArray(1, 1);		// Get pointer to far friendly player data
const CPlayer*const* pFarFoe = M2BReadPlayerArray(1, 0);		// Get pointer to far enemy player data
const CBall* pBall = M2BReadBall();		// Get pointer to ball
CVector InterceptPoint;			// Point where interception is feasible

Cycles = CyclesToIntercept(m_Normal, MAXMODELSTEPS, InterceptPoint);		// First calculate my interception time
//cout << GetTime() << " For me, cycles to intercept: " << Cycles << endl;
if (Cycles < 0)					// It means we can't get there anyway
	return 0;					// A teammate should be closer ! So return 0

for (int i=0; i<11; i++)		// For all teammates
	{
	// For close friends
	if ((pCloseFriend[i]->GetConf() > m_MobConfThr) && (pCloseFriend[i]->GetUNum() != 1))	// If remembered and NOT the goalie
		{
		temp = CyclesToIntercept(m_Normal, Cycles, 0, pCloseFriend[i]);	// Then calculate player's interception time
		if ((temp >= 0) && (temp < Cycles))								// If he is faster than me
			{
			Count++;													// Increase count
			FriendCycles = Min(temp, FriendCycles);						// Update friend cycle
			//cout << GetTime() << " Found better close friend, cycles to intercept: " << Cycles << endl;
			BestFriend = *pCloseFriend[i];								// Update best friend
			if (Count >= Min)											// If we reach Min required
				Result = 0;												// Set result to 0
			}
		}
		
	// For far friends
	if (pFarFriend[i]->GetConf() > m_MobConfThr)						// If remembered
		{
		temp = CyclesToIntercept(m_Normal, Cycles, 0, pFarFriend[i]);		// Then calculate player's interception time
		if ((temp >= 0) && (temp < Cycles))								// If he is faster than me
			{
			Count++;													// Increase count
			FriendCycles = Min(temp, FriendCycles);						// Update friend cycle
			//cout << GetTime() << " Found better far friend, cycles to intercept: " << Cycles << endl;
			BestFriend = *pFarFriend[i];								// Update best friend
			if (Count >= Min)											// If we reach Min required
				Result = 0;												// Set result to 0
			}
		}
	}

Cycles = Min(Cycles, FriendCycles);		// To compare against enemies, store minimum value

for (int j=0; j<11; j++)		// For all opponents
	{
	// For close foes
	if (pCloseFoe[j]->GetConf() > m_MobConfThr)							// If remembered
		{
		temp = CyclesToIntercept(m_Normal, Cycles, 0, pCloseFoe[j]);		// Then calculate player's interception time
		if ((temp >= 0) && (temp < Cycles))								// If he is faster than me
			{
			//cout << GetTime() << " Found better close foe, cycles to intercept: " << temp << endl;
			if (Result == 0)											// If teammate already closer
				return -2;												// return enemy is closer than closer friend
			else														// else
				return -1;												// return enemy is closer
			}
		}
		
	// For far foes
	if (pFarFoe[j]->GetConf() > m_MobConfThr)								// If remembered
		{
		temp = CyclesToIntercept(m_Normal, Cycles, 0, pFarFoe[j]);		// Then calculate player's interception time
		if ((temp >= 0) && (temp < Cycles))								// If he is faster than me
			{
			//cout << GetTime() << " Found better far foe, cycles to intercept: " << temp << endl;
			if (Result == 0)											// If teammate already closer
				return -2;												// return enemy is closer than closer friend
			else														// else
				return -1;												// return enemy is closer
			}
		}
	}

return Result;					// Return the result
}

// ISSHOT ////////////////////////////////////////////////////////////////
CVector CBehaviours::IsShot()						// Determines if there is an incoming shot and returns
{													// last point inbounds if it is else mag = -1
const CBall* pBall = M2BReadBall();					// Get pointer to ball data
CVector NewBallPos, NewBallVel;						// Used for calculations
CVector Result;										// This is where we keep the result

Result.SetMag(-1);									// Initialise to error

NewBallPos = pBall->GetGPos();						// Store current ball position
NewBallVel = pBall->GetVelocity();					// and velocity

for (int i=0; i<100; i++)							// Check for a number of steps
	{
	NewBallPos += NewBallVel;						// Update position due to velocity
	NewBallVel *= m_BallDecay;						// Decay velocity
	
	if ((NewBallPos.GetX() < -51) && (fabs(NewBallPos.GetY()) < 8)) // If ball going into the goal
		{
		//cout << GetTime() << " IS A SHOT... Vector is:";
		//NewBallPos.Display();
		
		return NewBallPos;							// Return this vector
		}
	}

return Result;										// If we reach this point, return error
}

// CLEARUSEFULNESS ///////////////////////////////////////////////////////
// Developed by Matthew Hunter ///////////////////////////////////////////
float CBehaviours::ClearUsefulness(float Angle)
{  
/* Clearing usefulness function, defined in (Stone 98) to be :
 *
 * usefulness(A) = (sin((3/2*A)+(PI/4))+1)/2
 *
 * for A in radians.
 *
 * A is the angle between the X axis and the direction being tested.
 * (theta in (Stone 98)).
 */
CVector GPos = GetGPos();								// Store global position here  
float testangle;										// The angle we are testing

if (GPos.GetY() >=0)
	testangle = ((3.0/2.0)*Angle)+(PI/4.0);
else
	testangle = ((3.0/2.0)*(-Angle))+(PI/4.0);
	
return ((sin(testangle)+1.0)/2.0);
}

// SENDUSEFULNESS ////////////////////////////////////////////////////////
// Developed by Matthew Hunter ///////////////////////////////////////////
float CBehaviours::SendUsefulness(float Angle)
{
/* Sending usefulness function, described in (Stone 98) :
 *
 * "...the usefulness function is linear, with slope determined by
 *  the agent's y position on the field. The closer the agent is to
 *  the sideline, the steeper the slope, and the more that it
 *  favors sending to the middle of the field."
 *
 */
CVector GPos = GetGPos();				// Store global position here
return -(GPos.GetY()*Angle);
}

// CENTERUSEFULNESS /////////////////////////////////////////////////////// 
// Developed by Matthew Hunter /////////////////////////////////////////// 
float CBehaviours::CentreUsefulness(float Angle,float Target,float Tightness)
{
/* Centering usefulness function:
 *
 * This works on the assumption that the player can make a guess as to the
 * general direction to clear/send the ball which should then be modified.
 * This differs from the clear and send usefulness which always prefer the
 * same direction.
 *
 * usefulness(A) = cos((Target-A)*tightness)
 *
 * for A in radians. Target is the preferred direction. Tightness is a factor
 * of how large an influence the difference in angle should have.
 *
 * e.g. for Tightness = 0, usefulness=1 for all A
 *
 * A is the angle between the X axis and the direction being tested.
 * (theta in (Stone 98)).
 */
float testangle = (Target-Angle)*Tightness;
float result = 0.0;
if (fabs(testangle)<(PI/2.0))
	{
	result=cos(testangle);
	}
	
return result;
}

// ESInfluence /////////////////////////////////////////////////////
// Developed by Matthew Hunter /////////////////////////////////////
float CBehaviours::ESInfluence(float A,float p, float q, float ta, float to)
{
  /* A straight implementation based on the description in Stone 98.
   * The large messy if checks that the player being tested is infront of
   * the player rather than behind, i.e. there is no need to avoid opponents
   * that are on the opposite side of the player from the angle being tested.
   */

float tb;			// tangent of angle B
float e=1.0;		// expected success of an opponent

if	(
		((A>45.0) && (A<=135.0) && (p>0)) || // North quadrant
		((A<-45.0) && (A>=-135.0) && (p<0)) || // South quadrant
		((A<=45.0) && (A>=-45.0) && (q>0)) || // East quadrant
		((A>135.0) && (A<-135.0) && (q<0)) // West quadrant
	)
	{
	tb = p/q;
	e = (ta - tb)/(1+(ta*tb));
	e = fabs(e);
	if (e>to)
		{
		// if the opponent is outside of the triangle
		// ignore it.
		 e=1.0;
		}
	}
return e;
}

// EXPECTEDSUCCESS /////////////////////////////////////////////////////
// Developed by Matthew Hunter ///////////////////////////////////////////
float CBehaviours::ExpectedSuccess(float A,float O)
{
/* Expected success function (Stone 98)
 *
 * "The expected degree of success is evalueated by looking at an
 * isosceles triangle with one vertex where the ball is, and
 * congruent sides extending in the direction of the target being
 * evaluated. For each opponent in the triangle, its distance from
 * the center line of the triangle (w) is divided by the distance from
 * the player on that line (d)."
 *
 * Notes:
 *
 * a) The final expected success is actually |e| (e<0 for opponents
 *      on the other side of the line).
 *
 */

const CPlayer* pPlayerClose = M2BReadPlayer(0, 'u');                     // Get pointer to close player
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);               // Get pointer to close enemy player data
// Should I include far enemy player array here ???
CVector GPos = GetGPos();                                				// Store global position here

// Calculation variables
float p,q;      // offset between opponent and player
float ta;        // tangent of angle A
float to;        // tangent of angle (O/2);

// Expected success variables
float etotal; // composite expected success.

etotal = 1.0; // etotal is a product
ta = tan(A);  // A is constant for all opponents
to = tan(O/2.0);// Triangle limit

if (pPlayerClose->GetConf() > m_MobConfThr)
	{
	q = pPlayerClose->GetGX() - GPos.GetX();
	p = pPlayerClose->GetGY() - GPos.GetY();
	etotal=etotal*ESInfluence(A,p,q,ta,to);
	}
 
for(int i=0; i<11; i++)
	{
	if (pCloseFoe[i]->GetConf() > m_MobConfThr)
		{
		q = pCloseFoe[i]->GetGX() - GPos.GetX();
		p = pCloseFoe[i]->GetGY() - GPos.GetY();
		etotal=etotal*ESInfluence(A,p,q,ta,to);
		}
	}
	
return etotal;
}

// SHOOTANGLE ///////////////////////////////////////////////////// 
// Developed by Matthew Hunter //////////////////////////////////// 
float CBehaviours::ShootAngle(float O,float accuracy, float HighPost,float LowPost)
{
/* ShootAngle
 *
 * Outer loop, tests angles and returns the best clearing angle
 *
 * NB angles are in radians.
 *
 * O is the angle between the congruent sides of the isosceles
 * triangle used in the expected success function.
 * it allows clearing sending to be different if desired.
 * Defaults to 30 degrees if O is 0
 *
 * accuracy is the step between angles that are tested
 * Defaults to 2 degrees if accuracy is 0
 *
 * High Post is the angle to the post with the highest y value
 * Low Post is the angle to the post with the lowest y value
 */

float testangle; // The angle to be tested
float testvalue; // the usefulness*expectedsuccess of the rest angle
float bestangle; // the best angle so far
float bestvalue; // the usefulness*expectedsuccess of the best angle

float targetAngle; // the prefered angle

O = Min(O, 89.0);			// O SHOULD NOT BE GREATER THAN 89
O = Max(O, 5.0);			// O SHOULD NOT BE LESS THAN 0

O = DEG2RAD * O;
accuracy = DEG2RAD * accuracy;
HighPost = DEG2RAD * HighPost;	// Should be Direction of (PostGlobal - MyGlobal)
LowPost = DEG2RAD * LowPost;	// Should be Direction of (PostGlobal - MyGlobal)

if (O<=0)
	{
	O = PI/3.0; // 30 degrees.
	}
if (accuracy<=0)
	{
	accuracy=PI/90.0;
	}

bestangle=0; // towards opponents end
bestvalue=0; // No testvalue should ever be < 0

// Prefer to send towards the closest post, but not _at_ the
// closest post, better to make sure it goes in the goal.
// This stuff sets the prefered direction towards the
// middle of the closest half of the goal.
targetAngle = (LowPost+HighPost)/2.0; // middle of goal.
if(fabs(HighPost-LowPost)>(DEG2RAD*10.0)) // If shallow angle on goal, aim for centre and hope
	{
	if (fabs(LowPost)<fabs(HighPost))
		{
		targetAngle=(targetAngle+(2*LowPost))/3.0;
		}
	else
		{
		targetAngle=(targetAngle+(2*HighPost))/3.0;
		}

	//cout << "Highpost = " << RAD2DEG * HighPost << ", LowPost = " << RAD2DEG * LowPost << "." << endl;

	for(testangle=LowPost;testangle<=HighPost;testangle+=accuracy)
		{
		// Find out how good the angle is
		testvalue = CentreUsefulness(testangle,targetAngle,0.7) * ExpectedSuccess(testangle,O);
		/*
			cout << GetTime() << " " << RAD2DEG*testangle << " : ";
			cout << "Use = " << CentreUsefulness(testangle,targetAngle,0.7) << ", ";    
			cout << "ES = " << ExpectedSuccess(testangle,O) << ", ";
			cout << "Result = " << testvalue << "." << endl;
		*/
		// Compare it with the best found so far.
		if (testvalue>bestvalue)
			{
			bestangle=testangle;
			bestvalue=testvalue;
			}
		}
	}
else
	{
	//cout << "Shallow angle, shoot and hope at " << targetAngle << "." << endl;
	bestangle=targetAngle;
	}
	
return (RAD2DEG * bestangle);
} 
 
// PASSANGLE ///////////////////////////////////////////////////// 
// Developed by Matthew Hunter /////////////////////////////////////////// 
float CBehaviours::PassAngle(float O,float accuracy, float direction,float deviation) 
{
/* PassAngle
 *
 * Outer loop, tests angles and returns the best passing angle
 *
 * NB angles are in radians.
 *
 * O is the angle between the congruent sides of the isosceles
 * triangle used in the expected success function.
 * it allows clearing sending to be different if desired.
 * Defaults to 30 degrees if O is 0
 *
 * accuracy is the step between angles that are tested
 * Defaults to 2 degrees if accuracy is 0
 *
 * direction is the intended direction for the pass
 * deviation is the maximum allowed deviation away from the intended
 * direction.
 */

float testangle; // The angle to be tested
float testvalue; // the usefulness*expectedsuccess of the rest angle
float bestangle; // the best angle so far
float bestvalue; // the usefulness*expectedsuccess of the best angle

float tightness;

O = Min(O, 89.0);			// O SHOULD NOT BE GREATER THAN 89
O = Max(O, 5.0);			// O SHOULD NOT BE LESS THAN 0

O = DEG2RAD * O;
accuracy = DEG2RAD * accuracy;
direction = DEG2RAD * direction;
deviation = DEG2RAD * deviation;

if (O<=0)
	{
	O = PI/3.0; // 30 degrees.
	}
if (accuracy<=0)
	{
	accuracy=PI/90.0;
	}

bestangle=0; // towards opponents end
bestvalue=0; // No testvalue should ever be < 0

tightness=PI/deviation;

for(testangle=direction-deviation;testangle<=direction+deviation; testangle+=accuracy)
	{
	// Find out how good the angle is
	testvalue = CentreUsefulness(testangle,direction,tightness) * ExpectedSuccess(testangle,O);
	
	// Compare it with the best found so far.
	if (testvalue>bestvalue)
		{
		bestangle=testangle;
		bestvalue=testvalue; 
		}
	}

return (RAD2DEG * bestangle);
}

// CLEARINGANGLE /////////////////////////////////////////////////////
// Developed by Matthew Hunter ///////////////////////////////////////////

float CBehaviours::ClearingAngle(float O,float accuracy)
{
/* clearingAngle
 *
 * Outer loop, tests angles and returns the best clearing angle
 *
 * NB angles are in radians.
 *
 * O is the angle between the congruent sides of the isosceles
 * triangle used in the expected success function.
 * it allows clearing sending to be different if desired.
 * Defaults to 30 degrees if O is 0
 *
 * accuracy is the step between angles that are tested
 * Defaults to 2 degrees if accuracy is 0
 */

float testangle; // The angle to be tested
float testvalue; // the usefulness*expectedsuccess of the rest angle
float bestangle; // the best angle so far
float bestvalue; // the usefulness*expectedsuccess of the best angle
CVector GPos = GetGPos();					// Store global position here 
CVector tmp;

tmp = CVector(Min(20, (52.5 - GPos.GetX())), 28.0 - GPos.GetY());		// Gives me point to aim at
											// The highest the X value, the less steep sending slope will be
float targetangle = DEG2RAD * tmp.GetDir();	// Direction to relative point to aim at

O = Min(O, 89.0);			// O SHOULD NOT BE GREATER THAN 89
O = Max(O, 5.0);			// O SHOULD NOT BE LESS THAN 0

O = DEG2RAD * O;
accuracy = DEG2RAD * accuracy;

if (O<=0)
	{
	O = PI/3.0; 		// 30 degrees.
	}
if (accuracy<=0)
	{
	accuracy=PI/90.0;
	}

bestangle = 0; 			// towards opponents end

/*
	For sending it is possible to generate a negative usefulness
	value (for angles away from the centre). While these are very
	bad (out of bounds etc.) if a player was close to the centre
	of the pitch and the middle of the pitch was blocked by opponents
	sending it out is still better than kicking it to the opponent.
	PI/2 = 90degrees = straight up or down depending on which side of
	the pitch the player is on. 40 is greater than the biggest magnitude
	of y.
*/

bestvalue=-(PI/2.0)*40;

for(testangle=-PI/2.0;testangle<=PI/2.0;testangle+=accuracy)
	{
	// Find out how good the angle is
	// 1.0 is tightness value.  Higher the value, more significant targetangle is
    testvalue = CentreUsefulness(testangle, targetangle, 1.0)*ExpectedSuccess(testangle,O);

	// Compare it with the best found so far.
	if (testvalue>bestvalue)
		{
		bestangle=testangle;
		bestvalue=testvalue;
		}
	}

return (RAD2DEG * bestangle);
}


// SENDINGANGLE /////////////////////////////////////////////////////
// Developed by Matthew Hunter ///////////////////////////////////////////
float CBehaviours::SendingAngle(float O,float accuracy)
{
/* sendingAngle
 *
 * Outer loop, tests angles and returns the best sending angle
 *
 * NB angles are in radians.
 *
 * O is the angle between the congruent sides of the isosceles
 * triangle used in the expected success function.
 * it allows clearing sending to be different if desired.
 * Defaults to 30 degrees if O is 0
 *
 * accuracy is the step between angles that are tested
 * Defaults to 2 degrees if accuracy is 0
 */

float testangle; // The angle to be tested
float testvalue; // the usefulness*expectedsuccess of the rest angle
float bestangle; // the best angle so far
float bestvalue; // the usefulness*expectedsuccess of the best angle
CVector GPos = GetGPos();					// Store global position here 
CVector tmp(Min(20, (52.5 - GPos.GetX())), -GPos.GetY());				// Gives me point to aim at
											// The highest the X value, the less steep sending slope will be
float targetangle = DEG2RAD * tmp.GetDir();	// Direction to relative point to aim at

O = Min(O, 89.0);			// O SHOULD NOT BE GREATER THAN 89
O = Max(O, 5.0);			// O SHOULD NOT BE LESS THAN 0

O = DEG2RAD * O;
accuracy = DEG2RAD * accuracy;

if (O<=0)
	{
	O = PI/3.0; 		// 30 degrees.
	}
if (accuracy<=0)
	{
	accuracy=PI/90.0;
	}

bestangle = 0; 			// towards opponents end

/*
	For sending it is possible to generate a negative usefulness
	value (for angles away from the centre). While these are very
	bad (out of bounds etc.) if a player was close to the centre
	of the pitch and the middle of the pitch was blocked by opponents
	sending it out is still better than kicking it to the opponent.
	PI/2 = 90degrees = straight up or down depending on which side of
	the pitch the player is on. 40 is greater than the biggest magnitude
	of y.
*/

bestvalue=-(PI/2.0)*40;

for(testangle=-PI/2.0;testangle<=PI/2.0;testangle+=accuracy)
	{
	// Find out how good the angle is
	// 1.0 is tightness value.  Higher the value, more significant targetangle is
    testvalue = CentreUsefulness(testangle, targetangle, 1.0)*ExpectedSuccess(testangle,O);

	// Compare it with the best found so far.
	if (testvalue>bestvalue)
		{
		bestangle=testangle;
		bestvalue=testvalue;
		}
	}

return (RAD2DEG * bestangle);
}

// CLEARBALL ////////////////////////////////////////////////////////////////
int	CBehaviours::ClearBall(float EndDist)						// Defensive clearing of the ball
{
const float GBFacing = GetBFacing();	// Store global body facing direction here
float Dir = ClearingAngle(40, 5);
//cout << GetTime() << " Player: " << GetUNum() << " Clearing global dir is: " << Dir;
Dir = NormaliseAngle(Dir - GBFacing);
//cout << " and with facing " << GBFacing << " goes to kick with direction " << Dir << endl;
Kick(EndDist, Dir, 1.0);	// Double kicks for cycle loss
return Kick(EndDist, Dir, 1.0);
}

// SENDBALL /////////////////////////////////////////////////////////////////
int	CBehaviours::SendBall(float EndDist)							// Offensive sending of the ball
{
const float GBFacing = GetBFacing();	// Store global body facing direction here
float Dir = SendingAngle(40, 2);
//cout << GetTime() << " Player: " << GetUNum() << " Sending global dir is: " << Dir;
Dir = NormaliseAngle(Dir - GBFacing);
//cout << " and with facing " << GBFacing << " goes to kick with direction " << Dir << endl;
Kick(EndDist, Dir, 1.0);	// Double kicks for cycle loss
return Kick(EndDist, Dir, 1.0);
}

// CROSSBALL ////////////////////////////////////////////////////////////////
int CBehaviours::CrossBall()						// Make a cross towards the penalty spot
{
CVector GPos = GetGPos();				// Store global position here
const float GBFacing = GetBFacing();	// Store global body facing direction here
CVector CrossSpot = CVector(40,0);		// The point where we want to cross
CVector target = CrossSpot - GPos;		// Target vector
target.SetDir(NormaliseAngle(target.GetDir() - GBFacing));
Kick(Point2KickVelocity(CrossSpot, m_SoftPass), target.GetDir(), 1.0);	// Double kicks for cycle loss
return Kick(Point2KickVelocity(CrossSpot, m_SoftPass), target.GetDir(), 1.0);
}

