/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's basic actions
 *				Basic actions are all those actions that the server allows.
 *				Bye, Move, Turn, Dash, Kick, Say, ChangeView, Catch, SenseBody
 *
 *				Non-basic actions are those that are combinations of the basic ones.
 * Created: 13/1/99
 * Last modified: 16/7/99
 */

#include <stdio.h> 
#include "../Agent_Classes/CBehaviours.H"

// SAYBYE ///////////////////////////////////////////////
void CBehaviours::SayBye()
{
int i,j;									// Used in for loops
sprintf(m_ActionBuffer, "(bye)");			// Put bye command in m_ActionBuffer

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
											
DoExit(0);									// Then exit normally
}

// MOVE ///////////////////////////////////////////////////
int CBehaviours::Move(int X, int Y)
{
sprintf(m_ActionBuffer, "(move %d %d)\n", X, Y);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
											
Wait4NewCycle();							// For exclusive actions we need to wait till the next
											// cycle.  This is not good since we can't send NON
											// exclusive actions BEHIND an exclusive one in the queue.
											// For now we just make sure to send any non-exclusive ones
											// BEFORE the exclusive ones.  In the future I'll change this.

return 1;
}

// BASICTURN //////////////////////////////////////////////////
int CBehaviours::BasicTurn(float Moment)
{
sprintf(m_ActionBuffer, "(turn %.3f)\n", Moment);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
											
Wait4NewCycle();							// For exclusive actions we need to wait till the next
											// cycle.  This is not good since we can't send NON
											// exclusive actions BEHIND an exclusive one in the queue.
											// For now we just make sure to send any non-exclusive ones
											// BEFORE the exclusive ones.  In the future I'll change this.
											
return 1;
}

// DASH //////////////////////////////////////////////////
int CBehaviours::Dash(float Power)
{
sprintf(m_ActionBuffer, "(dash %.3f)\n", Power);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.

Wait4NewCycle();							// For exclusive actions we need to wait till the next
											// cycle.  This is not good since we can't send NON
											// exclusive actions BEHIND an exclusive one in the queue.
											// For now we just make sure to send any non-exclusive ones
											// BEFORE the exclusive ones.  In the future I'll change this.
											
return 1;
}

// BASICKICK //////////////////////////////////////////////////
int CBehaviours::BasicKick(float Power, float Direction)
{
sprintf(m_ActionBuffer, "(kick %.3f %.3f)\n", Power, Direction);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.

Wait4NewCycle();							// For exclusive actions we need to wait till the next
											// cycle.  This is not good since we can't send NON
											// exclusive actions BEHIND an exclusive one in the queue.
											// For now we just make sure to send any non-exclusive ones
											// BEFORE the exclusive ones.  In the future I'll change this.
											
return 1;
}

// BASICTURNNECK /////////////////////////////////////////////////
int CBehaviours::BasicTurnNeck(float Angle)
{
sensebody SB = GetSB();						// Used to get our current head angle
sprintf(m_ActionBuffer, "(turn_neck %.3f)\n", Angle);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.

// If you want to know why this is done here, read the comments in Update.C
SB.HeadAngle += Angle;						// Add moment
SB.HeadAngle = NormaliseAngle(SB.HeadAngle);// normalise
SetSB(SB);									// and set SenseBody

return 1;
}

// SAY //////////////////////////////////////////////////////
int CBehaviours::Say(char* Msg)
{
sprintf(m_ActionBuffer, "(say %s)\n", Msg);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
											
return 1;
}

// CHANGEVIEW ///////////////////////////////////////////////
int CBehaviours::ChangeView(char* AngleWidth, char* Quality)
{
sensebody SB = GetSB();						// Used to get our current view mode params
sprintf(m_ActionBuffer, "(change_view %s %s)\n", AngleWidth, Quality);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.

// If you want to know why this is done here, read the comments for TurnNeck in Update.C
strncpy(SB.VMQuality, Quality, 10);			// Copy string
strncpy(SB.VMMode, AngleWidth, 10);			// Copy string
SetSB(SB);									// and set SenseBody

return 1;
}

// CATCH //////////////////////////////////////////////
int CBehaviours::Catch(float Direction)
{
sprintf(m_ActionBuffer, "(catch %.3f)\n", Direction);

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
										
Wait4NewCycle();							// For exclusive actions we need to wait till the next
											// cycle.  This is not good since we can't send NON
											// exclusive actions BEHIND an exclusive one in the queue.
											// For now we just make sure to send any non-exclusive ones
											// BEFORE the exclusive ones.  In the future I'll change this.
											
return 1;
}

// SENSEBODY /////////////////////////////////////////
int CBehaviours::SenseBody()
{
sprintf(m_ActionBuffer, "(sense_body)\n");

Lock();										// Lock Behaviours mutex
Changed();									// Notify mediator of the change
Unlock();									// Unlock Behaviours mutex

WaitToExec();								// Wait on a condition variable for the action to 
											// get executed so that
											// new actions do not overwrite it and it get's
											// lost.
											
return 1;
}
