/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's basic behaviours
 * Created: 8/3/99
 * Last modified: 24/7/99
 */

#include <sys/time.h>
#include <errno.h>
#include <iostream.h>
#include "Formations.H"
#include "../Agent_Classes/CPlayer.H"
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CGoal.H"
#include "../Agent_Classes/CBehaviours.H"

// INITIALMOVE /////////////////////////////////////////////////////////////////////////
void CBehaviours::InitialMove()		// Move player to formation (used before kick-off and after goals)
{
/*
	Although this has been modified so that it does not send continuous Move commands,
	this will not work at the beginning of a game when time stays at 0.  This is because
	localisation returns wrong results.  That happens because all flag times are set to 0,
	so all falgs are considered to be NEW even if they are not.  Hence the closest flag
	is picked at first, but then when the player Moves, if another flag is NOT closer
	than the one it remembers, it keeps using the old one.  Since this is not a major
	problem, I left it like this for now...
*/

int unum = GetUNum();		// Store player's unum
CVector GPos = GetGPos();	// Get global position
CVector temp, res;			// Stores the place where we are ment to move and the result of the difference
							// between where we are, and where we want to be

switch (m_FormationID)
	{
	case -1:				// If losing formation
		if (m_LosingFormation == 334)
			{
			// First do some customisations for kick_off
			if (unum == 6)				// If center mid, move him back a bit
				temp = CVector(-10, 7);
			else if (unum == 9)			// If left forward, move him back a bit
				temp = CVector(-10, -7);
			else if (unum == 10)		// If kick off taker
				temp = CVector(-1, 1);
			else						// else for rest of team
				{
				if (ThreeThreeFour[unum-1][0] >= 0)
					temp = CVector(-2, ThreeThreeFour[unum-1][1]);
				else
					temp = CVector(ThreeThreeFour[unum-1][0], ThreeThreeFour[unum-1][1]);
				}

			}
		else
			cerr << "WARNING: CBehaviours: InitialMove: What losing formation is that -" << m_LosingFormation << "- ?" << endl;
		break;
	case 0:					// If normal formation
		if (m_NormalFormation == 433)
			{
			// First do some customisations for kick_off
			if (unum == 10)				// If kick off taker
				temp = CVector(-1, 1);
			else if (unum == 7)			// If center mid, move him back a bit
				temp = CVector(-9, -5);
			else						// else for rest of team
				{
				if (FourThreeThree[unum-1][0] >= 0)
					temp = CVector(-2, FourThreeThree[unum-1][1]);
				else
					temp = CVector(FourThreeThree[unum-1][0], FourThreeThree[unum-1][1]);
				}
			}
		else
			cerr << "WARNING: CBehaviours: InitialMove: What normal formation is that -" << m_NormalFormation << "- ?" << endl;
		break;
	case 1:					// If winning formation
		if (m_WinningFormation == 532)
			{
			// First do some customisations for kick_off
			if (unum == 8)				// If center mid, move him back a bit
				temp = CVector(-10, -7);
			else if (unum == 10)		// If kick off taker
				temp = CVector(-1, 1);
			else if (unum == 11)		// If right forward, move him back a bit
				temp = CVector(-10, 7);
			else						// else for rest of team
				{
				if (FiveThreeTwo[unum-1][0] >= 0)
					temp = CVector(-2, FiveThreeTwo[unum-1][1]);
				else
					temp = CVector(FiveThreeTwo[unum-1][0], FiveThreeTwo[unum-1][1]);
				}
			}
		else
			cerr << "WARNING: CBehaviours: InitialMove: What winning formation is that -" << m_WinningFormation << "- ?" << endl;
		break;
	default:
		cerr << "WARNING: CBehaviours: InitialMove: What formation ID is that -" << m_FormationID << "- ?" << endl;
		break;
	}
	
res = GPos - temp;							// Compute difference
if (res.GetMag() > m_MinDistToPoint)		// If difference larger than wanted
	Move((int)rint(temp.GetX()), (int)rint(temp.GetY()));
	
}

// WAIT4NEWVISUAL //////////////////////////////////////////////////////////////////////
int CBehaviours::Wait4NewVisual()
{
struct timeval tv;
struct timespec ts;
int status;

Lock();								// Lock mutex

if (gettimeofday(&tv, NULL) < 0)	// If gettimeofday fails, display error
	cerr << "WARNING: CBehaviours: Wait4NewVisual: gettimeofday returned error..." << endl;	// and display warning
		
ts.tv_sec = tv.tv_sec;				// Convert and store the timeval to
ts.tv_nsec = tv.tv_usec * 1000;		// the timespec structure
	
ts.tv_nsec += 350000000;		// Increase by 350 msec (max time for visual data plus a bit)
//ts.tv_nsec += 300000000;			// Increase by max time for visual

if (m_IsNewVisual == 1)				// If predicate is true here
	{
	#ifdef DEBUG_PREDS
	if ((GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))
		cerr << "WARNING: CBehaviours: Wait4NewVisual: Skipping behaviours predicate at: " << GetTime() << endl;
	#endif	
	//usleep(20000);					// Wait for 20ms
	//m_IsNewVisual = 0;				// And force entry into the loop
	}
	
while (m_IsNewVisual == 0)										// Test Predicate
	{
	status = pthread_cond_timedwait(&m_Cond, &m_Mutex, &ts);	// Wait on condition variable
	if (status == ETIMEDOUT)		// If time out
		{
		#ifdef DEBUG_CONDV
		cerr << GetTime() << " WARNING: CBehaviours: Wait4NewVisual: Condition variable timed out..." << endl;
		#endif
		break;
		}
	}
Unlock();							// Unlock mutex

if (status == ETIMEDOUT)			// If the condition variable timed out
	return 0;						// return false
	
return 1;							// else return true
}

// WAITTOEXEC //////////////////////////////////////////////////////////////////////
void CBehaviours::WaitToExec()
{
struct timeval tv;
struct timespec ts;
int status;

ExecLock();							// Lock Exec mutex

if (gettimeofday(&tv, NULL) < 0)	// If gettimeofday fails, display error
	cerr << "ERROR: CBehaviours: WaitToExec: gettimeofday returned error..." << endl;
		
ts.tv_sec = tv.tv_sec;				// Convert and store the timeval to
ts.tv_nsec = tv.tv_usec * 1000;		// the timespec structure
	
ts.tv_nsec += 2 * m_RecvStep * 1000;	// Increase by twice server's polling time to make sure

#ifdef DEBUG_PREDS
if ((GetIsNewAction() == 0) && (GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))
	cerr << "WARNING: CBehaviours: WaitToExec: Skipping behaviours predicate at: " << GetTime() << endl;
#endif	
	
while (GetIsNewAction() == 1)		// Test Predicate
	{
	status = pthread_cond_timedwait(&m_ExecCond, &m_ExecMutex, &ts);	// Wait on exec condition variable
	if (status == ETIMEDOUT)		// If time out
		{
		#ifdef DEBUG_CONDV
		if (GetTime())
			cerr << GetTime() << " WARNING: CBehaviours: WaitToExec: Condition variable timed out..." << endl;
		#endif
		break;
		}
	}
ExecUnlock();						// Unlock Exec mutex
}

// WAIT4NEWCYCLE //////////////////////////////////////////////////////////////////////
void CBehaviours::Wait4NewCycle()
{
struct timeval tv;
struct timespec ts;
int status;

NewCycleLock();						// Lock NewCycle mutex

if (gettimeofday(&tv, NULL) < 0)	// If gettimeofday fails, display error
	cerr << "ERROR: CBehaviours: WaitToExec: gettimeofday returned error..." << endl;
		
ts.tv_sec = tv.tv_sec;				// Convert and store the timeval to
ts.tv_nsec = tv.tv_usec * 1000;		// the timespec structure
	
ts.tv_nsec += 2*m_SimStep;			// Increase by twice the sim step to make sure
									// Might change the timedwait to wait here

#ifdef DEBUG_PREDS
if ((m_IsNewCycle == 1) && (GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))
	cerr << "WARNING: CBehaviours: Wait4NewCycle: Skipping behaviours predicate at: " << GetTime() << endl;
#endif	
	
while (m_IsNewCycle == 0)			// Test Predicate
	{
	status = pthread_cond_timedwait(&m_NewCycleCond, &m_NewCycleMutex, &ts);	// Wait on NewCycle condition variable
	if (status == ETIMEDOUT)		// If time out
		{
		#ifdef DEBUG_CONDV
		if ((GetTime()!=0) && (GetTime()!=3000) && (GetTime()!=6000))
			cerr << GetTime() << " WARNING: CBehaviours: Wait4NewCycle: Condition variable timed out..." << endl;
		#endif
		break;
		}
	}
NewCycleUnlock();						// Unlock Exec mutex
}

// LOOKAROUND ////////////////////////////////////////////////////////////////////
void CBehaviours::LookAround()
{
sensebody SB = GetSB();				// Used to get our current field of view
int i;								// for loop counter

if (!strncmp(SB.VMMode, "wide", 4))	// If it is wide
	{
	for (i=0; i<2; i++)					// Do a complete circle (2*180=360)
		{
		Turn(180);						// Turn action
		while (!Wait4NewVisual()) {};	// Wait for visual data
		}
	}
else									// else (probably normal view)
	{
	for (i=0;i<4;i++)					// Do a complete circle (4*90=360)
		{
		Turn(90);						// Turn action
		while (!Wait4NewVisual()) {};	// Wait for visual data
		}
	}
}

// COLLISION /////////////////////////////////////////////////////////////////////
float CBehaviours::Collision()				// Return collision direction
{
sensebody SB = GetSB();											// Used to get our current field of view
const CPlayer* pPlayerClose = M2BReadPlayer(0, 'u');			// Get pointer to close player
const CPlayer*const* pCloseFriend = M2BReadPlayerArray(0, 1);	// Get pointer to close friend player data
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
int i;															// For loop counter


// The angle used here is sin^(-1)(m_PlayerSize/(2*m_PlayerSize)), i.e. sin^(-1)(1/2) = 30

if ((pPlayerClose->GetConf() > (m_MobConfDrop - m_Epsilon)) && 
(pPlayerClose->GetRMag() < (m_PlayerSize*2 + m_CollisionBuffer)) && (fabs(pPlayerClose->GetRDir() - SB.HeadAngle) < 40))
	return pPlayerClose->GetRDir();
	
for (i=0; i<11; i++)
	{
	if ((pCloseFriend[i]->GetConf() > (m_MobConfDrop - m_Epsilon)) && 
	(pCloseFriend[i]->GetRMag() < (m_PlayerSize*2 + m_CollisionBuffer)) && 
	(fabs(pCloseFriend[i]->GetRDir() - SB.HeadAngle) < 40))
		return pCloseFriend[i]->GetRDir();
		
	if ((pCloseFoe[i]->GetConf() > (m_MobConfDrop - m_Epsilon)) && 
	(pCloseFoe[i]->GetRMag() < (m_PlayerSize*2 + m_CollisionBuffer)) &&	(fabs(pCloseFoe[i]->GetRDir() - SB.HeadAngle) < 40))
		return pCloseFoe[i]->GetRDir();
	}
return -999;					// No collision
}

// MOVETOPOS /////////////////////////////////////////////////////////////////////
int CBehaviours::MoveToPos (float Power)		// Move to your predefined global position (depending on formation)
{
float coldir;			// Collision direction
CVector CV;				// Store position to move here
int unum = GetUNum();	// Store player's unum

Power = AdjustPower(Power);		// Adjust power depending on stamina and time left

coldir = Collision();			// Check for collisions

if (coldir != -999)				// If collision exists
	{
//	ResetNeck();				// Align neck
	//cout << GetTime()<< " ---Collision detected at: " << coldir << endl;
	//Dash(-30);				// Back off
	if (coldir > 0)				// If player on the right
		Turn(-90 + coldir);		// turn left
	else
		Turn(90 + coldir);		// else turn right
	Dash(35);					// And dash
	Dash(35);					// 
	return 0;
	}	

switch (m_FormationID)
	{
	case -1:				// If losing formation
		if (m_LosingFormation == 334)
			{
			CV = CVector(ThreeThreeFour[unum-1][0], ThreeThreeFour[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToPos: What losing formation is that -" << m_LosingFormation << "- ?" << endl;
		break;
	case 0:					// If normal formation
		if (m_NormalFormation == 433)
			{
			CV = CVector(FourThreeThree[unum-1][0], FourThreeThree[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToPos: What normal formation is that -" << m_NormalFormation << "- ?" << endl;
		break;
	case 1:					// If winning formation
		if (m_WinningFormation == 532)
			{
			CV = CVector(FiveThreeTwo[unum-1][0], FiveThreeTwo[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToPos: What winning formation is that -" << m_WinningFormation << "- ?" << endl;
		break;
	default:
		cerr << "WARNING: CBehaviours: MoveToPos: What formation ID is that -" << m_FormationID << "- ?" << endl;
		break;
	}
	
return MoveToPos(CV, Power);	// And then call MoveToPos(Point, Power)
}

// MOVETOSTOPCLEARANCE ///////////////////////////////////////////////////////////
int CBehaviours::MoveToStopClearance(float Power)			// Move to block opponents clearance
{
float coldir;			// Collision direction
CVector CV;				// Store position to move here
int unum = GetUNum();	// Store player's unum

Power = AdjustPower(Power);		// Adjust power depending on stamina and time left
	
coldir = Collision();			// Check for collisions

if (coldir != -999)				// If collision exists
	{
//	ResetNeck();				// Align neck
	//cout << GetTime()<< " ---Collision detected at: " << coldir << endl;
	//Dash(-30);				// Back off
	if (coldir > 0)				// If player on the right
		Turn(-90 + coldir);		// turn left
	else
		Turn(90 + coldir);		// else turn right
	Dash(35);					// And dash
	Dash(35);					// 
	return 0;
	}	
	
switch (m_FormationID)
	{
	case -1:				// If losing formation
		if (m_LosingFormation == 334)
			{
			CV = CVector(ThreeThreeFourSC[unum-1][0], ThreeThreeFourSC[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToStopClearance: What losing formation is that -" << m_LosingFormation << "- ?" << endl;
		break;
	case 0:					// If normal formation
		if (m_NormalFormation == 433)
			{
			CV = CVector(FourThreeThreeSC[unum-1][0], FourThreeThreeSC[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToStopClearance: What normal formation is that -" << m_NormalFormation << "- ?" << endl;
		break;
	case 1:					// If winning formation
		if (m_WinningFormation == 532)
			{
			CV = CVector(FiveThreeTwoSC[unum-1][0], FiveThreeTwoSC[unum-1][1]);
			}
		else
			cerr << "WARNING: CBehaviours: MoveToStopClearance: What winning formation is that -" << m_WinningFormation << "- ?" << endl;
		break;
	default:
		cerr << "WARNING: CBehaviours: MoveToStopClearance: What formation ID is that -" << m_FormationID << "- ?" << endl;
		break;
	}
	
return MoveToPos(CV, Power);	// And then call MoveToPos(Point, Power)
}

// MOVETOPOS /////////////////////////////////////////////////////////////////////
int CBehaviours::MoveToPos(CVector CV, float Power)		// Move to a specified position dashing with a specified power
{
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
sensebody SB = GetSB();				// Used to get our current field of view
CVector tmp, tmp2;					// Temp vectors used for calculations
float coldir;						// Collision direction
const float GBFacing = GetBFacing();// Store global body facing direction here
const float GFacing = GetFacing();	// Store global facing direction here
CVector GPos = GetGPos();			// Get global position	

tmp = CV - GPos;					// Calculate target vector		
tmp.SetDir(NormaliseAngle(tmp.GetDir() - GBFacing));// And modify direction to account for a global body facing direction

if (tmp.GetMag() < m_MinDistToPoint)		// If close to point
	{
//	ResetNeck();							// Align neck with body
	return 1;								// Return true
	}

//cout << GetTime() << " *** MoveToPos with GPos X, Y: " << GPos.GetX() << ", " << GPos.GetY() 
//<< " GFacing: " << GFacing << " and GBFacing: " << GetBFacing() << " to X, Y: " << CV.GetX() 
//<< ", " << CV.GetY() << " resulted in TURN by: " << tmp.GetDir() << " and GO by: " << tmp.GetMag() << endl;

// Here we need to set our field of view depending on where the ball is

if ((pBall->GetRMag() > 20) && (strncmp(SB.VMMode, "wide", 4)))		// If ball far, and not already in wide
	ChangeView("wide", "high");
else if ((pBall->GetRMag() < 20) && (strncmp(SB.VMMode, "normal", 6)))// If ball close, and not already in normal
	ChangeView("normal", "high");

/*
	While we are moving we want to make sure we maintain visual with the ball...
	However to avoid turn_necking twice, we check if we turn first, and then
	perform the turn_neck.  So here we only calculate the direction, but don't
	actually perform the action.
*/

if (pBall->GetConf() > m_MobConfThr)					// If ball remembered
	{
	tmp2 = BallPredPos() - PredPos();					// See where ball and I will be in the next cycle
	tmp2.SetDir(NormaliseAngle(tmp2.GetDir() - GFacing));	// And modify direction to account for a global facing direction
	}
else
	{
	tmp2.SetMag(-1);									// else set error code
	}

/*
	Here we need to check if we are facing the point.
	It is important to check that CV and GPos are not
	the same vector otherwise turning is meaningless and
	we will just waste a cycle.
*/

if ((fabs(tmp.GetDir()) > m_MaxGoAngle) && (tmp.GetMag() > m_Epsilon))
	{
	// If ball goes outside normal visual field
//	if ((tmp2.GetMag() > 0) && (fabs(NormaliseAngle(tmp2.GetDir() - tmp.GetDir())) > 45))
//		TurnNeck(NormaliseAngle(tmp2.GetDir() - tmp.GetDir()));		// Turn your neck so that you can face the ball
		
	Turn(tmp.GetDir());						// Turn to point
	return 0;								// and return 0
	}
else	
	{	
	coldir = Collision();			// Check for collisions

	if (coldir != -999)				// If collision exists
		{
//		ResetNeck();				// Align neck
		//cout << GetTime()<< " ---Collision detected at: " << coldir << endl;
		//Dash(-30);				// Back off
		if (coldir > 0)				// If player on the right
			Turn(-90 + coldir);		// turn left
		else
			Turn(90 + coldir);		// else turn right
		Dash(35);					// And dash
		Dash(35);					// 
		return 0;
		}
	
//	if ((tmp2.GetMag() > 0) && (fabs(tmp2.GetDir()) > 45))	// If ball goes outside normal visual field
//		TurnNeck(tmp2.GetDir());							// Turn your neck so that you can face the ball
	
	for (int i=0; i<m_MaxMoveDashes; i++)	// Otherwise perform a max number of dashes
		{
		Power = Min(Power, (tmp.GetMag()/m_DashPowerRate)+10);	// If too close to point, we don't want to overshoot
																// and we don't want to just miss it either!
		Power = Min(Power, 100.1);		// Also make sure we didn't exceed 100
		Dash(Power);					// Dash
		GPos = GetGPos();				// Get our new updated GPos
		tmp = CV - GPos;				// Calculate target vector		
		if (tmp.GetMag() < m_MinDistToPoint)		// If close to point
			{
//			ResetNeck();							// Align neck with body
			return 1;								// Return true
			}
		}
	}

if (tmp.GetMag() < m_MinDistToPoint)	// If close to point
	{
//	ResetNeck();						// Align neck with body
	return 1;							// Return true
	}

return 0;			// Return false
}

// MOVETOPOSBW /////////////////////////////////////////////////////////////////////
int CBehaviours::MoveToPosBW(CVector CV, float Power)	// Move backwards to a specified position dashing with a specified power
{
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
sensebody SB = GetSB();				// Used to get our current field of view
CVector tmp, tmp2;					// Temp vectors used for calculations
float coldir;						// Collision direction
const float GBFacing = GetBFacing();// Store global body facing direction here
const float GFacing = GetFacing();	// Store global facing direction here
CVector GPos = GetGPos();			// Get global position	

tmp = CV - GPos;					// Calculate target vector		
tmp.SetDir(NormaliseAngle(tmp.GetDir() - GBFacing + 180));// And modify direction to account for a global body facing direction

if (tmp.GetMag() < m_MinDistToPoint)		// If close to point
	{
//	ResetNeck();							// Align neck with body
	return 1;								// Return true
	}

//cout << GetTime() << " *** MoveToPos with GPos X, Y: " << GPos.GetX() << ", " << GPos.GetY() 
//<< " GFacing: " << GFacing << " and GBFacing: " << GetBFacing() << " to X, Y: " << CV.GetX() 
//<< ", " << CV.GetY() << " resulted in TURN by: " << tmp.GetDir() << " and GO by: " << tmp.GetMag() << endl;

// Here we need to set our field of view depending on where the ball is

if ((pBall->GetRMag() > 20) && (strncmp(SB.VMMode, "wide", 4)))		// If ball far, and not already in wide
	ChangeView("wide", "high");
else if ((pBall->GetRMag() < 20) && (strncmp(SB.VMMode, "normal", 6)))// If ball close, and not already in normal
	ChangeView("normal", "high");

/*
	While we are moving we want to make sure we maintain visual with the ball...
	However to avoid turn_necking twice, we check if we turn first, and then
	perform the turn_neck.  So here we only calculate the direction, but don't
	actually perform the action.
*/

if (pBall->GetConf() > m_MobConfThr)					// If ball remembered
	{
	tmp2 = BallPredPos() - PredPos();					// See where ball and I will be in the next cycle
	tmp2.SetDir(NormaliseAngle(tmp2.GetDir() - GFacing));	// And modify direction to account for a global facing direction
	}
else
	{
	tmp2.SetMag(-1);									// else set error code
	}

/*
	Here we need to check if we are facing the point.
	It is important to check that CV and GPos are not
	the same vector otherwise turning is meaningless and
	we will just waste a cycle.
*/

if ((fabs(tmp.GetDir()) > m_MaxGoAngle) && (tmp.GetMag() > m_Epsilon))
	{
	// If ball goes outside normal visual field
//	if ((tmp2.GetMag() > 0) && (fabs(NormaliseAngle(tmp2.GetDir() - tmp.GetDir())) > 45))
//		TurnNeck(NormaliseAngle(tmp2.GetDir() - tmp.GetDir()));		// Turn your neck so that you can face the ball
		
	Turn(tmp.GetDir());						// Turn to point
	return 0;								// and return 0
	}
else	
	{	
	
//	if ((tmp2.GetMag() > 0) && (fabs(tmp2.GetDir()) > 45))	// If ball goes outside normal visual field
//		TurnNeck(tmp2.GetDir());							// Turn your neck so that you can face the ball
	
	for (int i=0; i<m_MaxMoveDashes; i++)	// Otherwise perform a max number of dashes
		{
		Power = Max(Power, -((tmp.GetMag()/m_DashPowerRate)+10));// If too close to point, we don't want to overshoot
																// and we don't want to just miss it either!
		Power = Max(Power, -100.1);		// Also make sure we didn't exceed -100
		Dash(Power);					// Dash
		GPos = GetGPos();				// Get our new updated GPos
		tmp = CV - GPos;				// Calculate target vector		
		if (tmp.GetMag() < m_MinDistToPoint)		// If close to point
			{
//			ResetNeck();							// Align neck with body
			return 1;								// Return true
			}
		}
	}

if (tmp.GetMag() < m_MinDistToPoint)	// If close to point
	{
//	ResetNeck();						// Align neck with body
	return 1;							// Return true
	}

return 0;			// Return false
}

// LOOKINGATBALL ////////////////////////////////////////////////////
float CBehaviours::LookingAtBall(int TurnTo)				// Find ball and return its distance
{
CBall *pBall = M2BReadNonConstBall();	// Get pointer to ball data

if (pBall->GetConf() > m_MobConfThr)	// If confidence above threashold
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pBall->GetRDir()) > m_Epsilon)	// and we are not facing the ball
			Turn(pBall->GetRDir());				// Turn to face ball
		}
	return pBall->GetRMag();			// Return ball's distance
	}

else									// Else, turn towards last known position
	{
	//ResetNeck();						// Make sure we are looking forward
	if (pBall->GetRDir() > 0.0)			// If ball on the right
		{
		pBall->SetRDir(999);			// Make sure it stays on the right after we update for the turn
		Turn(80);						// otherwise we might end up turning 80, -80, 80, ... and so on
		}
	else
		{
		pBall->SetRDir(-999);			// same as above
		Turn(-80);
		}
	
	while (!Wait4NewVisual()){};		// and wait for new visual data
	}

if (pBall->GetConf() > ((m_MobConfDrop * m_MobConfDrop)- m_Epsilon))	// If ball found
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pBall->GetRDir()) > m_Epsilon)	// and we are not facing the ball
			Turn(pBall->GetRDir());				// Turn to face ball
		}
	return pBall->GetRMag();					// return it's distance
	}

return 999;										// otherwise ball not found so return 999
}

// LOOKINGATGOAL ////////////////////////////////////////////////////
float CBehaviours::LookingAtGoal(int TurnTo)	// Find enemy goal and return its distance	
{
CGoal *pGoal;							// Get pointer to enemy goal here

if (GetSide() == 'l')					// If my side is left
	pGoal = M2BReadNonConstGoal('r');	// then the enemy goal is the right one
else									// otherwise
	pGoal = M2BReadNonConstGoal('l');	// it is the left one

if (pGoal->GetConf() > m_StatConfThr)		// If confidence above threashold
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pGoal->GetRDir()) > m_Epsilon)	// and we are not facing the goal
			Turn(pGoal->GetRDir());				// Turn to face goal
		}
	return pGoal->GetRMag();			// Return goal's distance
	}
else									// Else, turn towards last known position
	{
	//ResetNeck();						// Make sure we are looking forward
	if (pGoal->GetRDir() > 0.0)			// If goal on the right
		{
		pGoal->SetRDir(999);			// Make sure it stays on the right after we update for the turn
		Turn(80);						// otherwise we might end up turning 80, -80, 80, ... and so on
		}
	else
		{
		pGoal->SetRDir(-999);			// same as above
		Turn(-80);
		}
	
	while (!Wait4NewVisual()){};		// and wait for new visual data
	}

if (pGoal->GetConf() > (m_StatConfDrop - m_Epsilon))// If goal found
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pGoal->GetRDir()) > m_Epsilon)	// and we are not facing the goal
			Turn(pGoal->GetRDir());				// Turn to face goal
		}
	return pGoal->GetRMag();					// return it's distance
	}

return 999;										// otherwise goal not found so return 999
}

// LOOKINGATMYGOAL /////////////////////////////////////////////////
float CBehaviours::LookingAtMyGoal(int TurnTo)	// Find my goal and return its distance
{
CGoal *pGoal;							// Get pointer to my goal here

if (GetSide() == 'l')					// If my side is left
	pGoal = M2BReadNonConstGoal('l');	// then my goal is the left one
else									// otherwise
	pGoal = M2BReadNonConstGoal('r');	// it is the right one

if (pGoal->GetConf() > m_StatConfThr)		// If confidence above threashold
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pGoal->GetRDir()) > m_Epsilon)	// and we are not facing the goal
			Turn(pGoal->GetRDir());				// Turn to face goal
		}
	return pGoal->GetRMag();			// Return goal's distance
	}

else									// Else, turn towards last known position
	{
	//ResetNeck();						// Make sure we are looking forward
	if (pGoal->GetRDir() > 0.0)
		{
		pGoal->SetRDir(999);			// Make sure it stays on the right after we update for the turn
		Turn(80);						// otherwise we might end up turning 80, -80, 80, ... and so on
		}
	else
		{
		pGoal->SetRDir(-999);			// same as above
		Turn(-80);
		}
		
	while (!Wait4NewVisual()){};		// and wait for new visual data
	}

if (pGoal->GetConf() > (m_StatConfDrop - m_Epsilon))// If goal found
	{
	if (TurnTo)							// If TurnTo is TRUE
		{
		if (fabs(pGoal->GetRDir()) > m_Epsilon)	// and we are not facing the goal
			Turn(pGoal->GetRDir());				// Turn to face goal
		}
	return pGoal->GetRMag();					// return it's distance
	}

return 999;										// otherwise goal not found so return 999
}

// ADJUSTPOWER ///////////////////////////////////////////////////////////////
float CBehaviours::AdjustPower(float Power)			// Adjust power of movement depending on stamina
{
sensebody SB = GetSB();				// Used to get our current stamina

// NEED TO PARAMETRISE THESE.  AND REC_DEC_THR IS ALREADY IN SERVER.CONF
// If in second half and over 1000 cycles left
if ((GetTime() >= 3000) && (GetTime() < 6000) && ((6000 - GetTime()) > 1000) && (SB.Stamina <= 1150))
	{
	Power = Min(Power, (SB.Stamina - 1050));				// Do not go below recover_dec_thr
	Power = Max(0.0, Power);								// But stay positive
	//cerr << GetTime() << ": Player " << GetUNum() << " reach rec_dec_thr. Adjusting power to: " << Power << endl;
	}
 // If in first half and over 1000 cycles left
else if ((GetTime() < 3000) && ((3000 - GetTime()) > 1000) && (SB.Stamina <= 1150))
	{
	Power = Min(Power, (SB.Stamina - 1050));				// Do not go below recover_dec_thr
	Power = Max(0.0, Power);								// But stay positive
	//cerr << GetTime() << ": Player " << GetUNum() << " reach rec_dec_thr. Adjusting power to: " << Power << endl;
	}
	
return Power;
}

// CALCPOW4GOALIE ///////////////////////////////////////////////////////////
float CBehaviours::CalcPow4Goalie()			// Calculate movement power for goalie
{
int PlayMode = P2BReadPlayMode(1);	// Get current play mode
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
CVector GPos = GetGPos();			// Get global position	
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)

if (((pBall->GetRMag() < 30) || (pBall->GetGX() < -25)) && (PlayMode == 20))	// If ball close
	Power = m_Fast;
	
return Power;							// and return value
}

// CALCPOW4DEF ///////////////////////////////////////////////////////////
float CBehaviours::CalcPow4Def()			// Calculate movement power for defenders
{
int PlayMode = P2BReadPlayMode(1);	// Get current play mode
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
CVector GPos = GetGPos();			// Get global position	
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)

// If ball in danger zone and behind me and game is on
if ((pBall->GetGX() < -25) && (pBall->GetGX() < GPos.GetX()) && (PlayMode == 20))
	Power = m_Fast;
else if (pBall->GetGX() < GPos.GetX() && (PlayMode == 20))	// If ball behind me and game on
	Power = m_Normal;					// m_Fast caused them to drain their energy very quickly
else if ((pBall->GetRMag() > 25) || (PlayMode != 20))		// If ball far away or not play on
	Power = m_Slow;						// take my time
	
return Power;							// and return value
}

// CALCPOW4MID ///////////////////////////////////////////////////////////
float CBehaviours::CalcPow4Mid()			// Calculate movement power for midfielders
{
int PlayMode = P2BReadPlayMode(1);	// Get current play mode
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
CVector GPos = GetGPos();			// Get global position	
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)

if ((pBall->GetRMag() > 40) || (PlayMode != 20))			// If ball far away or not play on
	Power = m_Slow;						// take my time
	
return Power;							// and return value
}

// CALCPOW4FW ///////////////////////////////////////////////////////////
float CBehaviours::CalcPow4Fw()				// Calculate movement power for forwards
{
int PlayMode = P2BReadPlayMode(1);	// Get current play mode
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
CVector GPos = GetGPos();			// Get global position	
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)

if ((pBall->GetGX() > GPos.GetX()) && (PlayMode == 20))	// If ball in front of me and play on
	Power = m_Fast;						// RUUUUUUN
// If ball far away and not pulling away from offside
else if ((GPos.GetX() - pBall->GetGX()) > 30)
	Power = m_Slow;						// take my time
	
return Power;							// and return value
}

// RECHARGE //////////////////////////////////////////////////////////////
void CBehaviours::Recharge()			// Check if we can afford to inrease our stamina
{
int Time = GetTime();					// Store current time
int PlayMode = P2BReadPlayMode(1);		// Get current play mode
int Event = PlayMode;					// Used to check if mode changed

if (m_ScoreDiff > 0)					// If we are winning, we can afford to recharge
	{
	/*
		OOPS:  It might be worth recharging even if we are loosing if the ball is far away.
		Anyway...this is a last minute hack and I'll improve it in the future...
		Recharging should be a function of both score and current stamina, NOT just score...
		The important thing to note is that the team should not stall if we are winning,
		BUT at the same time we don't want the player's to NOT have enough energy to
		defend their goal.  About 20 cycles should not be a problem...
	*/
	while ((PlayMode != 20) && ((PlayMode == Event)) && ((GetTime() - Time) < 20))
		{
		PlayMode = P2BReadPlayMode(1);		// Check if playmode has changed
		LookingAtBall(1);
		}
	}
}
