/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations for the Comm functions
 *				InitConnection, CloseConnection, SendMessage, ReceiveMessage
 * Created: 11/1/99
 * Last modified: 15/6/99
 */

#include <netdb.h> 
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include "../Agent_Classes/CActuators.H"
#include "../Agent_Classes/CSensors.H"

// INITCONNECTION /////////////////////////////////////
Socket CActuators::InitConnection(const char* host, int port) const
{
struct hostent      *host_ent;
struct in_addr      *addr_ptr;
struct sockaddr_in  cli_addr;
int                 sockfd, val;
Socket              sock;

sock.socketfd = -1;

if((host_ent = (struct hostent *)gethostbyname(host)) == NULL)
	{
	if(inet_addr(host) == -1)		// Check if a numeric address
		{
		return sock;
		}
	}
else
	{
	addr_ptr = (struct in_addr *) *host_ent->h_addr_list;
	host = inet_ntoa(*addr_ptr);
	}

// Open UDP socket ///////////////////////////////////////
if( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
	return sock;		// Can't open socket
	}

/*
	The commented code makes the socket non-blocking, i.e. returns
	EWOULDBLOCK if no datagram has arrived when recvfrom is called.
	In a multi-threaded approach this is NOT needed and in fact can
	cause serious problems. If the socket is set as blocking (default),
	the recvfrom call will block until data is available or an error
	occurs.  That means the Sensors thread will go to sleep.  When data
	arrives, it will wake up, process the data, call recvfrom and go to
	sleep again until new data arrives (if data hasn't already arrived).
	
	IF we uncomment the following code and make the socket non-blocking,
	the Sensors thread will never go to sleep.  It will keep polling and
	receiving EWOULDBLOCK. This can drain the resources and the other
	threads (especially the Actuators) might fail to meet the timing
	requirements.
	
	SO: DO NOT UNCOMMENT THE FOLLOWING CODE
	
	And this begs the question:  Why have you left the code there if you
	are not using it anyway???
	
	And my answer is: ?
*/
/*
val = fcntl(sockfd, F_GETFL, 0);
#if defined NewsOS || defined IRIX
	val |= O_NDELAY;
#else
	val |= O_NONBLOCK;
#endif
fcntl(sockfd, F_SETFL, val);
*/
// Bind any local address ///////////////////////////////
bzero((char *) &cli_addr, sizeof(cli_addr));
cli_addr.sin_family         = AF_INET;
cli_addr.sin_addr.s_addr    = htonl(INADDR_ANY);
cli_addr.sin_port           = htons(0);


if(bind(sockfd, (struct sockaddr *) &cli_addr, sizeof(cli_addr)) < 0)
	{
	return sock;		// Can't bind local address
	}

// Fill in the structure with the address of the server ////
sock.socketfd = sockfd;

bzero((char *) &sock.serv_addr, sizeof(sock.serv_addr));
sock.serv_addr.sin_family		= AF_INET;
sock.serv_addr.sin_addr.s_addr	= inet_addr(host);
sock.serv_addr.sin_port			= htons(port);

return sock;

}

// CLOSECONNECTION ////////////////////////////////////
void CActuators::CloseConnection() const
{
close(m_Socket.socketfd);
}

// SENDMESSAGE ////////////////////////////////////////
int CActuators::SendMessage(char* buf) const
{
int n;

n = strlen(buf);
//printf( "send %d : %s", ntohs(m_Socket.serv_addr.sin_port),buf);

if( sendto(m_Socket.socketfd, buf, n, 0, (struct sockaddr *)&m_Socket.serv_addr, sizeof(m_Socket.serv_addr)) != n)
	return (-1);
else
	{
	return 0;
	}
}

// RECEIVEMESSAGE /////////////////////////////////////
int CSensors::ReceiveMessage(char* buf, int size)
{
int n;
unsigned int servlen;
struct sockaddr_in  serv_addr;
Socket *Sock=&m_Socket;

servlen = sizeof(serv_addr);
n = recvfrom(Sock->socketfd, buf, size, 0, (struct sockaddr *)&serv_addr, &servlen);

if(n < 0)
	if( n == -1 && errno == EWOULDBLOCK)
		{
		buf[0] = '\0';
		return 0;
		}
	else
		return (-1);
else
	{
	//printf( "recv %d : %s", ntohs(serv_addr.sin_port), buf);
	Sock->serv_addr.sin_port = serv_addr.sin_port;
	buf[n] = '\0';

	if(n == 0)
		return 0;
	else
		return 1;
	}

}

