/*
 * Author: Kostas Kostiadis
 * Description: Contains the functions responsible for connecting/reconnecting the agent
 *				Init, Reconnect
 * Created: 13/1/99
 * Last modified: 8/7/99
 */

#include <stdio.h>
#include <iostream.h>
#include "../Agent_Classes/CGetParams.H" 
#include "../Agent_Classes/CAgent.H"

// INIT /////////////////////////////////////////////////////////
// Initialise agent and place him in the appropriate position depending on the uniform number
void CAgent::Init(char* Goalie, CGetParams Params)
{
int n, junk;						// Used for ReceiveMessage return code, and for GetPlayMode
char SendBuffer[MAXSENDDATALEN];	// Used to store init string
char Buffer[MAXVISUALLEN];			// Used to receive message after init command
char PlayMode[MAXPLAYMODELEN];		// Used to store play mode
char* tmp=Buffer;					// Look ar Buffer
tmp++;								// Skip the '('

if (!strncmp(Goalie, "g", 1))											// If goalie
	sprintf(SendBuffer, "(init %s (version 5.00) (goalie))\n", m_TName);	// Put init command for goalie in SendBuffer
else																	// else
	sprintf(SendBuffer, "(init %s (version 5.00))\n", m_TName);			// Put init command for player in SendBuffer

if (SendMessage(SendBuffer) == -1)                                		// and send it. If error while sending
	Abort("ERROR: Init: Cannot initialise client...", 20);				// Display error and exit
		
while (!(n = ReceiveMessage(Buffer, MAXVISUALLEN)))						// Receive reply from server
	{}

if (n == -1)															// If error on receive
	Abort("ERROR: Init: Cannot initialise client...", 20);				// Display error and exit

else if (n > 0)					// Otherwise
	{
	switch (*tmp)
		{
		case 'i':				// Init command acknowledged
			sscanf(tmp, "init %c %d  %[^)]", &m_Side, &m_UNum, &PlayMode);		// Scan until ')' and store side, uniform number and play mode info
			if (m_Side == 'r')													// If right side player
				pMemory->RevertSigns();											// Revert the signs of all the markers
			pMemory->InitPlayerArrays(m_Side, Params);							// Initialise Friend/Foe arrays
			pPlayMode->SetPlayMode(PlayMode);									// Set play mode
			pBehaviours->SetSide(m_Side);										// Set side in behaviours
			pBehaviours->SetUNum(m_UNum);										// Set uniform number in behaviours
			pBehaviours->InitPositioner();										// Initialise positioner module
			cout << endl;
			cout << "Agent Connected to Server: " << endl;
			cout << "Team Name   = " << m_TName << endl;
			cout << "Side        = " << m_Side << endl;
			cout << "Uniform Num = " << m_UNum << endl;
			cout << "Play mode ID is: " << pPlayMode->GetPlayMode(1) << endl << endl;
			break;
		case 'e':															// Error command
			Abort("ERROR: Init: Error returned from server...", 30);		// Not good! Display error and exit
		default:															// Default???
			Abort("ERROR: Init: Default ???", 39);							// Not good! Display error and exit
		}
	}
// After the init command, the server returns the new socket.
// However this new socket only goes to the pSensors object that received the message.
// Here we want to pass this new socket to the pActuators object as well
SetASocket(GetSSocket());
}

// RECONNECT //////////////////////////////////////////////////////
// Reconnect agent and place him in the appropriate position depending on the uniform number
void CAgent::Reconnect(CGetParams Params)
{
int n, junk;						// Used for receive message return code, and for GetPlayMode
char SendBuffer[MAXSENDDATALEN];	// Used to store reconnect string
char Buffer[MAXVISUALLEN];			// Used to receive message after reconnect command
char PlayMode[MAXPLAYMODELEN];		// Used to store play mode
char* tmp=Buffer;					// Look at Buffer
tmp++;								// Skip the '('

sprintf(SendBuffer, "(reconnect %s %d)\n", m_TName, m_UNum);	// Put reconnect command in SendBuffer

if (SendMessage(SendBuffer) == -1)								// and send it.  If error while sending
	Abort("ERROR: Reconnect: Cannot reconnect client...", 20);		// Display error and exit
	
while (!(n = ReceiveMessage(Buffer, MAXVISUALLEN)))				// Receive reply from server
	{}	
if (n == -1)													// If error
	Abort("ERROR: Reconnect: Cannot reconnect client...", 20);	// Display error and exit

else if (n > 0)													// Otherwise
	{
	switch (*tmp)
		{
		case 'r':													// Reconnect command acknowledged
			sscanf(tmp, "reconnect %c [^)]", &m_Side, &PlayMode);	// Store side, uniform number and play mode info.
			if (m_Side == 'r')										// If right side player
				pMemory->RevertSigns();								// Revert the signs of all the markers
			pMemory->InitPlayerArrays(m_Side, Params);				// Initialise Friend/Foe arrays
			pPlayMode->SetPlayMode("before_kick_off");				// Set play mode (don't pass PlayMode here since
																	// when you are reconnecting it is NULL)
			pBehaviours->SetSide(m_Side);							// Set side in behaviours
			pBehaviours->SetUNum(m_UNum);							// Set uniform number in behaviours
			pBehaviours->InitPositioner();							// Initialise positioner module
			cout << endl;
			cout << "Agent Reconnected to Server: " << endl;
			cout << "Team Name   = " << m_TName << endl;
			cout << "Side        = " << m_Side << endl;
			cout << "Uniform Num = " << m_UNum << endl;
			cout << "Play mode ID is: " << pPlayMode->GetPlayMode(1) << endl << endl;
			break;
		case 'e':															// Error command
			Abort("ERROR: Reconnect: Error returned from server...", 30);	// Not good! Display error and exit
		default:															// Error command
			Abort("ERROR: Reconnect: Default ???", 39);						// Not good! Display error and exit
		}
	}
// After the init command, the server returns the new socket.
// However this new socket only goes to the pSensors object that received the message.
// Here we want to pass this new socket to the pActuators object as well
SetASocket(GetSSocket());
}					

