/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's forward play styles
 * Created: 17/3/99
 * Last modified: 24/7/99
 */

#include <iostream.h> 
#include <stdlib.h>
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CGoal.H" 
#include "../Agent_Classes/CBehaviours.H"

/*
	The play mode IDs have the follows format:
		0 indicates before_kick_off
		Positive numbers indicate event for my side
		(i.e. corner_kick_l if I am left)
		Negative numbers indicate event for enemy side
		(i.e. corner_kick_l if I am right)
		The IDs for each mode are the same
		(i.e. if 1 indicates kick_off_? then
		-1 indicates also kick_off_?) with the sign
		determining if the event if for or against
		my team.
		
		These IDs are as follows:
			0:	before_kick_off
			1:	kick_off
			2:	kick_in
			3:	corner_kick
			4:	goal_kick
			5:	free_kick
			6:	offside
			
			7:	foul_?_UNum		// However it is foul_? without UNum !!!
			8:	goal_?_Point
			9:	goalie_catch_ball
			
		And then we have the modes that are only positive
		since they don't depend on the side.  These are:
		
			20:	play_on
			21:	half_time
			22:	time_over
			23: time_up_without_a_team
			24:	time_extended
			25:	drop_ball
		
*/

/*
	The positioning behaviours (Devensive/OffensivePosition) are
	assuming that we can see the ball
*/

// PLAYASFORWARD //////////////////////////////////////////////////////////
void CBehaviours::PlayAsForward(int PreviousPlayMode, int PlayMode)
{
const CBall* pBall = M2BReadBall();		// Get ball data
CVector GPos = GetGPos();				// Store global position here
int closest;							// Closer candidate test results
int randmove = (rand() % 6) + 1;		// Random move tolerance

CGoal *pGoal;							// Get pointer to enemy goal here

if (GetSide() == 'l')					// If my side is left
	pGoal = M2BReadNonConstGoal('r');	// then the enemy goal is the right one
else									// otherwise
	pGoal = M2BReadNonConstGoal('l');	// it is the left one
	
if (PlayMode >= 20)						// Special modes
	{
	switch (PlayMode)
		{
		case 20:							// Play on
		
////////////////////////////////////////////// TESTING BEHAVIOURS //////////////////////////////////////////
		/*
			if (GetTime() < 150)
				SetFormationID(-1);
			else if ((GetTime() > 150) && (GetTime() < 300))
				SetFormationID(0);
			else if (GetTime() > 300)
				SetFormationID(1);
		*/		
		//	MoveToPos(50);
		//	MoveToStopClearance(50);
			//LookingAtBall();
		

		/*			
		if (LookingAtBall()	!= 999)			// Look at ball and if found
			if (pBall->GetRMag() < 5)
				{
				if (ChaseBall(50))
					Kick(100, 0);
				}
			else
				OffensivePosition();			// Go to best offensive position
		*/
		/*
			if (LookingAtBall() != 999)
				{
				//Turn(pBall->GetRDir());
				closest = ClosestToBall(1);
				if ((closest != 0) && (closest != -2))			// If me
					{
					if (ChaseBall(m_Normal))
						{
						if (!Pass(DTPass("0", 0)))
							//Turn(5);
							
							{
							if (LookingAtGoal() != 999)
								ShootToGoal(1);
							else
								ShootToGoal(2);
							}
						else
							ShootToGoal(2);
						
						}
					}
				else
					OffensivePosition();	
				}
		*/

////////////////////////////////////////////////////////////////////////////////////////////////////////////		

			if (LookingAtBall() != 999)		// Always need to know where ball is
				{
				// If in good position to shoot
				if (pBall->IsKickable() && (GPos.GetX() > 45) && (fabs(GPos.GetY()) < 10))
					ShootToGoal(3);					// Take a shot
					
				closest = ClosestToBall(1);			// Get closest status
				if (abs(closest) == 1)				// If I am the closest
					{
					if (ChaseBall(m_Fast))			// Chase the ball and if you got it
						{
						if (!CleverDribble(CVector(53, 0), m_Fast, 2))	// Start dribbling up field
							{											// and if that fails
							if (!pBall->IsKickable())				// If dribbling was aborted
								ChaseBall(m_Fast);					// just before a dash, ball might be unkickable
								
							// If in good position to shoot
							if (pBall->IsKickable() && (GPos.GetX() > 38) && (fabs(GPos.GetY()) < 5))
								ShootToGoal(3);					// Take a shot
							// If in good position to shoot
							else if (pBall->IsKickable() && (GPos.GetX() > 45) && (fabs(GPos.GetY()) < 10))
								ShootToGoal(3);					// Take a shot
							else
								{
								if (GPos.GetX() < 45)				// If not too forward
									{
									if (!Pass(DTPass("73", 1)))			// If cannot pass
										SendBall(999);					// send the ball
									}
								else
									{
									if (!Pass(DTPass("7", 0)))			// If cannot pass
										ShootToGoal(3);					// Take a shot
									}
								}
							}
						}
					}
				else								// else if a teammate
					{
					if (OffensivePosition())		// Go to best offensive position and if there
						if (pBall->GetRMag() > 30)	// and ball is not too close
							LookAround();			// Look around
					}
				}
				
			break;
		
		case 21:							// Half time
			SayBye();
			break;
		
		case 22:							// Time over
			SayBye();
			break;
			
		case 23:							// Time up without a team
			SayBye();
			break;
			
		case 24:							// Time extended
			SayBye();
			break;
			
		case 25:							// Drop ball
			LookingAtBall();
			break;
			
		default:
			cerr << "WARNING: CBehaviours: PlayAsForward: What PlayMode ID is that - " << PlayMode << " -?" << endl;
			break;
		}
	}

else if ((PlayMode > 0) && (PlayMode < 20))		// My team has won something
	{
	switch (PlayMode)
		{
		case 1:									// Kick off
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("5430", 0)))	// Try to pass and if that fails
							ClearBall(999);			// perform a defensive clearance
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
		
		case 2:									// Kick in
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				if (pBall->GetGX() > 35)
					closest = ClosestToBall(3);	// Get closest status
				else
					closest = ClosestToBall(1);	// Get closest status
					
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("5", 1)))	// Try to pass and if that fails
							SendBall(999);			// perform a defensive clearance
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
		
		case 3:									// Corner Kick
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("760", 0)))// Try to pass and if that fails
							ShootToGoal(1);			// Take a shot
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
			
		case 4:								// Goal Kick
			if (LookingAtBall()	!= 999)		// Look at ball and if found
				if (OffensivePosition())	// Go to best offensive position
					if (LookingAtBall(1))
						Recharge();			// Check if we can afford to inrease our stamina
			break;
			
		case 5:									// Free Kick
			// *** PREVIOUS MIGHT HAVE BEEN goalie_catch_ball_MYTEAM
			// *** PREVIOUS MIGHT HAVE BEEN offside_ENEMYTEAM
			// *** PREVIOUS MIGHT HAVE BEEN foul_ENEMYTEAM
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if ((pBall->GetGX() > 45) && (fabs(pBall->GetGY()) < 10))
							ShootToGoal(3);
						else
							{
							if (!Pass(DTPass("532", 1)))// Try to pass and if that fails
								{
								if (pBall->GetGX() < -20)	// If in my half (a bit in)
									ClearBall(999);			// perform a defensive clearance
								else						// else if in enemy half
									SendBall(999);			// perform an offensive sending
								}
							}
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
			
		case 6:									// Offside
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("53", 0)))	// Try to pass and if that fails
							SendBall(999);			// perform an offensive sending
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
			
		case 7:									// Foul
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("3", 0)))	// Try to pass and if that fails
							SendBall(999);			// perform an offensive sending
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
			
		case 8:									// Goal
			InitialMove();
			Wait4NewVisual();
			LookingAtBall(1);
			break;
			
		case 9:									// Goalie catch ball
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (ChaseBall(m_Slow))		// Go to it
						{
						Recharge();				// Check if we can afford to inrease our stamina
							
						LookAround();				// Then look around
						if (!Pass(DTPass("5", 1)))	// Try to pass and if that fails
							ClearBall(999);			// perform a defensive clearance
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (OffensivePosition())	// Go to best offensive position
						if (LookingAtBall(1))
							Recharge();			// Check if we can afford to inrease our stamina
					}
				}
			break;
			
		default:
			cerr << "WARNING: CBehaviours: PlayAsForward: What PlayMode ID is that - " << PlayMode << " -?" << endl;
			break;
		}
	}
else if (PlayMode < 0)							// Enemy team has won something
	{
	switch (PlayMode)
		{
		case -1:								// Kick off
			LookingAtBall(1);					// Look at ball and save your energy!
			break;
		
		case -2:								// Kick in
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				{
				if (pBall->GetGX() > 35)
					closest = ClosestToBall(3);	// Get closest status
				else
					closest = ClosestToBall(1);	// Get closest status
					
				closest = ClosestToBall(1);		// Get closest status
				if (abs(closest) == 1)			// If you are the closest teammate to the ball
					{
					if (pBall->GetGY() > 0)
						{
						if (MoveToPos((pBall->GetGX()-3+randmove), pBall->GetGY() - 10, m_Slow))
							LookingAtBall(1);
						}
					else
						{
						if (MoveToPos((pBall->GetGX()-3+randmove), pBall->GetGY() + 10, m_Slow))
							LookingAtBall(1);
						}
					}
				else							// else if you are not the closest to the ball
					{
					if (DefensivePosition())		// Go to best defensive position
						LookingAtBall(1);
					}
				}
			break;
		
		case -3:								// Corner Kick
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				if (DefensivePosition())		// Go to best defensive position
					LookingAtBall(1);
			break;
			
		case -4:								// Goal Kick
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				if (MoveToStopClearance(m_Normal))	// Go to stop clearance fixed position
					LookingAtBall(1);
			break;
			
		case -5:								// Free Kick
			// *** PREVIOUS MIGHT HAVE BEEN goalie_catch_ball_ENEMYTEAM
			// *** PREVIOUS MIGHT HAVE BEEN offside_MYTEAM
			// *** PREVIOUS MIGHT HAVE BEEN foul_MYTEAM
			if (PreviousPlayMode == -9)				// If previous was goalie_catch_ball
				{
				if (MoveToStopClearance(m_Normal))	// Go to stop clearance fixed position
					LookingAtBall(1);
				}
			else									// For all other modes
				{
				if (LookingAtBall()	!= 999)			// Look at ball and if found
					if (DefensivePosition())		// Go to best defensive position
						LookingAtBall(1);
				}
			break;
			
		case -6:								// Offside
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				if (DefensivePosition())		// Go to best defensive position
					LookingAtBall(1);
			break;
			
		case -7:								// Foul
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				if (DefensivePosition())		// Go to best defensive position
					LookingAtBall(1);
			break;
			
		case -8:								// Goal
			InitialMove();
			Wait4NewVisual();
			LookingAtBall(1);
			break;

		case -9:								// Goalie catch ball
			if (LookingAtBall()	!= 999)			// Look at ball and if found
				if (DefensivePosition())		// Go to best defensive position
					LookingAtBall(1);
			break;
						
		default:
			cerr << "WARNING: CBehaviours: PlayAsForward: What PlayMode ID is that - " << PlayMode << " -?" << endl;
			break;
		}
	}
	
else if (PlayMode == 0)						// Before kick off
	{
	if (PreviousPlayMode == 0)				// If just started or reconnected
		{
		InitialMove();
		Wait4NewVisual();
		LookingAtBall(1);
		}
	else									// else, it is half_time (or time_extended etc.)
		{
		SayBye();							// so say bye
		}
	}
	
else
	cerr << "WARNING: CBehaviours: PlayAsForward: What PlayMode ID is that - " << PlayMode << " -?" << endl;
}
