/*
 * Author: Kostas Kostiadis
 * Description: Contains all the global parameters like max team name length, max hostname length etc.
 * Created: 11/1/99
 * Last modified: 12/7/99
 */

#ifndef _GLOBALPARAMS_H
#define _GLOBALPARAMS_H

//#define DEBUG_SENSORS		// Receiving data
//#define DEBUG_ACTUATORS	// Sending actions
//#define DEBUG_SYNC		// Locking/Unlocking
//#define DEBUG_PREDS		// Predicate skipping
//#define DEBUG_CONDV		// Conditional variable time-outs

#include <arpa/inet.h>

// MCPLPos Stuff ////////////////////////////////////
//#define		USEVMAP
//#define		BROKEN_STATUS
////////////////////////////////////////////////////

#define		PI					3.14159265358979323846	// pi
#define		DEG2RAD				(PI/180.0)				// Convert from degrees to radians
#define		RAD2DEG				(180.0/PI)				// Convert from radians to degrees
#define		Min(X, Y)			((X) < (Y) ? (X) : (Y))	// Min of two values
#define		Max(X, Y)			((X) > (Y) ? (X) : (Y))	// Max of two values

// BUFFER LENGTHS ///////////////////////////////////
/*
	The *LEN values are the maximum values for the size of the
	array of characters that will hold the corresponding data
*/

// In CGetParams and CAgent /////////////////////////
#define		MAXTEAMNAMELEN		30						// Max team name length
#define		MAXHOSTNAMELEN		128						// Max hostname length
#define		MAXACTIONCOUNT		10						// Max number of actions sent to the server per cycle
#define		MAXACTIONLEN		30						// Max action length

// In CPlayMode /////////////////////////////////////
#define		MAXPLAYMODELEN		50						// Max play mode length

// In CActuators ////////////////////////////////////
#define		MAXSENDDATALEN		640						// Max send data length (normally 512)

// In CSensors //////////////////////////////////////
#define		MAXSENDERLEN		25						// Max sender can be "online_coach_right" (so normally 19)
#define		MAXVISUALLEN		2048					// Max visual data length
#define		MAXAUDIOLEN			640						// Max audio data length (normally 512)
#define		MAXSENSEBODYLEN		512						// Max sense body data legth

// In CyclesToClosestPointOnPath /////////////////////
#define		MAXMODELSTEPS		100				// Max number of steps to model ball's or player's trajectory

// SOCKET INFO ////////////////////////////////////////////////////////////////////////////////
typedef struct socket
{
	int		socketfd;				// Socket structure
	struct	sockaddr_in	serv_addr;	//
}Socket;

// AUDIO INFO //////////////////////////////////////////////////
typedef struct audio
{
	int		Time;					// Time of message
	char	Sender[MAXSENDERLEN];	// Either "referee", "self", or "player"
	int		Direction;				// Direction
	char	Message[MAXAUDIOLEN];	// The message
}Audio;

// SENSE BODY INFO /////////////////////////////////////////////
typedef struct sensebody
{
	int		Time;			// Time of info
	char	VMQuality[10];	// View mode quality
	char	VMMode[10];		// View mode
	float	Stamina;		// Stamina
	float	Effort;			// Effort
	float	Speed;			// Speed
	float	HeadAngle;		// Head angle
	
	int		Kick;			// Number of kicks
	int		Dash;			// dashes
	int		Turn;			// turns
	int		Say;			// say
	int		TurnNeck;		// and turn_neck commands send
	
}SenseBody;

// POINT ////////////////////////////////////////////////////////
typedef struct point
{
        float   X;			// X co-ordinate
        float   Y;			// Y co-ordinate
}Point;

#endif
