/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's ball interception behaviours
 *              Some of this code is based on CMU's 98 low-level behaviours
 * Created: 18/4/99
 * Last modified: 24/7/99
 */

#include <iostream.h>
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CPlayer.H"
#include "../Agent_Classes/CBehaviours.H"

// Active intercept ///////////////////////////////////////////////////////
// CHASECLOSEBALL //////////////////////////////////////////////////////////////////////
CVector CBehaviours::ChaseCloseBall(float DashPower)	// Chase a ball that is close (2 step look-ahead)
{
float Angle, tmp;						// The angle I might need to turn and a temp angle
CVector NewPos, NewBallPos, NewBallVel;	// Used for calculations
CVector temp;							//
const CBall* pBall = M2BReadBall();		// Get pointer to ball data
const float GBFacing = GetBFacing();	// Store global body facing direction here
CVector GPos = GetGPos();				// Store global position here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity

// To remove the stats, delete all the ridiculusly tabbed lines below //
/*
				cout << GetTime() << " CCB - STATS:" << endl;
				cout << " Ball Pos = ";
				NewBallPos = pBall->GetGPos();
				NewBallPos.Display();
				cout << " New Ball Pos = ";
*/
NewBallPos = pBall->GetGPos() + pBall->GetVelocity();	// Ball's global position for the next cycle
//				NewBallPos.Display();
NewBallVel = pBall->GetVelocity() * m_BallDecay;		// Ball's global velocity for the next cycle
//				cout << " My Pos = ";
//				GPos.Display();
NewPos = PredPos();										// My global position for the next cycle
//				cout << " My New Pos = ";
//				NewPos.Display();
temp = NewBallPos - NewPos;								// Difference of the two positions

// 1st test.  Will doing nothing get us there in the next cycle?
//cout << GetTime() << " CCB - Test 1: Will doing nothing get us there? ...";
if (temp.GetMag() < m_Kickable)							// If yes
	{
//	cout << "YES!" << endl;
	return GPos;
	}
//cout << "NO." << endl;

// 2nd test.  Will a single dash get us there?

NewPos = PredPosAfterDash(DashPower);					// My global position after a dash
//				cout << " My New Pos after a DASH with " << DashPower << " ";
//				NewPos.Display();
//cout << GetTime() << " CCB - Test 2: Will a single dash get us there? ...";
temp = NewBallPos - NewPos;								// Difference of the two positions
if (temp.GetMag() < m_Kickable)							// If yes
	{
//	cout << "YES!" << endl;
	return (GPos + Polar2CVector(DashPower * m_DashPowerRate, GBFacing));
	}
//cout << "NO." << endl;

// Doing a 2 cycle look-ahead
NewBallPos += NewBallVel;								// Ball's global position after 2 cycles
//				cout << " Ball Pos after 2 cycles ";
//				NewBallPos.Display();
NewBallVel *= m_BallDecay;								// Ball's global velocity after 2 cycles

// 3rd test.  Will 2 dashes get us there?
NewPos = PredPosAfterDashing(2, DashPower);				// My global position after 2 dashes
//				cout << " My Pos after 2 DASHES with " << DashPower << " ";
//				NewPos.Display();
//cout << GetTime() << " CCB - Test 3: Will 2 dashes get us there? ...";
temp = NewBallPos - NewPos;								// Difference of the two positions
if (temp.GetMag() < m_Kickable)							// If yes
	{
//	cout << "YES!" << endl;
	return (GPos + Polar2CVector(2 * DashPower * m_DashPowerRate, GBFacing));
	}
//cout << "NO." << endl;
	
// 4th test.  Will a turn and a dash get us there?
NewPos = PredPosAfterTurn(2, 0, 0, 0);
//				cout << " My Pos after a TURN and a DASH with " << DashPower << " ";
//				NewPos.Display();
//cout << GetTime() << " CCB - Test 4: Will a dash and a turn get us there? ...";
temp = NewBallPos - NewPos;		// Check by how much I have to turn
tmp = temp.GetDir();			// Store this angle since we might pass it to Polar2CVector later
Angle = NormaliseAngle(temp.GetDir() - GBFacing);		//
if (fabs(Angle) > m_MaxGoAngle)							// If turning angle larger than m_MaxGoAngle
	{
	NewPos = PredPosAfterTurn(2, Angle, DashPower, 1);	// My global position after a turn and a dash
	temp = NewBallPos - NewPos;							// Difference of the two positions
	if (temp.GetMag() < m_Kickable)						// If yes
		{
//		cout << "YES!" << endl;
		return (GPos + Polar2CVector(DashPower * m_DashPowerRate, tmp));
		}
//	cout << "NO." << endl;
	}
/*
else
	{
	cout << "  Angle is smaller than " << m_MaxGoAngle << endl;
	}
*/
//cout << GetTime() << " CCB - RETURNING NO CAN DO!" << endl;
temp.SetMag(-1);										// Set magnitude of returned vector to -1
return temp;											// If we reach this....return error
}

// CYCLESTOINTERCEPT //////////////////////////////////////////////////////////////////
// Calculate the number of cycles it takes for a given player to intercept the ball
int CBehaviours::CyclesToIntercept(float Power, int MaxLookAhead, int IsMe, const CPlayer* P, CVector& InterceptPoint)
{
const CBall* pBall = M2BReadBall();		// Get pointer to ball
CVector PlayerPPos, BallPPos;			// Predicted positions of objects
CVector temp, temp2, temp3, EndPos;		// Used for temp calculations, and to store the end-position
CVector NewBallPos, NewBallVel;			// Updated ball position, and velocity
CVector OldBallPos, OldBallVel;			// Storing in case we need to go back
float BallAngle, TurnAngle, PlayerAngle;// Angle to ball and to turn, and player's angle
float FinalDist;						// Distance to ball after the calculations
int Cycle, CycleStep, MaxCycle, RunCycle;// Cycle stuff used for optimisation
int Result;								// Store the result here

NewBallPos = pBall->GetGPos();			// Init here once
NewBallVel = pBall->GetVelocity();		//
InterceptPoint = NewBallPos;			// Init intercept point to be where ball is now

if ((P == 0) && (!IsMe))				// Not me but player is NULL ???
	return -1;							// abort with error

CycleStep = m_InterceptStep;
// The following is done so that we don't miss an interception point if
// the step is too high
MaxCycle = (MaxLookAhead + m_InterceptStep - 1);
MaxCycle -= (MaxCycle % m_InterceptStep);

for (Cycle=0; Cycle<=MaxCycle; Cycle+=CycleStep)
	{
	// Calculate predicted position of player
	if (IsMe)								// If it's me we are doing this for
		{
		PlayerPPos = PredPos();				// Predict my pos
		PlayerAngle = NormaliseAngle(GetBFacing());	// And store my global body facing direction
		}
	else									// else
		{
		if (P->GetConf() < m_MobConfThr)		// If not remembered
			{
			//cout << GetTime() << " CTI - Could not remember player.  Aborting..." << endl;
			return -1;						// abort
			}
		PlayerPPos = PPredPos(P);			// Predict player's pos
		PlayerAngle = NormaliseAngle(P->GetGBFacing());	// and store player's global body facing direction
		}
	
	
	temp = NewBallPos - PlayerPPos;			// Check if we need to turn to the ball	
	BallAngle = temp.GetDir();				//
	TurnAngle = BallAngle - PlayerAngle;	//
	
	if (fabs(TurnAngle) < m_MaxGoAngle)		// If angle less than threashold
		TurnAngle = 0.0;					// set it to 0

	if (IsMe)								// If it's me
		{
		EndPos = PredPosAfterTurn(Cycle, TurnAngle, Power, (TurnAngle != 0.0));
		}
	else									// else if it is another player
		{
		RunCycle = Cycle;
		if (TurnAngle != 0.0)
			RunCycle--;
		RunCycle = Max(0, RunCycle);
		
		temp2 = Polar2CVector(Power * m_DashPowerRate, BallAngle);
		EndPos = PPredPos(RunCycle, temp2, P);
		if (EndPos.GetMag() < 0)			// Check for error return value
			{
			//cout << GetTime() << " CTI - Could not model player's position.  Aborting..." << endl;
			return -1;						// and abort if necessary
			}
		}
	
	temp = EndPos - PlayerPPos;
	temp2 = NewBallPos - PlayerPPos;
	temp3 = NewBallPos - EndPos;
	FinalDist = temp3.GetMag();
	
	if ((FinalDist <= 1) || (temp.GetMag() > (temp2.GetMag() + m_Kickable)))
		{
		if (FinalDist <= 1)
			Result = Cycle;
		else
			{
			Result = Cycle+1;
			NewBallPos += NewBallVel;
			}
	
		if ((CycleStep > 1) && (Cycle != 0))
			{
			Cycle -= CycleStep;
			NewBallPos = OldBallPos;
			NewBallVel = OldBallVel;
			CycleStep = 1;
			MaxCycle = MaxLookAhead;
			}
		else
			{
			if (Result > MaxLookAhead)
				return -1;
			else
				{
				InterceptPoint = NewBallPos;
				return Result;
				}
			}
		}
	
	// Update ball position estimation
	OldBallPos = NewBallPos;
	OldBallVel = NewBallVel;
	for (int i=0; i< CycleStep; i++)
		{
		NewBallPos += NewBallVel;
		NewBallVel *= m_BallDecay;
		}
	}	// End for loop ! (at last)

return -1;			// If we reach this, return error
}

///////////////////////////////////////////////////////////////////////////
	
// Passive intercept //////////////////////////////////////////////////////
int	CBehaviours::CyclesToClosestPointOnPath(int MaxLookAhead, CVector &Point)	// Calculate the point that is
{																				// closest to the ball's path
const CBall* pBall = M2BReadBall();		// Get pointer to ball data
CVector BallVel = pBall->GetVelocity();	// Get ball's velocity here
CVector GPos = GetGPos();				// Store global position here
CVector BallTraj;						// Used to compute ball's trajectory
float Cycles;							// Number of cycle to intercept
float temp;								// Used for calculations

if ((fabs(BallVel.GetX()) < m_Epsilon) && (fabs(BallVel.GetY()) < m_Epsilon))	// If ball has near zero velocity
	{
	Point = pBall->GetGPos();					// Then closest point is where ball is
	}
else											// else
	{
	//cout << GetTime() << " CTCPOP - Calling GetClosestPointToLine..." << endl;
	Point = GetClosestPointToLine(GPos, pBall->GetGPos(), BallVel);		// Calculate closest point to line
	if (Point.GetMag() < 0)						// If magnitude is negative
		{
		//cout << GetTime() << " CTCPOP - GetClosestPointToLine returned error..." << endl;
		Point = pBall->GetGPos();				// For safety set point to where ball is now
		return -1;								// but return failure
		}
	
	/*
		Here we need to check if the point is outside the pitch.
		If it is we can adjust it so that we can proceed with a
		passive intercept at the clipped point.
		However for now, if the point is outside the pitch we just
		abort the passive intercept.
	*/
	
	if ((fabs(Point.GetX()) > (m_PitchLength/2.0)) || (fabs(Point.GetY()) > (m_PitchWidth/2.0)))
		{
		//cout << GetTime() << " CTCPOP - Point is outside pitch!" << endl;
		Point = pBall->GetGPos();				// For safety set point to where ball is now
		return -1;								// but return failure
		}
	}
	
Cycles = 0.0;									// Set cycles to 0

BallTraj = Point - pBall->GetGPos();			// Now calculate number of cycles

// If ball is NOT coming towards us
if ((SignF(BallTraj.GetX()) != SignF(BallVel.GetX())) || (SignF(BallTraj.GetY()) != SignF(BallVel.GetY())))
	{
	//cout << GetTime() << " CTCPOP - Ball not coming towards us..." << endl;
	return -1;									// Abort passive chase
	}
	
if ((BallTraj.GetMag() < m_Epsilon) && (BallVel.GetMag() < m_Epsilon))
	temp = 0.0;
else
	temp = BallTraj.GetMag() / BallVel.GetMag() * (m_BallDecay - 1) + 1;
	
if (temp < 0.0)								// Ball will never get there so we need to adjust
	{
	Point = pBall->GetGPos() + BallTraj * SumInfGeomSeries(BallVel.GetMag(), m_BallDecay) / BallTraj.GetMag();
	Cycles = 999;
	//cout << GetTime() << " CTCPOP - Ball will never get there..." << endl;
	}
else if (temp == 0.0)						// Ball is probably stopped
	{
	Point = pBall->GetGPos();
	Cycles = 999;
	//cout << GetTime() << " CTCPOP - Temp is 0...using BallGPos..." << endl;
	}
else											// else
	{
	Cycles = log(temp) / log(m_BallDecay);		// calculate number of cycles to point
	//cout << GetTime() << " CTCPOP - All is cool..." << endl;
	}

//cout << GetTime() << " CTCPOP - Point is: ";
//Point.Display();
//cout << GetTime() << " CTCPOP - After " << Cycles << " cycles." << endl;

if (Cycles > MaxLookAhead)						// If cycles exceeded our lookahead
	return -1;									// return no can do
else											// else
	return (int)ceil(Cycles);					// return the value rounded upwards
}
///////////////////////////////////////////////////////////////////////////

// CHASEBALL ///////////////////////////////////////////////////////////////////////////
// Chase the ball with a given power and if passive mode is on, try that first
int CBehaviours::ChaseBall(float Power, int IsPassive) 
{
CVector GPos = GetGPos();				// Get global position
const CBall* pBall = M2BReadBall();		// Get pointer to ball data
CVector temp;							// Store the result
int Cycles;								// Number of cycles to intercept point

if (pBall->GetConf() < m_MobConfThr)		// If confidence on ball too low
	{
	if (LookingAtBall() != 999)			// Look for it and if found
		{
		if (pBall->IsKickable())		// If it is kickable
			return 1;					// return true
		}
		
	return 0;							// If we reach this, return false
	}

if (pBall->IsKickable())	// Check if ball is kickable
	{
	return 1;							// and return 1
	}
else									// else (if not kickable)
	{
	temp = ChaseCloseBall(Power);		// Check if close intercept is possible
	if (temp.GetMag() > 0)				// and if yes
		{
		//cout << GetTime() << " CB - CCB returned ok for close chase with vector: ";
		//temp.Display();
		if (MoveToIntercept(temp, Power) == 1)	// Move for the close intercept point
			{
			return 1;							// If reached, return 1
			}
		}
	else								// Else if close intercept is NOT possible
		{		
		//cout << GetTime() << " CB - CCB returned NO for close chase!" << endl;
		if (IsPassive)											// If passive mode is ON
			{
			Cycles = CyclesToClosestPointOnPath(temp);			// Try passive first
			if (Cycles < 0)										// If that failed, do an active chase
				{
				//cout << GetTime() << " CTCPOP - Failed. Switching to ative chase..." << endl;
				Cycles = CyclesToIntercept(Power, MAXMODELSTEPS, temp);
				//cout << GetTime() << " CTI - Returned cycle: " << Cycles << " and vector: ";
				}
			}
		else													// else if passive NOT ON
			{
			Cycles = CyclesToIntercept(Power, MAXMODELSTEPS, temp);	// Do an active chase
			//cout << GetTime() << " CTI - Returned cycle: " << Cycles << " and vector: ";
			}
			
		//temp.Display();
		if (MoveToIntercept(temp, Power) == 1)					// Then move towards that point, and if point reached
			{													
			return 1;											// return 1
			}
		}
	}

return 0;								// If we reach here, return 0
}

// MOVETOINTERCEPT //////////////////////////////////////////////////////
int CBehaviours::MoveToIntercept(CVector CV, float Power)		// Move to a given point with a given power
{
sensebody SB = GetSB();				// Used to get our current field of view
const CBall* pBall = M2BReadBall();	// Get pointer to ball data
float coldir;						// Collision direction
CVector tmp,tmp2;					// Temp vectors
const float GBFacing = GetBFacing();// Store global body facing direction here
const float GFacing = GetFacing();	// Store global facing direction here
CVector GPos = GetGPos();			// Get global position

tmp = CV - GPos;				// Calculate target vector		
tmp.SetDir(NormaliseAngle(tmp.GetDir() - GBFacing));	// And modify direction to account for a global body facing direction

//cout << GetTime() << " *** MoveToIntercept with GPos X, Y: " << GPos.GetX() << ", " << GPos.GetY() << " and GBFacing: "
//<< GetBFacing() << " to X, Y: " << CV.GetX() << ", " << CV.GetY() << " resulted in TURN by: " << tmp.GetDir()
//<< " and GO by: " << tmp.GetMag() << endl;

if (pBall->IsKickable())				// Check if ball is kickable
	{
//	ResetNeck();						// Align neck with body
	return 1;							// return true
	}

// Here we need to set our field of view depending on where the ball is

if ((pBall->GetRMag() > 30) && (strncmp(SB.VMMode, "wide", 4)))		// If ball far, and not already in wide
	ChangeView("wide", "high");
else if ((pBall->GetRMag() < 30) && (strncmp(SB.VMMode, "normal", 6)))// If ball close, and not already in normal
	ChangeView("normal", "high");
	
/*
	While we are moving we want to make sure we maintain visual with the ball...
	However to avoid turn_necking twice, we check if we turn first, and then
	perform the turn_neck.  So here we only calculate the direction, but don't
	actually perform the action.
*/

if (pBall->GetConf() > m_MobConfThr)					// If ball remembered
	{
	tmp2 = BallPredPos() - PredPos();					// See where ball and I will be in the next cycle
	tmp2.SetDir(NormaliseAngle(tmp2.GetDir() - GFacing));	// And modify direction to account for a global facing direction
	}
else
	{
	tmp2.SetMag(-1);									// else set error code
	}	

/*
	Here we need to check if we are facing the point.
	It is important to check that CV and GPos are not
	the same vector otherwise turning is meaningless and
	we will just waste a cycle.
*/

if ((fabs(tmp.GetDir()) > m_MaxGoAngle) && (tmp.GetMag() > m_Epsilon))
	{
	// If ball goes outside normal visual field
//	if ((tmp2.GetMag() > 0) && (fabs(NormaliseAngle(tmp2.GetDir() - tmp.GetDir())) > 45))
//		TurnNeck(NormaliseAngle(tmp2.GetDir() - tmp.GetDir()));		// Turn your neck so that you can face the ball
		
	Turn(tmp.GetDir());				// Turn to point
	return 0;						// and return 0
	}
else	
	{
	coldir = Collision();			// Check for collisions

	if (coldir != -999)				// If collision exists
		{
//		ResetNeck();				// Align neck
		//cout << GetTime()<< " ---Collision detected at: " << coldir << endl;
		//Dash(-30);					// Back off
		if (coldir > 0)				// If player on the right
			Turn(-90 + coldir);		// turn left
		else
			Turn(90 + coldir);		// else turn right
		Dash(35);					// And dash
		Dash(35);					// 
		return 0;
		}
			
//	if ((tmp2.GetMag() > 0) && (fabs(tmp2.GetDir()) > 45))	// If ball goes outside normal visual field
//		TurnNeck(tmp2.GetDir());							// Turn your neck so that you can face the ball		
				
	for (int i=0; i<m_MaxInterceptDashes; i++)	// Otherwise perform a max number of dashes
		{
		Power = Min(Power, (tmp.GetMag()/m_DashPowerRate)+50);	// If too close to point, we don't want to overshoot
																// and we don't want to just miss it either!
		Power = Min(Power, 100.1);		// Also make sure we didn't exceed 100
		Dash(Power);					// Dash
		GPos = GetGPos();				// Get our new updated GPos
		tmp = CV - GPos;				// Calculate target vector	
		if (pBall->IsKickable())		// Check if ball is kickable
			{
//			ResetNeck();				// Align neck with body
			return 1;					// Return true
			}
		}
	}

if (pBall->IsKickable())				// Check if ball is kickable
	{
//	ResetNeck();						// Align neck with body
	return 1;							// Return true
	}

return 0;								// Return false
}

// GOALIECHASE ///////////////////////////////////////////////////////
int CBehaviours::GoalieChase(int IsPassive)	// Chase behaviour specialised for the goalie
{										// Return 0: false, 1: kickable, 2: catchable
CVector GPos = GetGPos();				// Get global position
CBall* pBall = M2BReadNonConstBall();	// Get pointer to ball data
const float GBFacing = GetBFacing();	// Store global body facing direction here
int Time = GetTime();					// Store current time here
CVector temp;							// Store the result
float CatchDir;							// The direction of the catch command
int Cycles;								// Number of cycles to intercept point
int result = 0;							// Result of the chase

if ((GetTime() - pBall->GetTime()) > 6)	// If ball not seen for 6 cycles
	{
	if (LookingAtBall() != 999)			// Look for it and if found
		{
		CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
		if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
			return 2;
			
		// If ball is kickable (not after it's just been caught)
		else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
			return 1;
		}
		
	return 0;							// If we reach this, return false
	}

CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
	return 2;
			
// If ball is kickable (not after it's just been caught)
else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
	return 1;
	
else									// else (if not kickable)
	{
	temp = ChaseCloseBall(100);			// Check if close intercept is possible
	if (temp.GetMag() > 0)				// and if yes
		{
		result = GoalieMoveToIntercept(temp, 3);	// Move for the close intercept point
		if (result != 0)				// If succeeded
			return result;				// return result
		}
	else								// Else if close intercept is NOT possible
		{
		if (IsPassive)					// If is passive chase
			{
			Cycles = CyclesToClosestPointOnPath(temp);			// Try passive first
			if (Cycles < 0)										// If that failed, do an active chase
				{
				Cycles = CyclesToIntercept(100, MAXMODELSTEPS, temp);
				}
			}
		else							// If not passive
			{
			Cycles = CyclesToIntercept(100, MAXMODELSTEPS, temp);
			}
					
		result = GoalieMoveToIntercept(temp, Cycles);			// Then move towards that point, and if point reached
		if (result != 0)				// If succeeded
			return result;				// return result
		}
	}

return 0;								// If we reach here, return 0
}

// GOALIEMOVETOINTERCEPT ///////////////////////////////////////////////
int	CBehaviours::GoalieMoveToIntercept(CVector CV, int Cycles)		// Same as move to intercept but check for catching as well
{
sensebody SB = GetSB();				// Used to get our current field of view
CBall* pBall = M2BReadNonConstBall();// Get pointer to ball data
float CatchDir;						// The direction of the catch command
int Time = GetTime();				// Store current time here
CVector tmp,tmp2;					// Temp vectors
const float GBFacing = GetBFacing();// Store global body facing direction here
const float GFacing = GetFacing();	// Store global facing direction here
CVector GPos = GetGPos();			// Get global position
float Power = 100.0;				// Chase dash power
int InterCycles = Min(Cycles, 1);	// Max go for 1 chase cycles

tmp = CV - GPos;				// Calculate target vector		
tmp.SetDir(NormaliseAngle(tmp.GetDir() - GBFacing));	// And modify direction to account for a global body facing direction

CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
	return 2;
			
// If ball is kickable (not after it's just been caught)
else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
	return 1;

// Here we need to set our field of view depending on where the ball is

if ((pBall->GetRMag() > 30) && (strncmp(SB.VMMode, "wide", 4)))		// If ball far, and not already in wide
	ChangeView("wide", "high");
else if ((pBall->GetRMag() < 30) && (strncmp(SB.VMMode, "normal", 6)))// If ball close, and not already in normal
	ChangeView("normal", "high");
	
/*
	While we are moving we want to make sure we maintain visual with the ball...
	However to avoid turn_necking twice, we check if we turn first, and then
	perform the turn_neck.  So here we only calculate the direction, but don't
	actually perform the action.
*/

if (pBall->GetConf() > m_MobConfThr)					// If ball remembered
	{
	tmp2 = BallPredPos() - PredPos();					// See where ball and I will be in the next cycle
	tmp2.SetDir(NormaliseAngle(tmp2.GetDir() - GFacing));	// And modify direction to account for a global facing direction
	}
else
	{
	tmp2.SetMag(-1);									// else set error code
	}	

/*
	Here we need to check if we are facing the point.
	It is important to check that CV and GPos are not
	the same vector otherwise turning is meaningless and
	we will just waste a cycle.
*/

if ((fabs(tmp.GetDir()) > m_MaxGoAngle) && (tmp.GetMag() > m_Epsilon))
	{
	// If ball goes outside normal visual field
//	if ((tmp2.GetMag() > 0) && (fabs(NormaliseAngle(tmp2.GetDir() - tmp.GetDir())) > 45))
//		TurnNeck(NormaliseAngle(tmp2.GetDir() - tmp.GetDir()));		// Turn your neck so that you can face the ball
		
	Turn(tmp.GetDir());				// Turn to point
	CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
	if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
		return 2;
			
	// If ball is kickable (not after it's just been caught)
	else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
		return 1;	
		
	return 0;						// if we reach this, return 0
	}
else	
	{
	
//	if ((tmp2.GetMag() > 0) && (fabs(tmp2.GetDir()) > 45))	// If ball goes outside normal visual field
//		TurnNeck(tmp2.GetDir());							// Turn your neck so that you can face the ball		
				
	for (int i=0; i<InterCycles; i++)	// Otherwise perform a max number of dashes
		{
		Power = Min(100, (tmp.GetMag()/m_DashPowerRate)+50);	// If too close to point, we don't want to overshoot
																// and we don't want to just miss it either!
		Power = Min(Power, 100.1);		// Also make sure we didn't exceed 100
		Dash(Power);					// Dash
		GPos = GetGPos();				// Get our new updated GPos
		tmp = CV - GPos;				// Calculate target vector	
		
		CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
		if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
			return 2;
			
		// If ball is kickable (not after it's just been caught)
		else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
			return 1;
		}
	}

CatchDir = NormaliseAngle(GBFacing + pBall->GetRDir());	// Calculate catch direction
if (pBall->IsCatchable(GPos, CatchDir, Time))	// If ball is catchable
	return 2;
			
// If ball is kickable (not after it's just been caught)
else if ((pBall->IsKickable()) && (P2BReadPlayMode(1) != 5) && (P2BReadPlayMode(1) != 9))
	return 1;

return 0;								// Return false
}
