/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's passing behaviours
 * Created: 18/4/99
 * Last modified: 21/7/99
 */

#include "../Agent_Classes/CPlayer.H"
#include "../Agent_Classes/CBehaviours.H"

// DTPASS ///////////////////////////////////////////////////////
/*
	Modes are (string combining any of these):	
	0: Most likely			1: Only positive DT
	2: Most offensive		3: Most offensive pos only
	4: Most wide			5: Most wide pos only
	6: Most centred			7: Most centred pos only
	(e.g. "753" ORDER IS IMPORTANT)
	
	The boolean is used to determine if I want the receiver
	to have a higher X co-ord relative to me
*/
const CPlayer* CBehaviours::DTPass(char* Mode, int IsForward)
{
//CVector GPos = GetGPos();				// Store global position here // *DEBUG ONLY
const CPlayer*const* pCloseFriend = M2BReadPlayerArray(0, 1);	// Get pointer to close friendly player data
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
const CPlayer*const* pFarFriend = M2BReadPlayerArray(1, 1);		// Get pointer to far friendly player data
const CPlayer*const* pFarFoe = M2BReadPlayerArray(1, 0);		// Get pointer to far enemy player data

const CPlayer* pDTReceiver = 0;									// Candidate for the pass

pDTReceiver = DTBest(Mode, IsForward, pCloseFoe, pFarFoe, pCloseFriend, pFarFriend);

if (pDTReceiver == 0)
	{
	//cerr << GetTime() << " DTPass: Candidate was NULL..." << endl;
	return 0;
	}

//cout << GetTime() << " Player: " << GetUNum() << " DTPass with mode: " << Mode << " and my GPos being: " << endl;
//GPos.Display();
//cout << " returned the following receiver: " << endl;
//pDTReceiver->Display();

return pDTReceiver;
}

// OFFENSIVEPASS ////////////////////////////////////////////////
const CPlayer* CBehaviours::OffensivePass()			// Find best candidate for a pass
{
int i,j;													// For loop counters
CVector tmp;												// Temp vector
int best=-1, temp;											// Best candidate so far and temp candidate
float maxx=0;												// Closest to the enemy goal distance so far
CVector GPos = GetGPos();									// Store global position here
const CPlayer*const* pCloseFriend= M2BReadPlayerArray(0, 1);// Get pointer to close friendly player data
const CPlayer*const* pCloseFoe= M2BReadPlayerArray(0, 0);	// Get pointer to close enemy player data

for (i=0; i<11; i++)			// For all friendly players
	{
	// If feasible candidate for a pass
	if ((pCloseFriend[i]->GetConf() > m_MobConfThr) && (pCloseFriend[i]->GetRMag() > 5) && 
	(pCloseFriend[i]->GetRMag() < 20))
		{
		temp = i;				// Mark temp candidate
		for (j=0; j<11; j++)	// For all enemy players
			{
			// If feasible candidate
			if (pCloseFoe[j]->GetConf() > m_MobConfThr)
				{
				// If too close to candidate
				tmp = pCloseFriend[i]->GetGPos() - pCloseFoe[j]->GetGPos();	// We want to get the distance between the two
				if (tmp.GetMag() < 5)					// If distance less than 5
					{
					j = 11;						// Candidate not good so abort...
					temp = -1;					// and unmark temp candidate
					}
				}
			}
		if ((temp != -1) && (pCloseFriend[i]->GetGX()) > maxx)	// If still good, check if closer to enemy goal
			{
			best = i;							// and if so, make this one the best candidate
			maxx = pCloseFriend[i]->GetGX();	// and update best and maxx
			}
		}
	}
	
if ((best != -1) && (pCloseFriend[best]->GetGX() > GPos.GetX()))// If best candidate found and is more offensive than me
	return pCloseFriend[best];	// return him
else						// otherwise
	return 0;				// return NULL pointer
}

// PASS ///////////////////////////////////////////////////
// Pass the ball to a given player in such a way so that the ball has TargSpeed
// when it gets there...
int	CBehaviours::Pass(const CPlayer* Player, float TargSpeed)
{
if (Player != 0)										// Just check for NULL pointer here
	{
	Kick(Point2KickVelocity(Player->GetGPos(),TargSpeed), Player->GetRDir(), 1.0);		// make the pass
	return 1;											// and return OK
	}
else													// else
	return 0;											// return false
}
