/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's positioning behaviours
 * Created: 21/3/99
 * Last modified: 24/7/99
 */

#include <iostream.h> 
#include "Formations.H"
#include "../Agent_Classes/CGoal.H"
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CPlayer.H"
#include "../Agent_Classes/CBehaviours.H"

// INITPOSITIONER ///////////////////////////////////////////////////////////////
void CBehaviours::InitPositioner()	// Initialise positioner object
{
/*
	In here we need to make sure that ALL the pointers to MCPLPosiitoner get
	initialised.  All the possible pointers are:
	
		// 334 Formations ///////////////////////////////////////////////////////
		m_GPS334N
		m_GPS334Defend
		m_GPS334BallTracker
		m_GPS334OffsideTrap
		/////////////////////////////////////////////////////////////////////////
		
		// 433 Formations ///////////////////////////////////////////////////////
		m_GPS433N
		m_GPS433Defend
		m_GPS433BallTracker
		m_GPS433OffsideTrap
		/////////////////////////////////////////////////////////////////////////
		
		// 532 Formations ///////////////////////////////////////////////////////
		m_GPS532N
		m_GPS532Defend
		m_GPS532BallTracker
		m_GPS532OffsideTrap
		////////////////////////////////////////////////////////////////////////
		
	The initialisation is done based on m_FormationID, and m_UNum.  The important
	thing is to have correct fall-back options for the correspondin roles/uniform numbers
	
	The A -> B notation means specialisation A with fall-back to specialisation B
	* Denotes that specialisation does not exist in findpos.ini
*/

int UNum = GetUNum();		// Get my uniform number here

MCPLSetShirt(UNum);

switch (UNum)
	{
	case 1:					// This is the goalie in all formations
		/*
			The goalie should always BallTrack in all formations
		*/
	
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BallTracker
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal *
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BallTracker
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal *
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BallTracker
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 2:				// This is the sweeper in 433 and defender in 334 and 532
		/*
			As sweeper player 2 should OffsideTrap -> BallTrack
			As defender player 2 should OffsideTrap -> Defend (Tracker) -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");												// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);				// Fall-back to normal
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334Defend);	// Fall-back to Defend *
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BallTracker
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal *
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");												// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);				// Fall-back to normal
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532Defend);	// Fall-back to Defend *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 3:				// This is the defender in 334 and 433 and the sweeper in 532
		/*
			In 532 OffsideTrap -> BallTrack
			In 334 OffsideTrap -> Defend (Marker) -> Normal (BoxMapLR)
			In 433 OffsideTrap -> Defend (Tracker) -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");												// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);				// Fall-back to normal
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334Defend);	// Fall-back to Defend *
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");												// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);				// Fall-back to normal
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433Defend);	// Fall-back to Defend *
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");												// Normal is BallTracker
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);				// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);		// Fall-back to normal *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);		// Fall-back to normal
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 4:			// This is a defender in all formations
		/*
			In 433 and 532 OffsideTrap -> Defend (Marker) -> Normal (BoxMapLR)
			In 334 OffsideTrap -> Defend (Tracker) -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");												// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);				// Fall-back to normal
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334Defend);	// Fall-back to Defend *
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");												// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);				// Fall-back to normal
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433Defend);	// Fall-back to Defend *
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");												// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);				// Fall-back to normal
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532Defend);	// Fall-back to Defend *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 5:			// As midfielder in 334, as defender in 433, as sweeper in 532
		/*
			In 334 BallTrack -> Normal (BoxMapLR)
			In 433 OffsideTrap -> Defend (Tracker) -> Normal (BoxMapLR)
			In 532 OffsideTrap -> BallTrack
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *	
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");												// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);				// Fall-back to normal
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433Defend);	// Fall-back to Defend *
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433Defend);	// Fall-back to Defend	
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");												// Normal is BallTracker
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);				// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);		// Fall-back to normal *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);		// Fall-back to normal
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 6:		// As midfielder in 334 and 433, as defender in 532
		/*
			In 334 and 433 BallTrack -> Normal (BoxMapLR)
			In 532 OffsideTrap -> Defend (Tracker) -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");												// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);				// Fall-back to normal
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532Defend);	// Fall-back to Defend *
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532Defend);	// Fall-back to Defend
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 7:		// As midfielder in all formations
		/*
			In all formations BallTrack -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 8:		// As forward in 334, as midfielder in 433 and 532
		/*
			In all formations BallTrack -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 9:		// As forward in 334 and 433, as midfielder in 532
		/*
			In all formations BallTrack -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 10:	// As forward in all formations
		/*
			In all formations BallTrack -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	case 11:	// As forward in all formations
		/*
			In all formations BallTrack -> Normal (BoxMapLR)
		*/
		
		// 334 Formation //////////////////////////////////////////////////////////////////////
		m_GPS334N = MCPLFormation("334");											// Normal is BoxMapLR
		if (m_GPS334N != 0)							// If creation was succesfull
			{
			m_GPS334Defend = MCPLSpecialisation("334", "Defend", m_GPS334N);			// Fall-back to normal *
			m_GPS334BallTracker = MCPLSpecialisation("334", "BallTracker", m_GPS334N);	// Fall-back to normal
			m_GPS334OffsideTrap = MCPLSpecialisation("334", "OffsideTrap", m_GPS334N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 334!" << endl;
			}	
		// 433 Formation //////////////////////////////////////////////////////////////////////
		m_GPS433N = MCPLFormation("433");											// Normal is BoxMapLR
		if (m_GPS433N != 0)							// If creation was succesfull
			{
			m_GPS433Defend = MCPLSpecialisation("433", "Defend", m_GPS433N);			// Fall-back to normal *
			m_GPS433BallTracker = MCPLSpecialisation("433", "BallTracker", m_GPS433N);	// Fall-back to normal
			m_GPS433OffsideTrap = MCPLSpecialisation("433", "OffsideTrap", m_GPS433N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 433!" << endl;
			}	
		// 532 Formation //////////////////////////////////////////////////////////////////////
		m_GPS532N = MCPLFormation("532");											// Normal is BoxMapLR
		if (m_GPS532N != 0)							// If creation was succesfull
			{
			m_GPS532Defend = MCPLSpecialisation("532", "Defend", m_GPS532N);			// Fall-back to normal *
			m_GPS532BallTracker = MCPLSpecialisation("532", "BallTracker", m_GPS532N);	// Fall-back to normal
			m_GPS532OffsideTrap = MCPLSpecialisation("532", "OffsideTrap", m_GPS532N);	// Fall-back to normal *
			}
		else										// and if it couldn't, it displays a warning
			{
			cerr << "WARNING: InitPositioner: MCPLFormation returned NULL for formation 532!" << endl;
			}	
		break;
	default:							// Default ???
		cerr << "WARNING: CBehaviours: InitPositioner: What unifrom number is that -" << UNum << "- ?" << endl;
		break;
	}
}

// PRODUCEMCPLSTATUS ////////////////////////////////////////////////////////////
void CBehaviours::ProduceMCPLStatus()	// Using memory, create or update the MCPLStatus
{
int teamcount = 0, opponentcount = 0;	// Counts are stored here
float last = 0, offside = 0;			// Y co-ordinate for the offside line and the last player

// Gather all data from Memory through the Mediator //////////////////////////////
const CPlayer*const* pCloseFriend = M2BReadPlayerArray(0, 1);	// Get pointer to close friendly player data
const CPlayer*const* pCloseFoe = M2BReadPlayerArray(0, 0);		// Get pointer to close enemy player data
const CPlayer*const* pFarFriend = M2BReadPlayerArray(1, 1);		// Get pointer to far friendly player data
const CPlayer*const* pFarFoe = M2BReadPlayerArray(1, 0);		// Get pointer to far enemy player data
const CPlayer*const* pUnknown = M2BReadPlayerArray(1, 2);		// Get pointer to (very far) unknown player data
const CBall* pBall = M2BReadBall();		// Get pointer to ball
const CGoal *pMyGoal;					// Get pointer to my goal here
const CGoal *pGoal;						// Get pointer to enemy goal here

if (GetSide() == 'l')					// If my side is left
	{
	pMyGoal = M2BReadGoal('l');			// then my goal is the left one
	pGoal = M2BReadGoal('r');			// and enemy goal is the right one
	}
else									// otherwise
	{
	pMyGoal = M2BReadGoal('r');			// it is the right one
	pGoal = M2BReadGoal('l');			// and enemy goal is the left one
	}

// Store goals and ball /////////////////////////////////////////////////////////
m_MCPLStatus.owngoal.X = pMyGoal->GetGX();	// My goal
m_MCPLStatus.owngoal.Y = pMyGoal->GetGY();	//

//cout << GetTime() << " ProduceMCPLStatus: MyGoal (X,Y): " << m_MCPLStatus.owngoal.X
//		<< ", " << m_MCPLStatus.owngoal.Y << endl;

m_MCPLStatus.goal.X = pGoal->GetGX();		// Enemy goal
m_MCPLStatus.goal.Y = pGoal->GetGY();		//

//cout << GetTime() << " ProduceMCPLStatus: EnemyGoal (X,Y): " << m_MCPLStatus.goal.X
//		<< ", " << m_MCPLStatus.goal.Y << endl;

m_MCPLStatus.ball.X = pBall->GetGX();		// Ball
m_MCPLStatus.ball.Y = pBall->GetGY();		//

//cout << GetTime() << " ProduceMCPLStatus: ball (X,Y): " << m_MCPLStatus.ball.X
//		<< ", " << m_MCPLStatus.ball.Y << endl;

// Store players ////////////////////////////////////////////////////////////////
for (int i=0; i<11; i++)
	{
	// Close friends
	if (pCloseFriend[i]->GetConf() > m_MobConfThr)		// If remembered
		{
		if (teamcount > 21)								// For safety only...we shouldn't reach that!!!
			{
			cerr << GetTime() << " ProduceMCPLStatus: Exceeded 21 team-mates ???" << endl;
			teamcount = 21;								// so just keep overwritting the last teammate rather than going post fence
			}
		
		m_MCPLStatus.teammates[teamcount].X = pCloseFriend[i]->GetGX();		// Store team-mate
		m_MCPLStatus.teammates[teamcount].Y = pCloseFriend[i]->GetGY();		//
		
		//cout << GetTime() << " ProduceMCPLStatus: CF (X,Y): " << m_MCPLStatus.teammates[teamcount].X 
		//<< ", " << m_MCPLStatus.teammates[teamcount].Y << endl;
		
		teamcount++;									// Increase teamcount
		}
	
	// Far friends
	if (pFarFriend[i]->GetConf() > m_MobConfThr)			// If remembered
		{
		if (teamcount > 21)								// For safety only...we shouldn't reach that!!!
			{
			cerr << GetTime() << " ProduceMCPLStatus: Exceeded 21 team-mates ???" << endl;
			teamcount = 21;								// so just keep overwritting the last teammate rather than going post fence
			}
		
		m_MCPLStatus.teammates[teamcount].X = pFarFriend[i]->GetGX();		// Store team-mate
		m_MCPLStatus.teammates[teamcount].Y = pFarFriend[i]->GetGY();		//
		
		//cout << GetTime() << " ProduceMCPLStatus: FF (X,Y): " << m_MCPLStatus.teammates[teamcount].X 
		//<< ", " << m_MCPLStatus.teammates[teamcount].Y << endl;
		
		teamcount++;									// Increase teamcount
		}
		
	// Close foes
	if (pCloseFoe[i]->GetConf() > m_MobConfThr)			// If remembered
		{
		if (opponentcount > 21)							// For safety only...we shouldn't reach that!!!
			{
			cerr << GetTime() << " ProduceMCPLStatus: Exceeded 21 opponents ???" << endl;
			opponentcount = 21;							// so just keep overwritting the last teammate rather than going post fence
			}
		
		m_MCPLStatus.opponents[opponentcount].X = pCloseFoe[i]->GetGX();		// Store team-mate
		m_MCPLStatus.opponents[opponentcount].Y = pCloseFoe[i]->GetGY();		//
		
		//cout << GetTime() << " ProduceMCPLStatus: CO (X,Y): " << m_MCPLStatus.opponents[opponentcount].X
		//<< ", " << m_MCPLStatus.opponents[opponentcount].Y << endl;
		
		opponentcount++;								// Increase teamcount
		
		if (pCloseFoe[i]->GetGX() > last)				// Opponent is further in
			{
			offside = last;								// Shift offside for the new value
			last = pCloseFoe[i]->GetGX();				// and update last
			}
		}
	
	// Far foes
	if (pFarFoe[i]->GetConf() > m_MobConfThr)				// If remembered
		{
		if (opponentcount > 21)							// For safety only...we shouldn't reach that!!!
			{
			cerr << GetTime() << " ProduceMCPLStatus: Exceeded 21 opponents ???" << endl;
			opponentcount = 21;							// so just keep overwritting the last teammate rather than going post fence
			}
		
		m_MCPLStatus.opponents[opponentcount].X = pFarFoe[i]->GetGX();		// Store team-mate
		m_MCPLStatus.opponents[opponentcount].Y = pFarFoe[i]->GetGY();		//
		
		//cout << GetTime() << " ProduceMCPLStatus: FO (X,Y): " << m_MCPLStatus.opponents[opponentcount].X
		//<< ", " << m_MCPLStatus.opponents[opponentcount].Y << endl;
		
		opponentcount++;								// Increase teamcount
		
		if (pCloseFoe[i]->GetGX() > last)				// Opponent is further in
			{
			offside = last;								// Shift offside for the new value
			last = pCloseFoe[i]->GetGX();				// and update last
			}
		}
	}
	
// Checking far away player to update offside line (is it worth it?)
for (int j=0; j<22; j++)
	{
	// Very Far players
	if (pUnknown[j]->GetConf() > m_MobConfThr)			// If remembered
		{	
		if (pUnknown[j]->GetGX() > last)				// Opponent is further in
			{
			offside = last;								// Shift offside for the new value
			last = pUnknown[j]->GetGX();				// and update last
			}
		}
	}

m_MCPLStatus.teamcount = teamcount;						// Store teamcount
m_MCPLStatus.opponentcount = opponentcount;				// Store opponentcount

//cout << GetTime() << " ProduceMCPLStatus: - Teamcount: " << teamcount << endl;
//cout << GetTime() << " ProduceMCPLStatus: - Opponentcount: " << opponentcount << endl;

if ((pBall->GetConf() > m_MobConfThr) && (pBall->GetGX() > offside))	// Check ball's confidence and position
	{
	offside = pBall->GetGX();							// and if ball further in, offside line is where ball is !
	}

m_MCPLStatus.offside = offside - (2.0 * m_MinDistToPoint);	// Store offside line (a bit further back to make sure)
//cout << GetTime() << " PRODUCEMCPLSTATUS - Offisde line at X: " << m_MCPLStatus.offside << endl;
}

// DEFENSIVEPOSITION ////////////////////////////////////////////////////////////
int CBehaviours::DefensivePosition()	// Move towards best defensive position given your unum & the current formation
{
const CBall* pBall = M2BReadBall();	// Get pointer to ball
int UNum = GetUNum();		// Get my uniform number here
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)
Point P;					// Desired point to move to
CVector P2CV, tmp, tmp2;			// Used in the keeper calculations to check for backward dashing)
const float GBFacing = GetBFacing();// Store global body facing direction here

if ((m_GPS334N == 0) || (m_GPS433N == 0) || (m_GPS532N == 0))	// If either of these is NULL
	{
	return MoveToPos(Power);				// Just go to your home position
	}
else
	{
	ProduceMCPLStatus();					// Using memory, create or update the MCPLStatus

	switch (m_FormationID)
		{
		case -1:							// If losing formation
			if (m_LosingFormation == 334)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = ThreeThreeFour[UNum-1][0];	// Move to home position
							P.Y = ThreeThreeFour[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Left Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Centre Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Right Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Left Mid
						if (pBall->GetGY() < 10)				// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 6:	// Centre Mid
						if (fabs(pBall->GetGY()) < 23)			// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 7:	// Right Mid
						if (pBall->GetGY() > -10)				// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Left Wing
						if ((pBall->GetGY() < 10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 9:	// Left Forward
						if ((pBall->GetGY() < 10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 10: // Right Forward
						if ((pBall->GetGY() > -10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Wing
						if ((pBall->GetGY() > -10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS334BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What losing formation is that -" << m_LosingFormation << "- ?" << endl;
				
			break;
			
		case 0:								// If normal formation
			if (m_NormalFormation == 433)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = FourThreeThree[UNum-1][0];	// Move to home position
							P.Y = FourThreeThree[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Sweeper
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Left Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Centre Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Right Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 6:	// Left Mid
						if (pBall->GetGY() < 10)				// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 7:	// Centre Mid
						if (fabs(pBall->GetGY()) < 23)			// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Right Mid
						if (pBall->GetGY() > -10)				// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 9: // Left Wing
						if ((pBall->GetGY() < 10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 10: // Centre Forward
						if ((fabs(pBall->GetGY()) < 23) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Wing
						if ((pBall->GetGY() > -10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS433BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What normal formation is that -" << m_NormalFormation << "- ?" << endl;
				
			break;
			
		case 1:								// If winning formation
			if (m_WinningFormation == 532)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = FiveThreeTwo[UNum-1][0];	// Move to home position
							P.Y = FiveThreeTwo[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Left Half-Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Left Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Centre Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Right Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 6:	// Right Half-Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 7:	// Left Mid
						if (pBall->GetGY() < 10)				// If ball on my area
							{
							P = m_GPS532BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Centre Mid
						if (fabs(pBall->GetGY()) < 23)			// If ball on my area
							{
							P = m_GPS532BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 9:	// Right Mid
						if (pBall->GetGY() > -10)				// If ball on my area
							{
							P = m_GPS532BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 10: // Left Forward
						if ((pBall->GetGY() < 10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS532BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Forward
						if ((pBall->GetGY() > -10) && (pBall->GetGX() > -10))			// If ball on my area
							{
							P = m_GPS532BallTracker->findposition(GetPMCPLStatus());	// Generate target point
							}
						else									// else
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What winning formation is that -" << m_WinningFormation << "- ?" << endl;
				
			break;
			
		default:							// Default ???
			cerr << GetTime() << " WARNING: CBehaviours: DefensivePosition: What formation ID is that -" << m_FormationID << "- ?" << endl;
			break;
		}
	}

Power = AdjustPower(Power);			// Adjust power depending on stamina and time left

if (UNum == 1)						// If this is our goalie, he might need to move backwards
	{
	P2CV = CVector(P.X, P.Y);		// Convert point to vector
	tmp = P2CV - GetGPos();			// Calculate target vector		
	tmp.SetDir(NormaliseAngle(tmp.GetDir() - GBFacing));// Modify direction to account for a global body facing direction

	tmp2 = pBall->GetGPos() - GetGPos();					// Now calculate direction to ball
	tmp2.SetDir(NormaliseAngle(tmp2.GetDir() - GBFacing));	//
	
	if (fabs(tmp.GetDir() - tmp2.GetDir()) <= 90)
		return MoveToPos(P, Power);		// Move to point using forward movement
	else
		return MoveToPosBW(P, -Power);	// Move to point using backward movement
	}
	
return MoveToPos(P, Power);			// Move to the point just generated
}

// OFFENSIVEPOSITION //////////////////////////////////////////////////////////////
int CBehaviours::OffensivePosition()	// Move towards best offensive position given your unum & the current formation
{
const CBall* pBall = M2BReadBall();	// Get pointer to ball
int UNum = GetUNum();		// Get my uniform number here
float Power = m_Normal;		// The power used in the MoveToPos behaviour (defaults to normal, changes if needed)
Point P;					// Desired point to move to

if ((m_GPS334N == 0) || (m_GPS433N == 0) || (m_GPS532N == 0))	// If either of these is NULL
	{
	return MoveToPos(Power);				// Just go to your home position
	}
else
	{
	ProduceMCPLStatus();					// Using memory, create or update the MCPLStatus

	switch (m_FormationID)
		{
		case -1:							// If losing formation
			if (m_LosingFormation == 334)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = ThreeThreeFour[UNum-1][0];	// Move to home position
							P.Y = ThreeThreeFour[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Left Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Centre Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Right Back
						P = m_GPS334OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Left Mid
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 6:	// Centre Mid
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 7:	// Right Mid
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Left Wing
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 9:	// Left Forward
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 10: // Right Forward
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Wing
						P = m_GPS334N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What losing formation is that -" << m_LosingFormation << "- ?" << endl;
				
			break;
			
		case 0:								// If normal formation
			if (m_NormalFormation == 433)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = FourThreeThree[UNum-1][0];	// Move to home position
							P.Y = FourThreeThree[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Sweeper
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Left Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Centre Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Right Back
						P = m_GPS433OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 6:	// Left Mid
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 7:	// Centre Mid
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Right Mid
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 9: // Left Wing
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 10: // Centre Forward
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Wing
						P = m_GPS433N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What normal formation is that -" << m_NormalFormation << "- ?" << endl;
				
			break;
			
		case 1:								// If winning formation
			if (m_WinningFormation == 532)
				{
				switch (UNum)
					{
					case 1:	// Goalie
						if (pBall->GetGX() > 0)					// If ball in enemy half
							{
							P.X = FiveThreeTwo[UNum-1][0];	// Move to home position
							P.Y = FiveThreeTwo[UNum-1][1];	//
							}
						else									// else if ball in our half
							{
							P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
							}
						
						P.X = Min(-49.0, P.X);					// This stops keeper from going too far out
						Power = CalcPow4Goalie();				// Calculate power for the move
						break;
					case 2:	// Left Half-Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 3:	// Left Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 4:	// Centre Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 5:	// Right Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 6:	// Right Half-Back
						P = m_GPS532OffsideTrap->findposition(GetPMCPLStatus());	// Generate target point
						if (pBall->GetGX() > 25)			// If ball deep in enemy half
							return MoveToStopClearance(m_Normal); // Pull offside
						Power = CalcPow4Def();					// Calculate power for the move
						break;
					case 7:	// Left Mid
						P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 8:	// Centre Mid
						P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 9:	// Right Mid
						P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Mid();					// Calculate power for the move
						break;
					case 10: // Left Forward
						P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					case 11: // Right Forward
						P = m_GPS532N->findposition(GetPMCPLStatus());	// Generate target point
						Power = CalcPow4Fw();					// Calculate power for the move
						break;
					default:
						cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What unum is that -" << UNum << "- ?" << endl;
						break;
					}
				}
			else
				cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What winning formation is that -" << m_WinningFormation << "- ?" << endl;
				
			break;
			
		default:							// Default ???
			cerr << GetTime() << " WARNING: CBehaviours: OffensivePosition: What formation ID is that -" << m_FormationID << "- ?" << endl;
			break;
		}
	}

Power = AdjustPower(Power);			// Adjust power depending on stamina and time left
	
return MoveToPos(P, Power);			// Move to the point just generated
}
