/*
 * Author: Kostas Kostiadis
 * Description: Contains all the implementations for the agent's predicting behaviours
 * Created: 29/5/99
 * Last modified: 16/7/99
 */

#include <iostream.h>
#include "../Agent_Classes/CBall.H"
#include "../Agent_Classes/CBehaviours.H"
#include "../Agent_Classes/CMemory.H"

// For ball /////////////////////////////////////////////////////
// PREDBALLPOSAFTERKICK /////////////////////////////////////////
CVector CBehaviours::BallPredPosAfterKick(int Steps, float KickPower, float KickDirection)	// Predict ball's position after a kick
{
const float GBFacing = GetBFacing();		// Store global body facing direction here
const CBall *pBall = M2BReadBall();		// Get pointer to ball data
CVector NewPos = pBall->GetGPos();			// Store ball's global position here
CVector NewVelocity = pBall->GetVelocity();	// Store ball's global velocity temporarily here

NewVelocity += Polar2CVector(KickPower/CalcKickPower(1), KickDirection + GBFacing);	// Update velocity due to kick
if (NewVelocity.GetMag() > m_BallSpeedMax)						// Check if we exceeded max velocity
		NewVelocity *= m_BallSpeedMax/NewVelocity.GetMag();		// and if yes, adjust
		
for (int i=0; i<Steps; i++)										// For number of steps
	{
	NewPos += NewVelocity;										// Update position due to velocity
	NewVelocity *= m_BallDecay;									// Decay velocity
	}
                                       
return NewPos;
}      

// BALLWILLBEKICKABLE ////////////////////////////////////////
int	CBehaviours::BallWillBeKickable(int Steps, float DashPower)	// Will ball be kickable after a number of steps?
{
CVector BallPPos = BallPredPos(Steps);							// Predict ball's position
CVector PPos = PredPosAfterDashing(Steps, DashPower);			// Predict my position
CVector temp = PPos - BallPPos;									// Difference of two positions

return (temp.GetMag() < m_Kickable);	// Return TRUE if distance is less than kickable area
}

// For me ////////////////////////////////////////////////
// PREDPOSAFTERDASHING ///////////////////////////////////////
CVector CBehaviours::PredPosAfterDashing(int Steps, float Power)	// Predicted my position after continuous dashing
{
/*
	We should check and adjust stamina, effort, etc. but this is not
	done yet...
*/

const float GBFacing = GetBFacing();	// Store global body facing direction here
CVector NewPos = GetGPos();			// Store global position here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity 
CVector NewVelocity = *pVelocity;			// Store temporarily here

for (int i=0; i<Steps; i++)					// For the number of steps
	{
	NewVelocity += Polar2CVector(Power * m_DashPowerRate, GBFacing);// Update velocity because of Dash
	if (NewVelocity.GetMag() > m_PlayerSpeedMax)					// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();		// and if yes, adjust
	NewPos += NewVelocity;											// Update position because of velocity
	NewVelocity *= m_PlayerDecay;									// Decay velocity
	}
	
return NewPos;
}              

// PREDPOSAFTERTURN ///////////////////////////////////////
CVector CBehaviours::PredPosAfterTurn(int Steps, float Angle, float Power, int HasTurn)	// Predict my position after turn
{
/*
	We should check and adjust stamina, effort, etc. but this is not
	done yet...
*/

float GBFacing = GetBFacing();		// Store global body facing direction here
CVector NewPos = GetGPos();			// Store global position here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity 
float DashPower;					// Power that will be sent to the server
CVector NewVelocity = *pVelocity;	// Store temporarily here

for (int i=0; i<Steps; i++)			// For the specified number of steps
	{
	
	if ((i==0) && (HasTurn))		// If we turned, then this happens first
		{
		GBFacing += Angle;			// Update GBFacing 
		DashPower = 0;				// And zero the DashPower
		}
	else							// else (if no turn or step > 0)
		{
		DashPower = Power * m_DashPowerRate;					// Calculate the power that will be sent to the server
		}
	
	NewVelocity += Polar2CVector(DashPower, GBFacing);			// Our new velocity after a dash

	if (NewVelocity.GetMag() > m_PlayerSpeedMax)				// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();	// and if yes, adjust
		    
	NewPos += NewVelocity;										// Update position
	NewVelocity *= m_PlayerDecay;								// Decay velocity
	}

return NewPos;
}                                              

// For player ////////////////////////////////////////////////////////
// PPREDPOSAFTERDASHING /////////////////////////////////////////////
CVector CBehaviours::PPredPosAfterDashing(int Steps, float Power, const CPlayer* P)	// Predicted player's position after continuous dashing
{
CVector NewPos = P->GetGPos();			// Store global position here
CVector NewVelocity = P->GetVelocity();	// Store player's velocity 
float GBFacing = P->GetGBFacing();		// Store player's global body facing direction here

if (P->GetConf() < m_MobConfThr)		// If player not remembered
	{
	NewPos.SetMag(-1);				// Set to error
	return NewPos;					// and return
	}

for (int i=0; i<Steps; i++)					// For the number of steps
	{
	NewVelocity += Polar2CVector(Power * m_DashPowerRate, GBFacing);// Update velocity because of Dash
	if (NewVelocity.GetMag() > m_PlayerSpeedMax)					// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();		// and if yes, adjust
	NewPos += NewVelocity;											// Update position because of velocity
	NewVelocity *= m_PlayerDecay;									// Decay velocity
	}
	
return NewPos;
}
	
// PPREDPOSAFTERTURN ////////////////////////////////////////////////
// Predict player's position after turn
CVector CBehaviours::PPredPosAfterTurn(int Steps, float Angle, float Power, int HasTurn, const CPlayer* P)
{
CVector NewPos = P->GetGPos();			// Store global position here
CVector NewVelocity = P->GetVelocity();	// Store player's velocity 
float GBFacing = P->GetGBFacing();		// Store player's global body facing direction here
float DashPower;						// Power that will be sent to the server

if (P->GetConf() < m_MobConfThr)		// If player not remembered
	{
	NewPos.SetMag(-1);				// Set to error
	return NewPos;					// and return
	}

for (int i=0; i<Steps; i++)			// For the specified number of steps
	{
	
	if ((i==0) && (HasTurn))		// If he turned, then this happens first
		{
		GBFacing += Angle;			// Update GBFacing 
		DashPower = 0;				// And zero the DashPower
		}
	else							// else (if no turn or step > 0)
		{
		DashPower = Power * m_DashPowerRate;					// Calculate the power that will be sent to the server
		}
	
	NewVelocity += Polar2CVector(DashPower, GBFacing);			// Player's new velocity after a dash

	if (NewVelocity.GetMag() > m_PlayerSpeedMax)				// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();	// and if yes, adjust
		    
	NewPos += NewVelocity;										// Update position
	NewVelocity *= m_PlayerDecay;								// Decay velocity
	}

return NewPos;
}

// PPREDPOS /////////////////////////////////////////////////////////
CVector CBehaviours::PPredPos(int Steps, CVector Velocity, const CPlayer* P)	// Predict player's position using a velocity vector
{
CVector NewVelocity = P->GetVelocity();		// Store player's current velocity
CVector NewPos = P->GetGPos();				// Store player's current global position

for (int i=0; i<Steps; i++)					// For the number of steps
	{
	NewVelocity += Velocity;				// Update velocity
	
	if (NewVelocity.GetMag() > m_PlayerSpeedMax)				// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();	// and if yes, adjust
	
	NewPos += NewVelocity;					// Update position
	NewVelocity *= m_PlayerDecay;			// Decay velocity
	}
	
return NewPos;        						// Return the new predicted position
}

// Others ////////////////////////////////////////////////////////////
// WILLDASHBECOLLISION ///////////////////////////////////////////////
int	CBehaviours::WillDashBeCollision(float DashPower)		// Will dash lead to a collision with the ball?
{
CVector BallPPos = BallPredPos(1);							// Predict ball's position
CVector PPos = PredPosAfterDashing(1, DashPower);			// Predict my position
CVector temp = BallPPos - PPos;								// Difference of two positions

// Return TRUE if distance is less than or equal to player + ball size
return (temp.GetMag() <= (m_PlayerSize + m_BallSize + m_CollisionBuffer));	
}

// WILLKICKBECOLLISION ///////////////////////////////////////////////
int	CBehaviours::WillKickBeCollision(float Distance, float Direction)	// Will kick lead to a collision with the ball?
{
const CBall *pBall = M2BReadBall();		// Get pointer to ball data
const float GFacing = GetFacing();		// Store global body facing direction here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity
float KickPower;						// The power that will be sent in the kick command
float KickDirection;					// The direction of the kick command
CVector tmp;							// Temporary vector used for velocity corrections in the ball's trajectory

//cout << GetTime() << " WillKickBeCollision Dist, Dir: " << Distance << ", " << Direction << endl;
Direction = NormaliseAngle(Direction);			// Normalise the kick angle

if (fabs(pBall->GetMag()) > m_Epsilon)			// If ball has non zero speed
	{
	tmp = Polar2CVector(Distance, Direction) - pBall->GetRelativeVelocity(*pVelocity, GFacing);	// Correct for ball velocity
	KickDirection = NormaliseAngle(tmp.GetDir());							// Set kick direction
	KickPower = CalcKickPower(tmp.GetMag());								// and power
	//cout << "so KickPow, KickDir: " << KickPower << ", " << KickDirection << endl;
	}
else											// else if ball has near zero speed
	{
	KickDirection = Direction;					// Set kick direction
	KickPower = CalcKickPower(Distance);		// Calculate the kick power
	}

if (KickPower > m_MaxPower)						// If we have exceeded max power
	{
	/*
		Normally if we've exceeded max power here, the kick won't be used
		anyway.  So there is no point in doing the line circle intersection
		calculations.  Just use max power instead...
	*/
	//cout << GetTime() << " *** IN WILL KICKBECOLLISION EXCEEDED MAX KICK POWER *** " << endl;
	KickPower = m_MaxPower;						// kick as hard as possible
	}

CVector BallPPos = BallPredPosAfterKick(1, KickPower, KickDirection);	// Predict ball's position after the kick
CVector PPos = PredPos();									// Predict my position
CVector temp = BallPPos - PPos;								// Difference of two positions

// Return TRUE if distance is less than or equal to player + ball size
return (temp.GetMag() <= (m_PlayerSize + m_BallSize + m_CollisionBuffer));	
}

////////////////////////////////////////////////////////////////////////////////////
/*
	The following functions are IDENTICAL to some of the ones above, but are methods
	of the CMemory class.  The only difference is that CMemory has direct access
	to m_Ball data and hence no pointer via the Mediator is needed.
	
	The reason for doing this is because action updates should also update my
	global position after each action.  Hence, these behaviours are needed
	both by the CMemory AND by the CBehaviours classes.  One solution would be
	to attach them to the Mediator like the other globaly needed behaviours.
	However these are kept separate from the Mediator for performance reasons,
	and because the Mediator is kept independend of any behavioural methods.
	
	The increase in size is minimal...
*/
////////////////////////////////////////////////////////////////////////////////////

// For ball /////////////////////////////////////////////////////
// PREDBALLPOSAFTERKICK /////////////////////////////////////////
CVector CMemory::BallPredPosAfterKick(int Steps, float KickPower, float KickDirection)	// Predict ball's position after a kick
{
const float GBFacing = GetBFacing();		// Store global body facing direction here
CVector NewPos = m_Ball.GetGPos();			// Store ball's global position here
CVector NewVelocity = m_Ball.GetVelocity();	// Store ball's global velocity temporarily here

float ActualPower = KickPower / (m_KickPowerRate * (1 - 0.25*fabs(m_Ball.GetRDir()) / 180 - 0.25*(m_Ball.GetRMag()
 - m_PlayerSize - m_BallSize) / m_Kickable));

NewVelocity += Polar2CVector(ActualPower, KickDirection + GBFacing);	// Update velocity due to kick
if (NewVelocity.GetMag() > m_BallSpeedMax)						// Check if we exceeded max velocity
		NewVelocity *= m_BallSpeedMax/NewVelocity.GetMag();		// and if yes, adjust
		
for (int i=0; i<Steps; i++)										// For number of steps
	{
	NewPos += NewVelocity;										// Update position due to velocity
	NewVelocity *= m_BallDecay;									// Decay velocity
	}
                                       
return NewPos;
}      

// For player /////////////////////////////////////////////////////
// PREDPOSAFTERDASHING ///////////////////////////////////////
CVector CMemory::PredPosAfterDashing(int Steps, float Power)	// Predicted my position after continuous dashing
{
/*
	We should check and adjust stamina, effort, etc. but this is not
	done yet...
*/

const float GBFacing = GetBFacing();		// Store global body facing direction here
CVector NewPos = GetGPos();					// Store global position here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity 
CVector NewVelocity = *pVelocity;			// Store temporarily here

for (int i=0; i<Steps; i++)					// For the number of steps
	{
	NewVelocity += Polar2CVector(Power * m_DashPowerRate, GBFacing);// Update velocity because of Dash
	if (NewVelocity.GetMag() > m_PlayerSpeedMax)					// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();		// and if yes, adjust
	NewPos += NewVelocity;											// Update position because of velocity
	NewVelocity *= m_PlayerDecay;									// Decay velocity
	}
	
return NewPos;
}              

// PREDPOSAFTERTURN ///////////////////////////////////////
CVector CMemory::PredPosAfterTurn(int Steps, float Angle, float Power, int HasTurn)	// Predict my position after turn
{
/*
	We should check and adjust stamina, effort, etc. but this is not
	done yet...
*/

float GBFacing = GetBFacing();				// Store global body facing direction here
CVector NewPos = GetGPos();					// Store global position here
CVector *pVelocity = GetNonConstVelocity();	// Get pointer to my velocity 
float DashPower;							// Power that will be sent to the server
CVector NewVelocity = *pVelocity;			// Store temporarily here

for (int i=0; i<Steps; i++)			// For the specified number of steps
	{
	
	if ((i==0) && (HasTurn))		// If we turned, then this happens first
		{
		GBFacing += Angle;			// Update GFacing 
		DashPower = 0;				// And zero the DashPower
		}
	else							// else (if no turn or step > 0)
		{
		DashPower = Power * m_DashPowerRate;					// Calculate the power that will be sent to the server
		}
	
	NewVelocity += Polar2CVector(DashPower, GBFacing);			// Our new velocity after a dash

	if (NewVelocity.GetMag() > m_PlayerSpeedMax)				// Check if we exceeded max velocity
		NewVelocity *= m_PlayerSpeedMax/NewVelocity.GetMag();	// and if yes, adjust
		    
	NewPos += NewVelocity;										// Update position
	NewVelocity *= m_PlayerDecay;								// Decay velocity
	}

return NewPos;
}                                              
