/*
 * Author: Kostas Kostiadis
 * Description: Contains all the member function implementations of the CMemory class
 * 				responsible for updating the memory (e.g. after an action has been sent to the server).
 * Created: 28/5/98
 * Last modified: 23/7/99
 */
 
#include <stdio.h>
#include <iostream.h>
#include "../Agent_Classes/CMemory.H"

// Updating functions /////////////////////////////////////////
// UPDATE4ACTION //////////////////////////////////////////////
void CMemory::Update4Action(const char* sendbuffer)		// Update Memory model depending on the action just sent
{
//cout << GetTime() << " Updating for action: " << sendbuffer;
CBall *pBall = M2BReadNonConstBall();	// Get pointer to ball data
SenseBody SB = GetSB();	// Used to get our current speed
float moment;			// Store moment of turn or direction of kick here
float power;			// Store power of dash or kick here

if (*sendbuffer == '(')	// If indeed there is an action and NOT a 'N' character (from "NULL")
	sendbuffer++;		// Advance past "("
	
switch (*sendbuffer)
	{
	case 't':		// If turn
		if (!strncmp(sendbuffer, "turn_neck", 9))		// If not turn command but turn neck command
			{
			sscanf(sendbuffer, "turn_neck %f)", &moment);// Get requested moment
			Update4TurnNeck(moment);			
			}
		else
			{	
			// else it is a turn command
			sscanf(sendbuffer, "turn %f)", &moment);		// Get requested moment
		
			//cout << GetTime() << "\t\t\tIn update, original moment was: " << moment << " with speed: " << SB.Speed;
			moment = moment / (1.0 + m_InertiaMoment * SB.Speed);	// convert to actual moment and update relevant objects
			//cout << " and now it is: " << moment << " with speed: " << SB.Speed << endl;
			Update4Turn(moment);
			}
		break;
		
	case 'd':		// If dash
		sscanf(sendbuffer, "dash %f)", &power);		// Get requested power
		Update4Dash(power);
		break;
		
	case 'k':		// If kick
		sscanf(sendbuffer, "kick %f %f)", &power, &moment);		// Get requested power
		
		// Calculate power that will be sent to server
		//cout << GetTime() << " Kick with power, direction: " << power << ", " << moment << " results in a kick with ";
		power *= m_KickPowerRate * (1 - 0.25*fabs(m_Ball.GetRDir()) / 180 - 0.25*(m_Ball.GetRMag() - 
		m_PlayerSize - m_BallSize) / m_KickableMargin);
		//cout << "power, direction: " << power << ", " << moment << endl;
		Update4Kick(power, moment);
		break;

	case 'c':		// If catch or change_view
		if (!strncmp(sendbuffer, "catch", 5))		// if catch
			pBall->SetLastCatchCycle(GetTime());	// Set last catch cycle
		break;
	default:		// All other actions
		break;
	}
}

// UPDATE4TURN ///////////////////////////////////////////////////////
void CMemory::Update4Turn(float Moment)
{
CVector GPos = GetGPos();			// Store global position here
const float GFacing = GetFacing();	// Store global facing direction here
const float GBFacing = GetBFacing();// Store global body facing direction here
float temp;							// Temp variable
int i;								// For the for loop

//cout << GetTime() << " Update for turn using Moment: " << Moment << endl;
				
// Set new global facing direction here 
//cout << GetTime() << "\t\t\tUPDATING for TURN by: " << Moment;
temp = GFacing;								// Store current facing direction
//cout << " GFacing was: " << temp;
temp += Moment;					// Add Moment
temp = NormaliseAngle(temp);	// normalise
SetFacing(temp);				// and set temp as new facing direction
//cout << " now is: " << GetFacing() << endl;

// Set new global body direction here
temp = GBFacing;							// Store current body facing direction
//cout << " BFacing was: " << temp;
temp += Moment;					// Add Moment
temp = NormaliseAngle(temp);	// normalise
SetBFacing(temp);				// and set temp as new body facing direction
//cout << " now is: " << GetBFacing() << endl;

Changed();					// Inform mediator of the change
}

// UPDATE4DASH ///////////////////////////////////////////////////////
void CMemory::Update4Dash(float Power)
{
SenseBody SB = GetSB();						// Used to get our current speed
const float GBFacing = GetBFacing();		// Store global body facing direction here
CVector GPos = GetGPos();					// Store global position here
CVector* pVelocity = GetNonConstVelocity();	// Store our velocity here
CVector NewVelocity;						// Store the new velocity here
CVector tmp;								// Store various temporary stuff here

//cout << GetTime() << " Old stamina was: " << SB.Stamina << ", ";
SB.Stamina -= Power;
SB.Stamina += SB.Effort * m_StaminaIncMax;
if (SB.Stamina < 0)
	SB.Stamina = SB.Effort * m_StaminaIncMax;
SetSB(SB);
//cout << "new stamina is: " << SB.Stamina << endl;
		
//cout << GetTime() << " Old velocity was: " << pVelocity->GetMag() << " with direction: " << pVelocity->GetDir() << ". ";
Power *= m_DashPowerRate;	// Calculate Power that will be sent to server
//cout << "After dashing by: " << Power;
NewVelocity = *pVelocity + Polar2CVector(Power, GBFacing);	// Add new velocity to current velocity
if (NewVelocity.GetMag() > m_PlayerSpeedMax)				// If new velocity greater than max velocity
	NewVelocity.SetMag(m_PlayerSpeedMax/pVelocity->GetMag());// just set it to max
// Update global position using velocities
//cout << GetTime() << " Old GPos in Update4Dash: ";
//GPos.Display();
GPos += NewVelocity;	// Update global position
SetGPos(GPos);					// And set new global position
//cout << GetTime() << " New GPos in Update4Dash: ";
//GPos.Display();
NewVelocity *= m_PlayerDecay;			// Decay our velocity
SetVelocity(NewVelocity);				// and set the new velocity
SB.Speed = NewVelocity.GetMag();		// Update speed in SenseBody too
SetSB(SB);
//cout << " estimated new velocity is: " << NewVelocity.GetMag() << " with direction: " << NewVelocity.GetDir() << endl;
//cout << "X,Y components are: " << NewVelocity.GetX() << ", " << NewVelocity.GetY() << endl;
	
Changed();					// Inform mediator of the change
}

// UPDATE4KICK ///////////////////////////////////////////////////////
void CMemory::Update4Kick(float Power, float Moment)
{
const float GBFacing = GetBFacing();		// Store global body facing direction here
const float GFacing = GetFacing();			// Store global facing direction here
CVector GPos = GetGPos();					// Store global position here
const CVector *pVelocity = GetVelocity();	// Store my velocity here
CVector tmp;								// Used to calculate relative polars
CVector OldBallGPos, OldBallVel;			// Used to restore values in case of collisions

OldBallGPos = m_Ball.GetGPos();				// Store current velocity and position
OldBallVel = m_Ball.GetVelocity();			// here

if ( ((GetTime() - m_Ball.GetTime()) < 3) && (m_Ball.IsKickable()))	// Safeguard to check if ball was really kicked
	{
	// The resulting vector is the old ball velocity plus the kick vector
	//cout << GetTime() << " So from mag, dir: " << m_Ball.GetMag() << ", " << m_Ball.GetDir() << " ";
	m_Ball.SetVelocity(m_Ball.GetVelocity() + Polar2CVector(Power, Moment+GBFacing));		// Update ball's velocity
	if (m_Ball.GetMag() > m_BallSpeedMax)					// If ball exceeded max speed
		m_Ball.SetMag(m_BallSpeedMax/m_Ball.GetMag());		// set mag to max speed
		
	m_Ball.SetGPos(m_Ball.GetGPos() + m_Ball.GetVelocity());					// Update ball's global position
	tmp = m_Ball.GetGPos() - GPos;							// Check for collision
	if (tmp.GetMag() < m_ClosestDist)						// If collision occured
		{
		//cout << GetTime() << " *** COLLISION WITH THE BALL OCCURED *** " << endl;
		m_Ball.SetGPos(OldBallGPos);						// Restore old global
		/*
			Actually this is not very accurate.  If the ball was further
			away this will lead to a minor mess-up in the calculations.
			The ball should be placed on the same line connecting the
			player and oldballpos BUT at a distance m_ClosestDist.
			However placing the ball to its old position seems to work OKish.
		*/
		m_Ball.SetVelocity(OldBallVel * (-0.1));			// Update ball's speed
		}
	
		m_Ball.SetVelocity(m_Ball.GetVelocity() * m_BallDecay);	// Decay ball's velocity
	
	//cout << "we go to mag, dir: " << m_Ball.GetMag() * m_BallDecay << ", " << m_Ball.GetDir() << endl;
	}

else
	{
	//cout << GetTime() << " *** ACTION WAS KICK, BUT BALL WAS NOT REALLY KICKED ***" << endl;
	}

Changed();					// Inform mediator of the change
}

// UPDATE4TURNNECK ///////////////////////////////////////////////////////
void CMemory::Update4TurnNeck(float Moment)
{
CVector GPos = GetGPos();			// Store global position here
const float GFacing = GetFacing();	// Store global facing direction here
//SenseBody SB = GetSB();				// Used to get our current head angle
float temp;							// Temp variable
int i;								// For the for loop

//cout << GetTime() << " Update for turn neck using Moment: " << Moment << endl;
				
// Set new global facing direction here 
//cout << GetTime() << "\t\t\tUPDATING for TURN NECK by: " << Moment;
temp = GFacing;								// Store current facing direction
//cout << " GFacing was: " << temp;
temp += Moment;					// Add Moment
temp = NormaliseAngle(temp);	// normalise
SetFacing(temp);				// and set temp as new facing direction
//cout << " now is: " << GetFacing() << endl;

// Set new head angle in SB
/*
	Actions that can go on the same cycle have to update internal structures
	immediately.  Unlike other actions (that update at the end of the cycle)
	these actions (like turn_neck and change_view) have problems if the
	update is not done immediately.  For example imagine ResetNeck from
	SB.HeadAngle = 30.  The client will send numerous TurnNeck(30) commands
	if the SB structure is not updated.  This approach will break, if
	messages get lost, but that is the best I can do for now!  So the
	following code, is in the BasicAction.C file


SB.HeadAngle += Moment;			// Add moment
SB.HeadAngle = NormaliseAngle(SB.HeadAngle);	// normalise
SetSB(SB);						// and set SenseBody
*/

Changed();					// Inform mediator of the change
}

// UPDATE4NEWCYCLE ///////////////////////////////////////////////////
void CMemory::Update4NewCycle()				// Update Memory model for each new cycle
{
//cout << GetTime() << " Updating for new cycle..." << endl;
CVector* pVelocity = GetNonConstVelocity();	// Store our velocity here
CVector GPos = GetGPos();					// Store global position here
CVector tmp;								// Used to check for ball collisions
SenseBody SB = GetSB();						// Used to get our current speed
int i;										// for loop counter
int Dashed=0, Kicked=0;						// Used to decide on updates

for (i=0; i<GetActionCount(); i++)			// Check if I kicked or dashed in the last cycle
	{
	if (!strncmp(GetLastAction(i), "(dash", 5))
		Dashed = 1;									// and update as necessary
	if (!strncmp(GetLastAction(i), "(kick", 5))
		Kicked = 1;									//
	}

/*
if (Dashed && Kicked)						// Problem if this is true
	{
	cerr << GetTime() << " WARNING: Update4NewCycle: Critical Malfunction !!! You can't have dashed AND kicked in one cycle" << endl;
	cerr << " Action buffers are as follows:" << endl;
	for (i=0; i<GetActionCount(); i++)			// Check if I kicked or dashed in the last cycle
		{
		cerr << i << " action is: " << GetLastAction(i) << endl;
		}
	}
*/

// For me /////////////////////////////////////////////////////
if (!Dashed)						// If one of the actions actions was dash, velocity has been already decayed
	{								// and position has already been updated. If it was NOT a dash
	GPos += *pVelocity;				// Update global position
	SetGPos(GPos);
	*pVelocity *= m_PlayerDecay;	// If last action was NOT a dash, decay our velocity
	SetVelocity(*pVelocity);		// and set the new velocity
	SB.Speed = pVelocity->GetMag();	// Update speed in SenseBody too
	SetSB(SB);						//
	}
///////////////////////////////////////////////////////////////

// For the ball ///////////////////////////////////////////////
m_Ball.SetConf(m_Ball.GetConf() * m_MobConfDrop * m_MobConfDrop);// Update ball's confidence (drops twice as fast)
if (m_Ball.GetConf() > m_MobConfThr)							// If ball remembered
	{
	if (!Kicked)					// If one of the actions was kick, ball velocity has been already decayed
		{							// and position has already been updated.  If it was NOT a kick
		m_Ball.SetGPos(m_Ball.GetGPos() + m_Ball.GetVelocity());// Then update global position
		m_Ball.SetVelocity(m_Ball.GetVelocity() * m_BallDecay);	// If last action was NOT a kick, decay ball's velocity
		}
		
	// Check for collisions and 0 speed if collision detected
	tmp = m_Ball.GetGPos() - GetGPos();							// Difference between positions
	if (tmp.GetMag() < m_ClosestDist)							// If collision occured
		{
		//cout << GetTime() << " Player: " << GetUNum() << " Collision with the ball occured" << endl;
		m_Ball.SetVelocity(CVector(0, 0));						// Zero velocity
		}
	}
else															// else
	{
	m_Ball.SetVelocity(CVector(0, 0));							// Zero velocity
	m_Ball.SetPositionValid(0);									// Forget position
	m_Ball.SetVelocityValid(0);									// and velocity
	}
//m_Ball.Display();
////////////////////////////////////////////////////////////////

// For stationary objects //////////////////////////////////////
m_GoalClose.SetConf(m_GoalClose.GetConf() * m_StatConfDrop);	// Close objects (i.e. Flag, Line, Goal...)
m_FlagClose.SetConf(m_FlagClose.GetConf() * m_StatConfDrop);
m_LineClose.SetConf(m_LineClose.GetConf() * m_StatConfDrop);

m_LineL.SetConf(m_LineL.GetConf() * m_StatConfDrop);	// 4 lines
m_LineT.SetConf(m_LineT.GetConf() * m_StatConfDrop);
m_LineR.SetConf(m_LineR.GetConf() * m_StatConfDrop);
m_LineB.SetConf(m_LineB.GetConf() * m_StatConfDrop);

m_FlagC.SetConf(m_FlagC.GetConf() * m_StatConfDrop);	// 53 flags.  A center flag

m_FlagPLB.SetConf(m_FlagPLB.GetConf() * m_StatConfDrop);	// 6 penalty flags
m_FlagPLC.SetConf(m_FlagPLC.GetConf() * m_StatConfDrop);
m_FlagPLT.SetConf(m_FlagPLT.GetConf() * m_StatConfDrop);
m_FlagPRT.SetConf(m_FlagPRT.GetConf() * m_StatConfDrop);
m_FlagPRC.SetConf(m_FlagPRC.GetConf() * m_StatConfDrop);
m_FlagPRB.SetConf(m_FlagPRB.GetConf() * m_StatConfDrop);

m_FlagLB.SetConf(m_FlagLB.GetConf() * m_StatConfDrop);	// 10 flags on the lines
m_FlagGLB.SetConf(m_FlagGLB.GetConf() * m_StatConfDrop);	// Out of which 4 are goal flags
m_FlagGLT.SetConf(m_FlagGLT.GetConf() * m_StatConfDrop);	//
m_FlagLT.SetConf(m_FlagLT.GetConf() * m_StatConfDrop);
m_FlagCT.SetConf(m_FlagCT.GetConf() * m_StatConfDrop);
m_FlagRT.SetConf(m_FlagRT.GetConf() * m_StatConfDrop);
m_FlagGRT.SetConf(m_FlagGRT.GetConf() * m_StatConfDrop);	//
m_FlagGRB.SetConf(m_FlagGRB.GetConf() * m_StatConfDrop);	//
m_FlagRB.SetConf(m_FlagRB.GetConf() * m_StatConfDrop);
m_FlagCB.SetConf(m_FlagCB.GetConf() * m_StatConfDrop);

m_FlagLB30.SetConf(m_FlagLB30.GetConf() * m_StatConfDrop);	// And 36 flags outside the pitch
m_FlagLB20.SetConf(m_FlagLB20.GetConf() * m_StatConfDrop);
m_FlagLB10.SetConf(m_FlagLB10.GetConf() * m_StatConfDrop);
m_FlagL0.SetConf(m_FlagL0.GetConf() * m_StatConfDrop);
m_FlagLT10.SetConf(m_FlagLT10.GetConf() * m_StatConfDrop);
m_FlagLT20.SetConf(m_FlagLT20.GetConf() * m_StatConfDrop);
m_FlagLT30.SetConf(m_FlagLT30.GetConf() * m_StatConfDrop);
m_FlagTL50.SetConf(m_FlagTL50.GetConf() * m_StatConfDrop);
m_FlagTL40.SetConf(m_FlagTL40.GetConf() * m_StatConfDrop);
m_FlagTL30.SetConf(m_FlagTL30.GetConf() * m_StatConfDrop);
m_FlagTL20.SetConf(m_FlagTL20.GetConf() * m_StatConfDrop);
m_FlagTL10.SetConf(m_FlagTL10.GetConf() * m_StatConfDrop);
m_FlagT0.SetConf(m_FlagT0.GetConf() * m_StatConfDrop);
m_FlagTR10.SetConf(m_FlagTR10.GetConf() * m_StatConfDrop);
m_FlagTR20.SetConf(m_FlagTR20.GetConf() * m_StatConfDrop);
m_FlagTR30.SetConf(m_FlagTR30.GetConf() * m_StatConfDrop);
m_FlagTR40.SetConf(m_FlagTR40.GetConf() * m_StatConfDrop);
m_FlagTR50.SetConf(m_FlagTR50.GetConf() * m_StatConfDrop);
m_FlagRT30.SetConf(m_FlagRT30.GetConf() * m_StatConfDrop);
m_FlagRT20.SetConf(m_FlagRT20.GetConf() * m_StatConfDrop);
m_FlagRT10.SetConf(m_FlagRT10.GetConf() * m_StatConfDrop);
m_FlagR0.SetConf(m_FlagR0.GetConf() * m_StatConfDrop);
m_FlagRB10.SetConf(m_FlagRB10.GetConf() * m_StatConfDrop);
m_FlagRB20.SetConf(m_FlagRB20.GetConf() * m_StatConfDrop);
m_FlagRB30.SetConf(m_FlagRB30.GetConf() * m_StatConfDrop);
m_FlagBR50.SetConf(m_FlagBR50.GetConf() * m_StatConfDrop);
m_FlagBR40.SetConf(m_FlagBR40.GetConf() * m_StatConfDrop);
m_FlagBR30.SetConf(m_FlagBR30.GetConf() * m_StatConfDrop);
m_FlagBR20.SetConf(m_FlagBR20.GetConf() * m_StatConfDrop);
m_FlagBR10.SetConf(m_FlagBR10.GetConf() * m_StatConfDrop);
m_FlagB0.SetConf(m_FlagB0.GetConf() * m_StatConfDrop);
m_FlagBL10.SetConf(m_FlagBL10.GetConf() * m_StatConfDrop);
m_FlagBL20.SetConf(m_FlagBL20.GetConf() * m_StatConfDrop);
m_FlagBL30.SetConf(m_FlagBL30.GetConf() * m_StatConfDrop);
m_FlagBL40.SetConf(m_FlagBL40.GetConf() * m_StatConfDrop);
m_FlagBL50.SetConf(m_FlagBL50.GetConf() * m_StatConfDrop);

m_GoalL.SetConf(m_GoalL.GetConf() * m_StatConfDrop);	// 2 goals
m_GoalR.SetConf(m_GoalR.GetConf() * m_StatConfDrop);
/////////////////////////////////////////////////////////////////

// For players //////////////////////////////////////////////////
m_PlayerClose.SetConf(m_PlayerClose.GetConf() * m_MobConfDrop);		// The close player
	if (m_PlayerClose.GetConf() > m_MobConfThr)						// If player remembered
		m_PlayerClose.SetMag(m_PlayerClose.GetMag() * m_PlayerDecay);	// decay it's velocity
	else															// else
		m_PlayerClose.SetMag(0.0);									// set it to 0
	m_PlayerClose.SetGPos(m_PlayerClose.GetGPos() + m_PlayerClose.GetVelocity());	// Then update global position

for (i=0; i<11; i++)	// Also 2 arrays of close and far players (friends and foes)
	{
	m_Friend[i]->SetConf(m_Friend[i]->GetConf() * m_MobConfDrop);
	if (m_Friend[i]->GetConf() > m_MobConfThr)						// If player remembered
		m_Friend[i]->SetMag(m_Friend[i]->GetMag() * m_PlayerDecay);	// decay it's velocity
	else															// else
		m_Friend[i]->SetMag(0.0);									// set it to 0
	m_Friend[i]->SetGPos(m_Friend[i]->GetGPos() + m_Friend[i]->GetVelocity());	// Then update global position
		
	m_Foe[i]->SetConf(m_Foe[i]->GetConf() * m_MobConfDrop);
	if (m_Foe[i]->GetConf() > m_MobConfThr)							// If player remembered
		m_Foe[i]->SetMag(m_Foe[i]->GetMag() * m_PlayerDecay);		// decay it's velocity
	else															// else
		m_Foe[i]->SetMag(0.0);										// set it to 0
	m_Foe[i]->SetGPos(m_Foe[i]->GetGPos() + m_Foe[i]->GetVelocity());	// Then update global position
	
	m_FarFriend[i]->SetConf(m_FarFriend[i]->GetConf() * m_MobConfDrop);
	/*
		We don't get velocity information for far players in the current implementation
		
	if (m_FarFriend[i]->GetConf() > m_MobConfThr)								// If player remembered
		m_FarFriend[i]->SetMag(m_FarFriend[i]->GetMag() * m_PlayerDecay);	// decay it's velocity
	else																	// else
		m_FarFriend[i]->SetMag(0.0);										// set it to 0
	m_FarFriend[i]->SetGPos(m_FarFriend[i]->GetGPos() + m_FarFriend[i]->GetVelocity());	// Then update global position
	*/
	
	m_FarFoe[i]->SetConf(m_FarFoe[i]->GetConf() * m_MobConfDrop);
	/*
		We don't get velocity information for far players in the current implementation
		
	if (m_FarFoe[i]->GetConf() > m_MobConfThr)						// If player remembered
		m_FarFoe[i]->SetMag(m_FarFoe[i]->GetMag() * m_PlayerDecay);	// decay it's velocity
	else															// else
		m_FarFoe[i]->SetMag(0.0);									// set it to 0
	m_FarFoe[i]->SetGPos(m_FarFoe[i]->GetGPos() + m_FarFoe[i]->GetVelocity());	// Then update global position
	*/
	}

for (i=0; i<22; i++)								// And an array of very far players (unknown)
	{
	m_UnknownPlayer[i]->SetConf(m_UnknownPlayer[i]->GetConf() * m_MobConfDrop);
	/*
		We don't get velocity information for very far players in the current implementation
	
	if (m_UnknownPlayer[i]->GetConf() > m_MobConfThr)									// If player remembered
		m_UnknownPlayer[i]->SetMag(m_UnknownPlayer[i]->GetMag() * m_PlayerDecay);	// decay it's velocity
	else																			// else
		m_UnknownPlayer[i]->SetMag(0.0);											// set it to 0
	m_UnknownPlayer[i]->SetGPos(m_UnknownPlayer[i]->GetGPos() + m_UnknownPlayer[i]->GetVelocity());	// Then update global position
	*/
	}
////////////////////////////////////////////////////////
}

// UPDATEGLOBALS ///////////////////////////////////////////////////////////////////////
void CMemory::UpdateGlobals()	// Now that we've localised, calculate global positions of selected objects	
{
//cout << GetTime() << " Updating globals..." << endl;
CVector GPos = GetGPos();		// Store my global position here
float GFacing = GetFacing();	// Store my global facing direction here
int i;							// Used in the for loops

/*
	Update global co-ordinates for mobile objects.
	Just convert the relative polar to relative
	cartesian co-ordinates and add your global
	cartesian co-ordinates.
	
	X = MyGlobalX + RelativeDistance * cos (RelativeDirection + MyGlobalFacing)
	Y = MyGlobalY + RelativeDistance * sin (RelativeDirection + MyGlobalFacing)
	
	OR altarnatively in terms of vectors:
	
	MyGlobalPositionVector + Polar2CVector(RelativeDistance, RelativeDirection + MyGlobalFacing)
	
*/

// Update for the ball, if seen
if (m_Ball.GetConf() > ((m_MobConfDrop * m_MobConfDrop) - m_Epsilon))
	{
	m_Ball.SetGPos(GPos + Polar2CVector(m_Ball.GetRMag(), m_Ball.GetRDir() + GFacing));
	}
//m_Ball.Display();

// Update for close player, if seen
if (m_PlayerClose.GetConf() > (m_MobConfDrop - m_Epsilon))
	{	
	m_PlayerClose.SetGPos(GPos + Polar2CVector(m_PlayerClose.GetRMag(), m_PlayerClose.GetRDir() + GFacing));
	}
	
// Update for all friendly players, if seen
for (i=0; i<11; i++)
	{
	if (m_Friend[i]->GetConf() > (m_MobConfDrop - m_Epsilon))
		{	
		m_Friend[i]->SetGPos(GPos + Polar2CVector(m_Friend[i]->GetRMag(), m_Friend[i]->GetRDir() + GFacing));
		}
	if (m_FarFriend[i]->GetConf() > (m_MobConfDrop - m_Epsilon))
		{	
		m_FarFriend[i]->SetGPos(GPos + Polar2CVector(m_FarFriend[i]->GetRMag(), m_FarFriend[i]->GetRDir() + GFacing));
		}

// Update for all enemy players, if seen
	if (m_Foe[i]->GetConf() > (m_MobConfDrop - m_Epsilon))
		{
		m_Foe[i]->SetGPos(GPos + Polar2CVector(m_Foe[i]->GetRMag(), m_Foe[i]->GetRDir() + GFacing));						
		}
	if (m_FarFoe[i]->GetConf() > (m_MobConfDrop - m_Epsilon))
		{
		m_FarFoe[i]->SetGPos(GPos + Polar2CVector(m_FarFoe[i]->GetRMag(), m_FarFoe[i]->GetRDir() + GFacing));
		}
	}

// Update for all unknown players, if seen
for (i=0; i<22; i++)
	{
	if (m_UnknownPlayer[i]->GetConf() > (m_MobConfDrop - m_Epsilon))
		{
		m_UnknownPlayer[i]->SetGPos(GPos + Polar2CVector(m_UnknownPlayer[i]->GetRMag(), m_UnknownPlayer[i]->GetRDir() + GFacing));
		}
	}
}

// UPDATERELATIVES ///////////////////////////////////////////////////////////////
void CMemory::UpdateRelatives()		// Updates relative co-ords of mobile objects
{
//cout << GetTime() << " Updating relatives..." << endl;
const float GFacing = GetFacing();			// Store global facing direction here
CVector GPos = GetGPos();					// Store global position here
int GTime = GetTime();						// Get current time
int i;										// for loop counter

// The UpdateRPos function checks for confidence before updating
// mobile objects

// Update goals ////////////////////////////////////////////////////

m_GoalL.UpdateRPos(GPos, GFacing, GTime);	// Left goal
m_GoalR.UpdateRPos(GPos, GFacing, GTime);	// Right goal

// Update all lines ////////////////////////////////////////////////
/*
	If lines are not updated, Localisation can return wierd results
	since facing direction has changed, but line relative directions
	have not !
*/

m_LineL.UpdateRPos(GPos, GFacing, GTime);	// Left line
m_LineT.UpdateRPos(GPos, GFacing, GTime);	// Top line
m_LineR.UpdateRPos(GPos, GFacing, GTime);	// Right line
m_LineB.UpdateRPos(GPos, GFacing, GTime);	// Bottom line
					
// Update all flags ////////////////////////////////////////////////
/*
	If flags are not updated, Localisation can return wierd results
	since facing direction has changed, but flag relative directions
	have not !
*/				

m_FlagC.UpdateRPos(GPos, GFacing, GTime);

m_FlagPLB.UpdateRPos(GPos, GFacing, GTime);
m_FlagPLC.UpdateRPos(GPos, GFacing, GTime);
m_FlagPLT.UpdateRPos(GPos, GFacing, GTime);
m_FlagPRT.UpdateRPos(GPos, GFacing, GTime);
m_FlagPRC.UpdateRPos(GPos, GFacing, GTime);
m_FlagPRB.UpdateRPos(GPos, GFacing, GTime);

m_FlagLB.UpdateRPos(GPos, GFacing, GTime);
m_FlagGLB.UpdateRPos(GPos, GFacing, GTime);
m_FlagGLT.UpdateRPos(GPos, GFacing, GTime);
m_FlagLT.UpdateRPos(GPos, GFacing, GTime);
m_FlagCT.UpdateRPos(GPos, GFacing, GTime);
m_FlagRT.UpdateRPos(GPos, GFacing, GTime);
m_FlagGRT.UpdateRPos(GPos, GFacing, GTime);
m_FlagGRB.UpdateRPos(GPos, GFacing, GTime);
m_FlagRB.UpdateRPos(GPos, GFacing, GTime);
m_FlagCB.UpdateRPos(GPos, GFacing, GTime);
	
m_FlagLB30.UpdateRPos(GPos, GFacing, GTime);
m_FlagLB20.UpdateRPos(GPos, GFacing, GTime);
m_FlagLB10.UpdateRPos(GPos, GFacing, GTime);
m_FlagL0.UpdateRPos(GPos, GFacing, GTime);
m_FlagLT10.UpdateRPos(GPos, GFacing, GTime);
m_FlagLT20.UpdateRPos(GPos, GFacing, GTime);
m_FlagLT30.UpdateRPos(GPos, GFacing, GTime);
m_FlagTL50.UpdateRPos(GPos, GFacing, GTime);
m_FlagTL40.UpdateRPos(GPos, GFacing, GTime);
m_FlagTL30.UpdateRPos(GPos, GFacing, GTime);
m_FlagTL20.UpdateRPos(GPos, GFacing, GTime);
m_FlagTL10.UpdateRPos(GPos, GFacing, GTime);
m_FlagT0.UpdateRPos(GPos, GFacing, GTime);
m_FlagTR10.UpdateRPos(GPos, GFacing, GTime);
m_FlagTR20.UpdateRPos(GPos, GFacing, GTime);
m_FlagTR30.UpdateRPos(GPos, GFacing, GTime);
m_FlagTR40.UpdateRPos(GPos, GFacing, GTime);
m_FlagTR50.UpdateRPos(GPos, GFacing, GTime);
m_FlagRT30.UpdateRPos(GPos, GFacing, GTime);
m_FlagRT20.UpdateRPos(GPos, GFacing, GTime);
m_FlagRT10.UpdateRPos(GPos, GFacing, GTime);
m_FlagR0.UpdateRPos(GPos, GFacing, GTime);
m_FlagRB10.UpdateRPos(GPos, GFacing, GTime);
m_FlagRB20.UpdateRPos(GPos, GFacing, GTime);
m_FlagRB30.UpdateRPos(GPos, GFacing, GTime);
m_FlagBR50.UpdateRPos(GPos, GFacing, GTime);
m_FlagBR40.UpdateRPos(GPos, GFacing, GTime);
m_FlagBR30.UpdateRPos(GPos, GFacing, GTime);
m_FlagBR20.UpdateRPos(GPos, GFacing, GTime);
m_FlagBR10.UpdateRPos(GPos, GFacing, GTime);
m_FlagB0.UpdateRPos(GPos, GFacing, GTime);
m_FlagBL10.UpdateRPos(GPos, GFacing, GTime);
m_FlagBL20.UpdateRPos(GPos, GFacing, GTime);
m_FlagBL30.UpdateRPos(GPos, GFacing, GTime);
m_FlagBL40.UpdateRPos(GPos, GFacing, GTime);
m_FlagBL50.UpdateRPos(GPos, GFacing, GTime);

// Update ball //////////////////////////////////////////////////

m_Ball.UpdateRPos(GPos, GFacing, GTime);
//m_Ball.Display();
// Update players ///////////////////////////////////////////////

m_PlayerClose.UpdateRPos(GPos, GFacing, GTime);

for (i=0; i<11; i++)
	{
	m_Friend[i]->UpdateRPos(GPos, GFacing, GTime);
	m_FarFriend[i]->UpdateRPos(GPos, GFacing, GTime);
	m_Foe[i]->UpdateRPos(GPos, GFacing, GTime);						
	m_FarFoe[i]->UpdateRPos(GPos, GFacing, GTime);
	}

for (i=0; i<22; i++)
	{
	m_UnknownPlayer[i]->UpdateRPos(GPos, GFacing, GTime);
	}
}

// REMOVEGHOSTS ////////////////////////////////////////////////////////////////////
void CMemory::RemoveGhosts()				// // Forget any objects that should have been visible, but aren't
{
/*
	This function gets called right after we store the visual data we received
*/
SenseBody SB = GetSB();						// Used to get our current field of view
int Angle = 44;								// Max visible angle (+/-) depending on field of view
int i;										// For loop counter

if (!strncmp(SB.VMMode, "wide", 4))			// If wide field of view
	Angle = 89;								// we can see all objects from -90 to 90
else if (!strncmp(SB.VMMode, "normal", 6))	// If normal field of view
	Angle = 44;								// we can see all objects from -45 to 45
else if (!strncmp(SB.VMMode, "narrow", 6))	// If narrow field of view
	Angle = 22;								// we can see all objects from -22 to 22
else
	cerr << GetTime() << " WARNING: RemoveGhosts: What field of view is that -" << SB.VMMode << "- ?" << endl;

// Update ball //////////////////////////////////////////////////
// For the ball we test proximity sensors as well as normal ones
if ((m_Ball.GetConf() < (1 - m_Epsilon)) && (fabs(m_Ball.GetRDir()) < Angle) && (!m_Ball.GetClose()))
	m_Ball.SetConf(0);
if ((m_Ball.GetConf() < (1 - m_Epsilon)) && (m_Ball.GetRMag() < 3) && (m_Ball.GetClose()))
	m_Ball.SetConf(0);

// Update players ///////////////////////////////////////////////
// The close player we can forget if he is not seen and is within proximity range
if ((m_PlayerClose.GetConf() < (1 - m_Epsilon)) && 
((fabs(m_PlayerClose.GetRDir()) < Angle) || (m_PlayerClose.GetRMag()) < 3))
	m_PlayerClose.SetConf(0);

for (i=0; i<11; i++)
	{
	if ((m_Friend[i]->GetConf() < (1 - m_Epsilon)) && (fabs(m_Friend[i]->GetRDir()) < Angle))
		m_Friend[i]->SetConf(0);
	
	if ((m_FarFriend[i]->GetConf() < (1 - m_Epsilon)) && (fabs(m_FarFriend[i]->GetRDir()) < Angle))
		m_FarFriend[i]->SetConf(0);
		
	if ((m_Foe[i]->GetConf() < (1 - m_Epsilon)) && (fabs(m_Foe[i]->GetRDir()) < Angle))
		m_Foe[i]->SetConf(0);
		
	if ((m_FarFoe[i]->GetConf() < (1 - m_Epsilon)) && (fabs(m_FarFoe[i]->GetRDir()) < Angle))
		m_FarFoe[i]->SetConf(0);
	}

for (i=0; i<22; i++)
	{
	if ((m_UnknownPlayer[i]->GetConf() < (1 - m_Epsilon)) && (fabs(m_UnknownPlayer[i]->GetRDir()) < Angle))
		m_UnknownPlayer[i]->SetConf(0);
	}
}
