/*
 * Author: Kostas Kostiadis
 * Description: Contains the usage and main functions
 * Created: 14/12/98
 * Last modified: 20/6/99
 */

#include <stdlib.h>
#include <iostream.h>
#include "../Agent_Classes/CGetParams.H"
#include "../Agent_Classes/CAgent.H"

// USAGE //////////////////////////////////////////////////
void Usage()
{
cerr << "ERROR: Incorrect number/type of arguments..." << endl;			// Display error message
cerr << endl;
cerr << "Syntax: agent <Half> <Goalie/uniform number> [Filename]" << endl;
cerr << endl;
cerr << "Where:" << endl;
cerr << "\t Half: Which half (either 1 or 2)" << endl;
cerr << "\t Goalie/uniform number: Goalie flag or uniform number (g for goalie p for player, or 1-11 for uniform number)" << endl;
cerr << "\t\t\t If Half is 1, this parameter should be either g or p.  If Half is 2, this parameter should be 1-11" << endl;
cerr << "\t Filename: Specifies the configuration file to read for the agent parameters (default value \"agent.conf\")" << endl;
cerr << endl;
cerr << "Example: agent 1 g               -- Will connect a goalie reading parameters from the file \"agent.conf\"" << endl;
cerr << "or     : agent 2 4               -- Will reconnect player 4 reading parameters from the file \"agent.conf\"" << endl;
cerr << "or     : agent 1 p myagent.conf  -- Will connect a player reading parameters from the file \"myagent.conf\"" << endl;
cerr << endl;
cerr << "All parameters related to team name, port number, host name etc. can be changed by editing the *.conf file" << endl;
cerr << endl;
exit(1);																// and exit
}

// main ///////////////////////////////////////////////////
int main(int argc, char** argv)
{
cout << endl;																// Send program info
cout << "Essex Wizards '99 v1.0 by Kostas Kostiadis, (C) 1998-1999" << endl;	//
cout << "All rights reserved." << endl << endl;								//

int ExitCode=0;					// Exit code
Socket Sock;					// Used to store Socket info

char *Goalie;					// Used to determine if player is goalie
char *ConfigurationFile = 0;	// Where to read params from 
int Half, UNum;					// Used to determine 1st/2nd half and uniform number for reconnect
int PlayStyle;					// Used to determine formation

if (argc == 3)					// If 2 arguments specified
        {
        Goalie = argv[2];		// 2nd is goalie flag
        Half = atoi(argv[1]);	// 1nd is 1st/2nd half
        }
else if (argc == 4)				// If 3 arguments specified
        {
		ConfigurationFile = argv[3];// 3rd is configuration file
        Goalie = argv[2];		// 2nd is goalie flag
        Half = atoi(argv[1]);	// 1nd is 1st/2nd half
        }
else							// If invalid arguments
        Usage();				// Display usage and exit

CGetParams Params;				// Create a CGetParams object (default constructor is empty)
if (ConfigurationFile == 0)
	Params.LoadParams("agent.conf");		// Get parameters from server and agent configuration files
else
	Params.LoadParams(ConfigurationFile);	// Get parameters from server and agent (specified) configuration files

//Params.Display();
//exit(0);

CAgent Agent(Params);						// Create an agent using these parameters

Agent.SetTName(Params.GetAP_team_name());	// Set team name		
		
Sock = Agent.InitConnection(Params.GetAP_hostname(), Params.GetAP_port());

if (Sock.socketfd == -1)										// If connection denied
	Agent.Abort("ERROR: main: Unable to connect to server...", 10);	// Display error and exit

Agent.SetSSocket(Sock);			// Set m_Sensors socket
Agent.SetASocket(Sock);			// Set m_Actuators socket

if (Half == 1)						// For 1st half
        Agent.Init(Goalie, Params);	// Init the client
else if (Half == 2)					// For 2nd half
        {
        UNum = atoi(Goalie);		// Get uniform number
        Agent.SetUNum(UNum);		// Set uniform number and
        Agent.Reconnect(Params);	// Reconnect the client
        }
else
        {
        cerr << "ERROR: Half must be either 1 or 2..." << endl;
        Usage();
        }

ExitCode = Agent.MainLoop();	// Start the player's main loop

return(ExitCode);
}

