// File    : MCIni.cpp
//
// Purpose : C++ Configuration File class
// Author  : Matthew Hunter
// Version : 0.01 (08 Jun 1999)
//           0.02 (24 Jun 1999)
//                Added getclassed functions

#include "MCIni.h"
#include <fstream.h>
#include <iostream.h>

ostream& operator<<(ostream&s,MCIni d) {
  MCSettingList *current = d.head;
  while(current!=0) {
    s << *current << endl;
    current = current->next;
  }
  return s;
}

void MCIni::parse(MCString optionfile) {
  MCString file,line,token;
  MCString groupname("Global");
  MCString empty("");
 
  file = optionfile;
  while(file.length()>0) {
    // get the next line to process
    line = file.tokenise('\n');
    if (line!=empty) {
      // not a blank line
      if (line[0]=='[') {
	// New group heading
	if (line[line.length()-1]==']') {
	  // terminated ok
	  token=line.tokenise('['); // empty, just trims the '[' off of line
	  groupname=line.tokenise(']'); // get the groupname itself
	}
#ifndef NDEBUG
	else {
	  cout << "MCIni::parse: groupname incorrectly terminated.\n";
	}
#endif
      } else {
	token = line.tokenise('=');
	if (token!=empty) {
	  insert(groupname,token,line);
	}
      }
    }
  }
}

void MCIni::import(MCString filename) {
  MCString options = filename.readFile();
  parse(options);
}

void MCIni::export(MCString filename) {
  ofstream to((const char *)filename);
  if (to) {
    to << *this;
  }
}

void MCIni::insert(MCString g,MCString n, MCString v) {
  MCSettingList *add = find(g);
  if (add==0) { // add at start of list
    add = new MCSettingList();
    add->name = g;
    add->insert(n,v);
    add->next = head;
    head = add;
  } else {
    if (g==add->name) { // Item already in list
      add->insert(n,v);
    } else { // item not in list, add after current item
      MCSettingList *temp = new MCSettingList();
      temp->next = add->next;
      add->next = temp;
      temp->name=g;
      temp->insert(n,v);
    }
  }
}

void MCIni::insert(MCString g,MCString n,int v) {
  MCString value(v);
  insert(g,n,value);
}

void MCIni::insert(MCString g,MCString n,double v) {
  MCString value(v);
  insert(g,n,value);
}

MCString MCIni::get(MCString g,MCString n, MCString v) {
  MCSettingList *found = find(g);
  if (found==0) { // n is not in the list
    return v;
  } else {
    if (g==found->name) { // item is in list
      return found->get(n,v);
    } else { // item is not in the list
      return v;
    }
  }
}

int MCIni::get(MCString g,MCString n,int v) {
  MCString value(v);
  return (int)get(g,n,value);
}

double MCIni::get(MCString g,MCString n,double v) {
  MCString value(v);
  return (double)get(g,n,value);
}

MCString MCIni::getclassed(MCString g, MCString n, MCString v) {
  MCSettingList *found = find(g); // Find the group
  if (found==0) { // group definately not there
    return v;
  } else {
    if (g==found->name) { // found the group
      //      cout << "Checking " << n << " " << v << endl;
      return found->getclassed(n,v);
    } else {
      return v;
    }
  }
}

int MCIni::getclassed(MCString g, MCString n, int v) {
  MCString value(v);
  return (int)getclassed(g,n,value);
}

double MCIni::getclassed(MCString g, MCString n, double v) {
  MCString value(v);
  return (double)getclassed(g,n,value);
}

MCIni::MCIni() {
  head=0;
}

MCIni::MCIni(const MCIni& copyme) {
  // Copy constructor, so list is empty
  head=0;
  MCSettingList *source = copyme.head;
  MCSettingList *dest = 0;

  while(source!=0) {
    if (head==0) { // First item in list
      head = new MCSettingList(*source);
      dest = head;
    } else { // Other items
      dest->next = new MCSettingList(*source);
      dest=dest->next;
    }
    source = source->next;
  }
}

MCIni& MCIni::operator=(const MCIni& assignme) {
  MCSettingList *dest = 0;

  // Assignment so list may not be empty
  while(head!=0) {
    dest=head;
    head = head->next;
    delete dest;
  }

  MCSettingList *source = assignme.head;

  while(source!=0) {
    if (head==0) { // First item in list
      head = new MCSettingList(*source);
      dest = head;
    } else { // Other items
      dest->next = new MCSettingList(*source);
      dest=dest->next;
    }
    source = source->next;
  }
  return *this;
}

MCIni::~MCIni() {
  MCSettingList *done=0;

  while(head!=0) {
    done=head;
    head = head->next;
    delete done;
  }
}

MCSettingList *MCIni::find(MCString f) {
  if (head==0) return 0; // Fast get out clause

  MCSettingList *current = head;
  MCSettingList *retval = 0;

  while((current!=0) && (current->name<=f)) {
    retval = current; // retval points to the previous item.
    current = current->next;
  }

  return retval;
}




