// File    : MCMap.cpp
//
// Purpose : C++ Map class
// Author  : Matthew Hunter
// Version : 0.01 (08 Jun 1999)
//           0.02 (09 Jun 1999)
//                Now uses doubles for greater resolution.

#include "MCMap.h"
#include <string.h>
#include <fstream.h>
#include <iostream.h>

ostream& operator<<(ostream&s,MCMap d) {
  s << "P3" << endl;
  s << d.width << " " << d.height << endl;
  s << 255 << endl;
  int size = d.width*d.height;
  double max = d.mapdata[0];
  double min = d.mapdata[0];
  for(int i=1;i<size;i++) {
    if (d.mapdata[i]>max) max = d.mapdata[i];
    if (d.mapdata[i]<min) min = d.mapdata[i];
  }
  double scale=255/(max-min);
  for(int i=0;i<size;i++) {
    s << (int)((d.mapdata[i]-min)*scale) << " ";
    s << (int)((d.mapdata[i]-min)*scale) << " ";
    s << (int)((d.mapdata[i]-min)*scale) << endl;
  }
  return s;
}

MCMap MCMap::operator+(const MCMap& sum) const {
  if ((sum.width!=width) || (sum.height!=height)) {
    cout << "Maps must have same size to add.\n";
    return MCMap();
  }
  MCMap retval;
  int size = width*height;

  // Manually initialise the new map so the constructor does not
  // initialise all the values (we are going to write every value
  // anyway).
  retval.width=width;
  retval.height=height;
  retval.mapdata= new double[size];
  for(int i=0;i<size;i++) {
    retval.mapdata[i]==mapdata[i]+sum.mapdata[i];
  }
  return retval;
}

void MCMap::exportraw(MCString file) {
  ofstream to((const char*)file);
  if (to) {
    int x,y;
    for(y=0;y<height;y++) {
      for(x=0;x<width;x++) {
	to << x << "\t" << y << "\t" << mapdata[(y*width)+x] << endl;
      }
    }
  }
}

void MCMap::export(MCString file) {
  ofstream to((const char*)file);
  if (to) {
    to << *this;
  }
}

void MCMap::set(int x, int y, double value) {
  if ((x>=0) && (x<width) && (y>=0) && (y<height)) {
    mapdata[(y*width)+x]= value;
  }
}

double MCMap::get(int x, int y) {
  if ((x>=0) && (x<width) && (y>=0) && (y<height)) {
    return mapdata[(y*width)+x];
  } else {
    return 0.0;
  }
}

MCMap::MCMap() {
  width=0;
  height=0;
  mapdata=0;
}

MCMap::MCMap(const MCMap& copyme) {
  width=copyme.width;
  height=copyme.height;
  int size = width*height;
  mapdata = new double [size];
  if (mapdata==0) {
    width=0;
    height=0;
  } else {
    memmove(mapdata,copyme.mapdata,size*sizeof(double));
  }
}

MCMap::MCMap(int x,int y) {
  width = x;
  height = y;
  int size = width*height;
  mapdata = new double [size];
  if (mapdata==0) {
    width=0;
    height=0;
  } else {
    for (int j = 0; j<size;j++) {
      mapdata[j]=0.0;
    }
  }
}

MCMap& MCMap::operator=(const MCMap& assignme) {
  if (mapdata!=0) {
    delete[] mapdata;
  }

  width=assignme.width;
  height=assignme.height;
  int size = width*height;
  mapdata = new double[size];
  if (mapdata==0) {
    width=0;
    height=0;
  } else {
    memmove(mapdata,assignme.mapdata,size*sizeof(double));
  }
  return *this;
}

MCMap::~MCMap() {
  if (mapdata!=0) {
    delete[] mapdata;
  }
}



