// File    : MCSettingList.cpp
//
// Purpose : C++ SettingList class
// Author  : Matthew Hunter
// Version : 0.01 (08 Jun 1999)
//           0.02 (24 Jun 1999)

#include "MCSettingList.h"
#include <iostream.h>

ostream& operator<<(ostream&s,MCSettingList d) {
  MCSetting *current = d.head;
  s << "[" << d.name << "]" << endl;
  while(current!=0) {
    s << current->name << "=" << current->value << endl;
    current = current->next;
  }
  return s;
}

void MCSettingList::insert(MCString n, MCString v) {
  MCSetting *add = find(n);
  if (add==0) { // add at start of list
    add = new MCSetting(n,v);
    add->next = head;
    head = add;
  } else {
    if (n==add->name) { // Item already in list
      add->value = v;
    } else { // item not in list, add after current item
      MCSetting *temp = new MCSetting(n,v);
      temp->next = add->next;
      add->next = temp;
    }
  }
}

MCString MCSettingList::get(MCString n, MCString v) {
  MCSetting *found = find(n);
  if (found==0) { // n is not in the list
    return v;
  } else {
    if (n==found->name) { // item is in list
      return found->value;
    } else { // item is not in the list
      return v;
    }
  }
}

MCString MCSettingList::getclassed(MCString n, MCString v) {
  MCString entry;
  MCString root;
  MCString result;
  MCString empty;

  result=get(n,empty);
  if (result!=empty) {
    // Quick get out clause, if the entry is in return it.
    return result;
  }

  // Test the encompassing classes
  root=n;
  entry=root.tailtokenise(':');
  // Throw away the last class, it cannot be valid, if it were
  // the quick get out clause would have been triggered.
  result=root.tailtokenise(':');
  while(root.length()>0) {
    result=get((root+":")+entry,empty);
    if (result!=empty) {
      return result;
    }
    result = root.tailtokenise(':');
  }
  // Still haven't found what I'm looking for....
  result=get(entry,empty);
  if (result!=empty) {
    return result;
  }
  return v;
}

MCSettingList::MCSettingList() {
  name = MCString("Unknown");
  head=0;
  next=0;
}

MCSettingList::MCSettingList(const MCSettingList& copyme) {
  // Copy constructor, so list is empty
  head=0;
  MCSetting *source = copyme.head;
  MCSetting *dest = 0;
  name = copyme.name;

  while(source!=0) {
    if (head==0) { // First item in list
      head = new MCSetting(*source);
      dest = head;
    } else { // Other items
      dest->next = new MCSetting(*source);
      dest=dest->next;
    }
    source = source->next;
  }
  next=0;
}

MCSettingList& MCSettingList::operator=(const MCSettingList& assignme) {
  MCSetting *dest = 0;

  // Assignment so list may not be empty
  while(head!=0) {
    dest=head;
    head = head->next;
    delete dest;
  }

  MCSetting *source = assignme.head;
  name = assignme.name;

  while(source!=0) {
    if (head==0) { // First item in list
      head = new MCSetting(*source);
      dest = head;
    } else { // Other items
      dest->next = new MCSetting(*source);
      dest=dest->next;
    }
    source = source->next;
  }
  next=0;
  return *this;
}

MCSettingList::~MCSettingList() {
  MCSetting *done=0;

  while(head!=0) {
    done=head;
    head = head->next;
    delete done;
  }
}

MCSetting *MCSettingList::find(MCString f) {
  if (head==0) return 0; // Fast get out clause

  MCSetting *current = head;
  MCSetting *retval = 0;

  while((current!=0) && (current->name<=f)) {
    retval = current; // retval points to the previous item.
    current = current->next;
  }

  return retval;
}



