// File    : MCString.cpp
//
// Purpose : C++ String class
// Author  : Matthew Hunter
// Version : 0.01 (24 Mar 1999)
//           0.03 (07 Jun 1999)
//                Added readFile
//           0.04 (08 Jun 1999)
//                Added conversion functions for casting to integer and double
//                Added Constructors for creating from integer and double

#include "MCString.h"
#include "Mutil.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// Operations
// Define '+' as concatenation
MCString operator+(const char*cs, const MCString&s) {
  char *buf = new char[::strlen(cs) + s.length() + 1];
  ::strcpy(buf,cs);
  ::strcat(buf,s.rep->rep);
  MCString retval(&buf);
  return retval;
}

MCString MCString::operator+(const MCString&s) const {
  char *buf = new char[s.length() + length() + 1];
  ::strcpy(buf,rep->rep);
  ::strcat(buf,s.rep->rep);
  MCString retval(&buf);
  return retval;
}

MCString MCString::operator+(const char *s) const {
  char *buf = new char[::strlen(s) + length() + 1];
  ::strcpy(buf,rep->rep);
  ::strcat(buf,s);
  MCString retval(&buf);
  return retval;
}

MCString& MCString::operator+=(const MCString&s) {
  split();
  char *buf = new char[s.length() + length() + 1];
  ::strcpy(buf,rep->rep);
  ::strcat(buf,s.rep->rep);
  delete rep;
  rep = new SRep(&buf);
  return *this;
}

MCString& MCString::operator+=(const char *s) {
  split();
  char *buf = new char[::strlen(s) + length() + 1];
  ::strcpy(buf,rep->rep);
  ::strcat(buf,s);
  delete rep;
  rep = new SRep(&buf);
  return *this;
}


char& MCString::operator[](int index) {
  split();
  // Check whether the current string has multiple referants,
  // if so, seperate it out. This is not strictly necessary if
  // the [] are being used for reading, but are important
  // if they are being used for writing.
  if ((index<0) || (index>=length())) {
    index=0;
  }
  return rep->rep[index];
}

MCString::operator const char*() const {
  return rep->rep;
}

MCString::operator int() const {
  return atoi(rep->rep);
}

MCString::operator double() const {
  return atof(rep->rep);
}

int MCString::operator<(const MCString&s) const {
  if (compare(s)<0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::operator<=(const MCString&s) const {
  if (compare(s)<=0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::operator==(const MCString&s) const {
  if (compare(s)==0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::operator>=(const MCString&s) const {
  if (compare(s)>=0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::operator>(const MCString&s) const {
  if (compare(s)>0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::operator!=(const MCString&s) const {
  if (compare(s)!=0) {
    return -1;
  } else {
    return 0;
  }
}

int MCString::length() const {
  return ::strlen(rep->rep);
}

int MCString::compare(const MCString& s, int n) const {
  // Special case, the two strings a literally the same
  if (rep==s.rep) {
#ifndef NDEBUG
    cout << "MCString::compare\t: Special case triggered, reps identical.\n";
#endif
    return 0;
  }
  // Otherwise pass on to strcmp
  if (n==0) {
#ifndef NDEBUG
    cout << "MCString::compare\t: using strcmp on:\n";
    cout << "\t\t\t  " << rep->rep << endl;
    cout << "\t\t\t  " << s.rep->rep << endl;
#endif
    return ::strcmp(rep->rep,s.rep->rep);
  } else {
#ifndef NDEBUG
    cout << "MCString::compare\t: using strncmp on:\n";
    cout << "\t\t\t  " << rep->rep << endl;
    cout << "\t\t\t  " << s.rep->rep << endl;
    cout << "\t\t\t  " << n << endl;
#endif
    return ::strncmp(rep->rep,s.rep->rep,n);
  }
}

int MCString::index(const char ch) const {
  char *ptr = rep->rep;
  int offset=0;
  while(offset<length()) {
    if (ptr[offset]==ch) {
      return offset;
    }
    offset++;
  }
  return -1;
}

int MCString::rindex(const char ch) const {
  char *ptr = rep->rep;
  int offset=length()-1;
  while(offset>=0) {
    if (ptr[offset]==ch) {
      return offset;
    }
    offset--;
  }
  return -1;
}

MCString MCString::left(int n) const {
  n = min(n,length());
  char *buf = new char[n+1];
  ::strncpy(buf,rep->rep,n);
  buf[n]=0;
  MCString retval(&buf);
  return retval;
}

void MCString::tail(const int n) {
#ifndef NDEBUG
  cout << "MCString::tail: starting" << endl;
  cout << *this << endl;
#endif
  if (n<length()) {
    // There is some string left
    char *nstart=rep->rep+n;
    char *buf = new char[::strlen(nstart)+1];
    ::strcpy(buf,nstart);
    delete rep;
    rep = new SRep(&buf);
  } else {
    delete rep;
    rep=new SRep("");
  }
#ifndef NDEBUG
  cout << "MCString::tail: done" << endl;
  cout << *this << endl;
#endif
}

MCString MCString::shift(int n) {
#ifndef NDEBUG
  cout << "MCString::shift: Starting" << endl;
#endif
  split();
  MCString retval = left(n);
  tail(n);
#ifndef NDEBUG
  cout << "MCString::shift: Done" << endl;
#endif
  return retval;
}

MCString MCString::nexttoken(int n) {
  split();
  MCString retval = left(n);
  tail(n+1);
  return retval;
}

MCString MCString::tokenise(const char ch) {
  int offset=index(ch);
  if (offset==-1) {
    offset=length();
  }
  return nexttoken(offset);
}

MCString MCString::tailtoken(int n) {
  split();
  MCString head = left(n);
  tail(n+1);
  if (this->length()==0) {
    return head;
  } else {
    MCString retval(*this);
    *this=head;
    return retval;
  }
}

MCString MCString::tailtokenise(const char ch) {
  int offset=rindex(ch);
  if (offset==-1) {
    offset=length();
  }
  return tailtoken(offset);
}

MCString MCString::readFile(void) {
  FILE *textfile;
  MCString retval = MCString();

  textfile=fopen(rep->rep,"r");

  // If the file can not be opened, return an empty string.
  if (!textfile) return retval;
 
  // Find the length of the file by moving to the end.
  fseek(textfile,0,SEEK_END);
  int length = ftell(textfile);
  // Reset the file so it can be read from.
  rewind(textfile);

#ifndef NDEBUG
  cout << "readFile: Length is " << length << " characters.\n";
#endif

  // If the file is empty, return an empty string.
  if (!length) return retval;

  // Allocate the space needed to store it
  char *buf=new char[length+1];
  int got = fread(buf,length,1,textfile);
  fclose(textfile);

#ifndef NDEBUG
  cout << "readFile: read " << got << " items.\n";
#endif

  // Check the read was successful
  if (!got) return retval;

  // Create an MCString from the buffer, and return it.
  retval = MCString(&buf);
  return retval;
}


// Variations on the standard member functions
MCString::MCString(const char *s) {
  rep = new SRep(s);
}

MCString::MCString(int val) {
  char buf[32];
  sprintf(buf,"%d",val);
  rep = new SRep(buf);
}

MCString::MCString(double val) {
  char buf[32];
  sprintf(buf,"%f",val);
  rep = new SRep(buf);
}

// Output
ostream& operator<<(ostream&s,MCString d) {
  return s << d.rep->rep;
}

// Standard member functions
// Standard constructors
MCString::MCString() {
  rep = new SRep("");
}

MCString::MCString(const MCString& s) {
  rep = s.rep;
  rep->count++;
}

MCString::MCString(char **s) {
  rep = new SRep(s);
}

// Standard operations
MCString& MCString::operator=(const MCString& s) {
  s.rep->count++;
  if (--rep->count == 0) {
    delete rep;
  }
  rep = s.rep;
  return *this;
}

// Destructor
MCString::~MCString() {
  if (--rep->count <=0) { delete rep; }
}

void MCString::split() {
  if (rep->count>1) {
#ifndef NDEBUG
    cout << "MCString::split: Warning string has multiple referees, splitting" << endl;
#endif
    rep->count--;
    rep= new SRep(rep->rep);
#ifndef NDEBUG
    cout << "MCString::split: split complete." << endl;
#endif
  }
}

MCString::SRep::SRep(char **s) {
  rep = *s;
  count=1;
  *s=0;
}

MCString::SRep::SRep(const char *s) {
  ::strcpy(rep = new char[::strlen(s)+1],s);
  count=1;
}

MCString::SRep::~SRep() {
  delete rep;
}






