// File    : MCString.h
//
// Purpose : C++ String class
// Author  : Matthew Hunter
// Version : 0.01 (24 Mar 1999)
//           0.02 (31 Mar 1999)
//                Added nexttoken, tokenise
//           0.03 (07 Jun 1999)
//                Added readFile
//           0.04 (08 Jun 1999)
//                Added conversion functions for casting to integer and double
//                Added Constructors for creating from integer and double

#ifndef MCString_h
#define MCString_h

#include <iostream.h>

class MCString {
 public:
  // Operations
  // Define '+' as concatenation
  friend MCString operator+(const char*, const MCString&);
  MCString operator+(const MCString&) const;
  MCString operator+(const char *) const;
  MCString& operator+=(const MCString&);
  MCString& operator+=(const char *);

  // Subscript operator
  char& operator[](int);

  // Type conversion
  operator const char*() const;
  operator int() const;
  operator double() const;


  // Standard compare functions
  int operator<(const MCString&) const;
  int operator<=(const MCString&) const;
  int operator==(const MCString&) const;
  int operator>(const MCString&) const;
  int operator>=(const MCString&) const;
  int operator!=(const MCString&) const;

  // Functions
  int length() const; // strlen
  int compare(const MCString&, int n=0) const; // strcmp
  int index(const char) const; // index like, returns offset within
                               // string, not pointer
  int rindex(const char) const; // rindex like, returns offset within
                                // string, not pointer

  MCString left(const int) const; // Return a new string consisting of the
                                  // leftmost characters of the current string.
  void tail(const int); // Remove the lead characters from the string.
  MCString shift(const int); // Return a new string consisting of the
                             // leftmost characters of the current string,
                             // then shift the current string left by the
                             // same amount.
  MCString nexttoken(int); // As for shift, except that the character
                                 // at the index passed is assumed to be
                                 // a token and is not included in the new
                                 // returned string, and is discarded from
                                 // the original.
  MCString tokenise(const char); // The first occurence of char found in
                                 // the string is located and the string
                                 // broken at that point (as for nexttoken).
  MCString tailtoken(int); // As for nexttoken, except that the token is
                           // extracted from the end of the string
  MCString tailtokenise(const char);
  MCString readFile(void); // Treats the string as a filename to a text file,
                           // reads the contents of the file, returning the
                           // contents as a new string.

  // Output
  friend ostream& operator<<(ostream&s,MCString d);

  // Variations on the standard member functions
  MCString(const char *); // Initialise from 'C' style string
  MCString(int); // Create a string from an integer value
  MCString(double); // Create a string from a double or float value

  // Standard member functions
  // Standard constructors
  MCString();  // Default constructor
  MCString(const MCString&); // Copy constructor

  // Standard operations
  MCString& operator=(const MCString&); // assignment

  // Destructor
  ~MCString();
 private:
  class SRep {
  public:
    SRep(char **s);
    SRep(const char *s);
    ~SRep();
    char *rep;
    int count;
  };
  SRep *rep;
  MCString(char **s);
  void split();
};

#endif



