// File    : MCXfig.cpp
//
// Purpose : C++ Xfig class
// Author  : Matthew Hunter
// Version : 0.01 (5 Jul 1999)
//           0.02 (12 Jul 1999)
//                Seperated construction and creation of the associated
//                file.

#include "MCXfig.h"

#include <math.h>
#include <stdio.h>
#include <iostream.h>

#ifdef NDEBUG
#define XNDEBUG
#endif

#ifndef PI
#define PI 3.14159265358979323846
#endif

ostream& operator<<(ostream&s,MCXfig d) {
}

void MCXfig::comment(MCString text) {
  if (fp!=0) {
    fprintf(fp,"#%s\n",(const char *)text);
  }
}

void MCXfig::drawPoint(float x, float y) {
#ifndef XNDEBUG
  cout << "Drawing point (" << x << "," << y << ") with scale ";
  cout << scale << ".\n";
#endif
  fprintf(fp,"%d %d",(int)(450.0*scale*x),(int)(450.0*scale*y));
}

void MCXfig::drawBox(float x0, float y0,
		     float x1, float y1) {
  float x[5];
  float y[5];
  if (fp!=0) {
    x[0]=x0;
    y[0]=y0;
    x[1]=x1;
    y[1]=y0;
    x[2]=x1;
    y[2]=y1;
    x[3]=x0;
    y[3]=y1;
    x[4]=x0;
    y[4]=y0;
    drawPolyLine(2,5,x,y);
  }
}

void MCXfig::drawCircle(float cx, float cy, float r) {
  float rx,ry;

  if (fp!=0) {
    fprintf(fp,"1 "); // object code 2 for polylines
    fprintf(fp,"3 "); // sub type 2 for box
    fprintf(fp,"%d ",line_style);
    fprintf(fp,"%d ",thickness);
    fprintf(fp,"%d ",colour);
    fprintf(fp,"%d ",fill);
    fprintf(fp,"%d ",depth);
    fprintf(fp,"0 "); // pen style not used by xfig
    if (fill!=-1) {
      fprintf(fp,"20 "); // area_fill
    } else {
      fprintf(fp,"-1 ");
    }
    fprintf(fp,"0.000 "); // style val?
    fprintf(fp,"1 "); // direction
    fprintf(fp,"0.0000 "); // angle of x axis
    drawPoint(cx+ox,cy+oy); fputc(' ',fp);
    drawPoint(r,r); fputc(' ',fp);
    drawPoint(cx+ox,cy+oy); fputc(' ',fp);
    drawPoint(cx+r+ox,cy+r+oy); fputc('\n',fp);
  }
}

void MCXfig::drawArc(float x, float y,float radius, float a1, float a2) {
  float rad1,rad2,rad3;
  float cx,cy;

  cx = x+ox;
  cy = y+oy;

  rad1 = (a1*PI)/180.0;
  rad3 = (a2*PI)/180.0;
  rad2 = (rad1+rad2)/2.0;

  if (fp!=0) {
    fprintf(fp,"5 "); // object code 5 for arcs
    fprintf(fp,"1 "); // sub type 1 for open ended
    fprintf(fp,"%d ",line_style);
    fprintf(fp,"%d ",thickness);
    fprintf(fp,"%d ",colour);
    fprintf(fp,"%d ",fill);
    fprintf(fp,"%d ",depth);
    fprintf(fp,"0 "); // pen style not used by xfig
    if (fill!=-1) {
      fprintf(fp,"20 "); // area_fill
    } else {
      fprintf(fp,"-1 ");
    }
    fprintf(fp,"0.000 "); // style val?
    fprintf(fp,"%d ",cap);
    fprintf(fp,"1 "); // direction
    fprintf(fp,"%d ",head);
    fprintf(fp,"%d ",tail);
    drawPoint(cx,cy); fputc(' ',fp);

    drawPoint((radius*cos(rad1))+cx,(radius*sin(rad1))+cy); fputc(' ',fp);
    drawPoint((radius*cos(rad2))+cx,(radius*sin(rad2))+cy); fputc(' ',fp);
    drawPoint((radius*cos(rad3))+cx,(radius*sin(rad3))+cy); fputc('\n',fp);

    if (head) {
      fprintf(fp,"\n\t%d ",arrow);
      fprintf(fp,"%d ",arrow_style);
      fprintf(fp,"%f ",athick);
      fprintf(fp,"%f ",awidth);
      fprintf(fp,"%f ",aheight);
    }
    if (tail) {
      fprintf(fp,"\n\t%d ",arrow);
      fprintf(fp,"%d ",arrow_style);
      fprintf(fp,"%f ",athick);
      fprintf(fp,"%f ",awidth);
      fprintf(fp,"%f ",aheight);
    }
  }
}

void MCXfig::drawLine(float x0,float y0, float x1,float y1) {
  float x[2];
  float y[2];
  if (fp!=0) {
    x[0]=x0;
    y[0]=y0;
    x[1]=x1;
    y[1]=y1;
    drawPolyLine(1,2,x,y);
  }
}

void MCXfig::drawPolyLine(int type, int pcount, float *x, float *y) {
  int i;

  if (fp!=0) {
    fprintf(fp,"2 "); // object code 2 for polylines
    fprintf(fp,"%d ",type);
    fprintf(fp,"%d ",line_style);
    fprintf(fp,"%d ",thickness);
    fprintf(fp,"%d ",colour);
    fprintf(fp,"%d ",fill);
    fprintf(fp,"%d ",depth);
    fprintf(fp,"0 "); // pen style not used by xfig
    if (fill!=-1) {
      fprintf(fp,"20 "); // area_fill
    } else {
      fprintf(fp,"-1 ");
    }
    fprintf(fp,"0.000 "); // style val?
    fprintf(fp,"%d ",join);
    fprintf(fp,"%d ",cap);
    fprintf(fp,"7 "); // arc-box radius
    fprintf(fp,"%d ",head);
    fprintf(fp,"%d ",tail);
    fprintf(fp,"%d ",pcount); // number of points in line
    if (head) {
      fprintf(fp,"\n\t%d ",arrow);
      fprintf(fp,"%d ",arrow_style);
      fprintf(fp,"%f ",athick);
      fprintf(fp,"%f ",awidth);
      fprintf(fp,"%f ",aheight);
    }
    if (tail) {
      fprintf(fp,"\n\t%d ",arrow);
      fprintf(fp,"%d ",arrow_style);
      fprintf(fp,"%f ",athick);
      fprintf(fp,"%f ",awidth);
      fprintf(fp,"%f ",aheight);
    }
    fprintf(fp,"\n\t");
    for(i=0;i<pcount;i++) {
      drawPoint(x[i]+ox,y[i]+oy); fputc(' ',fp);
    }
    fputc('\n',fp);
  }
}

void MCXfig::drawText(MCString text,float x, float y, float w, float h) {
  int fontcode;

  if (fp!=0) {
    fprintf(fp,"4 "); // object code 4 for text
    fprintf(fp,"%d ",tjustify);
    fprintf(fp,"%d ",colour);
    fprintf(fp,"%d ",depth);
    fprintf(fp,"0 "); // pen style not used by xfig
    if (fontbase==-1) {
      fontcode=-1;
    } else {
      if (fontbase>31) {
	fontcode=fontbase;
      } else {
	fontcode=fontbase+bold+italic;
      }
    }
    fprintf(fp,"%d ",fontcode); // default font
    fprintf(fp,"%f ",tsize*scale);
    fprintf(fp,"%f ",tangle);
    fprintf(fp,"4 "); // arc-box radius
    fprintf(fp,"%f ",h*scale*450.0);
    fprintf(fp,"%f ",w*scale*450.0);
    fprintf(fp,"%d ",(int)((x+ox)*scale*450.0));
    fprintf(fp,"%d ",(int)((y+oy)*scale*450.0));
    fprintf(fp,"%s\\001\n",(const char *)text);
  }
}

void MCXfig::insertText(MCString text) {
  drawText(text,tx,ty,tw,ts);
  ty = ty + ts;
}

void MCXfig::plotPoint(float x, float y) {
  switch(pstyle) {
  case 0:
    drawLine(x-psize,y-psize,x+psize,y+psize);
    drawLine(x-psize,y+psize,x+psize,y-psize);
    break;
  case 1:
    drawBox(x-psize,y-psize,x+psize,y+psize);
    break;
  case 2:
    drawCircle(x,y,psize);
    break;
  }
}
  
void MCXfig::setOrigin(float x, float y) {
  ox=x;
  oy=y;
}

// Line style
void MCXfig::setDefaultLine(void) {
  line_style=-1;
}

void MCXfig::setSolidLine(void) {
  line_style=0;
}

void MCXfig::setDashedLine(void) {
  line_style=1;
}

void MCXfig::setDottedLine(void) {
  line_style=2;
}

void MCXfig::setDashedDottedLine(void) {
  line_style=3;
}

void MCXfig::setDashedDoubleDottedLine(void) {
  line_style=4;
}

void MCXfig::setDashedTripleDottedLine(void) {
  line_style=5;
}

// Join Style
void MCXfig::setMiterJoin(void) {
  join=0;
}

void MCXfig::setBevelJoin(void) {
  join=1;
}

void MCXfig::setRoundJoin(void) {
  join=2;
}

// Cap Style
void MCXfig::setButtCap(void) {
  cap=0;
}

void MCXfig::setRoundCap(void) {
  cap=1;
}

void MCXfig::setProjectingCap(void) {
  cap=2;
}

// Arrow Type
void MCXfig::setStickArrow(void) {
  arrow=0;
}

void MCXfig::setClosedArrow(void) {
  arrow=1;
}

void MCXfig::setIndentArrow(void) {
  arrow=2;
}

void MCXfig::setOutdentArrow(void) {
  arrow=4;
}

// Arrow Style
void MCXfig::setOutlineArrow(void) {
  arrow_style=0;
}

void MCXfig::setFilledArrow(void) {
  arrow_style=1;
}

// Arrow switches
void MCXfig::setArrowHeadOn(void) {
  head=1;
}

void MCXfig::setArrowHeadOff(void) {
  head=0;
}

void MCXfig::setArrowTailOn(void) {
  tail=1;
}

void MCXfig::setArrowTailOff(void) {
  tail=0;
}

// Arrow sizes
void MCXfig::setArrowThickness(float at) {
  athick=at;
}

void MCXfig::setArrowWidth(float w) {
  if (w>0) {
    awidth=w;
  } else {
    awidth=60.0;
  }
}

void MCXfig::setArrowHeight(float h) {
  if (h>0) {
    aheight=h;
  } else {
    aheight=120.0;
  }
}

// Colours
void MCXfig::setDefault(void) {
  colour = -1;
}

void MCXfig::setBlack(void) {
  colour = 0;
}

void MCXfig::setBlue(void) {
  colour = 1;
}

void MCXfig::setGreen(void) {
  colour = 2;
}

void MCXfig::setCyan(void) {
  colour = 3;
}

void MCXfig::setRed(void) {
  colour = 4;
}

void MCXfig::setMagenta(void) {
  colour = 5;
}

void MCXfig::setYellow(void) {
  colour = 6;
}

void MCXfig::setWhite(void) {
  colour = 7;
}

void MCXfig::setBlueShade(int s) {
  s = s > 3 ? 3 : s;
  s = s < 0 ? 0 : s;
  colour = 8 + s;
}

void MCXfig::setGreenShade(int s) {
  s = s > 2 ? 2 : s;
  s = s < 0 ? 0 : s;
  colour = 12 + s;
}

void MCXfig::setCyanShade(int s) {
  s = s > 2 ? 2 : s;
  s = s < 0 ? 0 : s;
  colour = 15 + s;
}

void MCXfig::setRedShade(int s) {
  s = s > 2 ? 2 : s;
  s = s < 0 ? 0 : s;
  colour = 18 + s;
}

void MCXfig::setMagentaShade(int s) {
  s = s > 2 ? 2 : s;
  s = s < 0 ? 0 : s;
  colour = 21 + s;
}

void MCXfig::setBrownShade(int s) {
  s = s > 2 ? 2 : s;
  s = s < 0 ? 0 : s;
  colour = 24 + s;
}

void MCXfig::setPinkShade(int s) {
  s = s > 3 ? 3 : s;
  s = s < 0 ? 0 : s;
  colour = 27 + s;
}

void MCXfig::setGold() {
  colour = 31;
}

// Other styles
void MCXfig::setThickness(int t) {
  t = t < 1 ? 1 : t;
  thickness=t;
}

void MCXfig::setDepth(int d) {
  d = d < 0 ? 0 : d;
  d = d > 999 ? 999 : d;
  depth=d;
}

void MCXfig::setFill(void) {
  fill = colour;
}

void MCXfig::setScale(float s) {
  scale=s;
}

// Text stuff
void MCXfig::setLeftJustify(void) {
  tjustify=0;
}

void MCXfig::setCentreJustify(void) {
  tjustify=1;
}

void MCXfig::setRightJustify(void) {
  tjustify=2;
}

void MCXfig::setFontSize(float s) {
  tsize=s;
}

void MCXfig::setFontAngle(float a) {
  tangle=a;
}

void MCXfig::setBoldOn(void) {
  bold=2;
}

void MCXfig::setBoldOff(void) {
  bold=0;
}

void MCXfig::setItalicOn(void) {
  italic=1;
}

void MCXfig::setItalicOff(void) {
  italic=0;
}

void MCXfig::setFontDefault(void) {
  fontbase=-1;
}

void MCXfig::setFontTimes(void) {
  fontbase=0;
}

void MCXfig::setFontAvantGarde(void) {
  fontbase=4;
}

void MCXfig::setFontBookman(void) {
  fontbase=8;
}

void MCXfig::setFontCourier(void) {
  fontbase=12;
}

void MCXfig::setFontHelvetica(void) {
  fontbase=16;
}

void MCXfig::setFontHelveticaNarrow(void) {
  fontbase=20;
}

void MCXfig::setFontNewCenturySchoolbook(void) {
  fontbase=24;
}

void MCXfig::setFontPalatino(void) {
  fontbase=28;
}

void MCXfig::setFontSymbol(void) {
  fontbase=32;
}

void MCXfig::setFontZapfChancery(void) {
  fontbase=33;
}

void MCXfig::setZapfDingbats(void) {
  fontbase=34;
}

void MCXfig::setLineSpacing(float s, float w) {
  ts = s;
  tw = w;
}

void MCXfig::setTextOrigin(float x,float y) {
  tx = x;
  ty = y;
}


void MCXfig::setPointSize(float ps) {
  if (ps>0) {
    psize=ps;
  }
}

void MCXfig::setCrossPoint(void) {
  pstyle=0;
}

void MCXfig::setSquarePoint(void) {
  pstyle=1;
}

void MCXfig::setCirclePoint(void) {
  pstyle=2;
}

MCXfig::MCXfig(MCString file) {
  openfile(file);
}

MCXfig::MCXfig() {
  fp=0;
}

MCXfig::MCXfig(const MCXfig& copyme) {
  fp=0;
}

MCXfig& MCXfig::operator=(const MCXfig& assignme) {
  fp=0;
}

MCXfig::~MCXfig() {
  closefile();
}

void MCXfig::openfile(MCString filename) {
  fp = fopen(filename,"w");
  if (fp!=0) {
#ifndef XNDEBUG
    cout << "Opened xfig file " << filename << " successfully.\n";
#endif
    fprintf(fp,"#FIG 3.2\n");
    fprintf(fp,"Landscape\n");
    fprintf(fp,"Center\n");
    fprintf(fp,"Metric\n");
    fprintf(fp,"Letter\n");
    fprintf(fp,"100.00\n");
    fprintf(fp,"Single\n");
    fprintf(fp,"-2\n");
    fprintf(fp,"1200 2\n");
  }
#ifndef XNDEBUG
  cout << "Initialising styles.\n";
#endif
  line_style=-1;
  join=0;;
  cap=0;
  arrow=0;
  arrow_style=1;
  head=0;
  tail=0;
  athick=1.0;
  awidth=60.0;
  aheight=120.0;
  colour=-1;
  thickness=1;
  depth=100;
  fill=-1;
  scale = 1.0;
  tjustify=0;
  tsize=16.0;
  tangle=0.0;
  fontbase=-1;
  italic=0;
  bold=0;
  psize=0.5;
  pstyle=0;
  tx=0;
  ty=0;
  ts=0;
  tw=0;
#ifndef XNDEBUG
  cout << "Done openfile.\n";
#endif
  
}

void MCXfig::closefile() {
#ifndef XNDEBUG
  cout << "Starting closefile.\n";
#endif
  if (fp!=0) {
#ifndef XNDEBUG
    cout << "closing file.\n";
#endif
    fclose(fp);
  }
#ifndef XNDEBUG
  cout << "Exitinging closefile.\n";
#endif
}




