// File    : MCXfig.h
//
// Purpose : C++ Xfig class
// Author  : Matthew Hunter
// Version : 0.01 (5 Jul 1999)
//           0.02 (12 Jul 1999)
//                Seperated construction and creation of the associated
//                file.
//           0.03 (15 Jul 1999)
//                Added drawArc

#ifndef MCXfig_h
#define MCXfig_h

#include "MCString.h"
#include <iostream.h>
#include <stdio.h>

class MCXfig {
 public:
  // Output
  friend ostream& operator<<(ostream&s,MCXfig);

  // Standard member functions
  MCXfig(MCString file);

  void openfile(MCString);
  void closefile();

  // Objects
  void comment(MCString);
  void drawBox(float, float, float, float);
  void drawCircle(float,float,float);
  void drawArc(float,float,float,float,float);
  void drawLine(float,float, float,float);
  void drawPolyLine(int type, int pcount, float *x, float *y);
  void drawText(MCString text,float x, float y, float w, float h);
  void insertText(MCString text);
  void plotPoint(float x, float y);

  void setOrigin(float,float);

  // Line style
  void setDefaultLine(void);
  void setSolidLine(void);
  void setDashedLine(void);
  void setDottedLine(void);
  void setDashedDottedLine(void);
  void setDashedDoubleDottedLine(void);
  void setDashedTripleDottedLine(void);

  // Join Style
  void setMiterJoin(void);
  void setBevelJoin(void);
  void setRoundJoin(void);

  // Cap Style
  void setButtCap(void);
  void setRoundCap(void);
  void setProjectingCap(void);

  // Arrow Type
  void setStickArrow(void);
  void setClosedArrow(void);
  void setIndentArrow(void);
  void setOutdentArrow(void);
  
  // Arrow Style
  void setOutlineArrow(void);
  void setFilledArrow(void);

  // Arrow switches
  void setArrowHeadOn(void);
  void setArrowHeadOff(void);
  void setArrowTailOn(void);
  void setArrowTailOff(void);

  // Arrow sizes
  void setArrowThickness(float);
  void setArrowWidth(float);
  void setArrowHeight(float);

  // Colours
  void setDefault(void);
  void setBlack(void);
  void setBlue(void);
  void setGreen(void);
  void setCyan(void);
  void setRed(void);
  void setMagenta(void);
  void setYellow(void);
  void setWhite(void);
  void setBlueShade(int);
  void setGreenShade(int);
  void setCyanShade(int);
  void setRedShade(int);
  void setMagentaShade(int);
  void setBrownShade(int);
  void setPinkShade(int);
  void setGold();

  // Other styles
  void setThickness(int);
  void setDepth(int);
  void setFill(void);
  void setScale(float);

  // Text stuff
  void setLeftJustify(void);
  void setCentreJustify(void);
  void setRightJustify(void);
  void setFontSize(float);
  void setFontAngle(float);
  void setBoldOn(void);
  void setBoldOff(void);
  void setItalicOn(void);
  void setItalicOff(void);
  void setFontDefault(void);
  void setFontTimes(void);
  void setFontAvantGarde(void);
  void setFontBookman(void);
  void setFontCourier(void);
  void setFontHelvetica(void);
  void setFontHelveticaNarrow(void);
  void setFontNewCenturySchoolbook(void);
  void setFontPalatino(void);
  void setFontSymbol(void);
  void setFontZapfChancery(void);
  void setZapfDingbats(void);

  void setLineSpacing(float,float);
  void setTextOrigin(float,float);

  // Point stuff
  void setPointSize(float);
  void setCrossPoint();
  void setSquarePoint();
  void setCirclePoint();

  // Standard constructors
  MCXfig();  // Default constructor
  MCXfig(const MCXfig&); // Copy constructor

  // Standard operations
  MCXfig& operator=(const MCXfig&); // assignment

  // Destructor
  ~MCXfig();
 private:
  void drawPoint(float x, float y);

  FILE *fp;
  int line_style;
  int join;
  int cap;
  int arrow;
  int arrow_style;
  int head;
  int tail;
  float athick;
  float awidth;
  float aheight;
  int colour;
  int fill;
  int thickness;
  int depth;
  float scale;
  int tjustify;
  float tsize;
  float tangle;
  int fontbase;
  int bold;
  int italic;
  float ox,oy;
  float psize;
  int pstyle;
  float tx,ty,ts,tw;
};

#endif



