// File    : MCPLBallTrackerPos.cpp
//
// Purpose : C++ PLBallTrackerPos class
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)
//                Code based on Marker

#include "MCPLBallTrackerPos.h"
#include "MCPLGeometry.h"

#include "MCPLPitchFig.h"

#include <iostream.h>

MCString MCPLBallTrackerPos::ident(void) {
  return MCString("BallTracker");
}

void MCPLBallTrackerPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}


point MCPLBallTrackerPos::findposition(MCPLStatus *status) {

  MCPLPitchFig diag;

  if (dinterval) {
    if (!(dicount % dinterval)) {
#ifdef MCPLDBBallTrackerPos
      cout << "Drawing diagram." << endl;
#endif
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      for(int i=0;i<status->teamcount;i++) {
	diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
      }
      for(int i=0;i<status->opponentcount;i++) {
	diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
      }
      diag.setDefault();
      diag.setFill();
    }
  }

  if (constraint.hmly) {
    if ((status->ball.X>constraint.hlimit.right) ||
	(status->ball.X<constraint.hlimit.left) ||
	(status->ball.Y>constraint.hlimit.top) ||
	(status->ball.Y<constraint.hlimit.bottom)) {
      if (dinterval) {
	if (!(dicount % dinterval)) {
	  diag.setFontSize(128);
	  diag.setRed();
	  diag.setDepth(20);
	  diag.setCentreJustify();
	  diag.drawText(MCString("Resorting to failsafe"),0,0,40,5);
	  diag.drawText(MCString("(Ball outside home area)"),0,10,40,5);
	  diag.setLeftJustify();
	}
      }
      // Object is too close
      if (next!=0) {
	return next->findposition(status);
      } else {
	return home;
      }
    }
  }

  MCPLPFLOAT bdist = separation(status->ball,status->owngoal,1);

  if ((constraint.fail) && (bdist<ranges.mindist)) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	diag.setCentreJustify();
	diag.drawText(MCString("Resorting to failsafe"),0,0,40,5);
	diag.drawText(MCString("(Proximity Failure)"),0,10,40,5);
	diag.setLeftJustify();
      }
    }
    // Object is too close
    if (next!=0) {
      return next->findposition(status);
    } else {
      return home;
    }
  }

  MCPLPFLOAT rdist = ranges.reldist;
  if ((ranges.reldist*bdist)<ranges.mindist) {
    rdist=ranges.mindist/bdist;
  }

#ifdef MCPLDBBallTrackerPos
  cout << "Relative distance is " << rdist << ".\n";
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setFill();
      diag.setBlack();
      diag.setDashedLine();
      diag.drawLine(status->ball.X,status->ball.Y,
		    status->owngoal.X,status->owngoal.Y);
      if (constraint.fmgl) {
	if (ranges.mindist!=0) {
	  diag.drawCircle(status->owngoal.X,status->owngoal.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(status->owngoal.X,status->owngoal.Y,
			  bdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(status->owngoal.X,status->owngoal.Y,rdist*bdist);
      } else {
	if (ranges.mindist!=0) {
	  diag.drawCircle(status->ball.X,status->ball.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(status->ball.X,status->ball.Y,bdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(status->ball.X,status->ball.Y,rdist*bdist);
      }
    }
  }

  point retval;

  retval=targetpoint(status->ball,status->owngoal,rdist);

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setSolidLine();
      diag.setFill();
      diag.setBlue();
      diag.setDepth(80);
      diag.setSquarePoint();
      diag.setPointSize(1);
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
  return retval;
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLBallTrackerPos::make(MCString method,
				    MCString player,
				    MCString spec,
				    MCIni& ini,
				    MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;
  if (method==ident()) {
    retval = new MCPLBallTrackerPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLBallTrackerPos::MCPLBallTrackerPos(MCString method,
			     MCString player,
			     MCString spec,
			     MCIni &ini,
			     MCPLPositioner *failsafe)
  :MCPLTrackerPos(method,player,spec,ini,failsafe) {

  resetoptions();
  readoptions(method,ini);
}

MCPLBallTrackerPos::MCPLBallTrackerPos(MCExemplar e): MCPLTrackerPos(e) {
}

MCPLBallTrackerPos::MCPLBallTrackerPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
  resetoptions();
}

MCPLBallTrackerPos::MCPLBallTrackerPos(const MCPLBallTrackerPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
  pitch = copyme.pitch;
  ranges = copyme.ranges;
  constraint = copyme.constraint;
}

MCPLBallTrackerPos& MCPLBallTrackerPos::operator=(const MCPLBallTrackerPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  pitch = assignme.pitch;
  ranges = assignme.ranges;
  constraint = assignme.constraint;
  return *this;
}

MCPLBallTrackerPos::~MCPLBallTrackerPos() {
}

static MCPLBallTrackerPos MCPLBallTrackerPosInstance(ExemplarInstance);













