// File    : MCPLBoxMapLRPos.cpp
//
// Purpose : C++ PLBoxMapLRPos class
// Author  : Matthew Hunter
// Version : 0.01 (23 Jun 1999)
//           0.02 (24 Jun 1999)
//                Extended to include home limits
//           0.03 (29 Jun 1999)
//                Modified to use new point structure.
//                Modified to use specialisation classed options
//                when reading defaults.
//           0.04 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.05 (20 Jul 1999)
//                Minor revisions to status handling.
//           0.06 (21 Jul 1999)
//                Minor modifications to constrainBall and
//                constrainBox to check offside limits after
//                checking home limits to ensure the offside
//                limits are enforced. This may lead to a
//                returned position being out of the home area if
//                there is not enough space between the offside
//                line and the edge of the home area.

#include "MCPLBoxMapLRPos.h"
#include "MCPLPitchFig.h"

#ifdef USEMAP
#include "MCMap.h"
#endif

#include <iostream.h>

static const int NOCONSTRAINT = 0;
static const int BALLCONSTRAINT = 1;
static const int BOXCONSTRAINT = 2;

point MCPLBoxMapLRPos::constrainBall(point ball,MCPLStatus *status) {
  // Constraint 1, ensure that the test area remains within the
  // area of the pitch
#ifdef MCPLDBBoxMapLRPos
  cout << "Constraining ball from ";
  cout << "(" << ball.X << "," << ball.Y << ").\n";
#endif
  if (constraint.boundary==BALLCONSTRAINT) {
    if (ball.X+box.width>pitch.right) {
      // Ball is too close to opponents goal
      ball.X = pitch.right-box.width;
    } else {
      // Cannot be too close to both ends, unless the box width
      // is set to something silly, in which case the user
      // desverves what they get.
      if (ball.X-box.width<pitch.left) {
	// Ball is too close to own goal
	ball.X = pitch.left+box.width;
      }
    }
    if (ball.Y+box.height>pitch.top) {
      // Ball is too close to opponents goal
      ball.Y = pitch.top-box.height;
    } else {
      if (ball.Y-box.height<pitch.bottom) {
	// Ball is too close to own goal
	ball.Y = pitch.bottom+box.height;
      }
    }
  }
  if (constraint.home==BALLCONSTRAINT) {
    if (ball.X+box.width>constraint.hlimit.right) {
      // Ball is too close to opponents goal
      ball.X = constraint.hlimit.right-box.width;
    } else {
      // Cannot be too close to both ends, unless the box width
      // is set to something silly, in which case the user
      // desverves what they get.
      if (ball.X-box.width<constraint.hlimit.left) {
	// Ball is too close to own goal
	ball.X = constraint.hlimit.left+box.width;
      }
    }
    if (ball.Y+box.height>constraint.hlimit.top) {
      // Ball is too close to opponents goal
      ball.Y = constraint.hlimit.top-box.height;
    } else {
      if (ball.Y-box.height<constraint.hlimit.bottom) {
	// Ball is too close to own goal
	ball.Y = constraint.hlimit.bottom+box.height;
      }
    }
  }
  if (constraint.offside==BALLCONSTRAINT) {
    if (ball.X+box.width>status->offside) {
      ball.X = status->offside-box.width;
    }
  }
#ifdef MCPLDBBoxMapLRPos
  cout << "Constraining ball to ";
  cout << "(" << ball.X << "," << ball.Y << ").\n";
#endif
  return ball;
}

point MCPLBoxMapLRPos::constrainBox(point centre,MCPLStatus *status) {
  // Constraint 1, ensure that the test area remains within the
  // area of the pitch
  MCPLPFLOAT bw2 = box.width/2.0;
  MCPLPFLOAT bh2 = box.height/2.0;

#ifdef MCPLDBBoxMapLRPos
  cout << "Constraining box from ";
  cout << "(" << centre.X << "," << centre.Y << ").\n";
#endif
  if (constraint.boundary==BOXCONSTRAINT) {
    if (centre.X+bw2>pitch.right) {
      // Ball is too close to opponents goal
      centre.X = pitch.right-bw2;
    } else {
      // Cannot be too close to both ends, unless the box width
      // is set to something silly, in which case the user
      // desverves what they get.
      if (centre.X-bw2<pitch.left) {
	// Ball is too close to own goal
	centre.X = pitch.left+bw2;
      }
    }
    if (centre.Y+bh2>pitch.top) {
      // Ball is too close to opponents goal
      centre.Y = pitch.top-bh2;
    } else {
      if (centre.Y-bh2<pitch.bottom) {
	// Ball is too close to own goal
	centre.Y = pitch.bottom+bh2;
      }
    }
  }
  if (constraint.home==BOXCONSTRAINT) {
    if (centre.X+bw2>constraint.hlimit.right) {
#ifdef MCPLDBBoxMapLRPos
      cout << "Too close to opponents end.\n";
#endif
      centre.X = constraint.hlimit.right-bw2;
    } else {
      // Cannot be too close to both ends, unless the box width
      // is set to something silly, in which case the user
      // desverves what they get.
      if (centre.X-bw2<constraint.hlimit.left) {
	// Ball is too close to own goal
	centre.X = constraint.hlimit.left+bw2;
      }
    }
    if (centre.Y+bh2>constraint.hlimit.top) {
#ifdef MCPLDBBoxMapLRPos
      cout << "Too close to top of home area.\n";
      cout << "top of box is at " << centre.Y+bh2 << endl;
      cout << "limit of home area is at " << constraint.hlimit.top << endl;
#endif
      centre.Y = constraint.hlimit.top-bh2;
    } else {
      if (centre.Y-bh2<constraint.hlimit.bottom) {
	// Ball is too close to own goal
	centre.Y = constraint.hlimit.bottom+bh2;
      }
    }
  }
  if (constraint.offside==BOXCONSTRAINT) {
    if (centre.X+bw2>status->offside) {
      centre.X = status->offside-bw2;
    }
  }

#ifdef MCPLDBBoxMapLRPos
  cout << "Constraining box to ";
  cout << "(" << centre.X << "," << centre.Y << ").\n";
#endif
  return centre;
}

point MCPLBoxMapLRPos::boxcentre(MCPLStatus *status) {
  // The boxmap method only consideres points within a small part of
  // the pitch, not the entire pitch, as used for simplemap.
  // A box centred on the ball is initially considered. This box is
  // broken into quarters. The size of these smaller boxes
  // is defined in the configuration file.
  // Each quarter is then tested to see which is closest to the home
  // location of the player. This is the box used to determine the best
  // position to move to.
  //
  // This basic system does not deal with the situation where all or part
  // of the selected box is 'out of play' e.g. due to an offside condition,
  // or the ball being close to the edge of the pitch.
  //
  // This can lead to a highly restricted choice, or possibly no valid
  // choices. For this reason the position of the boxes can be further
  // constrained by higher level rules.
  //
  // Three possible ways to apply these rules are as follows.
  // a) Constrain the large square by 'moving' the apparent location of
  //    the ball for the purposes of box selection. This has the advantage
  //    of ensuring that players are always trying to distribute themselves
  //    around the largest possible area. However it may also cause some
  //    players to move away from the ball.
  // b) Calculate the centre for each of the four quarters, and then constrain
  //    the location of each quarter individually before testing for
  //    suitability. This can lead to boxes overlapping, reducing the total
  //    area available, but ensures that players get as close as possible to
  //    the ball.
  // c) Simply discard any quarter which is marked 'unfit' by the higher level
  //    rules. This is not a preferred option since in extreme cases no
  //    quarter may be 'fit' (e.g. a defender may be constrained to stay
  //    in its own half, causing all quarters to be unfit if the ball moves
  //    some distance into the opponents half). Decisions must also be made
  //    at what point a quarter is declared unfit.
  MCPLPFLOAT dist=10000000; // In a galaxy far far away....
  MCPLPFLOAT newdist;
  int cent;
  int best;

  point ball;
  point half;
  point centres[4];

  ball = status->ball;

  if (constraint.ball) {
    // Constrain the 'centre' marker used for the box
    // calculations. Case a) above.
    ball = constrainBall(ball,status);
  }

  // Magnitude offsets to reach the centre of each quarter from
  // the ball location.
  half.X = (MCPLPFLOAT)box.width/2.0;
  half.Y = (MCPLPFLOAT)box.height/2.0;

  // Preload the centre of each quarter.
  centres[0].X = ball.X-half.X; // Bottom left
  centres[0].Y = ball.Y-half.Y;
  centres[1].X = ball.X-half.X; // Top left
  centres[1].Y = ball.Y+half.Y;
  centres[2].X = ball.X+half.X; // Top right
  centres[2].Y = ball.Y+half.Y;
  centres[3].X = ball.X+half.X; // Bottom right
  centres[3].Y = ball.Y-half.Y;

  if (constraint.box) {
    // Constrain the centre of each quarter
    // case b) above
    for(cent=0;cent<4;cent++) {
      centres[cent]=constrainBox(centres[cent],status);
    }
  }

  // Test each quarter to find the closest to the home location.
  best=0;
  for(cent=0;cent<4;cent++) {
    // find the distance between the two points
    // NB since sqrt(x)>sqrt(y) for all x>y passing
    // 2 as the third parameter prevents the (slow) sqrt
    // function being used.
    newdist=separation(home,centres[cent],2);
    if (newdist<dist) {
      best = cent;
      dist=newdist;
    }
  }
  return centres[best];
}



MCPLPFLOAT MCPLBoxMapLRPos::quality(point test,
				    MCPLStatus *status,
				    MCPLPFLOAT goaloffset) {
  MCPLPFLOAT dx,dy;
  MCPLPFLOAT t;
  MCPLPFLOAT influence;
  MCPLPFLOAT ilimit;
  int p;
  int pt;

  t=0.0;

  // Return a 'quality' assessment of a location P=(x,y) based on
  // objects on the pitch. A good quality location is the location
  // where:
  // sum(dist(P,Oi))+sum(dist(P,Ti))-dist(P,Ball)-dist(P,Goal)
  // is maximised.
  // Oi is the position of opponent i
  // Ti is the position of teammate i
  // dist(P,Q) is the distance between points P and Q
  // sum(f(i)) is the sum of f for all valid i

  if (use.opp>0) {
    // If the opponents are being considered, calculate a weighted score
    // based on the distance from the point passed, and the location of
    // each opponent.
    // NB. increase the value of t as the distance increases, further
    // from the opponent is better.
    if (status->opponentcount>MCPLSOPPONENTS) { // Bounds check
      pt=MCPLSOPPONENTS;
    } else {
      pt=status->opponentcount;
    }
    if ((use.opp==2) && (constraint.allows)) {
      ilimit = constraint.rlimit*constraint.rlimit;
    } else {
      ilimit = constraint.rlimit;
    }
    for(p=0;p<pt;p++) {
      influence = scale.opp*separation(test,status->opponents[p],use.opp);
      if ((constraint.rangel) && (influence>ilimit)) {
	influence = ilimit;
      }
      t+=influence;
    }
  }

  if (use.team>0) {
    // If the teammates are being considered, calculate a weighted score
    // based on the distance from the point passed, and the location of
    // each teammate.
    // NB. increase the value of t as the distance increases, further
    // from the teammate is better.
    if (status->teamcount>MCPLSTEAMMATES) { // Bounds check
      pt=MCPLSTEAMMATES;
    } else {
      pt=status->teamcount;
    }
    if ((use.team==2) && (constraint.allows)) {
      ilimit = constraint.rlimit*constraint.rlimit;
    } else {
      ilimit = constraint.rlimit;
    }
    for(p=0;p<pt;p++) {
      influence = scale.team*separation(test,status->teammates[p],use.team);
      if ((constraint.rangel) && (influence>ilimit)) {
	influence = ilimit;
      }
      t+=influence;
    }
  }

  if (use.ball>0) {
    // If the ball is being considered, calculate a weighted score based on
    // the distance from the point passed, and the location of the ball.
    // NB. reduce the value of t as the distance increases, closer to the
    // ball is better.
    if ((use.ball==2) && (constraint.allows)) {
      ilimit = constraint.rlimit*constraint.rlimit;
    } else {
      ilimit = constraint.rlimit;
    }
    influence = scale.ball*separation(test,status->ball,use.ball);
    if ((constraint.rangel) && (influence>ilimit)) {
      influence = ilimit;
    }
    t-=influence;
  }

  if (use.goal>0) {
    // If the goal is being considered, calculate a weighted score based on
    // the distance from the point passed, and the location of the opponents
    // goal.
    // NB. reduce the value of t as the distance increases, closer to the
    // goal is better.
    if ((use.ball==2) && (constraint.allows)) {
      ilimit = constraint.rlimit*constraint.rlimit;
    } else {
      ilimit = constraint.rlimit;
    }
    influence = scale.goal*separation(test,status->goal,use.goal) - goaloffset;
    if ((constraint.rangel) && (influence>ilimit)) {
      influence = ilimit;
    }
    t-=influence;
  }

  return t;
}

point MCPLBoxMapLRPos::findposition(MCPLStatus *status) {
  int x,y; // position within the virtual map
  int bw,bh; // Real size of the map in pixels

  point r; // position on the pitch
  MCPLPFLOAT value=-10000000.0; // reactor core, or somewhere else the
                            // player _really_ does not want to be.
  MCPLPFLOAT newvalue; // quality of the position being tested.
  point tbox; // The centre of the box in which positions will be
              // considered.
  point offset; // translation offset for the box.offset->pitch.offset
                // mapping.
  point retval;

  MCPLPFLOAT go; // Offset for the goal
  MCPLPFLOAT bo; // Offset for the ball

  int cmobile,nmobile; // Counters for prechecking players and opponents
  point tpos; // The closest position in the box the mobile being checked

  tbox = boxcentre(status);
  MCPLPitchFig diag;

  bw = (int)((MCPLPFLOAT)box.width/box.accuracy);
  bh = (int)((MCPLPFLOAT)box.height/box.accuracy);

#ifdef USEMAP
  MCMap pitchmap(bw,bh);
#endif

#ifdef MCPLDBBoxMapLRPos
  cout << "Selecting box with centre (" << tbox.X << "," << tbox.Y <<").\n";
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
#ifdef MCPLDBBoxMapLRPos
      cout << "Drawing diagram." << endl;
#endif
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      if (use.team) {
	for(int i=0;i<status->teamcount;i++) {
	  diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
	}
      }
      if (use.opp) {
	for(int i=0;i<status->opponentcount;i++) {
	  diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
	}
      }
      diag.setDefault();
      diag.setFill();

      diag.setThickness(4);

      if (constraint.offside) {
	diag.setMagenta();
	diag.drawLine(status->offside,pitch.top,status->offside,pitch.bottom);
      }
	
      diag.setBlue();
      diag.drawBox(tbox.X-(box.width/2.0),
	       tbox.Y-(box.height/2.0),
	       tbox.X+(box.width/2.0),
	       tbox.Y+(box.height/2.0));
    }
  }

  if ((constraint.rangel) && (use.goal)) { 
    tpos.X = tbox.X+(box.width/2.0);
    tpos.Y = status->goal.Y;
    if (tpos.Y<(tbox.Y-(box.height/2.0))) {
      tpos.Y = tbox.Y-(box.height/2.0);
    }
    if (tpos.Y>(tbox.Y+(box.height/2.0))) {
      tpos.Y = tbox.Y+(box.height/2.0);
    }
    go = separation(tpos,status->goal,use.goal);
  } else {
    go=0;
  }
  



#ifdef MCPLDBBoxMapLRPos
      cout << "Scanning box." << endl;
#endif

  offset.X = tbox.X+0.5-((MCPLPFLOAT)box.width/2.0);
  offset.Y = tbox.Y+0.5-((MCPLPFLOAT)box.height/2.0);

  for (y=0;y<bh;y++) {
    for (x=0;x<bw;x++) {
      r.X = ((MCPLPFLOAT)x*box.accuracy) + offset.X;
      r.Y = ((MCPLPFLOAT)y*box.accuracy) + offset.Y;
      newvalue=quality(r,status,go);

#ifdef USEMAP
      pitchmap.set(x,y,newvalue);
#endif

      if (newvalue>value) {
	retval = r;
	value=newvalue;
      }
    }
  }

#ifdef MCPLDBBoxMapLRPos
      cout << "Finished scan." << endl;
#endif

#ifdef USEMAP
  pitchmap.export(MCString("box.ppm"));
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setPointSize(1);
      diag.setRed();
      diag.setThickness(2);
      diag.setSquarePoint();
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
#ifdef MCPLDBBoxMapLRPos
      cout << "Finished findpos." << endl;
#endif
  return retval;
}

MCString MCPLBoxMapLRPos::ident(void) {
  return MCString("BoxMapLR");
}

void MCPLBoxMapLRPos::basediagram(MCPLPitchFig& diag) {
  diag.drawPitch();
  diag.setGold();
  diag.setFill();
  diag.drawPlayer(home.X,home.Y);
  diag.setDefault();
  diag.setFill();
  diag.setDepth(50);
  diag.setFontSize(96);
  diag.setFontTimes();
  diag.setLineSpacing(5,120);
  diag.setTextOrigin(-55,39);
  diag.insertText(name+" with specialisation "+special);
  diag.insertText(MCString("Positioner is ")+ident());

  MCString hometext;
  hometext = MCString("Home location is (")+MCString(home.X)+",";
  hometext+= MCString(home.Y)+").";
  diag.insertText(hometext);

  if (next!=0) {
    diag.insertText(MCString("Failsafe Positioner is a ")+next->ident());
  }

  diag.setFontSize(64);
  diag.setLineSpacing(2.5,80);
  diag.setTextOrigin(0,42.5);

  diag.insertText(MCString("Diagram interval is ")+MCString(dinterval));
  hometext = MCString("Box size is ");
  hometext+= MCString(box.width)+" by "+MCString(box.height);
  hometext+= MCString(" with accuracy ")+MCString(box.accuracy)+".";
  diag.insertText(hometext);

  hometext = MCString("Goal distance is ");
  switch(use.goal) {
  case 0:
    hometext+=MCString("inactive.");
    break;
  case 1:
    hometext+=MCString("active, with weight ")+MCString(scale.goal)+".";
    break;
  case 2:
    hometext+=MCString("squared, with weight ")+MCString(scale.goal)+".";
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Ball distance is ");
  switch(use.ball) {
  case 0:
    hometext+=MCString("inactive.");
    break;
  case 1:
    hometext+=MCString("active, with weight ")+MCString(scale.ball)+".";
    break;
  case 2:
    hometext+=MCString("squared, with weight ")+MCString(scale.ball)+".";
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Teammate distance is ");
  switch(use.team) {
  case 0:
    hometext+=MCString("inactive.");
    break;
  case 1:
    hometext+=MCString("active, with weight ")+MCString(scale.team)+".";
    break;
  case 2:
    hometext+=MCString("squared, with weight ")+MCString(scale.team)+".";
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Opponent distance is ");
  switch(use.opp) {
  case 0:
    hometext+=MCString("inactive.");
    break;
  case 1:
    hometext+=MCString("active, with weight ")+MCString(scale.opp)+".";
    break;
  case 2:
    hometext+=MCString("squared, with weight ")+MCString(scale.opp)+".";
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Boundary constraint is ");
  switch(constraint.boundary) {
  case NOCONSTRAINT:
    hometext+=MCString("Off.");
    break;
  case BALLCONSTRAINT:
    hometext+=MCString("using the Ball method.");
    break;
  case BOXCONSTRAINT:
    hometext+=MCString("using the Box.");
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Offside constraint is ");
  switch(constraint.offside) {
  case NOCONSTRAINT:
    hometext+=MCString("Off.");
    break;
  case BALLCONSTRAINT:
    hometext+=MCString("using the Ball method.");
    break;
  case BOXCONSTRAINT:
    hometext+=MCString("using the Box.");
    break;
  }
  diag.insertText(hometext);

  hometext = MCString("Home constraint is ");
  switch(constraint.home) {
  case NOCONSTRAINT:
    hometext+=MCString("Off.");
    break;
  case BALLCONSTRAINT:
    hometext+=MCString("using the Ball method.");
    break;
  case BOXCONSTRAINT:
    hometext+=MCString("using the Box method.");
    break;
  }
  diag.insertText(hometext);

  if (constraint.home) {
    diag.setYellow();
    diag.setThickness(4);
    diag.drawBox(constraint.hlimit.left,constraint.hlimit.top,
		  constraint.hlimit.right,constraint.hlimit.bottom);
  }  

  diag.setDefault();
  diag.setThickness(1);
  hometext = MCString("Range limiter constraint is ");
  if (constraint.rangel) {
    hometext+="On.";
    diag.insertText(hometext);
    if (constraint.allows) {
      diag.insertText(MCString("Allowing squared limit."));
    }
    diag.insertText(MCString("Range limit is ")+MCString(constraint.rlimit));
  } else {
    hometext+="Off.";
    diag.insertText(hometext);
  }

}

void MCPLBoxMapLRPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}

void MCPLBoxMapLRPos::resetoptions(void) {
  // This resets all the options to sensible default values
  use.goal=0; // Ignore goal
  use.ball=0; // Ignore ball
  use.team=0; // Ignore teammates
  use.opp=0; // Ignore opponents

  scale.ball=1;
  scale.goal=1;
  scale.team=1;
  scale.opp=1;

  box.width=20;
  box.height=20;
  box.accuracy=1.0;

  constraint.ball=0;
  constraint.box=0;
  constraint.boundary=NOCONSTRAINT;
  constraint.offside=NOCONSTRAINT;
  constraint.home=NOCONSTRAINT;
  constraint.hlimit.top=pitch.top;
  constraint.hlimit.left=pitch.left;
  constraint.hlimit.bottom=pitch.bottom;
  constraint.hlimit.right=pitch.right;
  constraint.rangel=0;
  constraint.allows=0;
  constraint.rlimit=0;
}

void MCPLBoxMapLRPos::readoptions(MCString section,MCIni &ini) {
  MCString sconstraint;
  MCString ballcon("Ball"),boxcon("Box");

#ifdef MCPLDBBoxMapLRPos
  cout << "Specialisation =  " << special << endl;
#endif
  // Read the options from the specified section of
  // the ini object. If no value is defined, use the
  // current value.
  use.goal = readOption(section,"Goal",ini,use.goal);
  use.ball = readOption(section,"Ball",ini,use.ball);
  use.team = readOption(section,"Team",ini,use.team);
  use.opp = readOption(section,"Opponent",ini,use.opp);

  scale.goal = readOption(section,"GoalMax",ini,scale.goal);
  scale.ball = readOption(section,"BallMax",ini,scale.ball);
  scale.team = readOption(section,"TeamMax",ini,scale.team);
  scale.opp  = readOption(section,"OpponentMax",ini,scale.opp);

  box.width = readOption(section,"Width",ini,box.width);
  box.height = readOption(section,"Height",ini,box.height);
  box.accuracy = readOption(section,"Accuracy",ini,box.accuracy);

  // Extract the constraint information
  sconstraint= readOption(section,"Boundary",ini,"Off");
  if (sconstraint==ballcon) {
    constraint.boundary=BALLCONSTRAINT;
    constraint.ball=1;
  } else {
    if (sconstraint==boxcon) {
      constraint.boundary=BOXCONSTRAINT;
      constraint.box=1;
    }
  }
  sconstraint= readOption(section,"Offside",ini,"Off");
  if (sconstraint==ballcon) {
    constraint.offside=BALLCONSTRAINT;
    constraint.ball=1;
  } else {
    if (sconstraint==boxcon) {
      constraint.offside=BOXCONSTRAINT;
      constraint.box=1;
    }
  }
  sconstraint= readOption(section,"Home",ini,"Off");
  if (sconstraint==ballcon) {
    constraint.home=BALLCONSTRAINT;
    constraint.ball=1;
  } else {
    if (sconstraint==boxcon) {
      constraint.home=BOXCONSTRAINT;
      constraint.box=1;
    }
  }
  if (constraint.home) {
    // Read the home constraints. These are 'unusual' in that
    // they depend on the player, not the method used
    // If the constraint is not specified use the limit of the pitch
    // in the given direction.
    constraint.hlimit.top=ini.getclassed(name,special+":HomeTop",pitch.top);
    if (constraint.hlimit.top>pitch.top) { // sanity check
      constraint.hlimit.top=pitch.top;
    }
    constraint.hlimit.left=ini.getclassed(name,special+":HomeLeft",pitch.left);
    if (constraint.hlimit.left<pitch.left) { // sanity check
      constraint.hlimit.left=pitch.left;
    }
    constraint.hlimit.bottom=ini.getclassed(name,special+":HomeBottom",
					    pitch.bottom);
    if (constraint.hlimit.bottom<pitch.bottom) { // sanity check
      constraint.hlimit.bottom=pitch.bottom;
    }
    constraint.hlimit.right=ini.getclassed(name,special+":HomeRight",pitch.right);
    if (constraint.hlimit.right>pitch.right) { // sanity check
      constraint.hlimit.right=pitch.right;
    }
  }

  constraint.rangel = readOption(section,"RangeLimiter",ini,constraint.rangel);
  constraint.allows = readOption(section,"AllowSquares",ini,constraint.allows);
  constraint.rlimit = readOption(section,"RangeLimit",ini,constraint.rlimit);
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLBoxMapLRPos::make(MCString method,
				    MCString player,
				    MCString spec,
				    MCIni& ini,
				    MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;

  if (method==ident()) {
    retval = new MCPLBoxMapLRPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLBoxMapLRPos::MCPLBoxMapLRPos(MCString method,
			     MCString player,
			     MCString spec,
			     MCIni &ini,
			     MCPLPositioner *failsafe)
  :MCPLPositioner(method,player,spec,ini,failsafe) {

  pitch.length = ini.get("Pitch","Length",105.0);
  pitch.width = ini.get("Pitch","Width",68.0);
  pitch.top = pitch.width/2.0;
  pitch.bottom = - pitch.top;
  pitch.right = pitch.length/2.0;
  pitch.left = - pitch.right;

  resetoptions();
  readoptions(method,ini);
}

MCPLBoxMapLRPos::MCPLBoxMapLRPos(MCExemplar e): MCPLPositioner(e) {
}

MCPLBoxMapLRPos::MCPLBoxMapLRPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
  resetoptions();
}

MCPLBoxMapLRPos::MCPLBoxMapLRPos(const MCPLBoxMapLRPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
  use = copyme.use;
  scale = copyme.scale;
  pitch = copyme.pitch;
  box = copyme.box;  
}

MCPLBoxMapLRPos& MCPLBoxMapLRPos::operator=(const MCPLBoxMapLRPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  use = assignme.use;
  scale = assignme.scale;
  pitch = assignme.pitch;
  box = assignme.box;  
  return *this;
}

MCPLBoxMapLRPos::~MCPLBoxMapLRPos() {
}

static MCPLBoxMapLRPos MCPLBoxMapLRPosInstance(ExemplarInstance);


