// File    : MCPLBoxMapLRPos.h
//
// Purpose : C++ PLBoxMapLRPos class
// Author  : Matthew Hunter
// Version : 0.01 (14 Jul 1999)
//                Copy of BoxMap

#ifndef MCPLBOXMAPLRPOS_h
#define MCPLBOXMAPLRPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLBoxMapLRPos: public MCPLPositioner {
 public:
  // Exemplar member functions
  point constrainBall(point,MCPLStatus*);
  point constrainBox(point,MCPLStatus*);
  point boxcentre(MCPLStatus *status);
  virtual MCPLPFLOAT quality(point,MCPLStatus*,MCPLPFLOAT);

  MCString ident(void);
  void basediagram(MCPLPitchFig&);
  void diagram(void);

  point findposition(MCPLStatus *);

  // Initialise all the options prior to use
  virtual void resetoptions(void);

  // Read the options from the named group within the
  // given configuration object.
  virtual void readoptions(MCString,MCIni&);

  // External functions, provide the positioning facilities
  MCPLPositioner *make(MCString,
		       MCString,
		       MCString,
		       MCIni&,
		       MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLBoxMapLRPos(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLBoxMapLRPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLBoxMapLRPos();  // Default constructor
  MCPLBoxMapLRPos(const MCPLBoxMapLRPos&); // Copy constructor

  // Standard operations
  MCPLBoxMapLRPos& operator=(const MCPLBoxMapLRPos&); // assignment

  // Destructor
  ~MCPLBoxMapLRPos();
 private:
 protected:
  struct {
    int goal;
    int ball;
    int team;
    int opp;
  } use; // Which elements will be considered for positioning
  struct {
    MCPLPFLOAT ball;
    MCPLPFLOAT goal;
    MCPLPFLOAT team;
    MCPLPFLOAT opp;
  } scale; // Relative weights of the different elements.
  struct {
    MCPLPFLOAT length;
    MCPLPFLOAT width;
    MCPLPFLOAT top;
    MCPLPFLOAT bottom;
    MCPLPFLOAT left;
    MCPLPFLOAT right;
  } pitch; // Size and boundaries of the pitch
  struct {
    int width;
    int height;
    MCPLPFLOAT accuracy;
  } box; // Size of the box considered during positioning.
  struct {
    int ball;
    int box;
    int boundary;
    int offside;
    int home;
    struct {
      MCPLPFLOAT top;
      MCPLPFLOAT left;
      MCPLPFLOAT bottom;
      MCPLPFLOAT right;
    } hlimit;
    int rangel;
    int allows;
    MCPLPFLOAT rlimit;
  } constraint;
};

#endif



