// File    : MCPLBoxMapPos.h
//
// Purpose : C++ PLBoxMapPos class
// Author  : Matthew Hunter
// Version : 0.01 (23 Jun 1999)
//           0.02 (24 Jun 1999)
//                Extended readoptions to take a specialisation
//                option so home constraints can be read
//           0.03 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.04 (14 Jul 1999)
//                Minor corrections.

#ifndef MCPLBOXMAPPOS_h
#define MCPLBOXMAPPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLBoxMapPos: public MCPLPositioner {
 public:
  // Exemplar member functions
  point constrainBall(point,MCPLStatus*);
  point constrainBox(point,MCPLStatus*);
  point boxcentre(MCPLStatus *status);
  virtual MCPLPFLOAT quality(point,MCPLStatus*);

  MCString ident(void);
  void basediagram(MCPLPitchFig&);
  void diagram(void);

  point findposition(MCPLStatus *);

  // Initialise all the options prior to use
  virtual void resetoptions(void);

  // Read the options from the named group within the
  // given configuration object.
  virtual void readoptions(MCString,MCIni&);

  // External functions, provide the positioning facilities
  MCPLPositioner *make(MCString,
		       MCString,
		       MCString,
		       MCIni&,
		       MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLBoxMapPos(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLBoxMapPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLBoxMapPos();  // Default constructor
  MCPLBoxMapPos(const MCPLBoxMapPos&); // Copy constructor

  // Standard operations
  MCPLBoxMapPos& operator=(const MCPLBoxMapPos&); // assignment

  // Destructor
  ~MCPLBoxMapPos();
 private:
 protected:
  struct {
    int goal;
    int ball;
    int team;
    int opp;
  } use; // Which elements will be considered for positioning
  struct {
    MCPLPFLOAT ball;
    MCPLPFLOAT goal;
    MCPLPFLOAT team;
    MCPLPFLOAT opp;
  } scale; // Relative weights of the different elements.
  struct {
    MCPLPFLOAT length;
    MCPLPFLOAT width;
    MCPLPFLOAT top;
    MCPLPFLOAT bottom;
    MCPLPFLOAT left;
    MCPLPFLOAT right;
  } pitch; // Size and boundaries of the pitch
  struct {
    int width;
    int height;
    MCPLPFLOAT accuracy;
  } box; // Size of the box considered during positioning.
  struct {
    int ball;
    int box;
    int boundary;
    int offside;
    int home;
    struct {
      MCPLPFLOAT top;
      MCPLPFLOAT left;
      MCPLPFLOAT bottom;
      MCPLPFLOAT right;
    } hlimit;
  } constraint;
};

#endif



