// File    : MCPLFixedPos.cpp
//
// Purpose : C++ PLPositioner class
// Author  : Matthew Hunter
// Version : 0.01 (23 Jun 1999)
//           0.02 (29 Jun 1999)
//                Modified to support new point structure.
//           0.03 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.

#include "MCPLFixedPos.h"
#include "MCPLPitchFig.h"
#include <iostream.h>

ostream& operator<<(ostream&s,MCPLFixedPos d) {
  s << "Fixed Positioner.\n";
  return s;
}

void MCPLFixedPos::display(ostream &s) {
  s << "Fixed Positioner: ";
  MCPLPositioner::display(s);
}

MCString MCPLFixedPos::ident(void) {
  return MCString("Fixed");
}

void MCPLFixedPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  setup.drawPitch();
  setup.setGold();
  setup.setFill();
  setup.drawPlayer(home.X,home.Y);
  setup.setDefault();
  setup.setFill();
  setup.setDepth(50);
  setup.setFontSize(96);
  setup.setFontTimes();
  setup.setLineSpacing(5,120);
  setup.setTextOrigin(-55,39);
  setup.insertText(name+" with specialisation "+special);
  setup.insertText(MCString("Positioner is ")+ident());

  MCString hometext;
  hometext = MCString("Home location is (")+MCString(home.X)+",";
  hometext+= MCString(home.Y)+").";
  setup.insertText(hometext);

  if (next!=0) {
    setup.insertText(MCString("Failsafe Positioner is a ")+next->ident());
  }
  setup.setFontSize(64);
  setup.setLineSpacing(2.5,80);
  setup.setTextOrigin(0,42.5);

  setup.insertText(MCString("Diagram interval is ")+MCString(dinterval));
}



// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLFixedPos::make(MCString method,
				   MCString player,
				   MCString spec,
				   MCIni& ini,
				   MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;

  if (method==ident()) {
    retval = new MCPLFixedPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLFixedPos::MCPLFixedPos(MCString method,
			   MCString player,
			   MCString spec,
			   MCIni &ini,
			   MCPLPositioner *failsafe)
  :MCPLPositioner(method,player,spec,ini,failsafe) {
}

MCPLFixedPos::MCPLFixedPos(MCExemplar e): MCPLPositioner(e) {
}

MCPLFixedPos::MCPLFixedPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
}

MCPLFixedPos::MCPLFixedPos(const MCPLFixedPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
}

MCPLFixedPos& MCPLFixedPos::operator=(const MCPLFixedPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  return *this;
}

MCPLFixedPos::~MCPLFixedPos() {
}

static MCPLFixedPos MCPLFixedPosInstance(ExemplarInstance);





