// File    : MCPLFixedRelPos.cpp
//
// Purpose : C++ PLPositioner class
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)

#include "MCPLFixedRelPos.h"
#include "MCPLPitchFig.h"
#include <iostream.h>

ostream& operator<<(ostream&s,MCPLFixedRelPos d) {
  s << "FixedRel Positioner.\n";
  return s;
}

void MCPLFixedRelPos::display(ostream &s) {
  s << "FixedRel Positioner: ";
  MCPLPositioner::display(s);
}

MCString MCPLFixedRelPos::ident(void) {
  return MCString("FixedRel");
}

void MCPLFixedRelPos::basediagram(MCPLPitchFig &diag) {
  diag.drawPitch();
  diag.setGold();
  diag.setFill();
  diag.drawPlayer(home.X,home.Y);
  diag.setDefault();
  diag.setFill();
  diag.setDepth(50);
  diag.setFontSize(96);
  diag.setFontTimes();
  diag.setLineSpacing(5,120);
  diag.setTextOrigin(-55,39);
  diag.insertText(name+" with specialisation "+special);
  diag.insertText(MCString("Positioner is ")+ident());

  MCString hometext;
  hometext = MCString("Home location is (")+MCString(home.X)+",";
  hometext+= MCString(home.Y)+").";
  diag.insertText(hometext);

  if (next!=0) {
    diag.insertText(MCString("Failsafe Positioner is a ")+next->ident());
  }
  diag.setFontSize(64);
  diag.setLineSpacing(2.5,80);
  diag.setTextOrigin(0,42.5);

  diag.insertText(MCString("Diagram interval is ")+MCString(dinterval));
  hometext = MCString("Ball offset is (")+MCString(balloffset.X);
  hometext+= MCString(",")+MCString(balloffset.Y)+").";
  diag.insertText(hometext);
}

void MCPLFixedRelPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}


point MCPLFixedRelPos::findposition(MCPLStatus *status) {
  point retval;
  point realoffset;

  MCPLPitchFig diag;

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      for(int i=0;i<status->teamcount;i++) {
	diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
      }
      for(int i=0;i<status->opponentcount;i++) {
	diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
      }
    }
  }

  realoffset=balloffset;

  if ((reverseX) && (status->ball.X<0)) {
    realoffset.X = -realoffset.X;
  }
  if ((reverseY) && (status->ball.Y<0)) {
    realoffset.Y = -realoffset.Y;
  }


  retval.X = status->ball.X + realoffset.X;
  retval.Y = status->ball.Y + realoffset.Y;

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setFill();
      diag.setSquarePoint();
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
  return retval;
}

void MCPLFixedRelPos::resetoptions(void) {
  balloffset.X=0;
  balloffset.Y=0;
  reverseX = 0;
  reverseY = 0;
}

void MCPLFixedRelPos::readoptions(MCString method, MCIni& ini) {
  MCString Offset;

  reverseX = readOption(method,"ReverseX",ini,reverseX);
  reverseY = readOption(method,"ReverseY",ini,reverseY);

  Offset=ini.getclassed(name,special+":BallOffset","");
#ifdef MCPLDBFixedRelPos
  cout << "Read ball offset as string " << Offset << "." << endl;
#endif
  if (Offset!=MCString("")) {
    balloffset=makepoint(Offset);
  }
}


// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLFixedRelPos::make(MCString method,
				   MCString player,
				   MCString spec,
				   MCIni& ini,
				   MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;

  if (method==ident()) {
    retval = new MCPLFixedRelPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLFixedRelPos::MCPLFixedRelPos(MCString method,
			   MCString player,
			   MCString spec,
			   MCIni &ini,
			   MCPLPositioner *failsafe)
  :MCPLPositioner(method,player,spec,ini,failsafe) {
  resetoptions();
  readoptions(method,ini);
}

MCPLFixedRelPos::MCPLFixedRelPos(MCExemplar e): MCPLPositioner(e) {
}

MCPLFixedRelPos::MCPLFixedRelPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
}

MCPLFixedRelPos::MCPLFixedRelPos(const MCPLFixedRelPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
}

MCPLFixedRelPos& MCPLFixedRelPos::operator=(const MCPLFixedRelPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  return *this;
}

MCPLFixedRelPos::~MCPLFixedRelPos() {
}

static MCPLFixedRelPos MCPLFixedRelPosInstance(ExemplarInstance);





