// File    : MCPLFixedRelPos.h
//
// Purpose : C++ PLFixedRelPos class
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)

#ifndef MCPLFIXEDRELPOS_h
#define MCPLFIXEDRELPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLFixedRelPos: public MCPLPositioner {
 public:
  // Output
  friend ostream& operator<<(ostream&s,MCPLPositioner);
  void display(ostream&s);

  // Exemplar member functions

  MCString ident(void);
  void basediagram(MCPLPitchFig&);
  void diagram(void);

  point findposition(MCPLStatus *);

  virtual void resetoptions(void);
  virtual void readoptions(MCString, MCIni&);

  // External functions, provide the positioning facilities

  MCPLPositioner *make(MCString,
		       MCString,
		       MCString,
		       MCIni&,
		       MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLFixedRelPos(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLFixedRelPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLFixedRelPos();  // Default constructor
  MCPLFixedRelPos(const MCPLFixedRelPos&); // Copy constructor

  // Standard operations
  MCPLFixedRelPos& operator=(const MCPLFixedRelPos&); // assignment

  // Destructor
  ~MCPLFixedRelPos();
 private:
 protected:
  point balloffset;
  int reverseX;
  int reverseY;
};

#endif



