// File    : MCPLGeometry.cpp
//
// Purpose : Geometry support
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)
//           0.02 (14 Jul 1999)
//                Minor corrections.

#include "MCPLGeometry.h"

// offsetpoint
//
// Calculates the point dist of the way between two points

// Calculate the point dist of the way from (0,0) to (x,y)
// e.g. for x = 10, y = 0 and dist = 0.1
// point returned will be (1,0)
point offsetpoint(MCPLPFLOAT x, MCPLPFLOAT y, MCPLPFLOAT dist) {
  point retval;

  retval.X=x*dist;
  retval.Y=y*dist;
  return retval;
}

// Calculates the point dist of the way between A and B
// e.g. for A=(5,7), B=(15,17) and dist = 0.5
// point returned will be (10,12)
point offsetpoint(point A, point B, MCPLPFLOAT dist) {
  point retval;

  retval.X=((B.X-A.X)*dist)+A.X;
  retval.Y=((B.Y-A.Y)*dist)+A.Y;
  return retval;
}

// intersect
//
// Calculates the point along a line through two points where
// the line intersects a second line
//
// xintersect calculates the point on the line through A and B
// that intersects the line x = X
// e.g. A = (-5,-7), B=(5,-11) and X = 0
// point returned is (0,-9)
point xintersect(point A, point B, MCPLPFLOAT X) {
  point retval;
  MCPLPFLOAT dx;
  MCPLPFLOAT dy;

  dx = (B.X-A.X);
  if (dx!=0) {
    dy = (B.Y-A.Y)/dx;
    retval.X = X;
    retval.Y = ((X-A.X)*dy)+A.Y;
  } else {
    // Line is parallel to x = X
    // return the midpoint of the points passed.
    retval.X = ((B.X-A.X)/2.0)+A.X;
    retval.Y = ((B.Y-A.Y)/2.0)+A.Y;
  }
  return retval;
}

// yintersect calculates the point on the line through A and B
// that intersects the line y = Y
// e.g. A = (-5,-6), B=(5,-10) and Y = 0
// point returned is (-20,0)
point yintersect(point A, point B, MCPLPFLOAT Y) {
  point retval;
  MCPLPFLOAT dx;
  MCPLPFLOAT dy;

  dy = (B.Y-A.Y);
  if (dy!=0) {
    dx = (B.X-A.X)/dy;
    retval.Y = Y;
    retval.X = ((Y-A.Y)*dx)+A.X;
  } else {
    // Line is parallel to y = Y
    // return the midpoint of the points passed.
    retval.X = ((B.X-A.X)/2.0)+A.X;
    retval.Y = ((B.Y-A.Y)/2.0)+A.Y;
  }
  return retval;
}

