// File    : MCPLMarkerPos.cpp
//
// Purpose : C++ PLMarkerPos class
// Author  : Matthew Hunter
// Version : 0.01 (6 Jul 1999)
//                Code based on Tracker
//           0.02 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.03 (14 Jul 1999)
//                Minor corrections.
//           0.04 (15 Jul 1999)
//                Fixed bug in reading options
//                Added support for from goal (actually from the end
//                of the bisector for Marker) and Proximity failures.
//           0.05 (20 Jul 1999)
//                Modified to use a local copy of the MCPLStatus object
//                rather than the original passed in by the caller.

#include "MCPLMarkerPos.h"
#include "MCPLGeometry.h"

#include "MCPLPitchFig.h"

#include <iostream.h>

MCString MCPLMarkerPos::ident(void) {
  return MCString("Marker");
}

void MCPLMarkerPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}


point MCPLMarkerPos::findposition(MCPLStatus *exstatus) {

  point mark;
  int useopp;
  MCPLPitchFig diag;
  MCPLStatus lstatus(*exstatus);
  MCPLStatus *status=&lstatus;

#ifdef MCPLDBMarkerPos
  cout << "Dumping external status info:" << endl;
  cout << *exstatus;
  cout << "Dumping local status info:" << endl;
  cout << *status;
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
#ifdef MCPLDBMarkerPos
      cout << "Drawing diagram." << endl;
#endif
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      for(int i=0;i<status->teamcount;i++) {
	diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
      }
      for(int i=0;i<status->opponentcount;i++) {
	diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
      }
      diag.setDefault();
      diag.setFill();
    }
  }

  if (constraint.asel) {
    useopp = autoselect(status);
  }

  if ((useopp<0) || (status->opponentcount==0)) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	diag.setCentreJustify();
	diag.drawText(MCString("Resorting to failsafe"),0,0,40,5);
	diag.drawText(MCString("(No opponents to mark)"),0,-10,40,5);
	diag.setLeftJustify();
      }
    }
    // No available opponents to track.
    if (next!=0) {
      return next->findposition(exstatus);
    } else {
      return home;
    }
  } else {
    mark = status->opponents[useopp];
  }

  MCPLPFLOAT gdist = separation(mark,status->owngoal,1);
  MCPLPFLOAT bdist = separation(mark,status->ball,1);

  point D = offsetpoint(mark,status->owngoal,10/gdist);
  point E = offsetpoint(mark,status->ball,10/bdist);
  point F = offsetpoint(D,E,0.5);

  MCPLPFLOAT fdist = separation(mark,F,1);

  if (gdist<bdist) {
    bdist=gdist;
  }
  point G = offsetpoint(mark,F,bdist/fdist);

  if ((constraint.fail) && (bdist<ranges.mindist)) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	diag.setCentreJustify();
	diag.drawText(MCString("Resorting to failsafe"),0,0,40,5);
	diag.drawText(MCString("(Proximity failure)"),0,-10,40,5);
	diag.setLeftJustify();
      }
    }
    // No available opponents to track.
    if (next!=0) {
      return next->findposition(exstatus);
    } else {
      return home;
    }
  }
  
  MCPLPFLOAT rdist = ranges.reldist;
  if ((ranges.reldist*bdist)<ranges.mindist) {
    rdist=ranges.mindist/bdist;
  }

#ifdef MCPLDBMarkerPos
  cout << "Relative distance is " << rdist << ".\n";
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setFill();
      diag.setBlack();
      diag.setDashedLine();
      diag.drawLine(mark.X,mark.Y,status->owngoal.X,status->owngoal.Y);
      diag.drawLine(mark.X,mark.Y,status->ball.X,status->ball.Y);
      if (ranges.outmark!=0) {
	diag.drawCircle(mark.X,mark.Y,ranges.outmark);
      }
      if (constraint.fmgl) {
	if (ranges.mindist!=0) {
	  diag.drawCircle(G.X,G.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(G.X,G.Y,bdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(G.X,G.Y,rdist*bdist);
      } else {
	if (ranges.mindist!=0) {
	  diag.drawCircle(mark.X,mark.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(mark.X,mark.Y,bdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(mark.X,mark.Y,rdist*bdist);
      }
      diag.setDefault();
      diag.setSolidLine();
      diag.setFill();
      diag.setCrossPoint();
      diag.setPointSize(1);
      diag.plotPoint(D.X,D.Y);
      diag.plotPoint(E.X,E.Y);
      diag.plotPoint(F.X,F.Y);
      diag.plotPoint(G.X,G.Y);
      diag.setDashedLine();
      diag.drawLine(mark.X,mark.Y,G.X,G.Y);
    }
  }

  point retval;

  retval=targetpoint(mark,G,rdist);

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setSolidLine();
      diag.setFill();
      diag.setBlue();
      diag.setDepth(80);
      diag.setSquarePoint();
      diag.setPointSize(1);
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
  return retval;
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLMarkerPos::make(MCString method,
				    MCString player,
				    MCString spec,
				    MCIni& ini,
				    MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;
  if (method==ident()) {
    retval = new MCPLMarkerPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLMarkerPos::MCPLMarkerPos(MCString method,
			     MCString player,
			     MCString spec,
			     MCIni &ini,
			     MCPLPositioner *failsafe)
  :MCPLTrackerPos(method,player,spec,ini,failsafe) {

  resetoptions();
  readoptions(method,ini);
}

MCPLMarkerPos::MCPLMarkerPos(MCExemplar e): MCPLTrackerPos(e) {
}

MCPLMarkerPos::MCPLMarkerPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
  resetoptions();
}

MCPLMarkerPos::MCPLMarkerPos(const MCPLMarkerPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
  pitch = copyme.pitch;
  ranges = copyme.ranges;
  constraint = copyme.constraint;
}

MCPLMarkerPos& MCPLMarkerPos::operator=(const MCPLMarkerPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  pitch = assignme.pitch;
  ranges = assignme.ranges;
  constraint = assignme.constraint;
  return *this;
}

MCPLMarkerPos::~MCPLMarkerPos() {
}

static MCPLMarkerPos MCPLMarkerPosInstance(ExemplarInstance);













