// File    : MCPLMarkerPos.h
//
// Purpose : C++ PLMarkerPos class
// Author  : Matthew Hunter
// Version : 0.01 (06 Jul 1999)
//                Derived from MCPLBoxMapPos.h
//           0.02 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.03 (12 Jul 1999)
//                Removal of basediagram, it is inherited from Tracker


#ifndef MCPLMARKERPOS_h
#define MCPLMARKERPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLTrackerPos.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLMarkerPos: public MCPLTrackerPos {
 public:
  // Exemplar member functions
  MCString ident(void);
  void diagram(void);

  // Find the best tracking position based on the status
  point findposition(MCPLStatus *);

  // External functions, provide the positioning facilities
  MCPLPositioner *make(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLMarkerPos(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLMarkerPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLMarkerPos();  // Default constructor
  MCPLMarkerPos(const MCPLMarkerPos&); // Copy constructor

  // Standard operations
  MCPLMarkerPos& operator=(const MCPLMarkerPos&); // assignment

  // Destructor
  ~MCPLMarkerPos();
 private:
 protected:
};

#endif



