// File    : MCPLOffsideTrapPos.cpp
//
// Purpose : C++ PLOffsideTrapPos class
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)
//                Code based on Tracker
//           0.02 (14 Jul 1999)
//                Added spanning to decide on target point if all
//                opponents are already covered.
//           0.03 (14 Jul 1999)
//                Minor corrections.
//           0.04 (20 Jul 1999)
//                Modified to use a local copy of the MCPLStatus object
//                rather than changing the original.
//                Changed size of distances array to match the maximum
//                number of players.
//           0.05 (21 Jul 1999)
//                Added option to set the maximum distance from the
//                offside line of the opponents to track.

#include "MCPLOffsideTrapPos.h"
#include "MCPLGeometry.h"

#include <iostream.h>

MCString MCPLOffsideTrapPos::ident(void) {
  return MCString("OffsideTrap");
}

void MCPLOffsideTrapPos::basediagram(MCPLPitchFig& diag) {
  diag.drawPitch();
  diag.setGold();
  diag.setFill();
  diag.drawPlayer(home.X,home.Y);
  diag.setDefault();
  diag.setFill();
  diag.setDepth(50);
  diag.setFontSize(96);
  diag.setFontTimes();
  diag.setLineSpacing(5,120);
  diag.setTextOrigin(-55,39);
  diag.insertText(name+" with specialisation "+special);
  diag.insertText(MCString("Positioner is ")+ident());

  MCString hometext;
  hometext = MCString("Home location is (")+MCString(home.X)+",";
  hometext+= MCString(home.Y)+").";
  diag.insertText(hometext);

  if (next!=0) {
    diag.insertText(MCString("Failsafe Positioner is a ")+next->ident());
  }

  diag.setFontSize(64);
  diag.setLineSpacing(2.5,80);
  diag.setTextOrigin(0,42.5);

  diag.insertText(MCString("Diagram interval is ")+MCString(dinterval));

  hometext = MCString("Minimum distance is ");
  hometext+= MCString(ranges.mindist);
  diag.insertText(hometext);
  hometext = MCString("Relative distance is ");
  hometext+= MCString(ranges.reldist*100)+"%.";
  diag.insertText(hometext);
  hometext = MCString("Outer Marker is ");
  hometext+= MCString(ranges.outmark);
  diag.insertText(hometext);

  hometext = MCString("Auto selection of target is ");
  if (constraint.asel) {
    hometext+= "On.";
    diag.insertText(hometext);
    if (constraint.useg) {
      diag.insertText(MCString("Ranking opponents by distance from goal."));
    } else {
      diag.insertText(MCString("Ranking opponents by distance from home."));
    }
  } else {
    hometext+= "Off.";
    diag.insertText(hometext);
  }

  hometext = MCString("Home constraint is ");
  if(constraint.home) {
    hometext+=MCString("On.");
    diag.insertText(hometext);
    diag.setYellow();
    diag.setThickness(4);
    diag.setDepth(140);
    diag.drawBox(constraint.hlimit.left,constraint.hlimit.top,
		  constraint.hlimit.right,constraint.hlimit.bottom);
    diag.setDepth(100);
  } else {
    hometext+=MCString("Off.");
    diag.insertText(hometext);
  }

  diag.setDefault();
  diag.insertText(MCString("Ball X Limit is ")+MCString(ranges.ballX));
  diag.insertText(MCString("Line Offset is ")+MCString(ranges.lineoff));
  diag.insertText(MCString("Line Maximum is ")+MCString(ranges.linemax));
  diag.insertText(MCString("Opponent Distance is ")+MCString(ranges.oppdist));

  diag.setMagenta();
  diag.setDashedLine();
  diag.drawLine(ranges.ballX,pitch.top,ranges.ballX,pitch.bottom);
  diag.setCyan();
  diag.setDottedLine();
  diag.drawLine(ranges.linemax,pitch.top,ranges.linemax,pitch.bottom);

}

void MCPLOffsideTrapPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}

point MCPLOffsideTrapPos::verifypoint(point T) {

  if (T.X>ranges.linemax) {
    T.X = ranges.linemax;
  }

  if (constraint.home) {
    // Need to check that the point found is within the players
    // home area
    if (T.X>constraint.hlimit.right) {
      T.X = constraint.hlimit.right;
    }
    if (T.X<constraint.hlimit.left) {
      T.X = constraint.hlimit.left;
    }
    if (T.Y>constraint.hlimit.top) {
      T.Y = constraint.hlimit.top;
    }
    if (T.Y<constraint.hlimit.bottom) {
      T.Y = constraint.hlimit.bottom;
    }
  }

  return T;
}

point MCPLOffsideTrapPos::findposition(MCPLStatus *exstatus) {

  MCPLStatus lstatus(*exstatus);
  MCPLStatus *status=&lstatus;

  // General variables
  point target;
  point retval;
  MCPLPitchFig diag;
  int i,j;

  // Variables for target point selection
  MCPLPFLOAT waright, waleft, watop, wabottom; // Work Area limits
  MCPLPFLOAT fixx; // the fixed offside line
  int activeoc; // Number of 'active' opponents.
  int activeindex; // index of the selected opponent.
  int marked; // is a teammate already marking the position

  point dtest; // distance comparison point.
  MCPLPFLOAT distances[MCPLSOPPONENTS];
  // distance of each opponent from dtest

  // Selection sort variables.
  int in,jn; // Counters for selection sort.
  int lowindex;
  MCPLPFLOAT lowkey;
  point swap;

  // Segmentation variables
  MCPLPFLOAT current;
  MCPLPFLOAT largest;
  MCPLPFLOAT middle;
  MCPLPFLOAT nextseg;
  MCPLPFLOAT segsize;

#ifdef MCPLDBOffsideTrapPos
  cout << "Dumping external status info:" << endl;
  cout << *exstatus;
  cout << "Dumping local status info:" << endl;
  cout << *status;
  cout << "findposition: OpponentCount is " << status->opponentcount << ".\n";
  for(i=0;i<status->opponentcount;i++) {
    cout << "findposition: Opponent " << i << " is at (";
    cout << status->opponents[i].X << ",";
    cout << status->opponents[i].Y << ").\n";
  }
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
#ifdef MCPLDBOffsideTrapPos
      cout << "Drawing diagram." << endl;
#endif
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      for(i=0;i<status->teamcount;i++) {
	diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
      }
      for(i=0;i<status->opponentcount;i++) {
	diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
      }
      diag.setDefault();
      diag.setFill();

    }
  }

  if (status->ball.X<ranges.ballX) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	
	diag.drawText(MCString("Resorting to failsafe"),-17,0,40,5);
      }
    }
    // No available opponents to track.
    if (next!=0) {
      return next->findposition(exstatus);
    } else {
      return home;
    }
  }

  if (!constraint.asel) {
    // Not autoselecting a 'target', so select a position
    // based purely on the location of the ball;
    target.X=status->ball.X+ranges.lineoff;
    target.Y=status->ball.Y;
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setDefault();
	diag.setFill();
	diag.setThickness(1);
	diag.setSolidLine();
	diag.setPointSize(1);
	diag.setCirclePoint();
	diag.plotPoint(target.X,target.Y);
      }
    }
    retval = verifypoint(target);
  } else {
    // Need to do more stuff to decide where to move to.

    // Base area, is the pitch, except for the right (highest X)
    // limit, which is as far right as the offside line, plus
    // the Opponent Distance.
    fixx = status->ball.X + ranges.lineoff;
    if (fixx>ranges.linemax) {
      fixx = ranges.linemax;
    }
    if (ranges.oppdist>0.0) {
      waright = fixx+ranges.oppdist;
    } else {
      waright = status->ball.X;
    }
    waleft = pitch.left;
    watop = pitch.top;
    wabottom = pitch.bottom;

    // If necessary further constrain the area based on the home
    // location of the player
    if (constraint.home) {
      // Do not apply the home X limit, because
      // the X location is going to be fixed anyway.
      if (waleft<constraint.hlimit.left) {
	waleft = constraint.hlimit.left;
      }
      if (watop>constraint.hlimit.top) {
	watop = constraint.hlimit.top;
      }
      if (wabottom<constraint.hlimit.bottom) {
	wabottom = constraint.hlimit.bottom;
      }
    }

    // Scan the list of opponents to find any that
    // are in the players work area, discarding the rest.
    activeoc=0;
    for(i=0; i<status->opponentcount; i++) {
      if (
	  (status->opponents[i].X<=waright) &&
	  (status->opponents[i].X>=waleft) &&
	  (status->opponents[i].Y<=watop) &&
	  (status->opponents[i].Y>=wabottom)) {
	status->opponents[activeoc].X=fixx;
	status->opponents[activeoc].Y=status->opponents[i].Y;
	activeoc++;
      }
    }

    status->opponentcount=activeoc;

    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setDefault();
	diag.setFill();
	diag.setRed();
	diag.setSolidLine();
	for(i=0; i<status->opponentcount; i++) {
	  diag.drawCircle(status->opponents[i].X,
			  status->opponents[i].Y,
			  ranges.outmark);
	}
	diag.setDefault();
	diag.setFill();
	diag.setDashedLine();
	diag.setThickness(4);
	diag.setBlue();
	diag.drawBox(waleft,wabottom,waright,watop);
      }
    }


    if (status->opponentcount<1) {
      // There are no opponents in the search area,
      // so aim at the centre (vertically) of the work area
      // Unlike tracker, the failsafe cannot be used, even
      // if there are no opponents, as the offside trap
      // should still be enforced.
      target.X = fixx;
      target.Y = (watop+wabottom)/2.0;

    } else {
    
      // If there are multiple options sort them based on distance
      // Preload distances with distance of each opponent from
      // players home or owngoal location
      if (constraint.useg) {
	dtest=status->owngoal;
      } else {
	dtest=home;
      }
      for(i=0;i<status->opponentcount;i++) {
	distances[i]=separation(dtest,status->opponents[i],1);
      }

      // Sort opponents based on distance from home location
      // NB Selection sort, not good, but better than
      // bubble sort, and players is much less than 100
      in = status->opponentcount-1;
      jn = status->opponentcount;
      for (i=0;i<in;i++) {
	lowindex=i;
	lowkey = distances[lowindex];
	for(j=i+1;j<jn;j++) {
	  if (distances[j]<lowkey) {
	    lowkey=distances[j];
	    lowindex=j;
	  }
	} 
	// swap the items
	swap = status->opponents[lowindex];
	status->opponents[lowindex]=status->opponents[i];
	distances[lowindex]=distances[i];
	status->opponents[i]=swap;
	distances[i]=lowkey;
      }

      // All the opponents left in the list are valid, try and find
      // one that is not already marked.
      activeindex=-1;
      i=0;
      while((activeindex<0) && (i<status->opponentcount)) {
	j = 0;
	marked=0;
	while((activeindex<0) && (j<status->teamcount)) {
	  if (
	      separation(status->teammates[j],
			 status->opponents[i],1)<ranges.outmark) {
	    marked = 1;
	  }
	  j++;
	}
	if (!marked) {
	  activeindex=i;
	}
	i++;
      }

#ifdef MCPLDBOffsideTrapPos
      cout << "Got active index is " << activeindex << "." << endl;
#endif

      if (activeindex<0) {
	// all the opponents are already marked.
	current = wabottom;
	middle = ((watop-wabottom)/2.0)+wabottom;
	largest=0.0;
	while(current<watop) {
	  nextseg=watop;
	  for(i=0;i<status->teamcount;i++) {
	    if (status->teammates[i].X<(fixx+ranges.outmark)) {
	      // Ignore teammates that are forward of the offside line
	      if ((status->teammates[i].Y<nextseg) &&
		  (status->teammates[i].Y>current)) {
		// if this teammate is between the end of the current
		// segment and the current start of the segment,
		// then update the end of the segment to be the Y value
		// of the teammate.
		nextseg = status->teammates[i].Y;
	      }
	    }
	  }
	  segsize = nextseg-current;
#ifdef MCPLDBOffsideTrapPos
	  cout << "Found segment between " << current << " and ";
	  cout << nextseg << ".";
	  cout << " Size is " << segsize << " largest is " << largest;
	  cout << "." << endl;
#endif
	  if (segsize>largest) {
	    largest=segsize;
	    middle = (segsize/2.0)+current;
	  }
	  current=nextseg;
	}
	target.X = fixx;
	target.Y = middle;
      } else {
	target = status->opponents[activeindex];
      }
    }
    retval = target;
  }

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setSolidLine();
      diag.setFill();
      diag.setBlue();
      diag.setDepth(80);
      diag.setSquarePoint();
      diag.setPointSize(1);
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
#ifdef MCPLDBOffsideTrapPos
  cout << "Dumping external status info:" << endl;
  cout << *exstatus;
  cout << "Dumping local status info:" << endl;
  cout << *status;
#endif

  return retval;
}

void MCPLOffsideTrapPos::resetoptions(void) {
  // This resets all the options to sensible default values
  ranges.mindist=5.0;
  ranges.reldist=0.0;
  ranges.outmark=10.0;
  ranges.ballX=10;
  ranges.lineoff=-10;
  ranges.linemax=0;
  ranges.oppdist=15;

  constraint.home=0;
  constraint.asel=0;
  constraint.useg=0;
  constraint.hlimit.top=pitch.top;
  constraint.hlimit.left=pitch.left;
  constraint.hlimit.bottom=pitch.bottom;
  constraint.hlimit.right=pitch.right;
}

void MCPLOffsideTrapPos::readoptions(MCString section,MCIni &ini) {
#ifdef MCPLDBOffsideTrapPos
  cout << "Specialisation =  " << special << endl;
#endif
  // Read the options from the specified section of
  // the ini object. If no value is defined, use the
  // current value.

  ranges.mindist=readOption(section,"MinimumDistance",ini,ranges.mindist);
  ranges.reldist=readOption(section,"RelativeDistance",ini,ranges.reldist);
  ranges.outmark=readOption(section,"OuterMarker",ini,ranges.outmark);
  ranges.ballX=readOption(section,"BallX",ini,ranges.ballX);
  ranges.lineoff=readOption(section,"LineOffset",ini,ranges.lineoff);
  ranges.linemax=readOption(section,"LineMax",ini,ranges.linemax);
  ranges.oppdist=readOption(section,"OpponentDistance",ini,ranges.oppdist);

#ifdef MCPLDBOffsideTrapPos
  cout << "Minimum distance is  " << ranges.mindist << ".\n";
  cout << "Relative distance is " << ranges.reldist << ".\n";
  cout << "Outer marker is      " << ranges.outmark << ".\n";
  cout << "Ball X Limit is      " << ranges.ballX << ".\n";
#endif

  constraint.home= readOption(section,"Home",ini,constraint.home);
  constraint.asel= readOption(section,"AutoSelect",ini,constraint.asel);
  constraint.useg= readOption(section,"UseGoalDist",ini,constraint.useg);

  if (constraint.home) {
    // Read the home constraints. These are 'unusual' in that
    // they depend on the player, not the method used
    // If the constraint is not specified use the limit of the pitch
    // in the given direction.
    constraint.hlimit.top=ini.getclassed(name,special+":HomeTop",pitch.top);
    if (constraint.hlimit.top>pitch.top) { // sanity check
      constraint.hlimit.top=pitch.top;
    }
    constraint.hlimit.left=ini.getclassed(name,special+":HomeLeft",pitch.left);
    if (constraint.hlimit.left<pitch.left) { // sanity check
      constraint.hlimit.left=pitch.left;
    }
    constraint.hlimit.bottom=ini.getclassed(name,special+":HomeBottom",
					    pitch.bottom);
    if (constraint.hlimit.bottom<pitch.bottom) { // sanity check
      constraint.hlimit.bottom=pitch.bottom;
    }
    constraint.hlimit.right=ini.getclassed(name,special+":HomeRight",
					   pitch.right);
    if (constraint.hlimit.right>pitch.right) { // sanity check
      constraint.hlimit.right=pitch.right;
    }
  }
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLOffsideTrapPos::make(MCString method,
				     MCString player,
				     MCString spec,
				     MCIni& ini,
				     MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;

  if (method==ident()) {
    retval = new MCPLOffsideTrapPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLOffsideTrapPos::MCPLOffsideTrapPos(MCString method,
			       MCString player,
			       MCString spec,
			       MCIni &ini,
			       MCPLPositioner* failsafe)
  :MCPLPositioner(method,player,spec,ini,failsafe) {

  pitch.length = ini.get("Pitch","Length",105.0);
  pitch.width = ini.get("Pitch","Width",68.0);
  pitch.top = pitch.width/2.0;
  pitch.bottom = - pitch.top;
  pitch.right = pitch.length/2.0;
  pitch.left = - pitch.right;

  resetoptions();
  readoptions(method,ini);
}

MCPLOffsideTrapPos::MCPLOffsideTrapPos(MCExemplar e): MCPLPositioner(e) {
}

MCPLOffsideTrapPos::MCPLOffsideTrapPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
  resetoptions();
}

MCPLOffsideTrapPos::MCPLOffsideTrapPos(const MCPLOffsideTrapPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
  pitch = copyme.pitch;
  ranges = copyme.ranges;
  constraint = copyme.constraint;
}

MCPLOffsideTrapPos& MCPLOffsideTrapPos::operator=(const MCPLOffsideTrapPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  pitch = assignme.pitch;
  ranges = assignme.ranges;
  constraint = assignme.constraint;
  return *this;
}

MCPLOffsideTrapPos::~MCPLOffsideTrapPos() {
}

static MCPLOffsideTrapPos MCPLOffsideTrapPosInstance(ExemplarInstance);


