// File    : MCPLOffsideTrapPos.h
//
// Purpose : C++ PLOffsideTrapPos class
// Author  : Matthew Hunter
// Version : 0.01 (13 Jul 1999)
//           0.02 (14 Jul 1999)
//                Minor corrections
//           0.03 (21 Jul 1999)
//                Added option to set the maximum distance from the
//                offside line of the opponents to track.

#ifndef MCPLOFFSIDETRAPPOS_h
#define MCPLOFFSIDETRAPPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLOffsideTrapPos: public MCPLPositioner {
 public:
  // Exemplar member functions

  // Find the best tracking position based on the status
  point findposition(MCPLStatus *);

  point verifypoint(point);

  MCString ident(void);
  void basediagram(MCPLPitchFig&);
  void diagram(void);

  // Initialise all the options prior to use
  virtual void resetoptions(void);

  // Read the options from the named group within the
  // given configuration object.
  virtual void readoptions(MCString,MCIni&);

  // External functions, provide the positioning facilities
  MCPLPositioner *make(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLOffsideTrapPos(MCString,MCString,MCString,MCIni&,MCPLPositioner *);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLOffsideTrapPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLOffsideTrapPos();  // Default constructor
  MCPLOffsideTrapPos(const MCPLOffsideTrapPos&); // Copy constructor

  // Standard operations
  MCPLOffsideTrapPos& operator=(const MCPLOffsideTrapPos&); // assignment

  // Destructor
  ~MCPLOffsideTrapPos();
 private:
 protected:
  struct {
    MCPLPFLOAT length;
    MCPLPFLOAT width;
    MCPLPFLOAT top;
    MCPLPFLOAT bottom;
    MCPLPFLOAT left;
    MCPLPFLOAT right;
  } pitch; // Size and boundaries of the pitch
  struct {
    int home; // Apply home area constraints
    int asel; // Auto select opponent from all the opponents
              // passed via the MCPLStatus object.
    int useg; // Rank opponents by distance from goal rather than
              // distance from home location
    struct {
      MCPLPFLOAT top;
      MCPLPFLOAT left;
      MCPLPFLOAT bottom;
      MCPLPFLOAT right;
    } hlimit; // Size and boundaries of the home location
  } constraint; // Constraints in force
  struct {
    MCPLPFLOAT mindist;
    MCPLPFLOAT reldist;
    MCPLPFLOAT outmark;
    MCPLPFLOAT ballX;
    MCPLPFLOAT lineoff;
    MCPLPFLOAT linemax;
    MCPLPFLOAT oppdist;
  } ranges;
};

#endif



