// File    : MCPLPositioner.cpp
//
// Purpose : C++ PLPositioner class
// Author  : Matthew Hunter
// Version : 0.02 (23 Jun 1999)
//                Modified make and constructor to accept specialisation.
//           0.03 (23 Jun 1999)
//                Added display function and minor tidying up.
//           0.04 (23 Jun 1999)
//                Added offensive and defensive functions to change the
//                offense mode of a Positioner.
//                Effects of the mode are determined by derived classes
//           0.05 (24 Jun 1999)
//                Changed constructor to used classed value for home
//                position.
//           0.06 (29 Jun 1999)
//                Modified to use alternative point structure
//           0.07 (12 Jul 1999)
//                Changed make, and constructor to support pass through
//                of positioners.
//           0.08 (12 Jul 1999)
//                Removed offensive and defensive functions and offense
//                member variable, they are no longer needed.
//                Added diagram function to save a file containing the
//                configuration for the positioner.
//           0.09 (12 Jul 1999)
//                Added readOption functions to allow Positioner options
//                to be overridden by values in the Player section of the
//                ini file.
//           0.10 (12 Jul 1999)
//                Added option for displaying diagrams during operation.

#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

#include <iostream.h>

ostream& operator<<(ostream&s,MCPLPositioner d) {
  d.display(s);
  return s;
}

MCPLPositioner *MCPLPositioner::list=0;

void MCPLPositioner::display(ostream&s) {
  s << name << ", home location = (";
  s << home.X << "," << home.Y << "). ";
}

// External functions, provide the positioning facilities
MCString MCPLPositioner::ident(void) {
  return MCString("Positioner");
}

void MCPLPositioner::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  setup.drawPitch();
  setup.setGold();
  setup.drawPlayer(home.X,home.Y);
}

MCString MCPLPositioner::readOption(MCString section,
				    MCString option,
				    MCIni& options,
				    MCString value) {
  MCString retval;

#ifdef MCPLDBPositioner
  cout << "Player  is " << name << "." << endl;
  cout << "Special is " << special << "." << endl;
  cout << "Section is " << section << "." << endl;
  cout << "Option  is " << option << "." << endl;
  cout << "Value   is " << value << "." << endl;
#endif

  // First try to read the default option from the appropriate
  // PSS.
  // Option name will be simply "specialisation:option", e.g. for
  // Specialisation "433:Kickoff" and option "Test" in section "TSection"
  // a classed check will check for option as :
  // 433:Kickoff:Test
  // 433:Test
  // Test
  // if none of these are found in TSection value will be returned.
  retval = options.getclassed(section,
			      special+":"+option,
			      value);

  // Then try and read the player specific option from the player
  // section. These options will take the form
  // "specialisation:section-option", continuing the example above
  // the second stage will test
  // 433:Kickoff:TSection-Test
  // 433:TSection-Test
  // TSection-Test
  retval = options.getclassed(name,
			      special+":"+section+"-"+option,
			      retval);

  return retval;
}

int MCPLPositioner::readOption(MCString section,
			       MCString option,
			       MCIni& options,
			       int value) {

  return (int)readOption(section,option,options,MCString(value));
}

double MCPLPositioner::readOption(MCString section,
				  MCString option,
				  MCIni& options,
				  double value) {
  return (double)readOption(section,option,options,MCString(value));
}

point MCPLPositioner::findposition(MCPLStatus *status) {
  // Do not try this at home kids, these are skilled professionals.
  return home;
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLPositioner::make(MCString player,
				     MCString spec,
				     MCIni& ini,
				     MCPLPositioner* failsafe) {
  MCPLPositioner *retval=0;
  MCPLPositioner *current;
  if (spec==MCString("")) {
    spec = ini.get(player,"Specialisation","433");
  }
  MCString Method = ini.get(player,spec+":Method","");

#ifdef MCPLDBPositioner
  cout << "MCPLPositioner::make : Creating Positioner." << endl;
  cout << "MCPLPositioner::make : Method = " << Method << "." << endl;
  cout << "MCPLPositioner::make : Specialisation = " << spec << "." << endl;
#endif

  if (Method!=MCString("")) {
    for (current = list; current; current = current->next) {
      if (retval = current->make(Method,player,spec,ini,failsafe))
	break;
    }
  }
  if (retval==0) {
    // Unable to create a positioner from the given information
#ifdef MCPLDBPositioner
    cout << "MCPLPositioner::make : Failed to create a Positioner." << endl;
    cout << "MCPLPositioner::make : Using failsafe." << endl;
#endif
    retval = failsafe;
  }
  return retval;
}

MCPLPositioner *MCPLPositioner::make(MCString method,
				     MCString player,
				     MCString spec,
				     MCIni& ini,
				     MCPLPositioner* failsafe) {

#ifdef MCPLDBPositioner
  cout << "MCPLPositioner::make(Positioner) : ";
  cout << "This probably should not have happened." << endl;
  cout << "MCPLPositioner::make(Positioner) : ";
  cout << "When trying to create " << method << "." << endl;
#endif
  return 0;
}

MCPLPositioner::MCPLPositioner(MCString method,
			       MCString player,
			       MCString spec,
			       MCIni& ini,
			       MCPLPositioner *failsafe) {
  name = player;
  special = spec;
  MCString shome = ini.getclassed(player,spec+":Home","(0,0)");
  dinterval = readOption(method,"DiagramInterval",ini,0);
  home = makepoint(shome);
  dicount = 0;
  next = failsafe;
#ifdef MCPLDBPositioner
  cout << "MCPLPositioner::constructor : Player         = ";
  cout << player << "." << endl;
  cout << "MCPLPositioner::constructor : Specialisation = ";
  cout << special << "." << endl;
  cout << "MCPLPositioner::constructor : failsafe       = ";
  if (failsafe) {
    cout << failsafe->ident();
  } else {
    cout << "none";
  }
  cout << "." << endl;
#endif
}

MCPLPositioner::MCPLPositioner(MCExemplar e) {
  // This inserts a new prototype positioner on
  // the list of available prototypes.
  next = list;
  list = this;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
}

MCPLPositioner::MCPLPositioner() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
}

MCPLPositioner::MCPLPositioner(const MCPLPositioner& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
}

MCPLPositioner& MCPLPositioner::operator=(const MCPLPositioner& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  return *this;
}

MCPLPositioner::~MCPLPositioner() {
}

MCExemplar ExemplarInstance;
static MCPLPositioner MCPLPosInstance(ExemplarInstance);
MCPLPositioner *positioner = &MCPLPosInstance;







