// File    : MCPLPositioner.h
//
// Purpose : C++ PLPositioner class
// Author  : Matthew Hunter
// Version : 0.01 (16 Jun 1999)
//           0.02 (23 Jun 1999)
//                Modified make and constructor to take a specialisation
//                value (specialisations are formations and set plays).
//           0.03 (23 Jun 1999)
//                Added display member function to show name and home
//                location, plus some minor tidying up.
//           0.04 (23 Jun 1999)
//                Added offensive and defensive functions to change the
//                offense mode of a Positioner.
//                Effects of the mode are determined by derived classes
//           0.05 (12 Jul 1999)
//                Added new version of function make that takes the method
//                name as the first argument, so each Positioner tested
//                does not need to look up the method name. This is the
//                version of make that should be overloaded by other
//                PSS's. Both forms of make also take a MCPLPositioner*
//                which is the positioner to fall back on if the positioner
//                being created should fail. The constructor has also
//                been extended to accept an MCPLPositioner*.
//           0.06 (12 Jul 1999)
//                Removed offensive and defensive functions and offense
//                member variable, they are no longer needed.
//                Added diagram function to save a file containing the
//                configuration for the positioner.
//           0.07 (12 Jul 1999)
//                Added readOption function to extract information from
//                the options.
//           0.08 (12 Jul 1999)
//                Added a variable to hold the interval between
//                diagrams.

#ifndef MCPLPositioner_h
#define MCPLPositioner_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLPositioner {
 public:
  // Output
  friend ostream& operator<<(ostream&s,MCPLPositioner);
  virtual void display(ostream&s);

  // Exemplar member functions
  virtual MCString ident(void); // Return the positioners identification
                                // string.
  virtual void diagram(void); // Save a diagram of the positioners
                              // configuration information.

  MCString readOption(MCString section,
		      MCString option,
		      MCIni& options,
		      MCString value);

  int readOption(MCString section,
		 MCString option,
		 MCIni& options,
		 int value);

  double readOption(MCString section,
		    MCString option,
		    MCIni& options,
		    double value);

  // External functions, provide the positioning facilities
  virtual point findposition(MCPLStatus*);

  // Build a new Positioner of the specified type from the list of
  // registered prototypes, using the specified options.
  // Create a positioner with the specified name and specialisation
  // (e.g. formation type)
  MCPLPositioner *make(MCString,
		       MCString,
		       MCIni&,
		       MCPLPositioner*);

  // this is the version of make that each new PSS should overload.
  virtual MCPLPositioner *make(MCString,
			       MCString,
			       MCString,
			       MCIni&,
			       MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLPositioner(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLPositioner(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLPositioner();  // Default constructor
  MCPLPositioner(const MCPLPositioner&); // Copy constructor

  // Standard operations
  MCPLPositioner& operator=(const MCPLPositioner&); // assignment

  // Destructor
  ~MCPLPositioner();
 private:
 protected:
  static MCPLPositioner *list;
  MCPLPositioner *next;
  
  // Common features of all position finding..
  MCString name; // The player's name
  MCString special; // The player's specialisation
  point home; // The player's home location
  int dinterval; // Interval between diagrams
  int dicount; // Count for generating diagrams.
};

// Used by derived classes to create new prototypes
extern MCExemplar ExemplarInstance;

// Used by the application to create a new MCPLPositioner object by name
extern MCPLPositioner *positioner;

#endif



