// File    : MCPLTest.cpp
//
// Purpose : Simple test for the MCPLPosLib Library
// Author  : Matthew Hunter
// Version : 0.01 (16 Jun 1999)

#include "MCPLPosLib.h"
#include "MCPLPitchFig.h"

#include "MCIni.h"
#include "MCString.h"

#include <iostream.h>

int main(int argc, char *argv[]) {
  MCIni test; // Configuration file for the test to run
  MCPLStatus status; // Status information to pass to the findposition fn.
  int shirt; // Players shirt number
  MCString role; // Players role
  MCString spoint; // string for string->point conversion
  int i; // count for reading opponent and teammate positions

  MCPLPositioner *pos,*ko,*d,*m;
  MCPLPitchFig pitch("pitch.fig");
  MCString defend,marker;
  int defendreps,markerreps;

  if (argc<2) {
    cout << "No initialisation file specified.\n";
    cout << "Using default shirt number = 1 and no status information.\n";
    shirt = 1;
  } else {
    MCString file(argv[1]);
    int ext = file.rindex('.');
    if (ext<0) {
      file+=".ini";
    }
    test.import(file);
    shirt = test.get("Player","shirt",-1);
    if (shirt<0) {
      cout << "No shirt number in config file, checking for role name.\n";
      role = test.get("Player","role","GoalPost");
      if (role==MCString("GoalPost")) {
	cout << "No role name given, defaulting to shirt number 1.\n";
	shirt=1;
      }

      // Read the specialisations to use
      defend = test.get("Player","defend","Defend");
      defendreps = test.get("Player","defendreps",1);
      marker = test.get("Player","marker","Marker");
      markerreps = test.get("Player","markerreps",1);

      // Get the information about the location of static items (goals)
      spoint = test.get("Static","goal","(52,0)");
      status.goal = makepoint(spoint);
      cout << "Goal location is x = " << status.goal.X << ", y = ";
      cout << status.goal.Y << ".\n";
      spoint = test.get("Static","owngoal","(52,0)");
      status.owngoal = makepoint(spoint);
      cout << "Own goal location is x = " << status.owngoal.X << ", y = ";
      cout << status.owngoal.Y << ".\n";

      // Offside line, default to opponents goal
      status.offside = test.get("Static","offside",status.goal.X);
      cout << "Offside line is at " << status.offside << endl;

      // Ball location
      spoint = test.get("Ball","ball","(0,0)");
      status.ball = makepoint(spoint);
      cout << "Own goal location is x = " << status.ball.X << ", y = ";
      cout << status.ball.Y << ".\n";
      pitch.drawBall(status.ball.X,status.ball.Y);


      // Team mate locations
      status.teamcount = test.get("Teammates","count",0);
      cout << status.teamcount << " teammates are on the pitch.\n";
      for(i=0;i<status.teamcount;i++) {
	spoint = test.get("Teammates",i+1,"(0,0)");
	status.teammates[i] = makepoint(spoint);
	cout << "Teammate located at x = " << status.teammates[i].X;
	cout << ", y = " << status.teammates[i].Y << ".\n";
	pitch.drawTeammate(status.teammates[i].X,status.teammates[i].Y);
      }

      // Opponent locations
      status.opponentcount = test.get("Opponents","count",0);
      cout << status.opponentcount << " opponents are on the pitch.\n";
      for(i=0;i<status.opponentcount;i++) {
	spoint = test.get("Opponents",i+1,"(0,0)");
	status.opponents[i] = makepoint(spoint);
	cout << "Opponent located at x = " << status.opponents[i].X;
	cout << ", y = " << status.opponents[i].Y << ".\n";
	pitch.drawOpponent(status.opponents[i].X,status.opponents[i].Y);
      }

    }
  }

  cout << status;

  if (MCPLPosLibInit()>0) {
    cout << "Successfully loaded configuration file.\n";
  } else {
    cout << "Oh dear, problem with configuration file.\n";
  }
  if (shirt>0) {
    MCPLSetShirt(shirt);
  } else {
    cout << role << endl;
    MCPLSetName(role);
  }

  pos = MCPLFormation("433");
  ko = MCPLSpecialisation("433","Kickoff",pos);
  d = MCPLSpecialisation("433",defend,pos);
  m = MCPLSpecialisation("433",marker,pos);

  if (pos!=0) {
    pos->diagram();
    point target = pos->findposition(&status);

    cout << "Received target point at (" << target.X;
    cout << "," << target.Y << ").\n";
    pitch.setDefault();
    pitch.setFill();
    pitch.setPointSize(1.0);
    pitch.setCrossPoint();
    pitch.plotPoint(target.X,target.Y);
    if (ko==pos) {
      cout << "No specialisation for kickoff\n";
    } else {
      ko->diagram();
      target = ko->findposition(&status);
      cout << "Received specialisation point at (" << target.X;
      cout << "," << target.Y << ").\n";
      pitch.setSquarePoint();
      pitch.plotPoint(target.X,target.Y);
    }
    if (d==pos) {
      cout << "No specialisation for defend\n";
    } else {
      d->diagram();
      for(int i=0;i<defendreps;i++) {
	target = d->findposition(&status);
      }
      cout << "Received specialisation point at (" << target.X;
      cout << "," << target.Y << ").\n";
      pitch.setCirclePoint();
      pitch.plotPoint(target.X,target.Y);
    }
    if (m==pos) {
	cout << "No specialisation for marker\n";
    } else {
      m->diagram();
      for(int i=0;i<markerreps;i++) {
	target = m->findposition(&status);
      }
      cout << "Received specialisation point at (" << target.X;
      cout << "," << target.Y << ").\n";
      pitch.setCirclePoint();
      pitch.plotPoint(target.X,target.Y);
      }
  } else {
    cout << "Error, unable to create a positioner.\n";
  }
}


