// File    : MCPLTrackerPos.cpp
//
// Purpose : C++ PLTrackerPos class
// Author  : Matthew Hunter
// Version : 0.01 (6 Jul 1999)
//                Code based on BoxMap
//           0.02 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.03 (14 Jul 1999)
//                Minor corrections.
//           0.04 (20 Jul 1999)
//                Changes to use local MCPLStatus object so as
//                not to corrupt the one passed in.

#include "MCPLTrackerPos.h"
#include "MCPLGeometry.h"

#include <iostream.h>

MCString MCPLTrackerPos::ident(void) {
  return MCString("Tracker");
}

void MCPLTrackerPos::basediagram(MCPLPitchFig& diag) {
  diag.drawPitch();
  diag.setGold();
  diag.setFill();
  diag.drawPlayer(home.X,home.Y);
  diag.setDefault();
  diag.setFill();
  diag.setDepth(50);
  diag.setFontSize(96);
  diag.setFontTimes();
  diag.setLineSpacing(5,120);
  diag.setTextOrigin(-55,39);
  diag.insertText(name+" with specialisation "+special);
  diag.insertText(MCString("Positioner is ")+ident());

  MCString hometext;
  hometext = MCString("Home location is (")+MCString(home.X)+",";
  hometext+= MCString(home.Y)+").";
  diag.insertText(hometext);

  if (next!=0) {
    diag.insertText(MCString("Failsafe Positioner is a ")+next->ident());
  }

  diag.setFontSize(64);
  diag.setLineSpacing(2.5,80);
  diag.setTextOrigin(0,42.5);

  diag.insertText(MCString("Diagram interval is ")+MCString(dinterval));

  hometext = MCString("Minimum distance is ");
  hometext+= MCString(ranges.mindist);
  diag.insertText(hometext);
  hometext = MCString("Relative distance is ");
  hometext+= MCString(ranges.reldist*100)+"%.";
  diag.insertText(hometext);
  hometext = MCString("Outer Marker is ");
  hometext+= MCString(ranges.outmark);
  diag.insertText(hometext);

  hometext = MCString("Auto selection of target is ");
  if (constraint.asel) {
    hometext+= "On.";
    diag.insertText(hometext);
    if (constraint.useg) {
      diag.insertText(MCString("Ranking opponents by distance from goal."));
    } else {
      diag.insertText(MCString("Ranking opponents by distance from home."));
    }
    if (constraint.semi) {
      diag.insertText(MCString("Ignoring teammates behind opponents."));
    } else {
      diag.insertText(MCString("Accepting teammates behind opponents."));
    }
    if (constraint.oppx) {
      diag.insertText(MCString("Ignoring positions behind opponent."));
    } else {
      diag.insertText(MCString("Accepting positions behind opponent."));
    }
  } else {
    hometext+= "Off.";
    diag.insertText(hometext);
  }
  if (constraint.hmly) {
    diag.insertText(MCString("Tracked object must be within home limits."));
  } else {
    diag.insertText(MCString("Tracked object may be outside home limits."));
  }
  if (constraint.fmgl) {
    diag.insertText(MCString("Measuring offset from goal."));
  } else {
    diag.insertText(MCString("Measuring offset from target."));
  }
  if (constraint.fail) {
    diag.insertText(MCString("Proximity Failure active."));
  } else {
    diag.insertText(MCString("Proximity Failure inactive."));
  }

  hometext = MCString("Home constraint is ");
  if(constraint.home) {
    hometext+=MCString("On.");
    diag.insertText(hometext);
    diag.setYellow();
    diag.setThickness(4);
    diag.setDepth(140);
    diag.drawBox(constraint.hlimit.left,constraint.hlimit.top,
		  constraint.hlimit.right,constraint.hlimit.bottom);
    diag.setDepth(100);
  } else {
    hometext+=MCString("Off.");
    diag.insertText(hometext);
  }
}

void MCPLTrackerPos::diagram(void) {
  MCPLPitchFig setup;

  setup.openfile(name+":"+special+".fig");
  basediagram(setup);
}

int MCPLTrackerPos::autoselect(MCPLStatus *status) {
  MCPLPFLOAT distances[MCPLSOPPONENTS];

  int i,j,in,jn;
  MCPLPFLOAT lowkey;
  int lowindex;
  int tracked;
  int target;
  point swap;
  point dtest;

#ifdef MCPLDBTrackerPos
  cout << "AutoSel: OpponentCount is " << status->opponentcount << ".\n";
  for(i=0;i<status->opponentcount;i++) {
    cout << "AutoSel: Opponent " << i << " is at (";
    cout << status->opponents[i].X << ",";
    cout << status->opponents[i].Y << ").\n";
  }
#endif


  // Preload distances with distance of each opponent from
  // players home or owngoal location
  if (constraint.useg) {
    dtest=status->owngoal;
  } else {
    dtest=home;
  }
  for(i=0;i<status->opponentcount;i++) {
    distances[i]=separation(dtest,status->opponents[i],1);
    // Check the opponent is in the home area, if not,
    // pretend it is further away to make sure that
    // opponents in the area are preferred.
    if ((constraint.home) && 
	(
	 (status->opponents[i].X>constraint.hlimit.right) ||
	 (status->opponents[i].Y>constraint.hlimit.top) ||
	 (status->opponents[i].X<constraint.hlimit.left) ||
	 (status->opponents[i].Y<constraint.hlimit.bottom)
	 )
	) {
      distances[i]+=100.0;
    }
  }

  // Sort opponents based on distance from home location
  // NB Selection sort, not good, but better than
  // bubble sort, and players is much less than 100
  in = status->opponentcount-1;
  jn = status->opponentcount;
  for (i=0;i<in;i++) {
    lowindex=i;
    lowkey = distances[lowindex];
    for(j=i+1;j<jn;j++) {
      if (distances[j]<lowkey) {
	lowkey=distances[j];
	lowindex=j;
      }
    } 
    // swap the items
    swap = status->opponents[lowindex];
    status->opponents[lowindex]=status->opponents[i];
    distances[lowindex]=distances[i];
    status->opponents[i]=swap;
    distances[i]=lowkey;
  }

#ifdef MCPLDBTrackerPos
  for(i=0;i<jn;i++) {
    cout << "AutoSel: Rank " << i << " Player is " << distances[i];
    cout << " away at (";
    cout << status->opponents[i].X << ",";
    cout << status->opponents[i].Y << ").\n";
  }
#endif

  // Now scan for the most suitable candidate
  target = -1;
  j=0;
  while((j<status->opponentcount) && (target<0)) {
    // Check that the opponent is in the players home area
    if ((constraint.home) &&
	(
	 (status->opponents[j].X>constraint.hlimit.right) ||
	 (status->opponents[j].Y>constraint.hlimit.top) ||
	 (status->opponents[j].X<constraint.hlimit.left) ||
	 (status->opponents[j].Y<constraint.hlimit.bottom)
	 )
	) {
      // if not, use the previous player
      // (unless this is the first player, in which case
      // we are stuck with it)
      // Opponents outside the home area are always sorted
      // further away than those inside, so as soon as
      // the first one is found it is not necessary to
      // test later opponents.
#ifdef MCPLDBTrackerPos
      cout << "j is (" << j << ").\n";
#endif 
      if (j==0) {
	if (!constraint.hmly) {
	  // allowing opponents outside home area
	  target=0;
	} else {
	  // not allowing opponents outside home area
	  j=status->opponentcount;
	}
      } else {
	target=j-1;
      }
#ifdef MCPLDBTrackerPos
      cout << "target is (" << target << ").\n";
#endif 
    } else {
      tracked=0;
      i=0;
      while((i<status->teamcount) && (!tracked)) {
	if ((status->teammates[i].X<status->opponents[j].X) ||
	    (!constraint.semi)) {
	  // Only consider teammates closer to the goal than
	  // the opponent (along the X axis) unless the
	  // semicircle rule has been disabled.
	  if (separation(status->opponents[j],
			 status->teammates[i],1)<ranges.outmark) {
	    // if the teamate is inside the opponents outer marker
	    // assume the teammate is tracking the opponent.
	    tracked=1;
	  }
	}
	i++;
      }
      if (!tracked) {
	target=j;
      }
    }
    j++;
  }

  if (target>=status->opponentcount) {
#ifdef MCPLDBTrackerPos
    cout << "target out of range (" << target << ").\n";
#endif 
    target = -1;
  }

  return target;
}

point MCPLTrackerPos::targetpoint(point S, point E, MCPLPFLOAT dist) {
  point retval,T;
  int changes=0;

  MCPLPFLOAT dx,dy;
  int dxz,dyz;
  MCPLPFLOAT xlim;

  if (constraint.fmgl) {
    dx = S.X-E.X;
    dy = S.Y-E.Y;
  } else {
    dx = E.X-S.X;
    dy = E.Y-S.Y;
  }

#ifdef MCPLDBTrackerPos
    cout << "dx is " << dx << ".\n";
    cout << "dy is " << dy << ".\n";
#endif

  retval=offsetpoint(dx,dy,dist);

  if (constraint.fmgl) {
    retval.X+=E.X;
    retval.Y+=E.Y;
  } else {
    retval.X+=S.X;
    retval.Y+=S.Y;
  }

  T=retval;

  if (constraint.home) {
    // Need to check that the point found is within the players
    // home area
    // Set up a zero flag so bad things(tm) do not occur
#ifdef MCPLDBTrackerPos
    cout << "Applying constraints.\n";
#endif
    if (fabs(dx)<0.0001) {
      dxz=1;
    } else {
      dxz=0;
    }
    
    if (fabs(dy)<0.0001) {
      dyz=1;
    } else {
      dyz=0;
    }

    if ((S.X>constraint.hlimit.right) || !constraint.oppx) {
      xlim=constraint.hlimit.right;
    } else {
      xlim=S.X;
    }

    if (retval.X>xlim) {
#ifdef MCPLDBTrackerPos
      cout << "Outside right constraint.\n";
#endif
      if (!dxz) {
	// if dx!=0 then the Y value can be safely calculated,
	// otherwise trust that the Y values will bring X back
	// into range.
	retval.X=xlim;
	retval.Y=(dy*((xlim-S.X)/dx))+S.Y;
      }
      // Mark the change even if the position was not recalculated,
      // it should have been
      changes++;
    }
    if (retval.Y>constraint.hlimit.top) {
#ifdef MCPLDBTrackerPos
      cout << "Outside top constraint.\n";
#endif
      if (!dyz) {
	retval.Y=constraint.hlimit.top;
	retval.X=(dx*((constraint.hlimit.top-S.Y)/dy))+S.X;
      }
      changes++;
    }
    if (retval.X<constraint.hlimit.left) {
#ifdef MCPLDBTrackerPos
      cout << "Outside left constraint.\n";
#endif
      if (!dxz) {
	retval.X=constraint.hlimit.left;
	retval.Y=(dy*((constraint.hlimit.left-S.X)/dx))+S.Y;
      }
      changes++;
    }
    if (retval.Y<constraint.hlimit.bottom) {
#ifdef MCPLDBTrackerPos
      cout << "Outside bottom constraint.\n";
#endif
      if (!dyz) {
	retval.Y=constraint.hlimit.bottom;
	retval.X=(dx*((constraint.hlimit.bottom-S.Y)/dy))+S.X;
#ifdef MCPLDBTrackerPos
	cout << "dx is " << dx << ".\n";
	cout << "S.Y is " << S.Y << ".\n";
	cout << "constraint.hlimit.bottom " << constraint.hlimit.bottom << ".\n";
	cout << "dy is " << dy << ".\n";
	cout << "S.X is " << S.X << ".\n";
	cout << "New x is " << retval.X << ".\n";
#endif
      }
      changes++;
    }
    if (changes>0) {
      // Had to adjust the point to allow for home boundaries
      if (
	  (retval.X>xlim) ||
	  (retval.Y>constraint.hlimit.top) ||
	  (retval.X<constraint.hlimit.left) ||
	  (retval.Y<constraint.hlimit.bottom)
	  ) {
	// Still outside of home area, which means the line does
	// pass through the home area, so have to do something else
	// In this case, just pick the closest point in the home location
	// to the originally selected target point
	if (T.X>xlim) {
	  retval.X=xlim;
	} else {
	  retval.X=T.X;
	}
	if (retval.X<constraint.hlimit.left) {
	  retval.X=constraint.hlimit.left;
	}
	if (T.Y>constraint.hlimit.top) {
	  retval.Y=constraint.hlimit.top;
	} else {
	  retval.Y=T.Y;
	}
	if (retval.Y<constraint.hlimit.bottom) {
	  retval.Y=constraint.hlimit.bottom;
	}
      }
    }
  }

  return retval;
}

point MCPLTrackerPos::findposition(MCPLStatus *exstatus) {
 
  point track;
  int useopp=0;
  MCPLPitchFig diag;
  MCPLStatus lstatus(*exstatus);
  MCPLStatus *status=&lstatus;

#ifdef MCPLDBTrackerPos
  int i;
  cout << "findposition: OpponentCount is " << status->opponentcount << ".\n";
  for(i=0;i<status->opponentcount;i++) {
    cout << "findposition: Opponent " << i << " is at (";
    cout << status->opponents[i].X << ",";
    cout << status->opponents[i].Y << ").\n";
  }
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
#ifdef MCPLDBTrackerPos
      cout << "Drawing diagram." << endl;
#endif
      diag.openfile(name+":"+special+":"+MCString(dicount)+".fig");
      basediagram(diag);
      diag.drawBall(status->ball.X,status->ball.Y);
      for(int i=0;i<status->teamcount;i++) {
	diag.drawTeammate(status->teammates[i].X,status->teammates[i].Y);
      }
      for(int i=0;i<status->opponentcount;i++) {
	diag.drawOpponent(status->opponents[i].X,status->opponents[i].Y);
      }
      diag.setDefault();
      diag.setFill();

    }
  }

  if (constraint.asel) {
    useopp = autoselect(status);
  }

  if ((useopp<0) || (status->opponentcount==0)) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	diag.setCentreJustify();
	diag.drawText(MCString("Resorting to failsafe"),-17,0,40,5);
	diag.drawText(MCString("(No available opponents)"),0,10,40,5);
	diag.setLeftJustify();
      }
    }
    // No available opponents to track.
    if (next!=0) {
      // pass on original status, not local copy.
      return next->findposition(exstatus);
    } else {
      return home;
    }
  } else {
    track = status->opponents[useopp];
  }

  MCPLPFLOAT gdist = separation(track,status->owngoal,1);

  if ((constraint.fail) && (gdist<ranges.mindist)) {
    if (dinterval) {
      if (!(dicount % dinterval)) {
	diag.setFontSize(128);
	diag.setRed();
	diag.setDepth(20);
	diag.setCentreJustify();
	diag.drawText(MCString("Resorting to failsafe"),0,0,40,5);
	diag.drawText(MCString("(Proximity Failure)"),0,10,40,5);
	diag.setLeftJustify();
      }
    }
    // Object is too close
    if (next!=0) {
      // pass on original status, not local copy.
      return next->findposition(exstatus);
    } else {
      return home;
    }
  }

  MCPLPFLOAT rdist = ranges.reldist;
  if ((ranges.reldist*gdist)<ranges.mindist) {
    rdist=ranges.mindist/gdist;
  }

#ifdef MCPLDBTrackerPos
  cout << "Relative distance is " << rdist << ".\n";
#endif

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setFill();
      diag.setBlack();
      diag.setDashedLine();
      diag.drawLine(track.X,track.Y,status->owngoal.X,status->owngoal.Y);
      if (ranges.outmark!=0) {
	diag.drawCircle(track.X,track.Y,ranges.outmark);
      }
      if (constraint.fmgl) {
	if (ranges.mindist!=0) {
	  diag.drawCircle(status->owngoal.X,status->owngoal.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(status->owngoal.X,status->owngoal.Y,
			  gdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(status->owngoal.X,status->owngoal.Y,rdist*gdist);
      } else {
	if (ranges.mindist!=0) {
	  diag.drawCircle(track.X,track.Y,ranges.mindist);
	}
	diag.setSolidLine();
	if (ranges.reldist>0.0) {
	  diag.drawCircle(track.X,track.Y,gdist*ranges.reldist);
	}
	diag.setRed();
	diag.setDottedLine();
	diag.drawCircle(track.X,track.Y,rdist*gdist);
      }
    }
  }

  point retval;

  retval=targetpoint(track,status->owngoal,rdist);

  if (dinterval) {
    if (!(dicount % dinterval)) {
      diag.setDefault();
      diag.setSolidLine();
      diag.setFill();
      diag.setBlue();
      diag.setDepth(80);
      diag.setSquarePoint();
      diag.setPointSize(1);
      diag.plotPoint(retval.X,retval.Y);
    }
  }

  dicount++;
  return retval;
}

void MCPLTrackerPos::resetoptions(void) {
  // This resets all the options to sensible default values
  ranges.mindist=5.0;
  ranges.reldist=0.0;
  ranges.outmark=10.0;

  constraint.home=0;
  constraint.oppx=1;
  constraint.asel=0;
  constraint.useg=0;
  constraint.semi=1;
  constraint.hmly=0;
  constraint.fmgl=0;
  constraint.fail=0;
  constraint.hlimit.top=pitch.top;
  constraint.hlimit.left=pitch.left;
  constraint.hlimit.bottom=pitch.bottom;
  constraint.hlimit.right=pitch.right;
}

void MCPLTrackerPos::readoptions(MCString section,MCIni &ini) {
#ifdef MCPLDBTrackerPos
  cout << "Specialisation =  " << special << endl;
#endif
  // Read the options from the specified section of
  // the ini object. If no value is defined, use the
  // current value.

  ranges.mindist=readOption(section,"MinimumDistance",ini,ranges.mindist);
  ranges.reldist=readOption(section,"RelativeDistance",ini,ranges.reldist);
  ranges.outmark=readOption(section,"OuterMarker",ini,ranges.outmark);

#ifdef MCPLDBTrackerPos
  cout << "Minimum distance is  " << ranges.mindist << ".\n";
  cout << "Relative distance is " << ranges.reldist << ".\n";
  cout << "Outer marker is      " << ranges.outmark << ".\n";
#endif

  constraint.home= readOption(section,"Home",ini,constraint.home);
  constraint.oppx= readOption(section,"OpponentX",ini,constraint.oppx);
  constraint.asel= readOption(section,"AutoSelect",ini,constraint.asel);
  constraint.useg= readOption(section,"UseGoalDist",ini,constraint.useg);
  constraint.semi= readOption(section,"SemiCircle",ini,constraint.semi);
  constraint.hmly= readOption(section,"HomeOnly",ini,constraint.hmly);
  constraint.fmgl= readOption(section,"FromGoal",ini,constraint.fmgl);
  constraint.fail= readOption(section,"ProximityFailure",ini,constraint.fail);

  if (constraint.home) {
    // Read the home constraints. These are 'unusual' in that
    // they depend on the player, not the method used
    // If the constraint is not specified use the limit of the pitch
    // in the given direction.
    constraint.hlimit.top=ini.getclassed(name,special+":HomeTop",pitch.top);
    if (constraint.hlimit.top>pitch.top) { // sanity check
      constraint.hlimit.top=pitch.top;
    }
    constraint.hlimit.left=ini.getclassed(name,special+":HomeLeft",pitch.left);
    if (constraint.hlimit.left<pitch.left) { // sanity check
      constraint.hlimit.left=pitch.left;
    }
    constraint.hlimit.bottom=ini.getclassed(name,special+":HomeBottom",
					    pitch.bottom);
    if (constraint.hlimit.bottom<pitch.bottom) { // sanity check
      constraint.hlimit.bottom=pitch.bottom;
    }
    constraint.hlimit.right=ini.getclassed(name,special+":HomeRight",
					   pitch.right);
    if (constraint.hlimit.right>pitch.right) { // sanity check
      constraint.hlimit.right=pitch.right;
    }
  }
}

// Build a new Positioner of the specified type using the specified
// options.
MCPLPositioner *MCPLTrackerPos::make(MCString method,
				     MCString player,
				     MCString spec,
				     MCIni& ini,
				     MCPLPositioner *failsafe) {

  MCPLPositioner *retval=0;

  if (method==ident()) {
    retval = new MCPLTrackerPos(method,player,spec,ini,failsafe);
  }
  return retval;
}

MCPLTrackerPos::MCPLTrackerPos(MCString method,
			       MCString player,
			       MCString spec,
			       MCIni &ini,
			       MCPLPositioner* failsafe)
  :MCPLPositioner(method,player,spec,ini,failsafe) {

  pitch.length = ini.get("Pitch","Length",105.0);
  pitch.width = ini.get("Pitch","Width",68.0);
  pitch.top = pitch.width/2.0;
  pitch.bottom = - pitch.top;
  pitch.right = pitch.length/2.0;
  pitch.left = - pitch.right;

  resetoptions();
  readoptions(method,ini);
}

MCPLTrackerPos::MCPLTrackerPos(MCExemplar e): MCPLPositioner(e) {
}

MCPLTrackerPos::MCPLTrackerPos() {
  next = 0;
  name = MCString("[X1]");
  home.X = 0.0;
  home.Y = 0.0;
  resetoptions();
}

MCPLTrackerPos::MCPLTrackerPos(const MCPLTrackerPos& copyme) {
  next = copyme.next;
  name = copyme.name;
  home = copyme.home;
  pitch = copyme.pitch;
  ranges = copyme.ranges;
  constraint = copyme.constraint;
}

MCPLTrackerPos& MCPLTrackerPos::operator=(const MCPLTrackerPos& assignme) {
  next = assignme.next;
  name = assignme.name;
  home = assignme.home;
  pitch = assignme.pitch;
  ranges = assignme.ranges;
  constraint = assignme.constraint;
  return *this;
}

MCPLTrackerPos::~MCPLTrackerPos() {
}

static MCPLTrackerPos MCPLTrackerPosInstance(ExemplarInstance);


