// File    : MCPLTrackerPos.h
//
// Purpose : C++ PLTrackerPos class
// Author  : Matthew Hunter
// Version : 0.01 (06 Jul 1999)
//                Derived from MCPLBoxMapPos.h
//           0.02 (12 Jul 1999)
//                Modifications to make use of the new features of
//                MCPLPositioner, including:
//                Revision of make to take extra arguments.
//                Revision of constructor to take extra arguments.
//                Addition of ident and diagram functions.
//           0.03 (14 Jul 1999)
//                Minor corrections
//           0.04 (15 Jul 1999)
//                Corrected bug in Home limits, added options for
//                measuring distance from goal and proximity failure.
//                Added option to limit targets to those within
//                the home limits.


#ifndef MCPLTRACKERPOS_h
#define MCPLTRACKERPOS_h

// Other PosLib Includes
#include "MCPLpoint.h"
#include "MCPLStatus.h"
#include "MCPLPositioner.h"
#include "MCPLPitchFig.h"

// MCLib includes
#include "MCExemplar.h"
#include "MCString.h"
#include "MCIni.h"

#include <iostream.h>

class MCPLTrackerPos: public MCPLPositioner {
 public:
  // Exemplar member functions

  // choose the best opponent to track
  int autoselect(MCPLStatus *);

  // Calculate the point distance dist of the way along the line
  // between S and E, and constrain the point to be within the
  // home boundaries if necessary
  point targetpoint(point S, point E, MCPLPFLOAT dist);

  // Find the best tracking position based on the status
  point findposition(MCPLStatus *);

  MCString ident(void);
  void basediagram(MCPLPitchFig&);
  void diagram(void);

  // Initialise all the options prior to use
  virtual void resetoptions(void);

  // Read the options from the named group within the
  // given configuration object.
  virtual void readoptions(MCString,MCIni&);

  // External functions, provide the positioning facilities
  MCPLPositioner *make(MCString,MCString,MCString,MCIni&,MCPLPositioner*);

  // Constructor for using the specified name and options.
  // Note that it is not necessary for the name to match the name
  // of the positioner being constructed.
  MCPLTrackerPos(MCString,MCString,MCString,MCIni&,MCPLPositioner *);

  // Create a prototype MCPL positioner
  // This constructor should never be invoked from a
  // program directly, use the 'positioner' variable
  // exported below to create instances.
  MCPLTrackerPos(MCExemplar); 

  // Standard member functions
  // Standard constructors
  MCPLTrackerPos();  // Default constructor
  MCPLTrackerPos(const MCPLTrackerPos&); // Copy constructor

  // Standard operations
  MCPLTrackerPos& operator=(const MCPLTrackerPos&); // assignment

  // Destructor
  ~MCPLTrackerPos();
 private:
 protected:
  struct {
    MCPLPFLOAT length;
    MCPLPFLOAT width;
    MCPLPFLOAT top;
    MCPLPFLOAT bottom;
    MCPLPFLOAT left;
    MCPLPFLOAT right;
  } pitch; // Size and boundaries of the pitch
  struct {
    int home; // Apply home area constraints
    int oppx; // Limit X travel to X location of opponent
    int asel; // Auto select opponent from all the opponents
              // passed via the MCPLStatus object.
    int useg; // Rank opponents by distance from goal rather than
              // distance from home location
    int semi; // Limit outer marker area to be on the goal side
              // of the selected opponent.
    int hmly; // Fail if none of the opponents are in the home area.
    int fmgl; // Measure relative distance from goal not trackee
    int fail; // Fail if the distance between the goal and
              // trackee is less than the minimum distance.
    struct {
      MCPLPFLOAT top;
      MCPLPFLOAT left;
      MCPLPFLOAT bottom;
      MCPLPFLOAT right;
    } hlimit; // Size and boundaries of the home location
  } constraint; // Constraints in force
  struct {
    MCPLPFLOAT mindist;
    MCPLPFLOAT reldist;
    MCPLPFLOAT outmark;
  } ranges;
};

#endif



