// File    : MCPLpoint.h
//
// Purpose : C++ point class
// Author  : Matthew Hunter
// Version : 0.01 (16 Jun 1999)
//           0.02 (23 Jun 1999)
//                Added makepoint to convert a string in the format
//                "(x,y)" to a point value when x and y are valid
//                doubles.
//           0.03 (23 Jun 1999)
//                Added separation functions to calculate the distance
//                between two points or two pairs of values.

#ifndef MCPLPOINT_H
#define MCPLPOINT_H

#include "MCString.h"
#include "math.h"

#ifndef MCPLPFLOAT
#define MCPLPFLOAT float
#endif

#ifndef EXTERNAL_POINT
typedef struct {
  MCPLPFLOAT X;
  MCPLPFLOAT Y;
} point;
#else
#include "GlobalParams.H"
#endif

point makepoint(MCString);

// Separation functions return the distance between two points, or two
// pairs of values.
//
// Notes:
// The value returned has a minimum value of 1, preventing divide
// by 0 errors.
// The optional final argument is nominally a 'power' figure, although
// currently any value other than 2 is equivalent to 1.
// ie. p = 1 result is the distance between the points, p=2 result is
// the square of the distance. This is useful for preventing the slow
// sqrt call when only the comparitive distances are important.
inline MCPLPFLOAT separation(MCPLPFLOAT x,MCPLPFLOAT y,
			     MCPLPFLOAT ox, MCPLPFLOAT oy, int p=1) {
  MCPLPFLOAT dx,dy;
  MCPLPFLOAT sep;

  dx = fabs(x-ox);
  dy = fabs(y-oy);
  sep = (dx*dx)+(dy*dy);
  if (p!=2) sep = sqrt(sep);
  return sep<1.0 ? 1 : sep;
}

inline MCPLPFLOAT separation(point a,point b, int p=1) {
  MCPLPFLOAT dx,dy;
  MCPLPFLOAT sep;

  dx = fabs(a.X-b.X);
  dy = fabs(a.Y-b.Y);
  sep = (dx*dx)+(dy*dy);
  if (p!=2) sep = sqrt(sep);
  return sep<1.0 ? 1 : sep;
}

#endif











